unit coscoeff;

    //
    // Cosine.pas
    //
    // Ephemerides of planets between 1900 and 2100 Chapront J., Francou G.
    // Bureau des Longitudes, Group : Dynamics of Solar System (1996)
    //
    // Tables and coefficients from the series96.xxx files converted to
    // pascal structures. - J. Burton
    //

interface

const

    //
    // Cosine Coefficients of Poisson Series
    //

    // mer
    merct   : array [1..20,1..224,0..2,1..3] of double = (
         (((       -20329.0,         6141.0,         4687.0),
           (         2553.0,         4881.0,         2322.0),
           (         5747.0,        -2192.0,         1265.0)),
          ((         -512.0,        -3852.0,        -1875.0),
           (        -5273.0,       -51098.0,       -26648.0),
           (        38636.0,         2153.0,        -4519.0)),
          ((          253.0,         1956.0,          946.0),
           (         2776.0,         -715.0,         -782.0),
           (            0.0,            0.0,            0.0)),
          ((          345.0,         1255.0,          605.0),
           (       -46664.0,        -1970.0,        33624.0),
           (            0.0,            0.0,            0.0)),
          ((        -4460.0,        17167.0,         9236.0),
           (       -39225.0,      -134863.0,       -66447.0),
           (            0.0,            0.0,            0.0)),
          ((         1001.0,        -1567.0,         -896.0),
           (        -3016.0,        -4796.0,        -2075.0),
           (            0.0,            0.0,            0.0)),
          ((       -16829.0,         9049.0,         6992.0),
           (       -20113.0,        -8831.0,         -943.0),
           (            0.0,            0.0,            0.0)),
          ((         -181.0,         2548.0,         1290.0),
           (        11187.0,         3570.0,          750.0),
           (            0.0,            0.0,            0.0)),
          ((         -926.0,          868.0,          593.0),
           (          -14.0,          791.0,          458.0),
           (            0.0,            0.0,            0.0)),
          ((         -274.0,          149.0,           99.0),
           (          -83.0,         -131.0,          -54.0),
           (            0.0,            0.0,            0.0)),
          ((          470.0,         -450.0,         -266.0),
           (           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          729.0,         -710.0,         -428.0),
           (           70.0,          -48.0,          -48.0),
           (            0.0,            0.0,            0.0)),
          ((         7728.0,         3938.0,         1143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -533.0,          -97.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1795.0,        -1711.0,         -777.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -289.0,          238.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3465.0,        -4583.0,        -2629.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -743.0,         1515.0,          976.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1084.0,        -2533.0,        -1765.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,            1.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3277.0,        -4412.0,        -1910.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          674.0,         -930.0,         -496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1267.0,         -732.0,         -378.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1011.0,          126.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          744.0,         -447.0,         -254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -307.0,           94.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1259.0,          634.0,          456.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -157.0,         1012.0,          503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34727.0,         1785.0,         5793.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2686.0,         6573.0,         2479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3212.0,         7808.0,         4243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          301.0,         -577.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1412.0,           81.0,         -332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5005.0,          560.0,          585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        76943.0,        18149.0,         4827.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     13767220.0,   3359427614.0,   1792507686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        64118.0,       -10784.0,       -13868.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1194.0,         -211.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3896.0,          124.0,         -608.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -68712.0,        11650.0,        17653.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63047.0,       -10828.0,       -16274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4612.0,        -2269.0,        -2103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1267.0,         -817.0,         -343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25599.0,        -9657.0,        -2270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -569.0,          766.0,          384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1159.0,         -604.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          687.0,         -765.0,         -468.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           33.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          291.0,         -226.0,         -136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1430.0,         2293.0,          958.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          482.0,         -589.0,         -339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1502.0,         2681.0,         1530.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          193.0,         -132.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          539.0,         -556.0,         -317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1428.0,         2275.0,         1011.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,          -80.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -362.0,         -720.0,         -326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -20.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3888.0,        -1801.0,        -1225.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          372.0,          523.0,          196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,          146.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          -43.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -247.0,          164.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -259.0,         -942.0,         -441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7633.0,           17.0,          759.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          526.0,          482.0,          199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1629.0,        -5775.0,        -2803.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,           78.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,         -133.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          122.0,           68.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -783.0,         -200.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -785.0,         2104.0,         1484.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1057.0,          459.0,          384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -83905539.0,    328931322.0,    184374521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17091.0,          437.0,        -1699.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6358.0,          904.0,         -129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2068.0,         -537.0,         -135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          278.0,          259.0,          138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           49.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,          -89.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          547.0,         -453.0,         -325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,          -37.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9715.0,        -5878.0,        -2100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,           26.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,         -685.0,         -341.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -364.0,          208.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,         -102.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           16.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,          -31.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          192.0,          602.0,          286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          111.0,         -165.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1049.0,          857.0,          577.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -24.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          -66.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,          -16.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          392.0,         1049.0,          512.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,          -14.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,         -169.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,           18.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1135.0,         -246.0,         -242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          109.0,          198.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           23.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,           -5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,         -340.0,         -174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1626.0,         -190.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          -47.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,        -1764.0,         -949.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,         -116.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,            9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -20.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,           51.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1410.0,          -76.0,          154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,         -268.0,         -111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -25174394.0,     45784061.0,     27066478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1073.0,          131.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4697.0,         1256.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           -8.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           55.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -17.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          153.0,          -76.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -15.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2271.0,        -2112.0,         -887.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,         -166.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,           26.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -17.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,          -30.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,          199.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,          -56.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -337.0,          162.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,          -16.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          325.0,          169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -35.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          300.0,           -4.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           55.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -82.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -379.0,         -127.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           -2.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,         -506.0,         -280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          -15.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,           19.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,          -12.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           34.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -646.0,         -160.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,         -129.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -6561219.0,      7073702.0,      4459594.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,          -36.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1226.0,          597.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -2.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          155.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,         -138.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -11.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           -7.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -11.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -452.0,         -650.0,         -301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -34.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           32.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,           21.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           88.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,           16.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -16.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -76.0,          -52.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,         -127.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,          -83.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,          -22.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1637394.0,      1096062.0,       755562.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          184.0,          155.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          136.0,           89.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,          -28.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,         -189.0,          -93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           22.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -16.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -27.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -32.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -398492.0,       154482.0,       123928.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -23.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           67.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -52.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -94904.0,        15407.0,        18096.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           19.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22089.0,         -887.0,         1823.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5003.0,        -1211.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1094.0,         -510.0,         -158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -228.0,         -170.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          -51.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        28544.0,        42147.0,        19162.0),
           (        -2414.0,         -162.0,          132.0),
           (       -32784.0,      -108665.0,       -53753.0)),
          ((         -553.0,        -5977.0,        -3041.0),
           (        39154.0,         8989.0,          314.0),
           (        16280.0,        80735.0,        38656.0)),
          ((          550.0,         3670.0,         1849.0),
           (        -4136.0,         -394.0,          204.0),
           (            0.0,            0.0,            0.0)),
          ((        -1097.0,          477.0,          361.0),
           (        49739.0,        10390.0,       -30974.0),
           (            0.0,            0.0,            0.0)),
          ((       -19933.0,        -3512.0,          118.0),
           (        59185.0,        67972.0,        29478.0),
           (            0.0,            0.0,            0.0)),
          ((         1702.0,          344.0,           27.0),
           (         5310.0,          629.0,         -295.0),
           (            0.0,            0.0,            0.0)),
          ((       -16755.0,         8610.0,         6756.0),
           (       -13974.0,         -146.0,         3245.0),
           (            0.0,            0.0,            0.0)),
          ((         -641.0,         2592.0,         1369.0),
           (       -13235.0,        -5299.0,        -1556.0),
           (            0.0,            0.0,            0.0)),
          ((          660.0,        -1346.0,         -784.0),
           (         -639.0,         -710.0,         -311.0),
           (            0.0,            0.0,            0.0)),
          ((          323.0,          250.0,           86.0),
           (         -319.0,         -192.0,          -59.0),
           (            0.0,            0.0,            0.0)),
          ((         -623.0,         -107.0,           14.0),
           (           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -361.0,         1449.0,          751.0),
           (            3.0,          -93.0,          -43.0),
           (            0.0,            0.0,            0.0)),
          ((        -4586.0,         5488.0,         3051.0),
           (            0.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          936.0,         1198.0,          500.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2692.0,        -2543.0,        -1082.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1100.0,         1755.0,          737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          411.0,        -7601.0,        -3729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3450.0,         4493.0,         1656.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3095.0,        -1642.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          554.0,         1647.0,          792.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3171.0,         3153.0,         1325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1483.0,         2141.0,          731.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,         -927.0,         -703.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          724.0,         2132.0,         1095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -708.0,        -2432.0,        -1295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          328.0,         1180.0,          595.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1875.0,        -1400.0,         -593.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3618.0,         4873.0,         2164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9450.0,       -64180.0,       -34812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21067.0,        38018.0,        18191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1413.0,        19650.0,         9923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5838.0,       -15003.0,        -7305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5285.0,        18074.0,         8974.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4850.0,       -23014.0,       -11525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15675.0,       -44853.0,       -25124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -207848954.0,  -3363502571.0,  -1774563349.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15445.0,        63327.0,        30519.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1446.0,        -5809.0,        -2679.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          884.0,        11245.0,         5485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7110.0,      -154127.0,       -77697.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6504.0,       142710.0,        71935.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3419.0,         4737.0,         2700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,        -2896.0,        -1463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26519.0,        11139.0,         2916.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,        -2756.0,        -1338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,        -1703.0,         -707.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,         2558.0,         1238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,         -298.0,         -150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          333.0,          178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1884.0,        -2684.0,        -1400.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -313.0,          824.0,          448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1429.0,        -2770.0,        -1571.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -147.0,          148.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -563.0,          436.0,          268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1692.0,         2067.0,          884.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -87.0,           10.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -374.0,         -749.0,         -342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,            5.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2404.0,        -3498.0,        -1478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -362.0,         -588.0,         -230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,          -81.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -46.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          203.0,           20.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,         -938.0,         -422.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7331.0,          923.0,         1217.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          276.0,          312.0,          130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5825.0,          484.0,          910.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -233.0,          321.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,           -1.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          175.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          561.0,         -401.0,         -264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2712.0,          859.0,          100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4573.0,         1527.0,          199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -44817235.0,    336693249.0,    184459269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2891.0,       -10705.0,        -5322.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2548.0,       -10632.0,        -5943.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2201.0,         6587.0,         3747.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          992.0,        -2501.0,        -1437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -255.0,          478.0,          281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          125.0,          -53.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          652.0,          509.0,          207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,         -149.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10123.0,        -5153.0,        -1670.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -233.0,           48.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -153.0,          228.0,          126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          626.0,          103.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -254.0,           -7.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,           -4.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -149.0,           49.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,          287.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -154.0,           12.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -824.0,          810.0,          529.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -157.0,           81.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          375.0,         -230.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -328.0,          195.0,          139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -315.0,        -1134.0,         -570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -110.0,           93.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          155.0,           68.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,           69.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          476.0,          979.0,          458.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,          188.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           28.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           10.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,          328.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1694.0,          -78.0,         -212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,           55.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2232.0,          -79.0,         -289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,           66.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          141.0,          -42.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -148.0,          -34.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1631.0,          377.0,          393.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1463.0,         -200.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     16695792.0,    -49077652.0,    -27943981.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,         -167.0,         -134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -482.0,         4139.0,         2246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,          -46.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,           82.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -19.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -166.0,         -130.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           17.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2512.0,         1844.0,          719.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,           -2.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,          -27.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -83.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           42.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           49.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          193.0,         -155.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,           58.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          312.0,         -192.0,         -133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           10.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          315.0,          157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -33.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,         -264.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -50.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -75.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -370.0,          -65.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           17.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -566.0,         -110.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,           23.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          140.0,          -98.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -327.0,          215.0,          113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          627.0,         -233.0,         -161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          362.0,          159.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -4712062.0,      8202126.0,      4869896.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -65.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          284.0,         -954.0,         -566.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,          -87.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -238.0,          386.0,          133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          215.0,         -349.0,         -121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           46.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -29.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -568.0,         -560.0,         -240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -12.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           10.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,           32.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -88.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           71.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           16.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,           29.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,           49.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,           25.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,          -66.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1244778.0,     -1448157.0,      -902744.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          153.0,           88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -117.0,          154.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,          -70.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           18.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          120.0,          168.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           23.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,          -16.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           -3.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           24.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -319900.0,       258248.0,       171163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -45.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,          -15.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -49.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        80852.0,       -44728.0,       -32290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20167.0,         7109.0,         5893.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4967.0,         -898.0,         -996.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1208.0,           29.0,          141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          290.0,           37.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,          -20.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((         -721.0,       -29935.0,       -14975.0),
           (        -9314.0,        -4128.0,        -1278.0),
           (       -62531.0,        17354.0,        13515.0)),
          ((        -5691.0,         6263.0,         3648.0),
           (       -56194.0,       -23431.0,        -4396.0),
           (        11293.0,       -18283.0,        -8212.0)),
          ((         3414.0,        -3608.0,        -2116.0),
           (         2204.0,         4477.0,         2165.0),
           (            0.0,            0.0,            0.0)),
          ((         1345.0,        -1912.0,        -1094.0),
           (       -42239.0,       -14374.0,        29903.0),
           (            0.0,            0.0,            0.0)),
          ((          678.0,       -16404.0,        -8490.0),
           (       -98468.0,       -64500.0,       -19889.0),
           (            0.0,            0.0,            0.0)),
          ((          829.0,         -237.0,         -188.0),
           (        -5059.0,        -2113.0,         -539.0),
           (            0.0,            0.0,            0.0)),
          ((       -16104.0,         9772.0,         7316.0),
           (       -25241.0,         -348.0,         5415.0),
           (            0.0,            0.0,            0.0)),
          ((         -510.0,         1649.0,          844.0),
           (         6503.0,         3059.0,         1327.0),
           (            0.0,            0.0,            0.0)),
          ((        -1314.0,          343.0,          343.0),
           (         1320.0,          140.0,          -95.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,          453.0,          229.0),
           (          205.0,          -50.0,          -63.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,         -709.0,         -374.0),
           (            7.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((          708.0,         -737.0,         -424.0),
           (         -148.0,          115.0,           81.0),
           (            0.0,            0.0,            0.0)),
          ((        -8585.0,        -3857.0,        -1053.0),
           (            0.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1140.0,          607.0,          220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2759.0,        -1811.0,         -734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          949.0,          651.0,          276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          557.0,        -5978.0,        -3154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2330.0,         2827.0,         1440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -929.0,        -3632.0,        -2179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          978.0,          248.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4499.0,        -4027.0,        -1603.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -490.0,          722.0,          340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1656.0,          925.0,          547.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1769.0,         -876.0,         -614.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1442.0,          132.0,          124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          654.0,          -24.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2592.0,          711.0,          549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2365.0,        -2181.0,        -1178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -64424.0,        24250.0,        18112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21549.0,        -6354.0,        -4912.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6113.0,        -9281.0,        -6115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8852.0,         2417.0,         1870.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10644.0,        -3062.0,        -2355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15719.0,         2757.0,         2743.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -56147.0,          227.0,         2390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    401361608.0,   3358554817.0,   1751872317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -49826.0,        -4412.0,         3924.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8866.0,         2983.0,         2388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9627.0,        -3605.0,        -2700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -118733.0,        51151.0,        35436.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       109416.0,       -47818.0,       -33031.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3831.0,           33.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2751.0,         1799.0,         1303.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25379.0,        -9690.0,        -2319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -840.0,         2277.0,         1181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1290.0,          270.0,          412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1035.0,        -1091.0,         -688.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,           69.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          255.0,         -353.0,         -197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2803.0,         -759.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          568.0,         -657.0,         -389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1520.0,         2714.0,         1553.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          210.0,         -130.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          458.0,         -566.0,         -333.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1544.0,         2193.0,          962.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,          -43.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -670.0,         -655.0,         -260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,            6.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3614.0,         2290.0,         1439.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          333.0,          601.0,          239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           80.0,           82.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,          -13.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -250.0,           68.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -487.0,        -1139.0,         -524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7118.0,         1967.0,         1725.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          350.0,          -20.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1218.0,         5011.0,         2433.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          235.0,          454.0,          205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          139.0,         -145.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,           99.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -537.0,         -981.0,         -408.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3792.0,         6829.0,         3799.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1314.0,         9623.0,         4962.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -5246023.0,    340831236.0,    182561003.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10445.0,        -3746.0,         -761.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11088.0,         3606.0,         2983.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6289.0,        -3773.0,        -2559.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2204.0,         1851.0,         1168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          496.0,         -385.0,         -246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -177.0,           48.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -231.0,          560.0,          351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,           39.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10458.0,        -4610.0,        -1344.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -169.0,          349.0,          194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -504.0,          -79.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,         -200.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           47.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,          -25.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -623.0,         -377.0,         -134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          207.0,           21.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -986.0,          879.0,          581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,           51.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,         -234.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,          161.0,           80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          529.0,          876.0,          408.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           62.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -157.0,         -242.0,         -111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,           92.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1051.0,          467.0,          347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          141.0,          206.0,           98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           10.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,          -21.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,         -381.0,         -187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1591.0,          406.0,          373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          104.0,         -146.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          194.0,         1972.0,         1024.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           80.0,          -23.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,           -9.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          -10.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -292.0,         -108.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -746.0,         2459.0,         1355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          248.0,         3418.0,         1745.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -7812849.0,     51182588.0,     28145261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -157.0,        -1485.0,         -747.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4063.0,         -402.0,          214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -139.0,         -235.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,          203.0,           98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -64.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           19.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -87.0,          107.0,           71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           33.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2761.0,        -1586.0,         -554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           90.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -120.0,          -36.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -44.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           16.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,          -53.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,          -24.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,          -33.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -292.0,          217.0,          145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,          -15.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,          308.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -35.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -288.0,           89.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           52.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,         -100.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -393.0,           58.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -42.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          571.0,          307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -31.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          152.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,         -347.0,         -232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,          -32.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -210.0,          761.0,          457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          664.0,          319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2660168.0,      8977588.0,      5070780.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          241.0,          169.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1089.0,         -392.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,           37.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,         -307.0,         -239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           87.0,          283.0,          218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           12.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           28.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -685.0,         -486.0,         -189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           23.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -16.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -25.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -77.0,           44.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           86.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,           16.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -22.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,           -5.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          144.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,          -11.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,          224.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          127.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -768593.0,      1703090.0,       989388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,          -82.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -149.0,          -46.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,            8.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,         -145.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           26.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,            5.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           34.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           52.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           47.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -210437.0,       337112.0,       201898.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -39.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,          -44.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           12.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -56217.0,        68196.0,        42261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14814.0,        13894.0,         8960.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3868.0,         2814.0,         1905.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1003.0,          558.0,          402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -258.0,          107.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           19.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -41955.0,       -14279.0,        -3727.0),
           (        -3841.0,           77.0,          363.0),
           (       117902.0,       -15803.0,       -16486.0)),
          ((        13039.0,         6045.0,         1961.0),
           (        25615.0,       -26839.0,       -15872.0),
           (      -113581.0,        49410.0,        33052.0)),
          ((        -8025.0,        -3369.0,        -1023.0),
           (         5594.0,          663.0,         -170.0),
           (            0.0,            0.0,            0.0)),
          ((        -1496.0,         -949.0,         -359.0),
           (        40095.0,        13865.0,       -32993.0),
           (            0.0,            0.0,            0.0)),
          ((        21697.0,         1876.0,        -1128.0),
           (       122867.0,       -62837.0,       -47166.0),
           (            0.0,            0.0,            0.0)),
          ((        -3178.0,        -1482.0,         -465.0),
           (         6403.0,         -729.0,        -1052.0),
           (            0.0,            0.0,            0.0)),
          ((       -14213.0,        10407.0,         7474.0),
           (       -13349.0,       -13166.0,        -2017.0),
           (            0.0,            0.0,            0.0)),
          ((         -499.0,         1838.0,          944.0),
           (        -9426.0,         7139.0,         4237.0),
           (            0.0,            0.0,            0.0)),
          ((         1765.0,        -1433.0,         -952.0),
           (           -3.0,         1666.0,          838.0),
           (            0.0,            0.0,            0.0)),
          ((         -465.0,          -73.0,            2.0),
           (          117.0,          157.0,           40.0),
           (            0.0,            0.0,            0.0)),
          ((          637.0,          350.0,          122.0),
           (            4.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((        -1239.0,         1109.0,          667.0),
           (           34.0,          151.0,           63.0),
           (            0.0,            0.0,            0.0)),
          ((         5411.0,        -7607.0,        -4153.0),
           (            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        -2588.0,          761.0,          598.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -333.0,        -1567.0,         -790.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3151.0,          247.0,          389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8087.0,        -4693.0,        -3032.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9310.0,         -473.0,          436.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6093.0,         1910.0,          792.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2830.0,          144.0,          396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6048.0,         3410.0,         1131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2332.0,         -543.0,          149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1461.0,        -1341.0,         -560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3374.0,          886.0,          523.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2977.0,        -1129.0,         -764.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1509.0,          432.0,          321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          950.0,          533.0,          229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6767.0,         1957.0,         1423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       102271.0,       -29636.0,       -23659.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -51344.0,         8697.0,         8319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15217.0,         1176.0,         1839.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17764.0,        -3093.0,        -2874.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19645.0,         1882.0,         2361.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18888.0,         1902.0,         -342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4383.0,        66676.0,        31642.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -593783933.0,  -3344636984.0,  -1724514118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4402.0,       -50921.0,       -26432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7018.0,        -9917.0,        -5460.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10711.0,        12319.0,         6883.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       137551.0,      -152719.0,       -85096.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -127964.0,       140885.0,        78608.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2606.0,         6169.0,         3104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3676.0,        -2004.0,        -1312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25336.0,         9583.0,         2169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          616.0,        -2775.0,        -1320.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -419.0,        -1977.0,         -768.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1197.0,         2284.0,         1210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,         -311.0,         -157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -140.0,          355.0,          195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -939.0,         1349.0,          625.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -303.0,          805.0,          438.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1413.0,        -2811.0,        -1590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,          295.0,          154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,          651.0,          366.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1762.0,         1996.0,          837.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,           73.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -568.0,         -630.0,         -270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           -5.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2925.0,         3295.0,         1330.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -371.0,         -573.0,         -220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,         -106.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           46.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          373.0,         -190.0,         -132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -343.0,         -986.0,         -465.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6840.0,         2618.0,         2047.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          295.0,          267.0,          112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5229.0,        -1684.0,        -1451.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          486.0,          196.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -180.0,          -59.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -390.0,           77.0,           88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2423.0,          373.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9640.0,        -6721.0,        -2386.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16563.0,        -4278.0,         -333.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     34387302.0,    341309552.0,    178703993.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2216.0,        14024.0,         7085.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10807.0,         4547.0,         3734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10560.0,         -125.0,        -1351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4726.0,         -830.0,          132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1190.0,          242.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -180.0,          -41.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -550.0,         -136.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -233.0,          -86.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10708.0,        -3763.0,         -872.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          173.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1213.0,          417.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -712.0,          -47.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          376.0,           44.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -122.0,           -8.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          287.0,           30.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -259.0,         -561.0,         -248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          533.0,          120.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1154.0,          880.0,          603.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          300.0,           81.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -357.0,         -358.0,         -147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          614.0,          249.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1078.0,        -1084.0,         -459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          247.0,           73.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -140.0,           83.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          175.0,           16.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -750.0,         -914.0,         -398.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,          189.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           26.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -16.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,           54.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          123.0,          366.0,          178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1454.0,         -667.0,         -495.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,          144.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2263.0,          154.0,          334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,          142.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -43.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          138.0,            0.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -503.0,           -5.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3453.0,         1936.0,          637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5014.0,         -305.0,         -719.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1261614.0,    -52050082.0,    -27666336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1384.0,          639.0,          502.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1512.0,        -4274.0,        -2428.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -308.0,           52.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,            1.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,           21.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -20.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          164.0,           77.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -19.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2951.0,         1252.0,          357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -69.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -191.0,          -82.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,          -45.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           29.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,           98.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,          -55.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,           75.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          260.0,         -255.0,         -161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           16.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           30.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          179.0,          285.0,          133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          -33.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,          236.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -49.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           18.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -27.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -82.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,          105.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,            6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          559.0,          -67.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,            8.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -297.0,          -34.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          613.0,           74.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           26.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1050.0,         -429.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -968.0,         -228.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -493087.0,      9367596.0,      5053980.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -197.0,          -33.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,         1006.0,          542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           70.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          380.0,         -225.0,         -238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -340.0,          192.0,          208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            0.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -17.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -759.0,         -371.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           18.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           23.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -22.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           55.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          -81.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -61.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           23.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          -19.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -166.0,            8.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           18.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          311.0,          131.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          202.0,           -1.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       240715.0,     -1843873.0,     -1009732.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,         -143.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          124.0,         -174.0,         -107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,           68.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           -9.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          188.0,          112.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           23.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           16.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,            2.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,          -37.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,           -9.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -80621.0,       383565.0,       213218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -28.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           60.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           12.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          -31.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        24213.0,       -82762.0,       -46715.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6933.0,        18317.0,        10503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1934.0,        -4127.0,        -2405.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -530.0,          943.0,          558.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,         -217.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           50.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        25534.0,        22020.0,         8834.0),
           (        12427.0,         2069.0,          -13.0),
           (       -21823.0,       -73971.0,       -35506.0)),
          ((        -8107.0,        -9939.0,        -4345.0),
           (        93460.0,        16325.0,        -1006.0),
           (        70186.0,        57471.0,        22534.0)),
          ((         4945.0,         5461.0,         2342.0),
           (          -47.0,        -2770.0,        -1443.0),
           (            0.0,            0.0,            0.0)),
          ((         1802.0,         1727.0,          721.0),
           (       -66974.0,        -3804.0,        42615.0),
           (            0.0,            0.0,            0.0)),
          ((         -239.0,        17210.0,         8839.0),
           (       143300.0,         9447.0,        -6690.0),
           (            0.0,            0.0,            0.0)),
          ((         1470.0,        -1266.0,         -778.0),
           (         3487.0,         -110.0,         -247.0),
           (            0.0,            0.0,            0.0)),
          ((       -16016.0,        10178.0,         7534.0),
           (        -3247.0,         9353.0,         8854.0),
           (            0.0,            0.0,            0.0)),
          ((          613.0,         2404.0,         1140.0),
           (        22589.0,         2516.0,         -562.0),
           (            0.0,            0.0,            0.0)),
          ((         -688.0,          805.0,          535.0),
           (        -1595.0,        -1058.0,         -443.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,          427.0,          226.0),
           (         -951.0,         -594.0,         -256.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,         -865.0,         -438.0),
           (            5.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((          412.0,         -670.0,         -369.0),
           (          158.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((         7660.0,         2210.0,          284.0),
           (            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,          753.0,          388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2667.0,        -1969.0,         -798.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          729.0,         1386.0,          670.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,        -7556.0,        -3872.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2434.0,         4901.0,         2485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1444.0,        -4974.0,        -2813.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          981.0,          862.0,          315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5395.0,        -3469.0,        -1256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1063.0,         -234.0,         -147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          463.0,        -1961.0,         -945.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          140.0,         2039.0,          903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -336.0,        -2181.0,        -1076.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,          948.0,          459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1338.0,         -149.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2335.0,         3914.0,         1833.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -50919.0,       -24956.0,        -7410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14095.0,        34064.0,        15735.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8875.0,        19666.0,        10134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4571.0,       -11041.0,        -5163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2754.0,        11959.0,         5800.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          762.0,       -13116.0,        -6889.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        77900.0,       -13014.0,       -12861.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    784651902.0,   3321743255.0,   1692535639.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        76295.0,        16423.0,         -954.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1864.0,        -7504.0,        -3623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4521.0,        10453.0,         4816.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -71527.0,      -121885.0,       -53409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        65724.0,       112945.0,        49603.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3356.0,         2097.0,          389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1697.0,        -2953.0,        -1441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26278.0,        -5780.0,         -167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,         -412.0,         -279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1008.0,        -1314.0,         -433.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          572.0,          580.0,          251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          -98.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,         -134.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2168.0,         1386.0,          472.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          281.0,         -452.0,         -243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1411.0,         2652.0,         1504.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,         -189.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,         -852.0,         -441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1631.0,         2311.0,         1009.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,         -175.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -800.0,         -311.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,         -114.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3177.0,        -2644.0,        -1565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          324.0,          517.0,          198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          189.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,          -87.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          399.0,          208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -347.0,         -893.0,         -413.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6491.0,         3566.0,         2492.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          904.0,         -183.0,         -191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3143.0,        -4420.0,        -1925.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          742.0,         -912.0,         -538.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -123.0,          336.0,          173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          572.0,         -763.0,         -458.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2438.0,         3191.0,         1912.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9728.0,       -10783.0,        -6624.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6254.0,       -13958.0,        -7913.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     73643208.0,    338119754.0,    172929593.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13176.0,         7579.0,         2500.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15105.0,        -8320.0,        -5970.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9644.0,         7529.0,         4978.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3328.0,        -2985.0,        -1923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -681.0,          769.0,          479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,         -163.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,         -576.0,         -329.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           74.0,         -117.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11171.0,        -2968.0,         -396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          156.0,           18.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -314.0,         -594.0,         -272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -166.0,          146.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          -69.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -54.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          582.0,          408.0,          152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,         -190.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -759.0,         1079.0,          662.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,          -69.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,          -78.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,         -123.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          736.0,         1021.0,          459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -79.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -199.0,           -9.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          -66.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          855.0,         -670.0,         -433.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,          172.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           41.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          -19.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           71.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,         -295.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1382.0,          889.0,          606.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          141.0,         -208.0,         -122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -804.0,        -1395.0,         -658.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,         -416.0,         -226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          167.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,         -145.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,          477.0,          274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2219.0,        -3342.0,        -1991.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1390.0,        -4902.0,        -2731.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     10302916.0,     51657872.0,     26518654.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -749.0,         2968.0,         1645.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5602.0,         -128.0,         -666.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -175.0,          284.0,          170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          109.0,         -128.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,           66.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -31.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,         -126.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -41.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3096.0,         -870.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,         -140.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,           27.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -13.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,          -59.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          179.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -75.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -223.0,          277.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -19.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -41.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          206.0,          284.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,          -15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          215.0,         -167.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           47.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           22.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -71.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -324.0,          180.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,         -521.0,         -256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          147.0,          -90.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -319.0,          292.0,          157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,          160.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          693.0,         -993.0,         -570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,         -875.0,         -514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1694473.0,      9354928.0,      4820136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          199.0,         -188.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1245.0,          345.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,         -158.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -920.0,         1197.0,          657.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          822.0,        -1075.0,         -590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -41.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -16.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -31.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -828.0,         -219.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -13.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -20.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,           64.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -12.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,           79.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,          -41.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -19.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           40.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,         -141.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           46.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          179.0,         -326.0,         -191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,         -126.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       303188.0,      1861026.0,       962423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          240.0,           77.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          191.0,          -21.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,           16.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -212.0,          -69.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           18.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           19.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           16.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -28.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,          -80.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -42.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        56869.0,       393043.0,       204003.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -53.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,           45.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -25.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,          -18.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -18.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10946.0,        86496.0,        45057.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2129.0,        19618.0,        10256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          411.0,         4555.0,         2390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,         1079.0,          568.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          259.0,          137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           62.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        35826.0,        -3489.0,        -4885.0),
           (       -12468.0,          430.0,         1376.0),
           (       -56626.0,       -13406.0,        -1865.0)),
          ((        -4724.0,         6279.0,         3602.0),
           (         -235.0,        35648.0,        19930.0),
           (        29111.0,        -2199.0,        -1126.0)),
          ((         2711.0,        -3271.0,        -1899.0),
           (        -4512.0,          538.0,          653.0),
           (            0.0,            0.0,            0.0)),
          ((         -651.0,         -589.0,         -228.0),
           (        72760.0,          -24.0,       -47992.0),
           (            0.0,            0.0,            0.0)),
          ((       -20296.0,         2261.0,         3109.0),
           (        -1116.0,       125222.0,        69053.0),
           (            0.0,            0.0,            0.0)),
          ((         1955.0,        -1080.0,         -726.0),
           (         3197.0,         1943.0,          775.0),
           (            0.0,            0.0,            0.0)),
          ((       -15647.0,        10694.0,         7758.0),
           (       -33477.0,        21726.0,        19285.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,         2577.0,         1314.0),
           (        -4436.0,        -5331.0,        -3064.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,        -1078.0,         -603.0),
           (         1309.0,        -2345.0,        -1470.0),
           (            0.0,            0.0,            0.0)),
          ((          396.0,         -131.0,         -116.0),
           (           23.0,         -371.0,         -247.0),
           (            0.0,            0.0,            0.0)),
          ((         -504.0,          515.0,          321.0),
           (           -3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((          395.0,         1090.0,          496.0),
           (          137.0,         -190.0,         -143.0),
           (            0.0,            0.0,            0.0)),
          ((        -3908.0,         7309.0,         3900.0),
           (           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         1446.0,         -370.0,         -349.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3314.0,         -965.0,         -172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1677.0,         -307.0,         -369.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2492.0,        -4139.0,        -1710.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4874.0,        -1593.0,        -1578.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3730.0,         2702.0,         2206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1002.0,          142.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4407.0,         2450.0,          863.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1906.0,          914.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -443.0,          893.0,          276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1510.0,         -857.0,         -508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1484.0,           63.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          686.0,          -24.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1948.0,         1033.0,          666.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4335.0,        -1001.0,         -939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8385.0,       -14999.0,        -7821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29292.0,        -6379.0,        -5414.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5006.0,         2057.0,          478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9280.0,         -403.0,          543.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9067.0,         2434.0,          423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11619.0,        -7964.0,        -3126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23685.0,       -58280.0,       -24730.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -973433710.0,  -3289987391.0,  -1656050816.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3202.0,        63044.0,        31955.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2301.0,         3410.0,         1932.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1902.0,        -1353.0,         -778.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7606.0,         4470.0,         1259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7060.0,        -3864.0,        -1038.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3111.0,          289.0,          650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -403.0,         -258.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27384.0,         7012.0,          714.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -496.0,        -1371.0,         -611.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -468.0,        -1184.0,         -442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          461.0,         1325.0,          589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,         -216.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          170.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          951.0,        -2908.0,        -1420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -201.0,          666.0,          359.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1362.0,        -2726.0,        -1543.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          188.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,          683.0,          355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1711.0,         1912.0,          799.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          116.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -864.0,         -491.0,         -169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,           73.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3332.0,        -2891.0,        -1100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -314.0,         -591.0,         -238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,         -115.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -40.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,         -129.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -702.0,         -702.0,         -286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5694.0,         3871.0,         2587.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          593.0,          599.0,          245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6227.0,          545.0,          978.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          364.0,         1029.0,          482.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -344.0,         -362.0,         -135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          587.0,          746.0,          322.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2939.0,        -3140.0,        -1290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14231.0,        10234.0,         3719.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14273.0,         9172.0,         3168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    112112313.0,    331285097.0,    165292520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7788.0,       -16169.0,        -7596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11123.0,        -4416.0,        -3671.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10502.0,          462.0,         1498.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4685.0,          176.0,         -463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1136.0,         -162.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          343.0,            0.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          643.0,          -39.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          101.0,           -1.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11350.0,        -2284.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          149.0,           71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -397.0,           18.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          997.0,          327.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -446.0,         -132.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          123.0,           38.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -278.0,          -32.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          380.0,          672.0,          295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -481.0,         -199.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -425.0,         1102.0,          639.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -332.0,          -31.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          510.0,         -116.0,         -116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -654.0,          -33.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -360.0,         -835.0,         -406.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -236.0,           -9.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          416.0,           84.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -187.0,           -2.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          956.0,          727.0,          279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,          167.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,           39.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          -13.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           53.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          200.0,          244.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1235.0,         -977.0,         -639.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -192.0,          -52.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2275.0,         -470.0,         -502.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -323.0,         -214.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          246.0,           95.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,         -104.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          429.0,          392.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4713.0,        -2337.0,         -716.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4025.0,        -2518.0,         -888.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -19096164.0,    -50013577.0,    -24728527.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2125.0,         1739.0,          681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,         4033.0,         2131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          151.0,          245.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,         -105.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           59.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -15.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -184.0,           33.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -14.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3153.0,          540.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          -34.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -117.0,          -95.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,           54.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,           82.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,         -287.0,         -147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          103.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          183.0,         -300.0,         -177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           29.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          247.0,          246.0,          105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -13.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -239.0,         -176.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          -44.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,          -73.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,          258.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          -22.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -476.0,          233.0,          176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,          -35.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          406.0,          185.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -942.0,         -434.0,         -150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -57.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1686.0,          748.0,          235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          622.0,          266.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3809493.0,      8939169.0,      4378671.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          424.0,          270.0,          152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -380.0,        -1230.0,         -632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          134.0,          -17.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -725.0,          -43.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          653.0,           34.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,           -2.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -835.0,         -106.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           33.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           71.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,          -66.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,           48.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           20.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,          -60.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          144.0,          -60.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           35.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -420.0,         -174.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,          -30.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -826822.0,     -1753192.0,      -850495.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          201.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          156.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -54.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          222.0,           23.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           14.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,           14.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,           34.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,           21.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       188897.0,       364531.0,       175074.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,           14.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,          -73.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -9.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,           -5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -44679.0,       -78893.0,       -37496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           20.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10829.0,        17575.0,         8262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2671.0,        -4000.0,        -1859.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          666.0,          926.0,          425.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -168.0,         -216.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,           50.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -46192.0,       -11329.0,        -1574.0),
           (        -5549.0,         2191.0,         1639.0),
           (         -609.0,         3632.0,         3155.0)),
          ((         8002.0,        -1338.0,        -1390.0),
           (      -114596.0,       -82327.0,       -27845.0),
           (       -59783.0,        10415.0,         9405.0)),
          ((        -4500.0,          421.0,          612.0),
           (         5358.0,         4727.0,         1787.0),
           (            0.0,            0.0,            0.0)),
          ((         -860.0,        -1239.0,         -559.0),
           (       -45500.0,        -2882.0,        48081.0),
           (            0.0,            0.0,            0.0)),
          ((        -2893.0,       -17004.0,        -8434.0),
           (      -203234.0,      -199658.0,       -79111.0),
           (            0.0,            0.0,            0.0)),
          ((         -404.0,          396.0,          237.0),
           (        -9147.0,        -5961.0,        -2075.0),
           (            0.0,            0.0,            0.0)),
          ((       -14009.0,        10571.0,         7572.0),
           (       -35362.0,        -5700.0,         5818.0),
           (            0.0,            0.0,            0.0)),
          ((         -374.0,         1987.0,         1012.0),
           (         1904.0,       -10764.0,        -5082.0),
           (            0.0,            0.0,            0.0)),
          ((         -705.0,          514.0,          384.0),
           (         2809.0,         1275.0,          279.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          538.0,          281.0),
           (          609.0,          444.0,          108.0),
           (            0.0,            0.0,            0.0)),
          ((          183.0,         -972.0,         -511.0),
           (            3.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         -348.0,         -643.0,         -286.0),
           (          -69.0,           87.0,           85.0),
           (            0.0,            0.0,            0.0)),
          ((        -8005.0,        -3113.0,         -745.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,         1075.0,          554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2067.0,        -1811.0,         -737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -638.0,         1407.0,          779.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1647.0,        -7972.0,        -4237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2022.0,         5103.0,         2902.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1790.0,        -5335.0,        -3203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -236.0,          700.0,          314.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5102.0,        -2569.0,         -798.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1340.0,         1220.0,          646.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -237.0,          307.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -494.0,          101.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          368.0,         -751.0,         -457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,          350.0,          197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -676.0,          758.0,          437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1820.0,         -369.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17391.0,        17070.0,        11186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8293.0,         6223.0,         3457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4963.0,        -5217.0,        -3351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          654.0,         -843.0,         -368.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,        -1088.0,         -786.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5731.0,         4352.0,         2793.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -53036.0,        31528.0,        22574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   1159598841.0,   3249419709.0,   1615139448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -64424.0,        -6963.0,         3482.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5749.0,          -48.0,          781.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5197.0,         -740.0,        -1145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -69763.0,        20678.0,        20140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63941.0,       -19387.0,       -18724.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3493.0,          962.0,          289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1641.0,         1677.0,         1194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26494.0,        -7055.0,         -896.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -102.0,         2166.0,         1130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1202.0,          149.0,          365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          622.0,        -1099.0,         -700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,           74.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          109.0,         -385.0,         -208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2962.0,          154.0,          352.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,         -752.0,         -430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1431.0,         2753.0,         1569.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,         -178.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,         -666.0,         -346.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1746.0,         2105.0,          894.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -38.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -906.0,         -259.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,            8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2808.0,         3064.0,         1740.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          270.0,          636.0,          267.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,           81.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           12.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,          209.0,          113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -776.0,         -966.0,         -413.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4749.0,         4854.0,         2988.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,           18.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2776.0,         4600.0,         2036.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -313.0,          630.0,          369.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          332.0,         -202.0,         -156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -547.0,          450.0,          296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2762.0,        -1957.0,        -1347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12110.0,        14007.0,         8700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9809.0,         9890.0,         6373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    149387018.0,    320893366.0,    155881303.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5252.0,        -6267.0,        -2743.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22398.0,        10241.0,         7791.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16085.0,        -9715.0,        -6843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6275.0,         4357.0,         2972.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1394.0,         -998.0,         -677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -355.0,          270.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          418.0,          534.0,          262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -278.0,          142.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11212.0,        -1638.0,          317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -139.0,          414.0,          230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -420.0,         -174.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -298.0,          -20.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          154.0,          -42.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           13.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          -73.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -843.0,           42.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          281.0,         -133.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -787.0,         1118.0,          685.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,           -6.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -119.0,         -194.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,           69.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          748.0,          770.0,          326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           40.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -261.0,         -107.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           69.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -686.0,          846.0,          506.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,          188.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           25.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -270.0,         -303.0,         -132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -826.0,         1278.0,          755.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,         -142.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1213.0,         1754.0,          797.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -187.0,           20.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,          -38.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,           21.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          324.0,          -57.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2902.0,         4533.0,         2714.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2779.0,         2843.0,         1816.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     27429334.0,     47159656.0,     22340514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2549.0,        -2441.0,        -1560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5136.0,          958.0,         1047.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          153.0,         -339.0,         -195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -120.0,          262.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          -81.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           52.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,          105.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           53.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3180.0,         -203.0,          226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          104.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -124.0,          -16.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,          -17.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -78.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -199.0,          123.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -77.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,          315.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -26.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          274.0,          227.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           -6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -153.0,          229.0,          135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           40.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           16.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          -85.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -181.0,          338.0,          198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -62.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          330.0,          520.0,          243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -73.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,          324.0,          192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          256.0,         -752.0,         -450.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          -26.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -737.0,         1624.0,          959.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -416.0,          451.0,          264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      5761594.0,      8139390.0,      3749116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          131.0,          288.0,          179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1063.0,          -15.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -167.0,          153.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          804.0,         -934.0,         -625.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -722.0,          847.0,          565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           33.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           22.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           15.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -821.0,            8.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           32.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,            5.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           79.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,           53.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           65.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -17.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           73.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,          114.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,            5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,          418.0,          243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -25.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1295139.0,      1527795.0,       681545.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -168.0,           24.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,           58.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,          -25.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,           22.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           18.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           21.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           22.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          107.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           27.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       302832.0,       300937.0,       129292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           14.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,            1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           22.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,           10.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           28.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        72583.0,        60975.0,        25032.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17684.0,        12540.0,         4862.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4357.0,         2590.0,          931.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1084.0,          529.0,          170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          271.0,          107.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           22.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        -9805.0,        26804.0,        14555.0),
           (        -2086.0,        -5700.0,        -2745.0),
           (        19506.0,       -85764.0,       -44802.0)),
          ((         1473.0,         -749.0,         -508.0),
           (        36457.0,       -39790.0,       -23446.0),
           (        23010.0,       101710.0,        47353.0)),
          ((        -1017.0,         1103.0,          651.0),
           (         4359.0,        -2260.0,        -1517.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,          231.0,           96.0),
           (        13466.0,        19479.0,       -38470.0),
           (            0.0,            0.0,            0.0)),
          ((        19801.0,        -3744.0,        -3833.0),
           (       146439.0,       -94097.0,       -65078.0),
           (            0.0,            0.0,            0.0)),
          ((        -1977.0,          -99.0,          136.0),
           (         7331.0,        -2780.0,        -2150.0),
           (            0.0,            0.0,            0.0)),
          ((       -13466.0,        11056.0,         7770.0),
           (         4162.0,       -11814.0,        -1554.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,         1907.0,          938.0),
           (       -13279.0,        12352.0,         7104.0),
           (            0.0,            0.0,            0.0)),
          ((          727.0,        -2052.0,        -1176.0),
           (         -863.0,         2161.0,         1145.0),
           (            0.0,            0.0,            0.0)),
          ((         -168.0,          206.0,          120.0),
           (          115.0,          405.0,          132.0),
           (            0.0,            0.0,            0.0)),
          ((          301.0,          -50.0,          -55.0),
           (            6.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,         1615.0,          812.0),
           (         -152.0,          175.0,           73.0),
           (            0.0,            0.0,            0.0)),
          ((         2493.0,        -8967.0,        -4569.0),
           (            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((         -641.0,         1978.0,         1050.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1955.0,        -2092.0,         -872.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -781.0,         1859.0,         1009.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1593.0,        -8300.0,        -4340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2487.0,         4416.0,         2367.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1489.0,        -1502.0,         -596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1081.0,         1537.0,          953.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6103.0,          782.0,         -211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1068.0,          299.0,          466.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -649.0,        -2649.0,        -1057.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,         3029.0,         1348.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,        -3023.0,        -1484.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -128.0,         1352.0,          675.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -388.0,         -383.0,         -126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -656.0,         6110.0,         3028.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32218.0,       -80168.0,       -43625.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4955.0,        40007.0,        20488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3488.0,        14190.0,         7031.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2015.0,       -14019.0,        -7220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4076.0,        13458.0,         7115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6281.0,        -9179.0,        -5554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        52996.0,        58401.0,        21919.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -1342642069.0,  -3200179507.0,  -1569927544.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9594.0,       -47394.0,       -26352.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2619.0,       -17595.0,        -8463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3114.0,        21956.0,        10553.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -48988.0,      -273976.0,      -130034.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        44754.0,       253485.0,       120416.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4043.0,         9369.0,         4021.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          892.0,        -4269.0,        -2282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27680.0,         8306.0,         1312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1358.0,        -3842.0,        -1734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1281.0,        -2611.0,        -1039.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          720.0,         3284.0,         1582.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,         -398.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          464.0,          236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -401.0,         1445.0,          615.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          957.0,          493.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1264.0,        -2859.0,        -1604.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          127.0,          334.0,          160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          679.0,          352.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1849.0,         1910.0,          781.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,           52.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -822.0,         -128.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,          -47.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3685.0,         2544.0,          899.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -338.0,         -632.0,         -256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -84.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           27.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          134.0,         -289.0,         -161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -708.0,         -863.0,         -370.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4108.0,         5376.0,         3207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -96.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5331.0,        -1861.0,        -1546.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          133.0,         -331.0,         -190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,          124.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -498.0,         -389.0,         -145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2631.0,         1536.0,          516.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15682.0,       -12774.0,        -4923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12044.0,       -12562.0,        -5228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    185049430.0,    307061198.0,    144801568.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5921.0,        16271.0,         7966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11825.0,         2173.0,         2495.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11576.0,         2550.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5038.0,        -2036.0,         -503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1276.0,          561.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -230.0,         -195.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -495.0,          364.0,          263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -238.0,         -145.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11299.0,         -623.0,          865.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           93.0,          131.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1131.0,          495.0,          138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -578.0,         -200.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          312.0,          117.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,          -31.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          229.0,           83.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -492.0,         -782.0,         -344.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          461.0,          239.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -858.0,          967.0,          615.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          235.0,          155.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -406.0,         -514.0,         -227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          506.0,          420.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1233.0,        -1014.0,         -404.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          134.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,         -123.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,           90.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1083.0,         -562.0,         -182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          188.0,          190.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -4.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           65.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          306.0,          283.0,          117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          541.0,        -1466.0,         -824.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          230.0,          239.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2051.0,          589.0,          540.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          237.0,          345.0,          155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,         -146.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,           76.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -263.0,          -59.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5393.0,         3604.0,         1318.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2598.0,         2431.0,          998.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -35098853.0,    -43165990.0,    -19412999.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1182.0,         -364.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -196.0,        -4281.0,        -2253.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -234.0,         -101.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          117.0,           87.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,           -5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          147.0,          -84.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3175.0,         -197.0,         -438.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,          -55.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -191.0,          -54.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -24.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           18.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           88.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           -8.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           67.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,         -330.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           38.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          297.0,          184.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           -2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          279.0,          100.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,          -38.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -33.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,          -43.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,          308.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           33.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          400.0,         -373.0,         -240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           80.0,           42.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -399.0,         -168.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          860.0,          384.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1828.0,         -865.0,         -249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -633.0,         -522.0,         -223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      7465983.0,      6990435.0,      2958850.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -350.0,         -229.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          439.0,         1135.0,          550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,           53.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          580.0,         -236.0,         -211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -523.0,          203.0,          184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,            5.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           26.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -808.0,          162.0,          171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           12.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,            6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -28.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           82.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,          -42.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -76.0,          -13.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -82.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,          108.0,           69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           16.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          473.0,          163.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           89.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1676619.0,     -1199975.0,      -466920.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -95.0,         -163.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,         -172.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           67.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          203.0,          -67.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -20.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           21.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,          -31.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,          -28.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -21.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       387591.0,       208398.0,        71083.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          -31.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,           64.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,           24.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -90997.0,       -35093.0,        -9300.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21531.0,         5369.0,          633.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5108.0,         -621.0,          199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1212.0,           -4.0,         -128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -287.0,           38.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,          -18.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        46179.0,       -28640.0,       -18860.0),
           (         7995.0,         -597.0,         -968.0),
           (       -28214.0,         9636.0,         6821.0)),
          ((       -11068.0,          907.0,         1463.0),
           (        64998.0,        27413.0,         9300.0),
           (        75130.0,       -40506.0,       -26780.0)),
          ((         6387.0,        -1170.0,        -1174.0),
           (        -3738.0,        -2568.0,         -961.0),
           (            0.0,            0.0,            0.0)),
          ((         2285.0,         -234.0,         -325.0),
           (       -35114.0,        -8363.0,        46953.0),
           (            0.0,            0.0,            0.0)),
          ((         4638.0,        18038.0,         8790.0),
           (        63010.0,        56646.0,        29231.0),
           (            0.0,            0.0,            0.0)),
          ((         1193.0,        -2745.0,        -1509.0),
           (          789.0,         1208.0,          809.0),
           (            0.0,            0.0,            0.0)),
          ((       -14654.0,        12224.0,         8487.0),
           (         2460.0,        16879.0,        14100.0),
           (            0.0,            0.0,            0.0)),
          ((         1206.0,         2004.0,          879.0),
           (        17321.0,        -1893.0,        -2021.0),
           (            0.0,            0.0,            0.0)),
          ((         -262.0,         1250.0,          714.0),
           (        -2084.0,        -1193.0,         -511.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,          196.0,           98.0),
           (        -1189.0,         -295.0,          -97.0),
           (            0.0,            0.0,            0.0)),
          ((         -364.0,         -426.0,         -186.0),
           (           10.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,        -1126.0,         -547.0),
           (           97.0,          -43.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8041.0,         2280.0,          315.0),
           (            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          208.0,         -599.0,         -300.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2951.0,         -450.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          944.0,         -484.0,         -295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2615.0,        -3974.0,        -1875.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3186.0,         -852.0,         -500.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2207.0,         -823.0,         -614.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1185.0,         -676.0,         -473.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6339.0,         -613.0,          271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1093.0,        -1226.0,         -637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          270.0,         -398.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          352.0,         -557.0,         -423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -606.0,           34.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          301.0,         -116.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -601.0,         1879.0,         1003.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2867.0,        -1373.0,         -870.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -46445.0,        35896.0,        22818.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17541.0,        -5550.0,        -4500.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9659.0,         4100.0,         2243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5490.0,         2180.0,         1535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3791.0,        -1345.0,         -920.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1785.0,        -2339.0,        -1599.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        52100.0,       -55174.0,       -37524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   1522082490.0,   3142347822.0,   1520511252.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        77956.0,         3415.0,        -7145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,         3589.0,         1733.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1825.0,        -4768.0,        -2474.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -35440.0,        71990.0,        39109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32775.0,       -66067.0,       -35852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          815.0,        -4692.0,        -2647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1669.0,          -44.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26965.0,        -5075.0,          297.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          557.0,         1201.0,          482.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -753.0,         -571.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          240.0,        -1028.0,         -525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           63.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,         -334.0,         -158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2655.0,          883.0,          227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,         -735.0,         -367.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1297.0,         2759.0,         1549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -163.0,         -277.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -246.0,         -816.0,         -392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1844.0,         2169.0,          915.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -90.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -943.0,           54.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          -21.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2271.0,        -3409.0,        -1854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          286.0,          585.0,          238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,          143.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,          -45.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          136.0,          324.0,          153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -577.0,         -954.0,         -429.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3531.0,         6071.0,         3503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          848.0,         -442.0,         -325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3922.0,        -3869.0,        -1541.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          681.0,        -1037.0,         -608.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -87.0,          358.0,          184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          586.0,         -638.0,         -406.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2770.0,         3351.0,         2078.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14870.0,       -11638.0,        -7904.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14273.0,        -9124.0,        -6590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    218713648.0,    289922540.0,    132164798.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10665.0,         4193.0,         1128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19521.0,        -8004.0,        -6425.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13386.0,         6939.0,         5206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4776.0,        -2731.0,        -2002.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1040.0,          690.0,          488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          152.0,          -79.0,          -59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -575.0,         -482.0,         -209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,         -113.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11535.0,          179.0,         1321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          326.0,           -5.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -429.0,         -506.0,         -212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           93.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -35.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          -26.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          750.0,            8.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -173.0,          -74.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -456.0,         1162.0,          673.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,            2.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,         -211.0,         -115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,           57.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1008.0,          707.0,          263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,            3.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -211.0,           45.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          -20.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          393.0,         -988.0,         -551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          159.0,          176.0,           80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           41.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -283.0,         -274.0,         -115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -339.0,         1577.0,          864.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,         -270.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1295.0,        -1079.0,         -432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,         -399.0,         -204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          133.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,         -111.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,          513.0,          281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3895.0,        -3834.0,        -2471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3932.0,        -3064.0,        -2077.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     41916675.0,     38126306.0,     16015465.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1708.0,         2025.0,         1266.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5710.0,        -1556.0,        -1432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -251.0,          213.0,          142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          146.0,          -87.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,           53.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           -7.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -159.0,          -98.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -26.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3110.0,          586.0,          638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,           -7.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,         -120.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           53.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -26.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -94.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          186.0,          125.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -87.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          345.0,          187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -17.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -35.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          325.0,          172.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           20.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         -260.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,           36.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -88.0,          -53.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          360.0,          190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -12.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -440.0,         -362.0,         -146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,            4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          216.0,         -145.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -433.0,          412.0,          264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          185.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          937.0,        -1319.0,         -800.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          648.0,         -661.0,         -428.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      8848566.0,      5540604.0,      2041328.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,         -341.0,         -180.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1176.0,         -254.0,         -263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          114.0,         -221.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -574.0,         1508.0,          852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          512.0,        -1352.0,         -763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -42.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,            5.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -36.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -774.0,          320.0,          251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -21.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,          -16.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           83.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           95.0,           33.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -68.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -11.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           84.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -149.0,          -75.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           35.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          174.0,         -425.0,         -245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,          -31.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1945559.0,       791631.0,       220973.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          191.0,         -103.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          159.0,         -111.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           38.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -180.0,          108.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           23.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -23.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           28.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           -1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,         -111.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -38.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       434898.0,        95739.0,         6014.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -39.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,          -23.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -37.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,           37.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -28.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        97491.0,         4611.0,        -7652.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -13.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21711.0,        -2720.0,        -3705.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4763.0,        -1479.0,        -1284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1019.0,         -535.0,         -391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          209.0,         -169.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -51.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -16550.0,        -9297.0,        -3362.0),
           (       -12970.0,         7411.0,         5027.0),
           (         2436.0,        12450.0,         5747.0)),
          ((         5553.0,         7820.0,         3487.0),
           (         8563.0,        37956.0,        21088.0),
           (       -63918.0,           77.0,         5616.0)),
          ((        -3084.0,        -4204.0,        -1869.0),
           (        -3598.0,         2360.0,         1663.0),
           (            0.0,            0.0,            0.0)),
          ((        -1886.0,         -392.0,          -21.0),
           (        58733.0,        -7610.0,       -59996.0),
           (            0.0,            0.0,            0.0)),
          ((       -17858.0,         6898.0,         5261.0),
           (        34966.0,       102767.0,        56269.0),
           (            0.0,            0.0,            0.0)),
          ((          758.0,        -1556.0,         -865.0),
           (         3889.0,          374.0,            7.0),
           (            0.0,            0.0,            0.0)),
          ((       -13557.0,        12153.0,         8345.0),
           (       -31698.0,        19262.0,        19639.0),
           (            0.0,            0.0,            0.0)),
          ((          410.0,         2366.0,         1163.0),
           (          529.0,          576.0,         -744.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,        -1153.0,         -636.0),
           (          970.0,        -2003.0,        -1289.0),
           (            0.0,            0.0,            0.0)),
          ((           64.0,         -223.0,         -133.0),
           (          -84.0,         -172.0,         -149.0),
           (            0.0,            0.0,            0.0)),
          ((          194.0,          573.0,          288.0),
           (           -1.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((          580.0,          894.0,          379.0),
           (          112.0,         -209.0,         -158.0),
           (            0.0,            0.0,            0.0)),
          ((        -1588.0,         7980.0,         4016.0),
           (            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((          192.0,         -737.0,         -423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2531.0,         -283.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,         -710.0,         -424.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1103.0,        -3355.0,        -1456.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -244.0,        -2659.0,        -1650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          241.0,         3369.0,         2154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,         -108.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5727.0,          577.0,         -202.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1120.0,          557.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          930.0,         1273.0,          352.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -697.0,        -1498.0,         -635.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          301.0,          597.0,          178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,         -256.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,         1666.0,          826.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,        -2416.0,        -1230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        24945.0,        -8751.0,        -7597.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3269.0,       -16789.0,        -7799.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7093.0,        -2302.0,         -702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          536.0,         3042.0,         1393.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -523.0,        -1863.0,         -842.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5033.0,        -1264.0,         -543.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -62345.0,       -41133.0,       -14173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -1697477518.0,  -3076110030.0,  -1467032965.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          343.0,        61792.0,        32100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1548.0,         3909.0,         1897.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3875.0,        -2404.0,         -832.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        59605.0,        15838.0,         1284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -54926.0,       -14383.0,        -1084.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4833.0,          906.0,         1216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          479.0,          365.0,          227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27323.0,         4103.0,         -806.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -602.0,         -918.0,         -384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -542.0,        -1079.0,         -390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          431.0,         1104.0,          476.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,         -178.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          160.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,        -3036.0,        -1392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,          650.0,          336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1280.0,        -2713.0,        -1528.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          211.0,          194.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          324.0,          674.0,          313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1756.0,         1834.0,          750.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,          145.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -989.0,            9.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,           66.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3994.0,        -2078.0,         -647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,         -624.0,         -259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,         -114.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -48.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,         -150.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -594.0,         -493.0,         -195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2709.0,         5845.0,         3316.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          909.0,          512.0,          175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6375.0,         1327.0,         1377.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          803.0,         1069.0,          467.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -523.0,         -426.0,         -152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          724.0,          642.0,          261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2877.0,        -3214.0,        -1370.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10368.0,        11744.0,         5095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9285.0,        15680.0,         7195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    250037642.0,    269657820.0,    118102257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9289.0,       -15787.0,        -7390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6433.0,        -4377.0,        -3025.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6145.0,          419.0,          898.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2733.0,          202.0,         -196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -585.0,         -156.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,          -50.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          521.0,         -481.0,         -335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           39.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11306.0,          873.0,         1665.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          108.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,          -37.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          759.0,          332.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -307.0,         -139.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,           36.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,          -41.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          474.0,          710.0,          312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -282.0,         -224.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -309.0,         1249.0,          704.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -192.0,          -38.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,          -62.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -246.0,          -72.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -831.0,         -606.0,         -231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,          -35.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          254.0,          -33.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           -9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1180.0,          307.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          147.0,          159.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           32.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           -7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           51.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          219.0,          135.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          211.0,        -1434.0,         -776.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,          -42.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1902.0,        -1050.0,         -768.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -379.0,         -219.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          280.0,           86.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,          -80.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          462.0,          391.0,          155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3324.0,        -3171.0,        -1324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3958.0,        -4515.0,        -1955.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -47722196.0,    -32161779.0,    -12229143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2733.0,         1613.0,          569.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1677.0,         3669.0,         1771.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          195.0,          263.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,         -113.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           64.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,          155.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2955.0,         -920.0,         -799.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -24.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           20.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -153.0,          -61.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,           37.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,           58.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -222.0,         -239.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           80.0,           84.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,         -347.0,         -184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           19.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          336.0,          125.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           22.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -306.0,           -9.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -32.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -11.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          -37.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,          364.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -28.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -274.0,          469.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,          -29.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          299.0,          170.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -746.0,         -380.0,         -111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,          -43.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1368.0,          753.0,          248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          701.0,          708.0,          305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      9851107.0,      3851484.0,      1035443.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          349.0,          234.0,           80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1017.0,         -955.0,         -400.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,          -44.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -467.0,          119.0,          115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          419.0,         -112.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -12.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -37.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -674.0,          409.0,          288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           25.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           82.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,          -14.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          -12.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,          -79.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,         -136.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,            9.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -399.0,         -149.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -167.0,         -124.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2084032.0,      -330042.0,        39880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          232.0,          109.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,          115.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -41.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,         -144.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -20.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           39.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,           15.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,           24.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       440269.0,       -26243.0,       -59685.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           14.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,          -27.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           45.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,            3.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -91223.0,        26492.0,        23613.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18187.0,       -10346.0,        -7413.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3390.0,         3265.0,         2096.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          556.0,         -930.0,         -555.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          249.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -63.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -20529.0,        37004.0,        20936.0),
           (         2822.0,         2525.0,         1035.0),
           (        -9063.0,       -75167.0,       -36522.0)),
          ((         6479.0,       -11143.0,        -6301.0),
           (       -56918.0,       -64907.0,       -26057.0),
           (        -8051.0,        90780.0,        48378.0)),
          ((        -3785.0,         6390.0,         3623.0),
           (         5941.0,         1875.0,          489.0),
           (            0.0,            0.0,            0.0)),
          ((        -1186.0,          110.0,          171.0),
           (       -66054.0,        10447.0,        62763.0),
           (            0.0,            0.0,            0.0)),
          ((        -7788.0,       -16952.0,        -7940.0),
           (       -77333.0,      -176789.0,       -81899.0),
           (            0.0,            0.0,            0.0)),
          ((         -208.0,         1506.0,          791.0),
           (        -4476.0,        -3865.0,        -1521.0),
           (            0.0,            0.0,            0.0)),
          ((       -12597.0,        10949.0,         7657.0),
           (       -18892.0,          853.0,         8473.0),
           (            0.0,            0.0,            0.0)),
          ((         -208.0,         2486.0,         1263.0),
           (         1191.0,       -17891.0,        -8674.0),
           (            0.0,            0.0,            0.0)),
          ((         -549.0,          533.0,          378.0),
           (         1602.0,          933.0,          220.0),
           (            0.0,            0.0,            0.0)),
          ((          154.0,          687.0,          339.0),
           (          244.0,          607.0,          227.0),
           (            0.0,            0.0,            0.0)),
          ((         -305.0,        -1247.0,         -607.0),
           (           -1.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((         -721.0,          -95.0,           26.0),
           (          -20.0,          -38.0,           10.0),
           (            0.0,            0.0,            0.0)),
          ((        -8520.0,        -2805.0,         -578.0),
           (            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          304.0,         2097.0,         1042.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2550.0,        -2190.0,         -842.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,         2898.0,         1484.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1670.0,       -10778.0,        -5390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          180.0,         9565.0,         4971.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -186.0,        -8536.0,        -4623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,         1792.0,          846.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5717.0,        -1141.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1009.0,         2216.0,         1117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -633.0,        -1131.0,         -598.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,         2463.0,         1229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -307.0,        -2793.0,        -1445.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          110.0,         1326.0,          670.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -328.0,         -528.0,         -244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1202.0,         4448.0,         2458.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17215.0,       -24589.0,       -10322.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -665.0,        42679.0,        21521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1892.0,         9114.0,         3756.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1447.0,       -13272.0,        -6576.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1001.0,        11824.0,         5762.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3725.0,        -5667.0,        -2512.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25118.0,        62582.0,        39033.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   1868353722.0,   3001629953.0,   1409628290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -55588.0,         7121.0,         9705.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7326.0,       -10790.0,        -4547.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7253.0,        13359.0,         5880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -100035.0,      -159666.0,       -69006.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        91985.0,       147581.0,        63827.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5554.0,         6985.0,         3066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1487.0,        -1047.0,         -261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26783.0,        -2675.0,         1402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          165.0,          407.0,          229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1194.0,         -759.0,         -113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          451.0,          544.0,          175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,          -40.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,         -132.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2762.0,          649.0,          535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          186.0,         -439.0,         -254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1329.0,         2626.0,         1496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,          -65.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -409.0,         -626.0,         -290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1930.0,         2061.0,          850.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          -89.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -829.0,          333.0,          245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -75.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1804.0,         3664.0,         1941.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          234.0,          606.0,          257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,          141.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -25.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          354.0,          175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1084.0,         -680.0,         -240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1143.0,         6495.0,         3482.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -366.0,         -190.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3951.0,         4711.0,         1958.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -547.0,          188.0,          172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          428.0,           64.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -700.0,          194.0,          187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2938.0,         -761.0,         -775.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9192.0,         6291.0,         4509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13662.0,         5001.0,         4350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    278681820.0,    246502927.0,    102774762.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5095.0,          560.0,          687.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19936.0,         4629.0,         4804.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13997.0,        -4562.0,        -4109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5421.0,         2246.0,         1852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1140.0,         -532.0,         -423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -225.0,          221.0,          146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          817.0,          343.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -263.0,          107.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11003.0,         1568.0,         2001.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          334.0,          183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -571.0,          -73.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -149.0,          -86.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,           -7.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           12.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -46.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -655.0,          351.0,          244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,         -169.0,         -107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -457.0,         1256.0,          724.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -17.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -189.0,          -82.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,          -20.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          998.0,          613.0,          217.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            8.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,           81.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           17.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -167.0,         1047.0,          559.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,          155.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           31.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           66.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -414.0,         -118.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          434.0,         1489.0,          739.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -293.0,          -70.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2084.0,         1293.0,          459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -423.0,           45.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          282.0,          -30.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,           32.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          460.0,         -100.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2404.0,         2145.0,         1424.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4817.0,         2206.0,         1719.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     52374312.0,     25420967.0,      8147872.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3160.0,        -1065.0,         -929.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4523.0,         2049.0,         1570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          184.0,         -222.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -144.0,          194.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          -53.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           48.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          201.0,           16.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           48.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2776.0,         1235.0,          949.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           82.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,           32.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -23.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            8.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -39.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,          169.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -65.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          348.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          355.0,           92.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           27.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          269.0,          136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,           25.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -14.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           28.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,          -16.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          194.0,          334.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -24.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          574.0,          263.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -31.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          142.0,           71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,         -350.0,         -189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          140.0,          -54.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -509.0,          817.0,          485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -685.0,          576.0,          394.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     10430989.0,      1997387.0,       -14741.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,           76.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -667.0,          591.0,          392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -183.0,          107.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          986.0,         -555.0,         -374.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -885.0,          506.0,          340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           20.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          -11.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           23.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -555.0,          499.0,          324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           24.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           31.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           82.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,           -3.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           65.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,            2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,           64.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,           18.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -7.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          -35.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,          291.0,          169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,           81.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2082889.0,      -153649.0,      -298070.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          120.0,           69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -110.0,          149.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          -59.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           13.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,          171.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           -5.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           18.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -15.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           76.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           43.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       403348.0,      -145664.0,      -119635.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           22.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           32.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           35.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           51.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        73042.0,       -54128.0,       -36488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           17.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11569.0,       -16208.0,        -9858.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1293.0,        -4347.0,        -2456.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,        -1080.0,         -572.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,         -248.0,         -123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,          -51.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        20336.0,        -2647.0,        -3000.0),
           (        -8188.0,       -10262.0,        -4552.0),
           (       -46523.0,       -46833.0,       -19235.0)),
          ((        -8932.0,         4357.0,         2984.0),
           (        30367.0,       -21042.0,       -12119.0),
           (        99763.0,        38618.0,        11352.0)),
          ((         5098.0,        -2230.0,        -1572.0),
           (          215.0,        -3706.0,        -2069.0),
           (            0.0,            0.0,            0.0)),
          ((         1749.0,         -607.0,         -468.0),
           (        40935.0,         7859.0,       -51472.0),
           (            0.0,            0.0,            0.0)),
          ((        16699.0,        -7489.0,        -5468.0),
           (        95741.0,       -46201.0,       -34376.0),
           (            0.0,            0.0,            0.0)),
          ((         -702.0,         -288.0,          -69.0),
           (         5113.0,        -1948.0,        -1438.0),
           (            0.0,            0.0,            0.0)),
          ((       -12595.0,        12399.0,         8397.0),
           (         2212.0,        12679.0,        12207.0),
           (            0.0,            0.0,            0.0)),
          ((          742.0,         1816.0,          829.0),
           (        -9352.0,         8605.0,         4706.0),
           (            0.0,            0.0,            0.0)),
          ((         -167.0,        -1965.0,        -1043.0),
           (        -1278.0,          307.0,          222.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          105.0,           50.0),
           (          -88.0,          -92.0,         -101.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,           83.0,           30.0),
           (            6.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         1107.0,         1132.0,          472.0),
           (         -214.0,          102.0,           54.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,        -8550.0,        -4125.0),
           (            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((          803.0,         1253.0,          556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3125.0,        -1087.0,         -214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          972.0,          910.0,          377.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3731.0,        -6241.0,        -2862.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2522.0,         1663.0,          561.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1738.0,          543.0,          672.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          307.0,          992.0,          551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5458.0,         -977.0,        -1043.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -179.0,         -254.0,          105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2203.0,        -1698.0,         -447.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2287.0,         1569.0,          421.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2033.0,        -1765.0,         -664.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          858.0,          761.0,          291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -873.0,          662.0,          449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3771.0,         3641.0,         1390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21060.0,       -59441.0,       -28510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27036.0,        19706.0,         7477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16226.0,         6698.0,         2203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8566.0,        -7524.0,        -3029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6583.0,         7598.0,         3206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2314.0,        -5768.0,        -2937.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        77645.0,        20140.0,         3874.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -2034232116.0,  -2919158594.0,  -1348480544.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18882.0,       -44964.0,       -25984.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7016.0,       -10513.0,        -4721.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9760.0,        12198.0,         5329.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -137114.0,      -147093.0,       -61427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       126640.0,       136485.0,        57112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6536.0,         3949.0,         1095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -620.0,        -2910.0,        -1552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28803.0,         4630.0,         -629.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2457.0,        -2705.0,        -1137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1641.0,        -2143.0,         -812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1604.0,         1979.0,          902.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -219.0,         -246.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,          337.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          392.0,         1697.0,          681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          171.0,          716.0,          365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1189.0,        -2746.0,        -1543.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          286.0,          202.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,          550.0,          256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1873.0,         1798.0,          717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           86.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -784.0,          341.0,          235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          -14.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4205.0,         1590.0,          390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -264.0,         -642.0,         -272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,         -106.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           42.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -172.0,         -314.0,         -146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -941.0,         -765.0,         -305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -473.0,         6609.0,         3491.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,         -166.0,          -93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4567.0,        -2590.0,        -1824.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          429.0,         -304.0,         -218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -191.0,            0.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,         -295.0,         -166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,         1357.0,          738.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6235.0,        -5994.0,        -2565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4117.0,        -8134.0,        -3922.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    304320698.0,    220699025.0,     86344861.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1819.0,        13568.0,         7103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -660.0,         3938.0,         2089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1120.0,          389.0,          148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -414.0,         -954.0,         -487.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          170.0,          276.0,          134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -76.0,         -158.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -317.0,          657.0,          410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -87.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10949.0,         2500.0,         2493.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          339.0,          107.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          781.0,          442.0,          148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,         -170.0,          -94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,           92.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -29.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           49.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -383.0,         -630.0,         -286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,          162.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -357.0,         1116.0,          636.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          128.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,         -356.0,         -181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          308.0,          166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1059.0,         -715.0,         -264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           94.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,         -225.0,         -136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,           87.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1175.0,          -74.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          175.0,          172.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           12.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,           55.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          401.0,          191.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -627.0,        -1582.0,         -769.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          188.0,          253.0,          115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1348.0,         1058.0,          710.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          387.0,          200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,         -153.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          103.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,         -320.0,         -175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2221.0,         1919.0,          783.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          559.0,         2344.0,         1186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -55757889.0,    -18065285.0,     -3869712.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          610.0,         -881.0,         -542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2002.0,        -3475.0,        -1637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,         -164.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,          128.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          -20.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           23.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,         -172.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2575.0,        -1587.0,        -1116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,          -20.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -179.0,          -26.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,           51.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,            4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           42.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,         -346.0,         -176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           17.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          349.0,           48.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           31.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          280.0,          -78.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          -24.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           16.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -23.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,            1.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          246.0,          267.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           34.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,         -544.0,         -307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           41.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,         -182.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          156.0,          429.0,          226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           23.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -634.0,         -594.0,         -262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -544.0,         -260.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     10561916.0,        57715.0,     -1064121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,         -289.0,         -189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          773.0,          856.0,          390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,            1.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -214.0,            2.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          -10.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           43.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -458.0,          612.0,          375.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           -4.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           -6.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -10.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           78.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,           17.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,           44.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           -4.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,          -50.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          146.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -40.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          233.0,          101.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,           43.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1941931.0,       626984.0,       536248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -164.0,          -99.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,          -53.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,           11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,         -192.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -15.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -12.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -33.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,           -6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       327510.0,      -250991.0,      -168027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,           18.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           52.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -45301.0,        74672.0,        44585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2981.0,       -19313.0,       -10626.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1084.0,         4498.0,         2290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -644.0,         -939.0,         -435.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          229.0,          169.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,          -23.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        -7153.0,       -47013.0,       -23672.0),
           (         3095.0,         4850.0,         2259.0),
           (        62072.0,        55090.0,        22123.0)),
          ((         5152.0,         6183.0,         2780.0),
           (       -36808.0,        19282.0,        16206.0),
           (       -52293.0,       -83344.0,       -41468.0)),
          ((        -2921.0,        -3974.0,        -1815.0),
           (        -3967.0,          118.0,          255.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,        -1109.0,         -577.0),
           (       -27548.0,        -4096.0,        56031.0),
           (            0.0,            0.0,            0.0)),
          ((        11351.0,        16868.0,         7567.0),
           (      -133878.0,        42035.0,        43677.0),
           (            0.0,            0.0,            0.0)),
          ((        -1294.0,        -3330.0,        -1597.0),
           (        -6344.0,          331.0,         1126.0),
           (            0.0,            0.0,            0.0)),
          ((       -11832.0,        13770.0,         9051.0),
           (        -9857.0,        15199.0,        15557.0),
           (            0.0,            0.0,            0.0)),
          ((          996.0,         1306.0,          535.0),
           (          824.0,        -7852.0,        -3394.0),
           (            0.0,            0.0,            0.0)),
          ((          556.0,         1150.0,          577.0),
           (         -306.0,        -1477.0,         -823.0),
           (            0.0,            0.0,            0.0)),
          ((         -156.0,          196.0,          118.0),
           (         -370.0,          -57.0,          -43.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,         -262.0,         -130.0),
           (            7.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        -1263.0,         -883.0,         -326.0),
           (           31.0,           31.0,           27.0),
           (            0.0,            0.0,            0.0)),
          ((         9861.0,         1193.0,         -361.0),
           (            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        -1661.0,         -725.0,         -204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1646.0,          218.0,          317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1397.0,         -883.0,         -298.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          181.0,        -2681.0,        -1472.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3727.0,        -2279.0,         -646.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2528.0,          218.0,         -449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -592.0,        -1205.0,         -581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5169.0,         1819.0,         1396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -189.0,        -1497.0,         -652.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2236.0,           10.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2850.0,        -1299.0,         -518.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2067.0,          726.0,          203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -984.0,         -443.0,         -142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2380.0,         2316.0,          977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2822.0,        -3975.0,        -1732.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        23439.0,        64824.0,        31580.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26634.0,       -23076.0,        -9726.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8058.0,        -3164.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9137.0,         8665.0,         3583.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10605.0,        -8552.0,        -3327.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9586.0,         5027.0,         1109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5137.0,       -78823.0,       -43647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2194642153.0,   2828865113.0,   1283727546.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        55043.0,         6415.0,        -3618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8534.0,         9470.0,         4070.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10159.0,       -13432.0,        -5933.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       120213.0,       185353.0,        84118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -110708.0,      -170851.0,       -77517.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5529.0,        -8428.0,        -3892.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,         2089.0,         1038.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28362.0,        -3787.0,         1070.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1989.0,         2208.0,          913.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -180.0,           21.0,          132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -870.0,        -2246.0,        -1066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          216.0,          109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,         -444.0,         -206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2848.0,          180.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -244.0,         -874.0,         -425.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1180.0,         2799.0,         1564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -331.0,         -260.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -592.0,         -559.0,         -235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2024.0,         1963.0,          790.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           19.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -730.0,          405.0,          274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           80.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1242.0,        -3945.0,        -2024.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          246.0,          645.0,          275.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,          101.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,          -14.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          329.0,          172.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -753.0,         -752.0,         -315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          335.0,         6754.0,         3476.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          557.0,         -183.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4205.0,        -4064.0,        -1614.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          263.0,         -472.0,         -268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          151.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          191.0,          -38.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -400.0,           12.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4889.0,        -2655.0,        -2167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1525.0,        -2766.0,        -1801.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    326690638.0,    192516061.0,     68983440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14137.0,        -1904.0,        -2393.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10410.0,          968.0,         -881.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5414.0,        -1258.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1425.0,          658.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -276.0,         -111.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,           88.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -758.0,         -232.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            0.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10676.0,         3243.0,         2864.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          327.0,          -30.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -274.0,         -537.0,         -243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -282.0,          200.0,          135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          -87.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           21.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -49.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          426.0,         -284.0,         -180.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -40.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -284.0,         1239.0,          693.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           40.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -296.0,         -175.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,          132.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1038.0,          392.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,           58.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -174.0,          102.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,           14.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,        -1092.0,         -550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          192.0,          173.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           16.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,           -5.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -326.0,         -125.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          836.0,         1405.0,          656.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,         -107.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1731.0,         -913.0,         -293.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,          -82.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          105.0,           -9.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,         -152.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1729.0,         -781.0,         -639.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          681.0,          -82.0,         -146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     57791630.0,     10270274.0,      -503274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -603.0,         -719.0,         -290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4569.0,        -1815.0,        -1458.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,         -101.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,           99.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -18.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           16.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -201.0,            5.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2302.0,         1886.0,         1246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,          -31.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -150.0,          -96.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           80.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -39.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,         -105.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          167.0,           49.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          -77.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,          351.0,          173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           -5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          369.0,           17.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           38.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -152.0,         -242.0,         -111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,           21.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           -6.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,           -4.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          341.0,          245.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -17.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -544.0,          -83.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          -13.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          343.0,           -5.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -686.0,           18.0,          115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,            5.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          783.0,         -320.0,         -285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,         -146.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     10238473.0,     -1885154.0,     -2068172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,         -105.0,         -125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          726.0,         -812.0,         -492.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,          -84.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -228.0,          455.0,          330.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          202.0,         -401.0,         -292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           28.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -13.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -331.0,          691.0,          404.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -25.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,            4.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -32.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,           71.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          -30.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,          -45.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,           29.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -164.0,           34.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -26.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          117.0,         -123.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,            2.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1670650.0,     -1058380.0,      -738539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,         -188.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,         -159.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           59.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          200.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           13.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -18.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           27.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -43.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       220008.0,      -332189.0,      -200256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -65.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -18.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           48.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -14.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11610.0,       -85453.0,       -46852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -16.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6120.0,       -19127.0,        -9583.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3226.0,        -3680.0,        -1632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1071.0,         -554.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,          -36.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           15.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        -9484.0,        34114.0,        18113.0),
           (        -1977.0,         6272.0,         3464.0),
           (       -53943.0,       -55423.0,       -24172.0)),
          ((        -1203.0,        -4677.0,        -2268.0),
           (        32321.0,        -3429.0,        -3581.0),
           (        15793.0,        67728.0,        34205.0)),
          ((          604.0,         3144.0,         1540.0),
           (        -1901.0,         3376.0,         2092.0),
           (            0.0,            0.0,            0.0)),
          ((         -687.0,         1654.0,          911.0),
           (        26732.0,         -558.0,       -61927.0),
           (            0.0,            0.0,            0.0)),
          ((       -16077.0,         9452.0,         6401.0),
           (        99745.0,          159.0,        -4873.0),
           (            0.0,            0.0,            0.0)),
          ((         1307.0,         -336.0,         -296.0),
           (         5074.0,        -3476.0,        -2152.0),
           (            0.0,            0.0,            0.0)),
          ((       -12311.0,        12386.0,         8376.0),
           (        -9566.0,         7532.0,        11694.0),
           (            0.0,            0.0,            0.0)),
          ((         1116.0,         2825.0,         1331.0),
           (         1278.0,        11822.0,         5243.0),
           (            0.0,            0.0,            0.0)),
          ((         -413.0,        -1372.0,         -689.0),
           (          653.0,           74.0,         -116.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,          -25.0,          -36.0),
           (          101.0,          269.0,           96.0),
           (            0.0,            0.0,            0.0)),
          ((          172.0,           56.0,           25.0),
           (            3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((         1323.0,          932.0,          335.0),
           (          101.0,          -35.0,          -37.0),
           (            0.0,            0.0,            0.0)),
          ((         -828.0,         7153.0,         3509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          869.0,          404.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3163.0,         -730.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          980.0,          931.0,          332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4816.0,        -6021.0,        -2519.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2687.0,         2436.0,          721.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1675.0,         -413.0,          339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          621.0,         1223.0,          554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4883.0,        -2310.0,        -1589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1723.0,         1328.0,          305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,         -155.0,         -290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          819.0,          823.0,          428.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1110.0,        -1366.0,         -692.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          574.0,          733.0,          347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -377.0,            9.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1530.0,         1766.0,          725.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13222.0,       -57168.0,       -28865.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16376.0,        16112.0,         7257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1801.0,        10427.0,         4979.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7540.0,        -8101.0,        -3611.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8968.0,         9123.0,         3959.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13976.0,        -8597.0,        -3572.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -65922.0,         7757.0,         7309.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -2349185289.0,  -2730990085.0,  -1215536520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7959.0,        61714.0,        33041.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4901.0,        -5030.0,        -2007.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5640.0,         8960.0,         4074.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -71949.0,      -132755.0,       -62560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        67020.0,       122925.0,        57866.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          877.0,         6846.0,         3787.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1230.0,        -1331.0,         -464.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28696.0,         2620.0,        -1710.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2128.0,        -1639.0,         -643.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1249.0,        -1481.0,         -548.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1840.0,         1969.0,          805.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -243.0,         -199.0,          -79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          127.0,          317.0,          147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1435.0,        -3041.0,        -1296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          281.0,          780.0,          367.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1136.0,        -2736.0,        -1526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          415.0,          164.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          624.0,          419.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1871.0,         1816.0,          726.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,           84.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -682.0,          564.0,          339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4346.0,        -1088.0,         -136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -270.0,         -668.0,         -286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -94.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,          -47.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,          -84.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -767.0,         -442.0,         -155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1214.0,         6399.0,         3212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          540.0,          -36.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5233.0,         2950.0,         2079.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          290.0,          340.0,          152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -285.0,         -141.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,           12.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -784.0,          171.0,          133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -169.0,          630.0,          544.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          105.0,        -1843.0,         -750.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    345555258.0,    162266204.0,     50881134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2717.0,       -12597.0,        -6411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3694.0,        -8231.0,        -3920.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3272.0,         4451.0,         1959.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1345.0,        -1552.0,         -658.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          439.0,          293.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,         -114.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          204.0,         -652.0,         -395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -179.0,          -10.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10153.0,         3997.0,         3206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -25.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,           50.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          423.0,          154.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -117.0,          -50.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -4.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          470.0,          440.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,         -107.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -197.0,         1270.0,          702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           34.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -351.0,          -40.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,           21.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1241.0,         -421.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,           -3.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,         -168.0,          -84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,           22.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1148.0,         -189.0,         -213.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          189.0,          158.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,           10.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          331.0,           30.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1064.0,        -1203.0,         -525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,           46.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          803.0,        -1654.0,         -966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -41.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,          -14.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,          -11.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,         -124.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          771.0,         -975.0,         -615.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          551.0,          298.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -58428765.0,     -2226925.0,      4864390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          285.0,          379.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2992.0,         3388.0,         1476.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,           49.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           16.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          161.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,           -3.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1962.0,        -2143.0,        -1347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           12.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,            3.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           64.0,            1.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,           11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -269.0,          -68.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,           26.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,         -329.0,         -158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -10.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          352.0,          -21.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           37.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -236.0,          162.0,          109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          -18.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,           16.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          345.0,          183.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -26.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          560.0,          294.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -19.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          141.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,         -291.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           38.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          560.0,          270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,           26.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      9475366.0,     -3747139.0,     -2983548.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          142.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1128.0,         -567.0,         -160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           -9.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          225.0,          -77.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -203.0,           67.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -37.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -173.0,          713.0,          399.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -10.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -5.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           -5.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           64.0,           62.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,           44.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          -63.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -14.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,           -6.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,         -134.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -21.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,         -126.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          -13.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1287363.0,      1418837.0,       891358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          230.0,           19.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          159.0,          -17.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,           21.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,         -192.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -18.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -11.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           22.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           33.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        91341.0,      -381394.0,      -213210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           29.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,           16.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           13.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           42.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        23596.0,        85059.0,        42995.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -14.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14180.0,       -15681.0,        -6908.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4680.0,         2072.0,          622.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1212.0,          -22.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          259.0,         -106.0,          -84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           48.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        35297.0,        19344.0,         7019.0),
           (         -158.0,         -927.0,         -514.0),
           (       -55681.0,       -51239.0,       -20733.0)),
          ((        -3329.0,        -5168.0,        -2390.0),
           (        26894.0,        -3015.0,        -3633.0),
           (        54503.0,        69656.0,        32965.0)),
          ((         2150.0,         2665.0,         1201.0),
           (         3282.0,         -902.0,         -690.0),
           (            0.0,            0.0,            0.0)),
          ((         -120.0,         -704.0,         -334.0),
           (       -46813.0,         3810.0,        65053.0),
           (            0.0,            0.0,            0.0)),
          ((       -13322.0,       -13878.0,        -5845.0),
           (        63138.0,       -54422.0,       -36204.0),
           (            0.0,            0.0,            0.0)),
          ((         1214.0,          677.0,          242.0),
           (         1143.0,         1437.0,          562.0),
           (            0.0,            0.0,            0.0)),
          ((       -11908.0,        12385.0,         8339.0),
           (         -258.0,        14699.0,        13962.0),
           (            0.0,            0.0,            0.0)),
          ((          893.0,         2330.0,         1098.0),
           (          763.0,       -12855.0,        -6211.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,          865.0,          517.0),
           (         -299.0,          552.0,          278.0),
           (            0.0,            0.0,            0.0)),
          ((          428.0,          449.0,          191.0),
           (          -54.0,          499.0,          240.0),
           (            0.0,            0.0,            0.0)),
          ((         -918.0,         -778.0,         -312.0),
           (           -1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((         -679.0,           22.0,           78.0),
           (           57.0,         -112.0,          -68.0),
           (            0.0,            0.0,            0.0)),
          ((        -9496.0,        -1056.0,          366.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1461.0,         1317.0,          540.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3505.0,        -1101.0,         -156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1658.0,         1831.0,          789.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6732.0,        -7863.0,        -3460.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5290.0,         6228.0,         2790.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4186.0,        -6028.0,        -2938.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1254.0,          966.0,          331.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5777.0,         1413.0,         1269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,         1815.0,          823.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1899.0,         -345.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2242.0,         1246.0,          459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2139.0,        -1690.0,         -727.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          969.0,          804.0,          334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -715.0,          219.0,          170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2279.0,         3341.0,         1596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -40604.0,        -3911.0,         1968.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26103.0,        29949.0,        12844.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8239.0,         3971.0,          346.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9368.0,        -9223.0,        -3899.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8665.0,         8092.0,         3243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7066.0,        -2451.0,         -653.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21126.0,        67423.0,        33475.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2497440719.0,   2625790069.0,   1144089316.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -42273.0,       -12699.0,        -1188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10129.0,        -8912.0,        -3516.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11229.0,        10483.0,         4289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -150180.0,      -128900.0,       -51335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       138577.0,       119217.0,        47522.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7815.0,         5423.0,         2021.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1637.0,        -1092.0,         -377.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26673.0,          -13.0,         2817.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          352.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1112.0,         -981.0,         -268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          308.0,          630.0,          289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,          -13.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,          -17.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2279.0,         1309.0,          778.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,         -284.0,         -156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1249.0,         2538.0,         1442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -305.0,           63.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -712.0,         -360.0,         -129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2093.0,         1941.0,          772.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,          -55.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -448.0,          736.0,          413.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          -82.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -670.0,         4008.0,         2005.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          204.0,          611.0,          264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,          159.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -25.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          298.0,          253.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1035.0,         -511.0,         -168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2482.0,         6427.0,         3095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,         -313.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3950.0,         4534.0,         1880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,          -86.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          188.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,          -49.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          294.0,          128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4898.0,        -3039.0,        -1999.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5677.0,        -3605.0,        -2394.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    360702968.0,    130275327.0,     32233884.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11742.0,         2180.0,         2319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8119.0,         -366.0,          881.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3426.0,          124.0,         -479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -994.0,          154.0,          263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,          -43.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,           66.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          818.0,          -31.0,         -116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,           65.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9824.0,         4624.0,         3504.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,          183.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -833.0,           76.0,          136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          162.0,         -222.0,         -124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,           68.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,           16.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -201.0,          502.0,          276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,         -111.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,         1288.0,          700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,           30.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -125.0,           16.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,           -7.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1204.0,          340.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           20.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,          153.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           10.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          393.0,         1013.0,          485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,          136.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           33.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,           20.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,          -26.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1407.0,         1069.0,          422.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -214.0,         -116.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2244.0,          861.0,          214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,          -98.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,           25.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -30.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          184.0,          -43.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,        -2172.0,        -1128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           77.0,        -1905.0,         -981.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     57651212.0,     -5869120.0,     -9107342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          465.0,          481.0,          171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3339.0,         2072.0,         1462.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -110.0,           17.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           47.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          141.0,          -98.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           24.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1657.0,         2350.0,         1426.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           34.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -148.0,           94.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -48.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           28.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,           30.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,          108.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,          -14.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,          316.0,          149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           21.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           20.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          352.0,          -64.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           31.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          223.0,          188.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,           11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,           34.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          407.0,           88.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           -2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          606.0,          -59.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -56.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,          113.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,            3.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -460.0,         -521.0,         -252.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -213.0,         -482.0,         -190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      8305023.0,     -5447440.0,     -3770469.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,         -140.0,         -134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -306.0,          890.0,          531.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,           39.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          683.0,          -19.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -611.0,           19.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,          -40.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           21.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          736.0,          396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           34.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           30.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,           53.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,          -57.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,           18.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           18.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          -20.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          130.0,          -84.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -28.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,          -68.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,          -83.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       817760.0,     -1684058.0,      -984392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          149.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          151.0,           82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -54.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,          181.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -22.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -18.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -40.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           11.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -17.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -46084.0,      -393807.0,      -205610.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           21.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           33.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           22.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           33.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -55706.0,       -73535.0,       -33514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19822.0,        -9566.0,        -3057.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5124.0,          -21.0,          520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1034.0,          516.0,          383.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -144.0,          218.0,          131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           62.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -24605.0,       -33368.0,       -14721.0),
           (       -13891.0,        -3725.0,         -700.0),
           (          -25.0,        10950.0,         5686.0)),
          ((           -7.0,        10686.0,         5415.0),
           (         6492.0,        -3171.0,         -217.0),
           (        35000.0,       -24140.0,       -16762.0)),
          ((           28.0,        -5854.0,        -2970.0),
           (        -2088.0,        -3682.0,        -1632.0),
           (            0.0,            0.0,            0.0)),
          ((          572.0,        -1665.0,         -903.0),
           (        32958.0,         6941.0,       -58228.0),
           (            0.0,            0.0,            0.0)),
          ((        14703.0,       -10519.0,        -6823.0),
           (        23866.0,        -8678.0,        -5205.0),
           (            0.0,            0.0,            0.0)),
          ((        -1609.0,         -742.0,         -220.0),
           (         1597.0,        -1546.0,         -830.0),
           (            0.0,            0.0,            0.0)),
          ((       -10207.0,        13730.0,         8876.0),
           (        -9107.0,        20366.0,        17764.0),
           (            0.0,            0.0,            0.0)),
          ((          481.0,         1394.0,          643.0),
           (         1600.0,         3233.0,          957.0),
           (            0.0,            0.0,            0.0)),
          ((         -765.0,        -1773.0,         -877.0),
           (         -998.0,        -1255.0,         -575.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,           29.0,           20.0),
           (         -173.0,         -491.0,         -258.0),
           (            0.0,            0.0,            0.0)),
          ((          401.0,          146.0,           35.0),
           (            0.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         1264.0,          384.0,           80.0),
           (         -144.0,           60.0,           60.0),
           (            0.0,            0.0,            0.0)),
          ((         -876.0,        -7938.0,        -3724.0),
           (            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          311.0,          439.0,          187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2668.0,           24.0,          353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,         -164.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3596.0,        -3316.0,        -1416.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -414.0,        -1480.0,         -750.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          790.0,         2787.0,         1535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -240.0,          281.0,          231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5004.0,        -2553.0,        -1794.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -762.0,         -899.0,         -175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1392.0,         -556.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          868.0,         -160.0,         -312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -860.0,         -226.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          288.0,           29.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1075.0,         1423.0,          669.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1780.0,          288.0,         -126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6076.0,       -27684.0,       -13763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6648.0,        -6378.0,        -3869.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8072.0,        -3413.0,        -2111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1332.0,         1449.0,          838.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -709.0,        -1515.0,         -750.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5313.0,         1261.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63039.0,       -29480.0,       -18044.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -2639046060.0,  -2513598425.0,  -1069601752.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35764.0,       -45520.0,       -27702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2227.0,        -3232.0,        -1510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3028.0,         1459.0,          560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -48704.0,        -5228.0,         1732.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        45287.0,         5267.0,        -1358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2429.0,        -1858.0,        -1467.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          193.0,        -1084.0,         -726.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28081.0,          645.0,        -2593.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2043.0,        -1165.0,         -444.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1289.0,        -1454.0,         -526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1082.0,          331.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -187.0,          -43.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          184.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1352.0,         1719.0,          641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,          403.0,          211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1196.0,        -2593.0,        -1467.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          352.0,           -6.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          655.0,          246.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1890.0,         1729.0,          676.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,           94.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -438.0,          700.0,          384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4460.0,          563.0,         -132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -188.0,         -648.0,         -285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,         -119.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           31.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -329.0,          -93.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1005.0,         -343.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3176.0,         5815.0,         2717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          209.0,          275.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3612.0,        -4050.0,        -2477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          759.0,          342.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -379.0,         -318.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          260.0,          225.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1702.0,        -1314.0,         -477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3790.0,        10080.0,         4716.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8102.0,         9114.0,         3664.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    371971913.0,     96878076.0,     13236942.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3742.0,         9151.0,         5301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5358.0,        12126.0,         5741.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4341.0,        -7360.0,        -3332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1934.0,         2470.0,         1057.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -353.0,         -551.0,         -244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           77.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          122.0,          613.0,          338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           66.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9460.0,         5327.0,         3843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          416.0,           96.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          654.0,          297.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          116.0,          -15.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,            1.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           -8.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,          -25.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -468.0,         -230.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          -13.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,         1262.0,          674.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,           61.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -67.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,          101.0,           71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1072.0,         -336.0,          -59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -77.0,           36.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,         -231.0,         -128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,           68.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1055.0,          423.0,          324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          189.0,          155.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,            9.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,          -26.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          350.0,          -43.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1467.0,         -801.0,         -274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,           10.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -486.0,         1721.0,          966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,          104.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          152.0,          -45.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           33.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          268.0,         -169.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3124.0,        -1560.0,         -472.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4598.0,          -96.0,          462.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -55477772.0,     13824596.0,     13130938.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2557.0,         -483.0,         -533.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3007.0,        -3411.0,        -1486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          198.0,          -70.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,           83.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          -12.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,         -122.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           -5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1301.0,        -2553.0,        -1497.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,           17.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -169.0,            8.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           26.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -10.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,            9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -32.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,           20.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -192.0,         -306.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -12.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -13.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          329.0,          -95.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           32.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,         -222.0,         -133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,           -9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,           59.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          399.0,          -16.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           39.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -154.0,         -553.0,         -280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,           34.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          165.0,         -121.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -422.0,          296.0,          239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,           17.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1056.0,         -270.0,         -289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          730.0,         -215.0,         -152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      6776927.0,     -6913860.0,     -4395510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           64.0,         -112.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          938.0,          562.0,          228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          154.0,          -23.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -748.0,           95.0,          226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          665.0,          -90.0,         -205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           11.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          120.0,          747.0,          388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -15.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -21.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           19.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           80.0,           44.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,           67.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           70.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           -6.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,           39.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,           99.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -35.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -186.0,          216.0,          134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          116.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -293161.0,      1836307.0,      1011409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -186.0,          -39.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,           66.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,          -40.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           14.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -142.0,         -161.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -24.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -17.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           -3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -67.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -50.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -179139.0,      -368231.0,      -178176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,          -18.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -24.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,           19.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        80539.0,        52381.0,        19644.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           15.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22089.0,        -1820.0,         1315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4472.0,        -2038.0,        -1552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -584.0,          919.0,          551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,         -260.0,         -137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,           54.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -28903.0,         -534.0,         2068.0),
           (         7159.0,         4144.0,         1517.0),
           (        88016.0,         2556.0,        -5557.0)),
          ((        11428.0,        -1422.0,        -1686.0),
           (       -85878.0,        -9280.0,         5287.0),
           (       -68724.0,       -52420.0,       -20368.0)),
          ((        -7049.0,          658.0,          935.0),
           (        -3412.0,         3058.0,         1888.0),
           (            0.0,            0.0,            0.0)),
          ((        -1314.0,          -23.0,           96.0),
           (       -23139.0,         2312.0,        62455.0),
           (            0.0,            0.0,            0.0)),
          ((        16428.0,        12980.0,         5093.0),
           (      -215704.0,        -8425.0,        24805.0),
           (            0.0,            0.0,            0.0)),
          ((        -2848.0,        -2056.0,         -805.0),
           (        -9076.0,        -1705.0,          289.0),
           (            0.0,            0.0,            0.0)),
          ((        -9395.0,        14061.0,         8989.0),
           (        -8543.0,        -1731.0,         6272.0),
           (            0.0,            0.0,            0.0)),
          ((          962.0,         1502.0,          641.0),
           (       -14206.0,       -12035.0,        -4321.0),
           (            0.0,            0.0,            0.0)),
          ((         1102.0,          813.0,          338.0),
           (         1320.0,         -892.0,         -610.0),
           (            0.0,            0.0,            0.0)),
          ((         -182.0,          355.0,          203.0),
           (          471.0,          256.0,           80.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,         -395.0,         -209.0),
           (            1.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((        -1784.0,           -8.0,          165.0),
           (          -59.0,          174.0,           79.0),
           (            0.0,            0.0,            0.0)),
          ((        10437.0,        -1121.0,        -1555.0),
           (           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2394.0,          537.0,          506.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1053.0,         -173.0,          106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2337.0,          563.0,          524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          505.0,        -4548.0,        -2476.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6446.0,         1955.0,         1742.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4200.0,        -2834.0,        -2108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1401.0,         -299.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3642.0,         3199.0,         1947.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -685.0,         -725.0,         -204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2925.0,        -1308.0,         -807.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3998.0,          778.0,          651.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2990.0,         -958.0,         -722.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1388.0,          387.0,          311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3684.0,          461.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5608.0,        -1209.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        58313.0,        34061.0,        12994.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43732.0,         1684.0,         4228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14578.0,         6745.0,         5526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14075.0,          309.0,         -984.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14288.0,          122.0,         1151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11176.0,        -2729.0,        -2303.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -56301.0,       -63952.0,       -26191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2773587012.0,   2394684649.0,    992258074.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        48689.0,        33482.0,        10859.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11688.0,         5062.0,         1572.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14347.0,        -7645.0,        -2632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       169938.0,       107867.0,        40226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -156342.0,       -99412.0,       -37114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7665.0,        -4981.0,        -1806.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          378.0,         1560.0,          806.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28538.0,         -686.0,         2656.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2519.0,         1556.0,          579.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,         -147.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1126.0,        -1711.0,         -787.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          221.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -227.0,         -302.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2425.0,         -874.0,         -527.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -293.0,         -649.0,         -318.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1153.0,         2692.0,         1511.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,         -101.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -770.0,         -200.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2160.0,         1777.0,          680.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           65.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,          667.0,          356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          104.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          138.0,        -4164.0,        -2032.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          204.0,          671.0,          296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           89.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -15.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          299.0,          -33.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1179.0,         -574.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4291.0,         5745.0,         2556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -209.0,         -105.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3614.0,        -3722.0,        -1517.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -784.0,         -304.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          392.0,          161.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -515.0,          305.0,          215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2951.0,        -1232.0,         -961.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13738.0,         1760.0,         2396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15088.0,         7717.0,         5618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    379252149.0,     62441426.0,     -5903029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21975.0,        -1075.0,        -2942.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -370.0,         2559.0,         1333.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4652.0,        -2959.0,        -2001.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2931.0,         1422.0,         1036.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          722.0,         -313.0,         -235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           71.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -586.0,          187.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -150.0,           36.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8785.0,         6043.0,         4154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          260.0,         -121.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,         -469.0,         -235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -602.0,          189.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          248.0,          -80.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,           26.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,          -37.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -119.0,         -513.0,         -239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          250.0,           24.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,         1249.0,          683.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          138.0,           85.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -509.0,          -90.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          431.0,          178.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          934.0,           69.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,           89.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,           84.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          112.0,           41.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -683.0,         -956.0,         -427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          236.0,          160.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -45.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -420.0,           71.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1726.0,          510.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,          121.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1761.0,         -671.0,         -158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -292.0,          265.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          171.0,         -142.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          122.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,         -462.0,         -251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2424.0,         3007.0,         1835.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -822.0,         4888.0,         2652.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     51963097.0,    -21446407.0,    -16837806.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          953.0,        -2238.0,        -1277.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4022.0,        -2084.0,        -1534.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,         -335.0,         -175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          245.0,          131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -75.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,          112.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           21.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -895.0,         2698.0,         1531.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          -63.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -146.0,          -60.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           89.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -40.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          -79.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,          -12.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -47.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          226.0,          296.0,          130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           22.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          323.0,         -138.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           20.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -280.0,         -119.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,            4.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,           62.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          425.0,          -80.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,            1.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -491.0,          208.0,          160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,          -11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          270.0,           85.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -498.0,         -287.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,         -154.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          249.0,         1053.0,          492.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,          899.0,          483.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      4957795.0,     -8083647.0,     -4832078.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          232.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          442.0,        -1260.0,         -690.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           58.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          300.0,         -575.0,         -247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -271.0,          528.0,          228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           22.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           31.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          278.0,          718.0,          355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -23.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           10.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -19.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,           32.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          -78.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,           12.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           -7.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           25.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,          -62.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,          122.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,          -12.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          268.0,          252.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          187.0,           32.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -251180.0,     -1865502.0,      -970675.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,         -202.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,         -175.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           73.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -12.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          179.0,          130.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -24.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           13.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -22.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           39.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,           19.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,          -12.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -295012.0,      -307224.0,      -133597.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,           15.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,          -53.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -6.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,            3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -94856.0,       -24371.0,        -3200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -7.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20596.0,         6240.0,         5466.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2857.0,         3659.0,         2251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,         1077.0,          574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          170.0,          216.0,           98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           24.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        43037.0,        21849.0,         7286.0),
           (         2611.0,          305.0,          -90.0),
           (      -141327.0,       -61588.0,       -19919.0)),
          ((       -13320.0,        -6285.0,        -2046.0),
           (        61227.0,       -46341.0,       -30748.0),
           (        96792.0,        94879.0,        41875.0)),
          ((         8125.0,         3621.0,         1135.0),
           (        -1223.0,         4526.0,         2424.0),
           (            0.0,            0.0,            0.0)),
          ((         1648.0,         1710.0,          733.0),
           (        32805.0,       -13630.0,       -68702.0),
           (            0.0,            0.0,            0.0)),
          ((       -14120.0,        12938.0,         7985.0),
           (       150297.0,       -85760.0,       -62102.0),
           (            0.0,            0.0,            0.0)),
          ((         2684.0,         -730.0,         -617.0),
           (         6179.0,        -5994.0,        -3816.0),
           (            0.0,            0.0,            0.0)),
          ((       -11519.0,        13475.0,         8871.0),
           (        21560.0,        10438.0,         8849.0),
           (            0.0,            0.0,            0.0)),
          ((         2343.0,         2347.0,          975.0),
           (          923.0,        18358.0,         9240.0),
           (            0.0,            0.0,            0.0)),
          ((        -1022.0,        -1360.0,         -622.0),
           (           90.0,         2519.0,         1333.0),
           (            0.0,            0.0,            0.0)),
          ((          266.0,            1.0,          -30.0),
           (           88.0,          752.0,          409.0),
           (            0.0,            0.0,            0.0)),
          ((         -126.0,         -166.0,          -66.0),
           (            6.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((         1958.0,          379.0,            6.0),
           (          120.0,           73.0,           52.0),
           (            0.0,            0.0,            0.0)),
          ((         -646.0,         7138.0,         3465.0),
           (           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2156.0,          337.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4149.0,         -208.0,          403.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2783.0,          919.0,          186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9798.0,        -4882.0,        -1556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8151.0,         2498.0,          343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5458.0,         -531.0,          524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2096.0,         1308.0,          473.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2742.0,        -4185.0,        -2350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2727.0,         1221.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1581.0,         -136.0,         -146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3428.0,          843.0,          241.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3432.0,        -1302.0,         -459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1767.0,          727.0,          246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1692.0,         -366.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5371.0,         2703.0,          912.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -72499.0,       -63615.0,       -27199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51303.0,        20013.0,         6453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16582.0,        11448.0,         4246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20525.0,        -9338.0,        -3174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        23595.0,         9932.0,         3123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26879.0,        -7803.0,        -1685.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -38817.0,        49809.0,        28210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -2900710891.0,  -2269355251.0,   -912258658.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19365.0,        64590.0,        35919.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13069.0,        -6058.0,        -1974.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17263.0,        10442.0,         4014.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -229303.0,      -157913.0,       -64059.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       212647.0,       146021.0,        59135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7637.0,         8264.0,         3985.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3033.0,        -1556.0,         -463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29848.0,          150.0,        -3060.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3577.0,        -1388.0,         -438.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1848.0,        -1471.0,         -516.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3082.0,         1959.0,          730.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -342.0,         -146.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,          370.0,          155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2551.0,        -2566.0,         -996.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          559.0,          755.0,          331.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1037.0,        -2697.0,        -1498.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          526.0,           57.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          719.0,           49.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2009.0,         1785.0,          694.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,            5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,          907.0,          461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,         -103.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4409.0,            0.0,          397.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -246.0,         -687.0,         -301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          -90.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,           -8.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,           -7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -799.0,         -522.0,         -203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4642.0,         5327.0,         2317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          332.0,         -736.0,         -411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4288.0,         4794.0,         2939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,         -627.0,         -334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,          168.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -184.0,         -921.0,         -453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          805.0,         3959.0,         1951.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3361.0,       -14639.0,        -7209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6704.0,       -14297.0,        -6641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    382448983.0,     27339393.0,    -24976632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          347.0,        -6549.0,        -3537.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7691.0,       -21156.0,       -10244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6596.0,        15885.0,         7574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2654.0,        -6268.0,        -2978.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          762.0,         1392.0,          642.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -274.0,         -257.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -267.0,         -424.0,         -216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,         -185.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8121.0,         6821.0,         4494.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,         -263.0,         -125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,          273.0,          103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          327.0,         -217.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          142.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -63.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,          110.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          543.0,         -114.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          168.0,           80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         1156.0,          617.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,          180.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -414.0,         -120.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          292.0,          258.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1385.0,         -293.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,           83.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -246.0,         -239.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,           70.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          879.0,         -653.0,         -426.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          218.0,          152.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          -14.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          349.0,            4.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1756.0,         -453.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          281.0,          223.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -251.0,        -2185.0,        -1131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          341.0,          253.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,         -131.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          162.0,           96.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,         -327.0,         -115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3982.0,         2441.0,          851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4989.0,         2009.0,          515.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -47191173.0,     28547068.0,     20136587.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2819.0,        -1327.0,         -402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4698.0,         3588.0,         1399.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -409.0,         -180.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          238.0,          158.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,          -49.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,            1.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,           55.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,            7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          445.0,        -2802.0,        -1540.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           39.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,           13.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,           53.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -26.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -29.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,          114.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          -29.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -247.0,         -257.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -30.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -33.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          292.0,         -162.0,         -118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           15.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,          258.0,          141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,           -1.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           46.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          322.0,         -126.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -22.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          282.0,          478.0,          220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,           -5.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -451.0,           50.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1000.0,          -73.0,          -96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,           15.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1660.0,           79.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -704.0,          277.0,          265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2925703.0,     -8905116.0,     -5060654.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -262.0,          -94.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1164.0,          -97.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -188.0,          -23.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1017.0,            4.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -911.0,           -5.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,            4.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          418.0,          664.0,          311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -30.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,           19.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           82.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -69.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -13.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -23.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           75.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,          -53.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           32.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          296.0,         -274.0,         -177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -96.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       778558.0,      1769543.0,       864859.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,          -65.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          189.0,         -155.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           84.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -20.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,          -81.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -21.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -29.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          -24.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           96.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           35.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -382332.0,      -216615.0,       -76162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           46.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           35.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           42.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,          -12.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -23.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        96743.0,        -6817.0,       -13655.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15593.0,        13236.0,         8686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          627.0,        -4494.0,        -2466.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          620.0,          950.0,          443.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -269.0,         -103.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,          -16.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           13.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        12825.0,       -17610.0,       -10045.0),
           (        -7918.0,         4509.0,         2912.0),
           (        14169.0,        10991.0,         3002.0)),
          ((         3350.0,         6906.0,         3228.0),
           (        88309.0,        45518.0,        14309.0),
           (       -11518.0,        22722.0,        10947.0)),
          ((        -1868.0,        -3847.0,        -1797.0),
           (         1795.0,        -3729.0,        -2016.0),
           (            0.0,            0.0,            0.0)),
          ((        -1571.0,        -1829.0,         -785.0),
           (       -54191.0,        20892.0,        76754.0),
           (            0.0,            0.0,            0.0)),
          ((       -15798.0,        -9484.0,        -3343.0),
           (       175142.0,        35774.0,        -1670.0),
           (            0.0,            0.0,            0.0)),
          ((          382.0,         -568.0,         -329.0),
           (         5851.0,         5583.0,         2204.0),
           (            0.0,            0.0,            0.0)),
          ((        -9681.0,        13961.0,         8952.0),
           (         7424.0,        38988.0,        25125.0),
           (            0.0,            0.0,            0.0)),
          ((          901.0,         1958.0,          913.0),
           (        -1016.0,        -1595.0,         -534.0),
           (            0.0,            0.0,            0.0)),
          ((          255.0,         1157.0,          605.0),
           (        -2778.0,          219.0,          455.0),
           (            0.0,            0.0,            0.0)),
          ((          351.0,          135.0,           35.0),
           (         -511.0,          -12.0,           76.0),
           (            0.0,            0.0,            0.0)),
          ((         -816.0,          -96.0,           28.0),
           (            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1001.0,          115.0,          147.0),
           (          118.0,         -201.0,         -149.0),
           (            0.0,            0.0,            0.0)),
          ((        -8935.0,         1058.0,         1361.0),
           (            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          885.0,          103.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2936.0,          332.0,          570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          795.0,          141.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6300.0,        -3182.0,        -1103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2619.0,          977.0,          307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2469.0,        -2098.0,        -1020.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          420.0,         -320.0,         -247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3807.0,         3880.0,         2339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -568.0,         1039.0,          459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -961.0,         1019.0,          486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          691.0,         -901.0,         -465.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -821.0,          270.0,          153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,         -128.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1223.0,         1189.0,          489.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -153.0,          166.0,          225.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3575.0,        33889.0,        16815.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2647.0,         1852.0,          903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2155.0,        -7432.0,        -4353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          462.0,          369.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2689.0,        -2172.0,         -649.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7874.0,         6173.0,         2181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        42954.0,        44848.0,        16088.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3020061127.0,   2137927500.0,    829808827.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -54450.0,       -30651.0,        -9439.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4234.0,        -4736.0,        -1963.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2873.0,         4261.0,         1939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34697.0,       -49042.0,       -21930.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        31884.0,        45290.0,        20286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3745.0,         1748.0,          426.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          359.0,         -474.0,         -336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27707.0,         2177.0,         4082.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1643.0,           -4.0,         -123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -458.0,         -974.0,         -358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1100.0,          272.0,          286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           47.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -237.0,           65.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1378.0,         1894.0,          957.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,         -174.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1159.0,         2457.0,         1390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -402.0,          206.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -763.0,           21.0,           69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2189.0,         1774.0,          675.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          859.0,          431.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -63.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          478.0,         4100.0,         1947.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          188.0,          629.0,          277.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          159.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -34.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          363.0,           86.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -905.0,         -175.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5349.0,         4496.0,         1812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          447.0,          -50.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3502.0,         3623.0,         1466.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1095.0,          146.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -242.0,          133.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          942.0,           49.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4271.0,         -128.0,          437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18172.0,        -3397.0,        -3830.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15066.0,        -3127.0,        -3338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    381529320.0,     -8067316.0,    -43787793.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18828.0,         1787.0,         3020.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          631.0,         1379.0,          567.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4938.0,         -992.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2611.0,          449.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -788.0,          -99.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          245.0,          -17.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          503.0,         -487.0,         -335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          114.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7683.0,         7171.0,         4636.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          183.0,           82.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1003.0,          133.0,          187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          356.0,         -221.0,         -149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -203.0,           67.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,            8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,           19.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          227.0,          622.0,          293.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -201.0,         -143.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          250.0,         1326.0,          681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,           16.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          225.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -87.0,         -133.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1259.0,          134.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -32.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          116.0,          175.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -35.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          863.0,          776.0,          315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          208.0,          110.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           43.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -27.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          -11.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -266.0,          118.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1675.0,           74.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -79.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1990.0,          241.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          152.0,         -175.0,         -113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           72.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          109.0,          -75.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -440.0,          254.0,          184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2708.0,        -4258.0,        -2543.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2490.0,        -2874.0,        -1789.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     41274478.0,    -34957653.0,    -22948677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1942.0,         1276.0,          880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2239.0,         2792.0,         1713.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -391.0,          184.0,          138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,          -61.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -88.0,           24.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,          -25.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,         -158.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,            4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -102.0,         2840.0,         1525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,           -4.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,          151.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -69.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           45.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           78.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          191.0,           -3.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,           34.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          280.0,          233.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           31.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           36.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          267.0,         -203.0,         -138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          300.0,           23.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           74.0,           -4.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -21.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           80.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          292.0,         -245.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           39.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          439.0,         -381.0,         -252.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           59.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,         -345.0,         -190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,          788.0,          435.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           95.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -378.0,        -1650.0,         -864.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -146.0,         -568.0,         -248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       767008.0,     -9343446.0,     -5071556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,         -568.0,         -361.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,         1258.0,          683.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -71.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,          693.0,          447.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,         -624.0,         -402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -25.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -27.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          524.0,          606.0,          270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -6.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           50.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,            6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -85.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,          -41.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           21.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -80.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,         -139.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           44.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -361.0,         -295.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,           14.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1253824.0,     -1554746.0,      -700969.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,          118.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,          131.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -50.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          217.0,           43.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -22.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -18.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -17.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -32.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,            2.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,            1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -432793.0,      -105026.0,       -11334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,            4.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,            8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,          -23.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           20.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -85961.0,        37124.0,        28730.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7932.0,        17974.0,        10424.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1737.0,         4360.0,         2150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1059.0,          576.0,          198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          284.0,          -37.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -48.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -53056.0,         8432.0,         9110.0),
           (        -5201.0,         1597.0,         1247.0),
           (        21904.0,       -51800.0,       -27997.0)),
          ((         5019.0,        -3455.0,        -2218.0),
           (       -12871.0,         6563.0,         5636.0),
           (        29971.0,       -19582.0,       -11905.0)),
          ((        -3157.0,         2017.0,         1313.0),
           (        -3303.0,        -2370.0,         -724.0),
           (            0.0,            0.0,            0.0)),
          ((         -197.0,          -74.0,          -25.0),
           (        24694.0,         3450.0,       -64176.0),
           (            0.0,            0.0,            0.0)),
          ((        11245.0,       -15086.0,        -8820.0),
           (       -42930.0,        19103.0,        15741.0),
           (            0.0,            0.0,            0.0)),
          ((        -1900.0,         1076.0,          731.0),
           (        -2100.0,        -1332.0,         -414.0),
           (            0.0,            0.0,            0.0)),
          ((        -8169.0,        13425.0,         8543.0),
           (       -16044.0,        16455.0,        15815.0),
           (            0.0,            0.0,            0.0)),
          ((          791.0,         1981.0,          926.0),
           (        14362.0,        -1188.0,        -2272.0),
           (            0.0,            0.0,            0.0)),
          ((        -1198.0,        -1720.0,         -789.0),
           (         -694.0,        -2727.0,        -1309.0),
           (            0.0,            0.0,            0.0)),
          ((         -146.0,          313.0,          170.0),
           (         -275.0,         -691.0,         -299.0),
           (            0.0,            0.0,            0.0)),
          ((          574.0,         -453.0,         -282.0),
           (           -5.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((         1130.0,          203.0,            2.0),
           (          -15.0,          -10.0,           32.0),
           (            0.0,            0.0,            0.0)),
          ((        -1980.0,        -8683.0,        -3999.0),
           (            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,         1524.0,          763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2259.0,         -382.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -497.0,         1445.0,          780.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3686.0,        -5217.0,        -2385.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2260.0,         3516.0,         1964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2482.0,         -939.0,         -555.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -476.0,         1549.0,          885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3540.0,        -4877.0,        -2831.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1158.0,          -50.0,          284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -868.0,        -1808.0,         -629.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,         2015.0,          885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -162.0,        -1999.0,         -931.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          891.0,          430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2168.0,         -873.0,         -588.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          384.0,         3172.0,         1434.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1982.0,       -63692.0,       -32165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5871.0,        21026.0,        11049.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3514.0,         8060.0,         4128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2630.0,        -8096.0,        -4326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3795.0,         8856.0,         4784.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7503.0,       -11183.0,        -6248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32888.0,       -44731.0,       -27744.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3131352285.0,  -2000801270.0,   -745148916.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        39237.0,       -45465.0,       -28122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1993.0,        -6362.0,        -3432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1711.0,         5327.0,         2899.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14348.0,       -51334.0,       -26888.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12685.0,        47641.0,        24923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -923.0,         -740.0,         -565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          514.0,        -1663.0,        -1022.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27373.0,        -1746.0,        -3771.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1786.0,         -918.0,         -368.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -986.0,        -1353.0,         -521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          677.0,           64.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,           39.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          147.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2080.0,         1230.0,          379.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,          271.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1217.0,        -2497.0,        -1419.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          346.0,         -197.0,         -122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          678.0,         -168.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1972.0,         1715.0,          657.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,           48.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,          837.0,          402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -33.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4421.0,         -466.0,         -627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,         -646.0,         -292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,         -133.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           37.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -307.0,          114.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1119.0,         -102.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6059.0,         3572.0,         1261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          484.0,          255.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3157.0,        -5006.0,        -2933.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          559.0,          650.0,          282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -301.0,         -464.0,         -195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          156.0,          764.0,          388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -466.0,        -3192.0,        -1645.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4846.0,        18931.0,         9403.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2014.0,        13619.0,         6965.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    376520331.0,    -43389703.0,    -62132271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1565.0,         2718.0,         1704.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5467.0,        24645.0,        12323.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4012.0,       -18650.0,        -9319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1701.0,         7359.0,         3660.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -291.0,        -1720.0,         -867.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          416.0,          219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          576.0,          206.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          306.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7013.0,         7580.0,         4786.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          365.0,          133.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          709.0,           10.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          334.0,          160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,         -195.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           56.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,         -169.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -548.0,          431.0,          270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,         -365.0,         -188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          244.0,         1458.0,          747.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,         -113.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          396.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -123.0,         -345.0,         -166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1109.0,          218.0,          239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,         -109.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,          -19.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -39.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -716.0,          840.0,          507.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          204.0,          116.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           28.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,          -63.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          313.0,         -234.0,         -152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1668.0,          384.0,          369.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,         -265.0,         -147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          263.0,         2240.0,         1159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,         -276.0,         -135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,          106.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -95.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          256.0,          131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5216.0,        -3448.0,        -1253.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3233.0,        -2201.0,         -813.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -34360475.0,     40524392.0,     25207279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3312.0,         1213.0,          283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4431.0,        -3178.0,        -1218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          251.0,           93.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,          -17.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,           15.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -24.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,           24.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          -19.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -309.0,        -2846.0,        -1484.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           42.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -159.0,           53.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           29.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,          -18.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -90.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           13.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -301.0,         -220.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -14.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -26.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          243.0,         -219.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,            1.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,         -272.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,            9.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           97.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          250.0,         -308.0,         -189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           46.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -401.0,         -402.0,         -173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,           38.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          438.0,         -102.0,         -115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1016.0,          236.0,          265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -51.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2089.0,         -332.0,         -428.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          647.0,           84.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1424917.0,     -9381145.0,     -4865091.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,            2.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1058.0,           22.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          209.0,          -40.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1044.0,          145.0,          272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          933.0,         -132.0,         -245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           -4.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -33.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           14.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          631.0,          507.0,          206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -18.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -36.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           32.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,           -4.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           88.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,           48.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -20.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           81.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          165.0,           -6.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -308.0,          354.0,          221.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -24.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1645124.0,      1235706.0,       490083.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,           62.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -157.0,          191.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          -93.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           24.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,           -3.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           18.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -20.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           -3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           37.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,         -118.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -24.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -441678.0,        16680.0,        54600.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -20.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -36.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -39.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          -36.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           22.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63953.0,       -62587.0,       -40057.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1080.0,        19655.0,        10390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3726.0,        -3295.0,        -1375.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1213.0,           48.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -209.0,          168.0,          111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -62.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // ven
    venct   : array [1..8,1..178,0..2,1..3] of double = (
         (((       -10670.0,       -80306.0,       -35331.0),
           (        38915.0,        44422.0,        37591.0),
           (         -349.0,         -610.0,         -279.0)),
          ((        24152.0,       -20179.0,       -10500.0),
           (       -57132.0,       -20077.0,       -19811.0),
           (            0.0,            0.0,            0.0)),
          ((       -20597.0,         3103.0,         2580.0),
           (        -9567.0,        -7367.0,        -5419.0),
           (            0.0,            0.0,            0.0)),
          ((         8307.0,         -745.0,         -825.0),
           (       -30312.0,       -18658.0,       -10960.0),
           (            0.0,            0.0,            0.0)),
          ((       -60881.0,       166947.0,        80206.0),
           (       126360.0,       -40580.0,       269259.0),
           (            0.0,            0.0,            0.0)),
          ((         2845.0,         3480.0,         1444.0),
           (        -6851.0,       -19704.0,       -14849.0),
           (            0.0,            0.0,            0.0)),
          ((       -17846.0,       -10508.0,        -4027.0),
           (        34358.0,         7549.0,         5366.0),
           (            0.0,            0.0,            0.0)),
          ((       -39831.0,       258034.0,       117493.0),
           (        -1031.0,         -518.0,         -419.0),
           (            0.0,            0.0,            0.0)),
          ((         2437.0,          -30.0,          -71.0),
           (          558.0,          327.0,          276.0),
           (            0.0,            0.0,            0.0)),
          ((        12000.0,       -27541.0,       -12064.0),
           (         1588.0,         2069.0,          636.0),
           (            0.0,            0.0,            0.0)),
          ((        -8810.0,        22804.0,         9541.0),
           (        -2741.0,        -2295.0,        -1054.0),
           (            0.0,            0.0,            0.0)),
          ((          298.0,          511.0,          194.0),
           (         1228.0,         7365.0,         2880.0),
           (            0.0,            0.0,            0.0)),
          ((         -282.0,        -2316.0,        -1056.0),
           (         2524.0,       -15938.0,        -6335.0),
           (            0.0,            0.0,            0.0)),
          ((        -1023.0,         6453.0,         3051.0),
           (          -55.0,           49.0,           27.0),
           (            0.0,            0.0,            0.0)),
          ((         3183.0,        -5217.0,        -2658.0),
           (          356.0,         -203.0,         -142.0),
           (            0.0,            0.0,            0.0)),
          ((        -6030.0,        18026.0,         8354.0),
           (         2553.0,         1100.0,          363.0),
           (            0.0,            0.0,            0.0)),
          ((         -652.0,          -89.0,          -16.0),
           (         -568.0,         -168.0,          -62.0),
           (            0.0,            0.0,            0.0)),
          ((         5522.0,        -3619.0,        -2200.0),
           (          125.0,          -19.0,          -16.0),
           (            0.0,            0.0,            0.0)),
          ((         5121.0,       -24382.0,       -13042.0),
           (            6.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         1716.0,         5111.0,         2614.0),
           (          -10.0,          -17.0,           -7.0),
           (            0.0,            0.0,            0.0)),
          ((         -598.0,         -591.0,         -321.0),
           (            2.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        11066.0,       -93908.0,       -41395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          205.0,        -1889.0,         -885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          822.0,         3281.0,         1517.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3916.0,         1067.0,          873.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,         -724.0,         -341.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1746.0,         2216.0,         1229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          925.0,        -3066.0,        -1578.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3436.0,        18591.0,         8834.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9091.0,       -40533.0,       -18798.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10400.0,       -30867.0,       -13542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8500.0,       -13871.0,        -5962.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9933.0,        23080.0,        16682.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23401.0,        -7794.0,        -1788.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20165.0,       -11530.0,        -4938.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -82868164.0,   6594674504.0,   2970172035.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13916.0,         9471.0,         5990.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21219.0,         4939.0,         2269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11489.0,       -16841.0,        -7358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          780.0,        -4855.0,        -2216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7095.0,        19181.0,         7697.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7863.0,        29610.0,        13260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,          522.0,          304.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -751.0,         -325.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          122.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          623.0,        -5408.0,        -1170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          336.0,          -37.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5454.0,        61326.0,        26812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,         -256.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           14.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,          301.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          257.0,         -955.0,         -387.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2179.0,        -8569.0,        -3750.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,         -361.0,         -176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -532.0,          600.0,          276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,         -169.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -730.0,           72.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -438.0,         2250.0,          618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -201.0,          280.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21182.0,      -103603.0,       -45239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1932.0,        -2510.0,        -1153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -899.0,         1037.0,          489.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1226.0,         -614.0,         -292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -838.0,         1429.0,          694.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1406.0,        -2571.0,        -1058.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15913.0,        23231.0,        11283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        41166.0,       -30740.0,       -15986.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -30650.0,        43689.0,        21258.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8687.0,          594.0,         -170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     15888629.0,     17536271.0,      6877587.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12628.0,         4887.0,         2845.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4259.0,          -53.0,         -236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2116.0,         -160.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2058.0,         -830.0,         -268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31370.0,       -54804.0,       -22693.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18709.0,         5663.0,         1427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1085.0,         -499.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,          239.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -284.0,           54.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           87.0,           38.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          642.0,         1248.0,          511.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          170.0,         -309.0,         -149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         -186.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          413.0,          409.0,          164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           13.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -546.0,          369.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          -17.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2097.0,         5279.0,         2206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -247.0,          507.0,          284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -13.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          111.0,          -27.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -24.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,         -237.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,           -5.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -872.0,         -706.0,         -270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          144.0,           -1.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -33.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -292.0,         -785.0,         -282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,           -5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1818.0,         3472.0,         1677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -678.0,        -1008.0,         -379.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,          176.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,          207.0,          109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       124272.0,        20608.0,         1382.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,           53.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           59.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -25.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -487.0,         -297.0,         -101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -187.0,          104.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          286.0,          428.0,          150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -12.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -32.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,           36.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -14.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,          -51.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -134.0,         -155.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           25.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          -72.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           58.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -14.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          577.0,         -328.0,         -194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,           58.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,          -14.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          -25.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,           10.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        45603.0,       -70052.0,       -34161.0),
           (        29730.0,      -228929.0,      -105041.0),
           (        -3103.0,         1729.0,          257.0)),
          ((        23683.0,       -11233.0,        -6369.0),
           (       -41370.0,       177042.0,        82136.0),
           (            0.0,            0.0,            0.0)),
          ((        -6410.0,       -15646.0,        -6721.0),
           (         2975.0,        21552.0,         9452.0),
           (            0.0,            0.0,            0.0)),
          ((         -500.0,         3310.0,         1650.0),
           (        18500.0,       100694.0,        41881.0),
           (            0.0,            0.0,            0.0)),
          ((       189143.0,        32098.0,         -192.0),
           (       -40277.0,      -131520.0,      -249261.0),
           (            0.0,            0.0,            0.0)),
          ((         3289.0,         2753.0,         1110.0),
           (        11769.0,        52525.0,        23658.0),
           (            0.0,            0.0,            0.0)),
          ((       -10311.0,       -15022.0,        -7162.0),
           (         2844.0,       -60036.0,       -27753.0),
           (            0.0,            0.0,            0.0)),
          ((      -180575.0,      -177881.0,       -69395.0),
           (          579.0,         3504.0,         1647.0),
           (            0.0,            0.0,            0.0)),
          ((          388.0,         1938.0,         1110.0),
           (           -5.0,        -1114.0,         -497.0),
           (            0.0,            0.0,            0.0)),
          ((       -13905.0,         5390.0,         5289.0),
           (         5525.0,         3124.0,         1024.0),
           (            0.0,            0.0,            0.0)),
          ((        11527.0,          533.0,        -2507.0),
           (         1510.0,         3229.0,         1370.0),
           (            0.0,            0.0,            0.0)),
          ((          446.0,         1221.0,          507.0),
           (        -3178.0,         9065.0,         4117.0),
           (            0.0,            0.0,            0.0)),
          ((        -1918.0,         1319.0,          612.0),
           (         3339.0,       -29293.0,       -12877.0),
           (            0.0,            0.0,            0.0)),
          ((         9761.0,        -4537.0,        -2195.0),
           (           34.0,           21.0,            7.0),
           (            0.0,            0.0,            0.0)),
          ((       -14066.0,         8660.0,         4173.0),
           (          104.0,          -99.0,          -61.0),
           (            0.0,            0.0,            0.0)),
          ((        18453.0,         1837.0,           91.0),
           (        -1606.0,        -1932.0,         -751.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          145.0,           14.0),
           (          -73.0,          259.0,          112.0),
           (            0.0,            0.0,            0.0)),
          ((         7200.0,        -4278.0,        -2520.0),
           (          117.0,         -124.0,          -68.0),
           (            0.0,            0.0,            0.0)),
          ((        20188.0,        -6456.0,        -7609.0),
           (           -6.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((        -1037.0,        -2552.0,         -299.0),
           (           13.0,           19.0,            8.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,         1065.0,          251.0),
           (           -1.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((        -9204.0,       -94827.0,       -41340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -676.0,        -1013.0,         -569.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          653.0,          584.0,          599.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3164.0,         2304.0,          892.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -246.0,           36.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1449.0,        -2181.0,        -1095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          326.0,          890.0,         -156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5214.0,       -10236.0,        -2266.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22454.0,        26950.0,         7880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        40446.0,        27719.0,        11718.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13921.0,        22957.0,        15784.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23218.0,       -26807.0,       -19002.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35810.0,         5976.0,         2194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15034.0,        12776.0,         3823.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3835930112.0,  -5498287249.0,  -2715410782.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16612.0,       -15754.0,        -5054.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11157.0,        20870.0,         7616.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21219.0,        -5082.0,         1297.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2159.0,        -5885.0,        -2971.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17770.0,        12852.0,         2877.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32584.0,         9541.0,         3823.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          657.0,         -192.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1087.0,          270.0,          132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          279.0,          -89.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -762.0,        -6354.0,        -2751.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -788.0,           82.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -62202.0,        20714.0,        10465.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -501.0,          230.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -18.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,           73.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          489.0,         -266.0,         -147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5560.0,         5178.0,         2068.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          518.0,         -298.0,         -157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          682.0,         -306.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -156.0,           34.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -286.0,           43.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1007.0,        -1890.0,        -1138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,         -210.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -111126.0,       -34253.0,        -9154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,         1721.0,          650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          257.0,         -856.0,         -326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -889.0,         1396.0,          594.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,         -707.0,         -215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2603.0,         1504.0,          690.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15376.0,         5203.0,         3940.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29892.0,         6478.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34892.0,        15752.0,         9872.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2405.0,        10366.0,         4243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -9447982.0,     20483748.0,      9809536.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6114.0,        -2345.0,         -521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2554.0,         1820.0,          627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1463.0,          208.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3656.0,         -306.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8108.0,        42194.0,        17885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26786.0,          241.0,        -1157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1846.0,          175.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          547.0,         -161.0,         -101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,          -83.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          105.0,          -28.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          936.0,         -935.0,         -509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          276.0,          162.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          143.0,           33.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          435.0,         -272.0,         -124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,          -18.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -588.0,          318.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           19.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -417.0,         5703.0,         2660.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          734.0,           10.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           21.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -70.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           13.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          241.0,           13.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -15.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           82.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1221.0,           40.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -258.0,          255.0,          137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,          -19.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          584.0,          755.0,          283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,          -36.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2988.0,        -2753.0,        -1442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -890.0,          788.0,          546.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,         -128.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,          -19.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28686.0,      -110760.0,       -51653.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -17.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           93.0,           39.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,           18.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -17.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -427.0,         -382.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          182.0,           80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -530.0,           87.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           13.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           42.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -78.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           19.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          104.0,           53.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           20.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -187.0,          -94.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           19.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           19.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,          -53.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           54.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -22.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          677.0,          310.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,            9.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -68.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          -32.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -29.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -11.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -5.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -50.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -20.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        81776.0,       -31041.0,       -18797.0),
           (        -2175.0,       -60042.0,      -109573.0),
           (        -2844.0,         2017.0,         1748.0)),
          ((        22011.0,         2693.0,         -120.0),
           (         6292.0,        70499.0,        94087.0),
           (            0.0,            0.0,            0.0)),
          ((         8710.0,        -7349.0,        -3680.0),
           (         -638.0,         6854.0,        12323.0),
           (            0.0,            0.0,            0.0)),
          ((        -2829.0,         5315.0,         2473.0),
           (        -4655.0,        -1535.0,        33182.0),
           (            0.0,            0.0,            0.0)),
          ((        14547.0,      -178410.0,       -81809.0),
           (        -4042.0,       -68605.0,       -59813.0),
           (            0.0,            0.0,            0.0)),
          ((        -1590.0,        -1678.0,         -777.0),
           (        -3762.0,        23244.0,        36619.0),
           (            0.0,            0.0,            0.0)),
          ((        13232.0,         7652.0,         3709.0),
           (       -15551.0,       -37923.0,       -33087.0),
           (            0.0,            0.0,            0.0)),
          ((       267192.0,       -30246.0,       -28029.0),
           (          960.0,         1893.0,         2144.0),
           (            0.0,            0.0,            0.0)),
          ((        -2813.0,        -2707.0,        -1261.0),
           (         -567.0,         -297.0,         -658.0),
           (            0.0,            0.0,            0.0)),
          ((       -45907.0,       -34606.0,       -13658.0),
           (         -815.0,         7059.0,         3746.0),
           (            0.0,            0.0,            0.0)),
          ((        49365.0,        28950.0,        10931.0),
           (           30.0,        -1096.0,          338.0),
           (            0.0,            0.0,            0.0)),
          ((         2019.0,          222.0,          -23.0),
           (        -6028.0,        -4640.0,         -901.0),
           (            0.0,            0.0,            0.0)),
          ((         1431.0,         -407.0,          -92.0),
           (         7439.0,        14548.0,         3146.0),
           (            0.0,            0.0,            0.0)),
          ((          281.0,        -4203.0,        -2517.0),
           (           49.0,          -50.0,          -36.0),
           (            0.0,            0.0,            0.0)),
          ((        -3217.0,         9245.0,         5055.0),
           (            6.0,          247.0,          157.0),
           (            0.0,            0.0,            0.0)),
          ((        -5280.0,       -14821.0,        -6565.0),
           (         1177.0,         1930.0,          662.0),
           (            0.0,            0.0,            0.0)),
          ((         -128.0,         -965.0,         -309.0),
           (          112.0,          -31.0,           49.0),
           (            0.0,            0.0,            0.0)),
          ((         6725.0,         4675.0,          238.0),
           (           -7.0,         -116.0,          -85.0),
           (            0.0,            0.0,            0.0)),
          ((        24207.0,         3113.0,        -1072.0),
           (            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((        -2182.0,         1788.0,          421.0),
           (           -9.0,          -10.0,           -7.0),
           (            0.0,            0.0,            0.0)),
          ((          401.0,          172.0,          294.0),
           (           -3.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       -29500.0,       -91158.0,       -39525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -275.0,        -1395.0,         -420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1614.0,         2475.0,          522.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3301.0,         -691.0,         -187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -417.0,         -261.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2267.0,        -1273.0,        -1053.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2644.0,         -748.0,          342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16888.0,        -1764.0,        -3945.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -39754.0,         8253.0,         8753.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34625.0,        13921.0,         5481.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16946.0,       -11508.0,       -12554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16640.0,        29078.0,        20709.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27571.0,        -7057.0,        -4188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3184.0,         6660.0,         5254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -6463401150.0,   2788595301.0,   1663448657.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6706.0,        14825.0,         2922.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8714.0,       -21031.0,        -7851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12849.0,        25208.0,         8587.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1761.0,        -8127.0,        -3222.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14063.0,       -11238.0,        -3738.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32724.0,       -12673.0,        -8844.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          829.0,          -72.0,         -247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1000.0,          625.0,          337.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          310.0,         -115.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1718.0,        -3642.0,        -2827.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -571.0,          642.0,          445.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -39366.0,       -50052.0,       -21283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -182.0,          444.0,          306.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,         -297.0,         -299.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -365.0,          -54.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1053.0,         -202.0,         -149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3225.0,        -6031.0,        -2582.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          228.0,          -67.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -832.0,           95.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          215.0,            5.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -535.0,          676.0,          277.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          674.0,        -2289.0,         -588.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -69.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -80051.0,        74564.0,        37714.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -459.0,         1253.0,          640.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          327.0,         -522.0,         -289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1815.0,         1004.0,          544.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -146.0,         -576.0,         -280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2966.0,         2103.0,         1142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14676.0,         3351.0,         2375.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16445.0,         8165.0,         2590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -33441.0,         5607.0,         4669.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8550.0,         7144.0,         3549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -24438639.0,      1182957.0,      2079307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7829.0,        -2443.0,        -1183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2256.0,         2136.0,          929.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          913.0,          122.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2655.0,        -1461.0,         -675.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -32590.0,       -38405.0,       -14020.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15169.0,         9568.0,         4308.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1023.0,         -580.0,         -278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -332.0,          189.0,          105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,           64.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,          -13.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1189.0,         -823.0,         -271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,          332.0,          141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,          208.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,         -573.0,         -242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -14.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -657.0,          270.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           17.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2467.0,         5194.0,         2626.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -304.0,         -599.0,         -259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           13.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,           32.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           20.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -139.0,          241.0,          126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -9.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -33.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1119.0,          476.0,          292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          189.0,         -208.0,         -115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           25.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -896.0,         -461.0,         -167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,          -30.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3894.0,         1688.0,         1025.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1454.0,          622.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -386.0,         -174.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,           38.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -119030.0,       -37118.0,        -9133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            5.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,           22.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -22.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -2.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -330.0,         -454.0,         -186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          223.0,          -33.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,         -479.0,         -241.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -10.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -44.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,           55.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          117.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -17.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,            3.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           21.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          -56.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,            4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           11.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -637.0,         -454.0,         -171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -13.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           71.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -43.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           -8.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -14.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           48.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           19.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            5.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        84216.0,        20766.0,         4267.0),
           (       -65102.0,       -44909.0,       -17998.0),
           (        -1237.0,        -3406.0,        -1204.0)),
          ((        20871.0,        16670.0,         6175.0),
           (        38943.0,        35824.0,        15563.0),
           (            0.0,            0.0,            0.0)),
          ((         9463.0,         3871.0,         1233.0),
           (          870.0,         3692.0,         1764.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,          564.0,          124.0),
           (        16064.0,        30357.0,        16041.0),
           (            0.0,            0.0,            0.0)),
          ((      -190693.0,         5125.0,        17169.0),
           (       -14045.0,       -96363.0,        58743.0),
           (            0.0,            0.0,            0.0)),
          ((        -5184.0,          431.0,          466.0),
           (         6445.0,        12996.0,         6195.0),
           (            0.0,            0.0,            0.0)),
          ((        31580.0,          153.0,        -1083.0),
           (       -29526.0,       -11258.0,        -4182.0),
           (            0.0,            0.0,            0.0)),
          ((      -163386.0,       219449.0,       106062.0),
           (          486.0,          421.0,          211.0),
           (            0.0,            0.0,            0.0)),
          ((        -7037.0,         -684.0,          -95.0),
           (         -659.0,         -598.0,         -213.0),
           (            0.0,            0.0,            0.0)),
          ((       -67054.0,       -30720.0,       -11510.0),
           (          486.0,         1907.0,          695.0),
           (            0.0,            0.0,            0.0)),
          ((        72115.0,        39736.0,        15359.0),
           (          -29.0,          605.0,          347.0),
           (            0.0,            0.0,            0.0)),
          ((          420.0,          355.0,           52.0),
           (         2009.0,        -6979.0,        -2835.0),
           (            0.0,            0.0,            0.0)),
          ((        -1734.0,        -1244.0,         -551.0),
           (         3335.0,        14084.0,         5496.0),
           (            0.0,            0.0,            0.0)),
          ((         3566.0,         5078.0,         2066.0),
           (          -68.0,          -71.0,          -29.0),
           (            0.0,            0.0,            0.0)),
          ((        -6373.0,        -6105.0,        -2296.0),
           (          433.0,          408.0,          154.0),
           (            0.0,            0.0,            0.0)),
          ((       -14989.0,         9723.0,         4979.0),
           (         -372.0,        -2938.0,        -1295.0),
           (            0.0,            0.0,            0.0)),
          ((         -764.0,         -211.0,          -49.0),
           (          335.0,           16.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((         6599.0,         7061.0,         2418.0),
           (          -68.0,         -170.0,          -77.0),
           (            0.0,            0.0,            0.0)),
          ((        11121.0,        17195.0,         6175.0),
           (           -3.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,        -1925.0,         -948.0),
           (            3.0,           12.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          170.0,          115.0),
           (            2.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((       -48325.0,       -84038.0,       -35935.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1181.0,        -2073.0,         -839.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1049.0,         2637.0,         1004.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3392.0,        -3652.0,        -1727.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -258.0,         -707.0,         -271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2150.0,         2860.0,         1445.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1415.0,        -4120.0,        -1559.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3455.0,        23023.0,         9402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5692.0,       -49372.0,       -20598.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -488.0,       -32850.0,       -14659.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22148.0,       -12954.0,         -276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20607.0,       -35299.0,       -20205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        30969.0,        10771.0,         2398.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14235.0,       -11334.0,        -5293.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   7194214386.0,    739232174.0,   -122841955.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10531.0,        15668.0,         6590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25126.0,       -11264.0,        -3744.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18622.0,       -10298.0,        -7539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1569.0,        -4096.0,        -1542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2287.0,       -29327.0,       -12323.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6068.0,       -23238.0,       -11442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,          446.0,          100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -493.0,          487.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,          158.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1717.0,         1409.0,        -1108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          232.0,         -847.0,         -342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        41231.0,       -47739.0,       -21843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,         -647.0,         -261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          428.0,          393.0,          133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           62.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          521.0,          292.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7991.0,         5734.0,         2178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          672.0,          227.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          758.0,          104.0,           82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,          -20.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          330.0,          142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          925.0,         2307.0,         1250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,          125.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        39772.0,       101626.0,        43351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1603.0,        -1085.0,         -466.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          668.0,          453.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1796.0,         -397.0,          -96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          341.0,          365.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3205.0,          996.0,          604.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13643.0,         9351.0,         5077.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8085.0,       -10538.0,        -5376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31605.0,         6574.0,         4966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9966.0,        -3415.0,         -977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -12840886.0,    -19333403.0,     -7884250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13597.0,         4578.0,         1299.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3506.0,          -72.0,          148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,          136.0,          144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1884.0,         -627.0,         -361.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        73485.0,         7374.0,        -1260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21056.0,         6511.0,         3717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1343.0,         -503.0,         -267.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -324.0,          136.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          134.0,         -119.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,           74.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -361.0,         1262.0,          608.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -331.0,          -46.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -174.0,           72.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -532.0,          -34.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,           14.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -701.0,          213.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          -22.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4360.0,         3610.0,         2102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -532.0,          493.0,          299.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -19.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          101.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,          -23.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           44.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -139.0,         -269.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           -7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,          -86.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          554.0,         -814.0,         -389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           34.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -39.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1171.0,          124.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,            3.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4167.0,         -511.0,         -495.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,        -1266.0,         -594.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -148.0,          113.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,         -126.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -46138.0,       104454.0,        49923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           64.0,          -19.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,            9.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -13.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -236.0,         -492.0,         -209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,         -191.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          511.0,          202.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,            3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           38.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,            9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,           55.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -175.0,           85.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          112.0,          -52.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          -41.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          776.0,         -131.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -72.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -46.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           20.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,            2.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,           -3.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        57130.0,        63170.0,        24828.0),
           (       -99776.0,       -56058.0,        10210.0),
           (        -3358.0,         -267.0,         -684.0)),
          ((        15584.0,        23698.0,         9680.0),
           (        74840.0,        72742.0,         5322.0),
           (            0.0,            0.0,            0.0)),
          ((       -10086.0,         6601.0,         3440.0),
           (        13500.0,          157.0,        -3840.0),
           (            0.0,            0.0,            0.0)),
          ((           74.0,        -2277.0,         -913.0),
           (        57735.0,         9381.0,       -15447.0),
           (            0.0,            0.0,            0.0)),
          ((        32084.0,       174609.0,        76549.0),
           (      -139195.0,        59339.0,      -163753.0),
           (            0.0,            0.0,            0.0)),
          ((          297.0,         2709.0,         1346.0),
           (        29038.0,         8402.0,        -7776.0),
           (            0.0,            0.0,            0.0)),
          ((        -1430.0,       -12152.0,        -6939.0),
           (       -38116.0,         8978.0,        12536.0),
           (            0.0,            0.0,            0.0)),
          ((      -114547.0,      -224034.0,       -92953.0),
           (         1506.0,          346.0,         -590.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         2353.0,         1462.0),
           (         -366.0,         -441.0,           38.0),
           (            0.0,            0.0,            0.0)),
          ((         1588.0,        27107.0,        15775.0),
           (        -4765.0,         5368.0,         2427.0),
           (            0.0,            0.0,            0.0)),
          ((        -5518.0,       -35221.0,       -19354.0),
           (         2031.0,          358.0,         -197.0),
           (            0.0,            0.0,            0.0)),
          ((         1259.0,         1025.0,          405.0),
           (         6783.0,         -676.0,         -944.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,         1469.0,          695.0),
           (       -15754.0,         1313.0,         2385.0),
           (            0.0,            0.0,            0.0)),
          ((         7081.0,          616.0,            1.0),
           (          -63.0,          -15.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       -13538.0,        -3564.0,         -988.0),
           (          264.0,          128.0,           16.0),
           (            0.0,            0.0,            0.0)),
          ((        12579.0,         9619.0,         3767.0),
           (         -802.0,         2772.0,         1339.0),
           (            0.0,            0.0,            0.0)),
          ((          690.0,          897.0,          315.0),
           (           48.0,         -421.0,         -220.0),
           (            0.0,            0.0,            0.0)),
          ((        -3039.0,          857.0,         1181.0),
           (         -154.0,            8.0,           26.0),
           (            0.0,            0.0,            0.0)),
          ((         2195.0,        32106.0,        14453.0),
           (            7.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((        -2004.0,        -6813.0,        -2524.0),
           (            2.0,          -17.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((          894.0,         2277.0,          840.0),
           (            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       -65670.0,       -74811.0,       -31449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -857.0,          154.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1717.0,           85.0,          295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4317.0,        -3472.0,        -1570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -254.0,          146.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1259.0,         1491.0,         1095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -486.0,         1381.0,          340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6195.0,        -7742.0,        -1645.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25147.0,        19763.0,         5578.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        43955.0,        20625.0,         8518.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16352.0,        17862.0,         7099.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9690.0,        27384.0,        16502.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19685.0,         1094.0,         2637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8412.0,       -10105.0,        -7152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -5813873472.0,  -4049919637.0,  -1454273858.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27577.0,       -12872.0,        -4313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4399.0,        23098.0,         9134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17407.0,        -8532.0,         -914.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2178.0,        -3677.0,        -2000.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24069.0,       -17032.0,        -8118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16589.0,       -21393.0,        -7571.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          475.0,          -53.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -357.0,          908.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          209.0,          -59.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -374.0,         5296.0,         1089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1006.0,          -53.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        62649.0,        23395.0,         9075.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -547.0,           58.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,         -199.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -157.0,         -244.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          582.0,          618.0,          274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5592.0,        -2596.0,         -964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          277.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -697.0,         -361.0,         -187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          143.0,          191.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,          312.0,          153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -500.0,         1815.0,          365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,           66.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       114953.0,        16259.0,          950.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          679.0,        -1006.0,         -421.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -153.0,          380.0,          160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,         -816.0,         -312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           85.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1814.0,         1750.0,          873.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18725.0,         4519.0,         3015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18479.0,       -11295.0,        -5953.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -36884.0,        -3669.0,          395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4630.0,        -7253.0,        -2815.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     12660027.0,    -18785873.0,     -9253271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7616.0,        10045.0,         3731.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1029.0,        -3075.0,        -1193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1877.0,         -104.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,         2396.0,          939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -56962.0,        16906.0,        10732.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1617.0,       -13333.0,        -5061.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,          780.0,          289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,         -169.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,           40.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,          -23.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1453.0,          214.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -242.0,         -270.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,         -231.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -173.0,          592.0,          259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -706.0,          142.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5882.0,         1791.0,         1431.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          592.0,          221.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -12.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           -4.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -20.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           34.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          295.0,          -15.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          132.0,           69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          969.0,          416.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,           87.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           64.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1143.0,          301.0,          167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,           27.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4209.0,         -797.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1427.0,          596.0,          525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,         -243.0,         -138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,         -181.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       111486.0,        52497.0,        16544.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           25.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           77.0,          -29.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -26.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           15.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -142.0,         -530.0,         -229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -10.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -217.0,          -52.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -337.0,          356.0,          244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,            4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -25.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -68.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           13.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,          -61.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,          149.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -15.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,          -24.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -55.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -306.0,          582.0,          275.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           71.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -12.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -39.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           31.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -15.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           16.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -46.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -28.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((         7346.0,        81637.0,        36131.0),
           (      -208307.0,       -17537.0,       -37802.0),
           (         -794.0,         -471.0,          257.0)),
          ((         5380.0,        21103.0,         9054.0),
           (       157504.0,        48632.0,        44942.0),
           (            0.0,            0.0,            0.0)),
          ((       -10076.0,        -2680.0,         -624.0),
           (        20426.0,         2932.0,         4955.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,           14.0,           49.0),
           (        53205.0,        -8026.0,        13220.0),
           (            0.0,            0.0,            0.0)),
          ((       185409.0,       -44335.0,       -34415.0),
           (      -166956.0,      -103822.0,       203991.0),
           (            0.0,            0.0,            0.0)),
          ((         3736.0,        -2178.0,        -1243.0),
           (        53529.0,         8255.0,        14549.0),
           (            0.0,            0.0,            0.0)),
          ((       -12467.0,        15928.0,         7927.0),
           (       -35317.0,        21189.0,         2490.0),
           (            0.0,            0.0,            0.0)),
          ((       284833.0,        43295.0,         4573.0),
           (         3342.0,         -300.0,          296.0),
           (            0.0,            0.0,            0.0)),
          ((         1041.0,        -3263.0,        -1486.0),
           (         -932.0,          116.0,         -175.0),
           (            0.0,            0.0,            0.0)),
          ((       -27064.0,       -19481.0,        -6117.0),
           (        -1497.0,        -2638.0,         -704.0),
           (            0.0,            0.0,            0.0)),
          ((        33280.0,        25707.0,         8411.0),
           (         2645.0,          527.0,          417.0),
           (            0.0,            0.0,            0.0)),
          ((         1819.0,         -883.0,         -508.0),
           (         7353.0,         6031.0,         2489.0),
           (            0.0,            0.0,            0.0)),
          ((          174.0,        -1280.0,         -524.0),
           (       -23428.0,       -17009.0,        -7353.0),
           (            0.0,            0.0,            0.0)),
          ((         5290.0,          689.0,         -146.0),
           (           24.0,           22.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((        -9440.0,        -2056.0,         -203.0),
           (          -92.0,           97.0,           91.0),
           (            0.0,            0.0,            0.0)),
          ((         6175.0,       -15380.0,        -7120.0),
           (         1023.0,        -2217.0,        -1156.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,         -515.0,         -203.0),
           (          345.0,          357.0,          198.0),
           (            0.0,            0.0,            0.0)),
          ((        -3186.0,         4726.0,         2684.0),
           (         -155.0,          -60.0,          -36.0),
           (            0.0,            0.0,            0.0)),
          ((       -20725.0,        14504.0,        10460.0),
           (            6.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((         3204.0,        -2995.0,        -2132.0),
           (           11.0,           17.0,            6.0),
           (            0.0,            0.0,            0.0)),
          ((         -585.0,          677.0,          473.0),
           (           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -79592.0,       -61284.0,       -25366.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1861.0,         -731.0,         -195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3700.0,          148.0,         -237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6120.0,        -3546.0,         -897.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -883.0,           99.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -984.0,        -1611.0,        -1000.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3795.0,         1832.0,         1176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21457.0,        -9735.0,        -6015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -42525.0,        24225.0,        13580.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20536.0,        22974.0,        10220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15542.0,       -17863.0,       -13171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24706.0,       -28355.0,       -16938.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        37256.0,          -14.0,        -1858.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9390.0,        -6244.0,        -1138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2727479141.0,   6171984106.0,   2604913795.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11610.0,         5156.0,          326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16573.0,       -12642.0,        -5281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10220.0,        22358.0,        10018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          644.0,        -7314.0,        -3221.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -32148.0,         -704.0,         2292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26519.0,         2689.0,         2058.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          251.0,          246.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          865.0,          154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,          -31.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1054.0,         6294.0,         2648.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,          146.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6790.0,        61080.0,        26874.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,          -12.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          493.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,          -59.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          345.0,          528.0,          223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7573.0,         5039.0,         1856.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          144.0,          597.0,          258.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          496.0,          482.0,          238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,         -137.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          703.0,          197.0,           82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1316.0,        -2368.0,        -1227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -90.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        64861.0,       -87212.0,       -42626.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          505.0,          738.0,          353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -212.0,         -211.0,         -111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1533.0,         -396.0,         -151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          254.0,         -839.0,         -371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1007.0,         3584.0,         1667.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17773.0,        -7761.0,        -2711.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        24269.0,         4206.0,          934.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34129.0,       -22486.0,        -8394.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          836.0,        -6779.0,        -2861.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     24327634.0,      2155903.0,      -568623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2543.0,         6225.0,         2618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2906.0,        -2689.0,        -1258.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2670.0,         -602.0,         -260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2430.0,         1255.0,          641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29653.0,        -7509.0,        -6048.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21890.0,       -16326.0,        -8356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1436.0,         1077.0,          562.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          414.0,         -260.0,         -149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          184.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          -96.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -308.0,        -1308.0,         -573.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,          -75.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,         -190.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          457.0,          353.0,          136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           -4.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -716.0,           78.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           16.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6226.0,         -173.0,          561.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -145.0,         -599.0,         -298.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,          -75.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,            8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,          296.0,          150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           26.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -80.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -608.0,         -867.0,         -357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          134.0,          -70.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          -37.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          960.0,         -696.0,         -320.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,          -31.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3686.0,         1953.0,          641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1530.0,          976.0,          243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -372.0,         -244.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          265.0,          -33.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        62258.0,       -95858.0,       -47077.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,          -80.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,           34.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -10.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,         -526.0,         -239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,          175.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -307.0,         -416.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           -9.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,            9.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           63.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -10.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -16.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,         -106.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -17.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          191.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -11.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,           19.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          -35.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -370.0,         -523.0,         -211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          -61.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          -25.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           15.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           -2.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           55.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           15.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -10.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -48045.0,        67824.0,        33293.0),
           (       131329.0,      -127310.0,       -68645.0),
           (         1252.0,        -2205.0,        -1254.0)),
          ((        -6509.0,        16713.0,         7824.0),
           (       -96630.0,        81342.0,        44545.0),
           (            0.0,            0.0,            0.0)),
          ((        -1039.0,       -15197.0,        -6686.0),
           (       -17251.0,        16019.0,         8592.0),
           (            0.0,            0.0,            0.0)),
          ((         1122.0,         2379.0,          949.0),
           (       -41254.0,        60468.0,        31612.0),
           (            0.0,            0.0,            0.0)),
          ((       -74919.0,      -162807.0,       -68042.0),
           (        72257.0,       -30469.0,      -279367.0),
           (            0.0,            0.0,            0.0)),
          ((        -1682.0,        -2720.0,        -1158.0),
           (       -34825.0,        40418.0,        22137.0),
           (            0.0,            0.0,            0.0)),
          ((        16490.0,         9306.0,         3638.0),
           (        49362.0,        -9791.0,        -8379.0),
           (            0.0,            0.0,            0.0)),
          ((      -219939.0,       159485.0,        82300.0),
           (        -2839.0,         1479.0,          863.0),
           (            0.0,            0.0,            0.0)),
          ((        -6005.0,         -513.0,            2.0),
           (         1053.0,         -523.0,         -322.0),
           (            0.0,            0.0,            0.0)),
          ((       -82927.0,         6333.0,         6456.0),
           (        -6366.0,         1782.0,         1176.0),
           (            0.0,            0.0,            0.0)),
          ((        81239.0,       -10009.0,        -7800.0),
           (         1166.0,         -787.0,         -270.0),
           (            0.0,            0.0,            0.0)),
          ((          223.0,         -172.0,         -155.0),
           (        -3913.0,         7253.0,         3502.0),
           (            0.0,            0.0,            0.0)),
          ((        -2130.0,          743.0,          351.0),
           (        15184.0,       -14091.0,        -7033.0),
           (            0.0,            0.0,            0.0)),
          ((         1227.0,          512.0,          283.0),
           (           29.0,          -14.0,           -9.0),
           (            0.0,            0.0,            0.0)),
          ((          559.0,        -4251.0,        -2029.0),
           (         -103.0,          387.0,          193.0),
           (            0.0,            0.0,            0.0)),
          ((       -16762.0,        -3637.0,         -766.0),
           (        -1551.0,         1393.0,          721.0),
           (            0.0,            0.0,            0.0)),
          ((        -1187.0,          333.0,          192.0),
           (          141.0,          258.0,          119.0),
           (            0.0,            0.0,            0.0)),
          ((        -1707.0,        -2227.0,         -140.0),
           (          -26.0,         -175.0,          -75.0),
           (            0.0,            0.0,            0.0)),
          ((       -33083.0,         3565.0,         6161.0),
           (           -3.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0)),
          ((         6750.0,        -1070.0,        -1079.0),
           (          -12.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((        -1094.0,          803.0,          481.0),
           (           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -91018.0,       -46033.0,       -18420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1847.0,         -226.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2814.0,         1440.0,          533.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4645.0,        -3335.0,         -897.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -534.0,         -212.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1610.0,        -1486.0,        -1008.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -871.0,        -1683.0,         -796.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7317.0,        15104.0,         6560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14843.0,       -39349.0,       -17159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9223.0,       -39354.0,       -16978.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22420.0,        17111.0,        15097.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11718.0,        26179.0,        17092.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23910.0,         5601.0,         3357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19874.0,         -957.0,         1424.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   1159242685.0,  -6482875795.0,  -2991195523.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11982.0,          534.0,          650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19650.0,       -15369.0,        -6018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17622.0,       -11647.0,        -6802.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          292.0,        -5465.0,        -2311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12837.0,        17262.0,         8765.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18005.0,        24093.0,        11619.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -228.0,          460.0,          164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          599.0,          544.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,          101.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2000.0,         3788.0,         2794.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          239.0,         -708.0,         -360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -66078.0,        10437.0,         5985.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,         -427.0,         -251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          319.0,          -43.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          206.0,         -127.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -618.0,          670.0,          296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8491.0,         -256.0,          283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -204.0,          295.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -257.0,         -733.0,         -340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          104.0,          235.0,          115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          336.0,         -223.0,         -160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1029.0,        -1336.0,         -149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,         -240.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -57823.0,       -95198.0,       -39454.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1580.0,         2095.0,          821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1182.0,         -856.0,         -328.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2850.0,          490.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -581.0,        -1787.0,         -723.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          796.0,         5624.0,         2427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16607.0,       -20759.0,        -8288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32025.0,        26691.0,         9937.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28113.0,       -42463.0,       -17306.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7272.0,        -1760.0,        -1231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      9541804.0,     20683703.0,      8706228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4901.0,        -5475.0,        -2203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3515.0,         1656.0,          549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -358.0,          461.0,          228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2429.0,         -566.0,         -118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -46699.0,        29060.0,        16201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10388.0,         8762.0,         3522.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -416.0,         -545.0,         -236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -88.0,          120.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,         -108.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           45.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1403.0,          451.0,          323.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          402.0,          134.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          204.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          403.0,         -434.0,         -201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -731.0,           22.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5670.0,        -2445.0,         -556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -478.0,          353.0,          237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -77.0,            1.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -202.0,         -231.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -34.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          155.0,           39.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          864.0,          501.0,          178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -44.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -16.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,         1016.0,          455.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,          -39.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2880.0,        -3003.0,        -1165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,        -1414.0,         -685.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          201.0,           67.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,         -115.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -101305.0,       -66156.0,       -23369.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           17.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,          -87.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -12.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,         -522.0,         -245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,          121.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          524.0,         -106.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,            9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          -14.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,          -58.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           -3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,          164.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -19.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -30.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          110.0,           50.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,            6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           13.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          724.0,          -13.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -16.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,           53.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            5.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          -14.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -13.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,          -15.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           16.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -4.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -85254.0,        28020.0,        17702.0),
           (       213644.0,       -54348.0,       -30983.0),
           (         1150.0,         3031.0,         1683.0)),
          ((       -17435.0,        13483.0,         7088.0),
           (      -131435.0,        75200.0,        36843.0),
           (            0.0,            0.0,            0.0)),
          ((         9268.0,        -1666.0,        -1189.0),
           (       -18807.0,         1293.0,          827.0),
           (            0.0,            0.0,            0.0)),
          ((         3051.0,          289.0,         -133.0),
           (       -68161.0,         5473.0,         1356.0),
           (            0.0,            0.0,            0.0)),
          ((      -172104.0,        82193.0,        50538.0),
           (       253460.0,       -18396.0,       135112.0),
           (            0.0,            0.0,            0.0)),
          ((        -1169.0,         1823.0,          896.0),
           (       -48748.0,        -3273.0,         -568.0),
           (            0.0,            0.0,            0.0)),
          ((        -2453.0,        -9587.0,        -4007.0),
           (        58753.0,        -8165.0,        -5637.0),
           (            0.0,            0.0,            0.0)),
          ((       -28036.0,      -245752.0,      -108543.0),
           (        -2876.0,          482.0,          280.0),
           (            0.0,            0.0,            0.0)),
          ((          938.0,         1730.0,          710.0),
           (          706.0,          364.0,          117.0),
           (            0.0,            0.0,            0.0)),
          ((         7097.0,        10975.0,         5400.0),
           (        -2612.0,        -6231.0,        -2511.0),
           (            0.0,            0.0,            0.0)),
          ((        -1328.0,        -9229.0,        -5167.0),
           (         -153.0,         -113.0,          -33.0),
           (            0.0,            0.0,            0.0)),
          ((         1552.0,         -210.0,         -175.0),
           (       -14174.0,         1563.0,         1327.0),
           (            0.0,            0.0,            0.0)),
          ((         1605.0,          503.0,          285.0),
           (        31597.0,        -3778.0,        -3425.0),
           (            0.0,            0.0,            0.0)),
          ((        -3282.0,         3768.0,         1418.0),
           (            8.0,          -10.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((         5063.0,        -9258.0,        -3895.0),
           (         -259.0,          121.0,           56.0),
           (            0.0,            0.0,            0.0)),
          ((         4190.0,        13254.0,         5526.0),
           (         2888.0,         -977.0,         -619.0),
           (            0.0,            0.0,            0.0)),
          ((          511.0,         1240.0,          589.0),
           (         -211.0,           62.0,           36.0),
           (            0.0,            0.0,            0.0)),
          ((        -8515.0,        -9908.0,        -4034.0),
           (          128.0,          -27.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((       -15335.0,        -5727.0,          963.0),
           (           -1.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((        -1571.0,        -5105.0,        -3060.0),
           (            5.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         1149.0,         1591.0,          885.0),
           (           -1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       -99751.0,       -29316.0,       -10966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -804.0,          126.0,          191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1672.0,        -1823.0,        -1164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1223.0,         -694.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,          507.0,          311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2388.0,          281.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1405.0,         1662.0,          923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10497.0,        -9753.0,        -5282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        30926.0,        17173.0,         8407.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        42287.0,         2225.0,        -2018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15881.0,         7940.0,        -2076.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15314.0,       -17517.0,       -11668.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29784.0,       -15065.0,        -8265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7161.0,        24003.0,        13567.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -4705802776.0,   4891519324.0,   2499519509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16754.0,       -23007.0,       -13137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8369.0,        30152.0,        14463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21348.0,       -12519.0,        -6072.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1868.0,        -5904.0,        -2510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9846.0,        28605.0,        13757.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8540.0,        22522.0,         8748.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,         -232.0,         -254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          700.0,          787.0,          152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,         -211.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1987.0,        -1201.0,         1140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,          891.0,          473.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -29469.0,       -55948.0,       -24000.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          386.0,          224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -578.0,         -461.0,         -220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           95.0,            3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -407.0,          -17.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6365.0,         2093.0,          650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -360.0,          509.0,          255.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          700.0,          312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,         -103.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          282.0,         -728.0,         -357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1073.0,         2498.0,         1247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,          -31.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -116027.0,         1170.0,         6854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -363.0,         1422.0,          676.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,         -973.0,         -448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1240.0,         1052.0,          453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          465.0,        -2142.0,         -971.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -632.0,         6736.0,         3025.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5407.0,       -26769.0,       -12216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          551.0,        42418.0,        18672.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4840.0,       -53393.0,       -24001.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2787.0,         5594.0,         2160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -15550424.0,     16690310.0,      8496490.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1611.0,       -14535.0,        -6305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          752.0,         4691.0,         1914.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1308.0,          550.0,          148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,        -3532.0,        -1494.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13868.0,       -69783.0,       -31854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7081.0,        25136.0,        10095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -585.0,        -1589.0,         -636.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          287.0,          413.0,          163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          161.0,         -144.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           93.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          889.0,         1057.0,          410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -210.0,          179.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          287.0,          133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -281.0,         -566.0,         -234.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           -7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -719.0,          -50.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4493.0,        -4198.0,        -1487.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          572.0,          241.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -5.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,           39.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,          -15.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          293.0,          -73.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -126.0,          -24.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1056.0,           19.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          150.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           46.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,        -1119.0,         -513.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           53.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1690.0,         3623.0,         1531.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1291.0,          557.0,          446.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          206.0,         -330.0,         -150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          204.0,           16.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -77202.0,        85343.0,        43311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -8.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,         -107.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -3.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          204.0,         -491.0,         -238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          187.0,         -126.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          478.0,          236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -26.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,          -51.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           12.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          112.0,           44.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           18.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          188.0,           88.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           16.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -20.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          -29.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,           56.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           48.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -456.0,          574.0,          285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           26.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,          -43.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,           15.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -30.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           -4.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          -36.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // emb
    embct   : array [1..5,1..208,0..2,1..3] of double = (
         (((      -104802.0,       131217.0,        46890.0),
           (        59426.0,      -984380.0,      -395439.0),
           (        16476.0,       -20580.0,       -10555.0)),
          ((        81136.0,      -216638.0,       -81218.0),
           (        -8909.0,        56673.0,        22964.0),
           (           -2.0,          -22.0,          -23.0)),
          ((      -184244.0,       223249.0,        93831.0),
           (        -6968.0,      -103804.0,       -42779.0),
           (            0.0,            0.0,            0.0)),
          ((       -22442.0,        79034.0,        33807.0),
           (         1480.0,         2585.0,         1250.0),
           (            0.0,            0.0,            0.0)),
          ((        38030.0,       -36804.0,       -16192.0),
           (        -4500.0,         6056.0,         3469.0),
           (            0.0,            0.0,            0.0)),
          ((         3313.0,         2096.0,         1170.0),
           (        22057.0,        14318.0,        -6354.0),
           (            0.0,            0.0,            0.0)),
          ((       186036.0,      -105705.0,       -47976.0),
           (       -12300.0,       160227.0,      -377661.0),
           (            0.0,            0.0,            0.0)),
          ((       -10928.0,        -3159.0,        -2682.0),
           (        25941.0,       -49780.0,       -19696.0),
           (            0.0,            0.0,            0.0)),
          ((          278.0,         -726.0,         -185.0),
           (        -8721.0,        -6621.0,        -2232.0),
           (            0.0,            0.0,            0.0)),
          ((         5880.0,          459.0,          357.0),
           (       -28309.0,        -9113.0,        -2866.0),
           (            0.0,            0.0,            0.0)),
          ((      -117184.0,       -79793.0,       -37703.0),
           (        10734.0,         5559.0,         2199.0),
           (            0.0,            0.0,            0.0)),
          ((        -1005.0,         1079.0,          318.0),
           (        14425.0,       -26782.0,       -16191.0),
           (            0.0,            0.0,            0.0)),
          ((         -217.0,         2655.0,         1169.0),
           (         3940.0,         1067.0,          -76.0),
           (            0.0,            0.0,            0.0)),
          ((          146.0,        -1643.0,         -794.0),
           (         -282.0,          476.0,          182.0),
           (            0.0,            0.0,            0.0)),
          ((         -267.0,         2142.0,         1157.0),
           (         -169.0,        10506.0,         4432.0),
           (            0.0,            0.0,            0.0)),
          ((         1791.0,         8138.0,         4403.0),
           (        -9100.0,        -4654.0,        -2347.0),
           (            0.0,            0.0,            0.0)),
          ((       -14213.0,        -8615.0,        -4446.0),
           (         -800.0,           46.0,          -21.0),
           (            0.0,            0.0,            0.0)),
          ((         6488.0,         4888.0,         2811.0),
           (        -1332.0,        -2560.0,        -1255.0),
           (            0.0,            0.0,            0.0)),
          ((         3162.0,         8470.0,         3493.0),
           (           -4.0,           46.0,           23.0),
           (            0.0,            0.0,            0.0)),
          ((         3554.0,        -3719.0,        -1539.0),
           (          -48.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((        -4494.0,         -332.0,         -567.0),
           (          -48.0,          -13.0,            5.0),
           (            0.0,            0.0,            0.0)),
          ((         9038.0,         -459.0,          820.0),
           (          106.0,            3.0,          -72.0),
           (            0.0,            0.0,            0.0)),
          ((        -9250.0,         -304.0,         -993.0),
           (            6.0,           -1.0,          -13.0),
           (            0.0,            0.0,            0.0)),
          ((       -59590.0,        35292.0,        15448.0),
           (            0.0,           -3.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((        -3056.0,         1276.0,          369.0),
           (            6.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((         1614.0,        -1681.0,         -506.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        86858.0,       -63919.0,       -21940.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -965.0,        -4855.0,        -1853.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7327.0,       -28895.0,       -12072.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        64290.0,       -14583.0,        -6673.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        46559.0,        22198.0,        11290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4021.0,         1625.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3634598463.0,   8545654587.0,   3706776192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25712.0,        -2690.0,         -221.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        43010.0,        43691.0,        18634.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19212.0,        -8712.0,        -2301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19744.0,        33503.0,        11941.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2940.0,        -3751.0,        -1376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1718.0,         1291.0,          481.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1423.0,         -594.0,         -260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28817.0,       -38529.0,       -16968.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          441.0,         -825.0,         -429.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1507.0,          303.0,          222.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3815.0,         1286.0,          298.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3504.0,         2174.0,         1254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1484.0,        -1615.0,         -729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6271.0,        -2124.0,         -877.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1676.0,         -661.0,         -427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3140.0,         -298.0,          144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -592.0,        -3804.0,        -2012.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8037.0,         2214.0,         -472.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1662.0,        -1356.0,         -522.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1704.0,         1671.0,          797.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1325.0,        -1736.0,         -843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -94204.0,         -793.0,        -1311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3908.0,        -3519.0,        -2037.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -933.0,         1327.0,          638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2536.0,        -4358.0,        -2086.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6229.0,          674.0,          237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1585.0,         2952.0,         1386.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5339.0,        -2137.0,        -1084.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5452.0,        25212.0,        11115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9467.0,       -17223.0,        -7521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9843.0,        -8191.0,        -3379.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5143.0,       -80601.0,       -35106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21477.0,        -3581.0,        -1336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1524.0,        -1250.0,         -759.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -41785116.0,     66533021.0,     28858259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1457.0,        -5068.0,        -1077.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,           59.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1207.0,        -1703.0,         -984.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -270.0,         3684.0,         1912.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1750.0,         1981.0,          632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          313.0,           73.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,         -221.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,            7.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11299.0,        49049.0,        21281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -415.0,          -17.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           77.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          489.0,          451.0,          199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          275.0,          148.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          158.0,         -447.0,         -136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          672.0,         -567.0,         -279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,         -167.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,            5.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -890.0,         -141.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,         -465.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          250.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -217.0,           81.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5826.0,         2745.0,         1673.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7211.0,        -3337.0,        -2007.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          365.0,          -63.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -840.0,          117.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4660.0,        -1549.0,         -549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1499.0,          -64.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1124.0,          139.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13114.0,        22210.0,        11216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17913.0,        -4426.0,        -3577.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3627.0,         1165.0,         1099.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1110.0,        -1743.0,         -741.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,           13.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -385.0,         -975.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -655052.0,       754319.0,       326870.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           25.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            4.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,         -237.0,         -115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -2.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           15.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,         1324.0,          581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -28.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           14.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          736.0,         -601.0,         -187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -39.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -578.0,         -496.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -7.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,           19.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,          -10.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,          -17.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,         -115.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -49.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            5.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          483.0,          536.0,          204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,          597.0,          261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -21.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,           45.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          -27.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          -19.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11774.0,         9400.0,         4055.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,          -79.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            1.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           -3.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,         -134.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -362.0,         -440.0,         -172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           77.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -13.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           26.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -16.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            6.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -369.0,           53.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,          -18.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           28.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,           55.0,          -59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           14.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           16.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           11.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -202.0,          143.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -7.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -16.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            8.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          166.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,           41.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -11.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -14.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          -45.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -32.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            7.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           36.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -26642.0,      -164735.0,       -68637.0),
           (       447434.0,       537782.0,       234251.0),
           (         -315.0,        19329.0,        10532.0)),
          ((        73570.0,       243123.0,       101817.0),
           (       -29641.0,       -14260.0,        -6311.0),
           (           19.0,            1.0,            9.0)),
          ((       -41286.0,      -280340.0,      -120421.0),
           (        37520.0,        40756.0,        18413.0),
           (            0.0,            0.0,            0.0)),
          ((        -3322.0,        56939.0,        24944.0),
           (       -18937.0,       -27843.0,       -12705.0),
           (            0.0,            0.0,            0.0)),
          ((       -43584.0,       -36435.0,       -15502.0),
           (       -15652.0,        -5015.0,        -2277.0),
           (            0.0,            0.0,            0.0)),
          ((        -1787.0,         5368.0,         2244.0),
           (        10437.0,       -20549.0,       -16500.0),
           (            0.0,            0.0,            0.0)),
          ((       120932.0,      -118417.0,       -54461.0),
           (        21812.0,       131280.0,      -290821.0),
           (            0.0,            0.0,            0.0)),
          ((        -1881.0,       -44764.0,       -18804.0),
           (       -10407.0,       112170.0,        48107.0),
           (            0.0,            0.0,            0.0)),
          ((         -481.0,         4493.0,         1871.0),
           (        -1875.0,         8229.0,         3217.0),
           (            0.0,            0.0,            0.0)),
          ((          959.0,         7690.0,         3220.0),
           (        11063.0,        -2559.0,        -1869.0),
           (            0.0,            0.0,            0.0)),
          ((      -114486.0,       -86106.0,       -40213.0),
           (         1141.0,            1.0,          152.0),
           (            0.0,            0.0,            0.0)),
          ((          907.0,         4181.0,         1669.0),
           (        38296.0,       -26022.0,       -11997.0),
           (            0.0,            0.0,            0.0)),
          ((         2555.0,        -1378.0,         -443.0),
           (        -2652.0,        -4203.0,        -1746.0),
           (            0.0,            0.0,            0.0)),
          ((        -1453.0,          201.0,           52.0),
           (         -255.0,          -29.0,           -8.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,         1348.0,          998.0),
           (        -7642.0,          260.0,         -285.0),
           (            0.0,            0.0,            0.0)),
          ((         4918.0,        -8110.0,        -2483.0),
           (         2512.0,        -2996.0,        -2080.0),
           (            0.0,            0.0,            0.0)),
          ((        -7075.0,         3327.0,          144.0),
           (         -961.0,         -171.0,          -54.0),
           (            0.0,            0.0,            0.0)),
          ((         3433.0,         1193.0,         1340.0),
           (         1511.0,        -2023.0,        -1017.0),
           (            0.0,            0.0,            0.0)),
          ((        11318.0,         1119.0,          124.0),
           (           33.0,          -38.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((         3341.0,         7706.0,         3715.0),
           (          -24.0,          -36.0,          -22.0),
           (            0.0,            0.0,            0.0)),
          ((         -641.0,        -4826.0,        -2536.0),
           (          -35.0,          -52.0,          -17.0),
           (            0.0,            0.0,            0.0)),
          ((         1939.0,        12227.0,         6466.0),
           (           12.0,           94.0,          110.0),
           (            0.0,            0.0,            0.0)),
          ((        -4535.0,       -10487.0,        -5571.0),
           (           13.0,           19.0,           13.0),
           (            0.0,            0.0,            0.0)),
          ((       -71423.0,        39238.0,        16645.0),
           (          -16.0,            3.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((         -854.0,        -4196.0,        -2069.0),
           (            3.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((         2088.0,         2859.0,         1583.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -89350.0,       -59538.0,       -34375.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2310.0,         4176.0,         4162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7588.0,       -17178.0,        -2083.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -56781.0,       -68848.0,       -29355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26634.0,        -8972.0,        -4988.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6658.0,       -12115.0,        -5239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -6530136237.0,   6946811783.0,   3012502056.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14820.0,        -8720.0,        -2958.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -46446.0,       -45053.0,       -19173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25574.0,       -36692.0,       -17989.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -56377.0,        53391.0,        26627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5974.0,        -9137.0,        -4439.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -345.0,         5454.0,         2512.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          462.0,         -753.0,         -349.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6802.0,       -47720.0,       -20744.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2462.0,         1914.0,         1007.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          634.0,        -2106.0,        -1002.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1205.0,         4972.0,         2345.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2947.0,        -6120.0,        -2947.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          357.0,          669.0,          371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1962.0,         5399.0,         2313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          180.0,         2456.0,         1121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1929.0,        -3513.0,        -1552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3556.0,         4974.0,         2240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2000.0,         7392.0,         1974.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -205.0,          640.0,          542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,        -1781.0,         -947.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,         2061.0,         1066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -64398.0,       -61720.0,       -28335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -974.0,         3212.0,         2088.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -215.0,         -997.0,         -486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1380.0,         2338.0,         1192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7324.0,        -4750.0,        -2359.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1485.0,         -457.0,         -288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1337.0,        -2107.0,         -766.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19094.0,       -12734.0,        -5734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14825.0,         -174.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17087.0,        -2828.0,        -1163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -96618.0,         3264.0,         1652.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22282.0,         6377.0,         2717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7782.0,         3197.0,         1437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -77927687.0,     27819012.0,     12063463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8715.0,       -10594.0,        -4831.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -969.0,          880.0,          409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2578.0,         1159.0,          480.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2476.0,        -3114.0,        -1313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4276.0,          980.0,          757.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,          216.0,           98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,           79.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -303.0,          153.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -39918.0,        33907.0,        15761.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,         -680.0,         -302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,          164.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          748.0,         -435.0,         -184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           23.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -194.0,         -609.0,         -295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          627.0,         1556.0,          696.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,         -228.0,         -107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,           94.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          510.0,         -421.0,         -191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -276.0,         -833.0,         -378.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          114.0,           95.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,         -234.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4870.0,        -1963.0,         -981.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5715.0,         1862.0,          953.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          261.0,          189.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -633.0,          196.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4381.0,        -2601.0,        -1061.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1165.0,        -1400.0,         -533.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1055.0,         1222.0,          455.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26090.0,         -615.0,        -1193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13156.0,       -13576.0,        -4930.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2445.0,        -2617.0,         -899.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2105.0,        -1209.0,         -518.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          -49.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1200.0,         1569.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1040602.0,      -113815.0,       -48948.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,          -95.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           38.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -44.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          171.0,           60.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -12.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -15.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1378.0,          314.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,            4.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           30.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -921.0,         -277.0,         -191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           38.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          589.0,         -314.0,         -404.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,         -106.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            3.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           87.0,           58.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -212.0,          -65.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,            7.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,          721.0,          328.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -197.0,         -568.0,         -246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           26.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,         -126.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -19.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -40.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,          -13.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13064.0,        -7726.0,        -3368.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,           68.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,          186.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          580.0,         -150.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -41.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           13.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -17.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -15.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           13.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          430.0,           81.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,          -15.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          -30.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -9.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           11.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -16.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -21.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -117.0,         -214.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,         -164.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           12.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,          -12.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           10.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           61.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -5.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -5.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -24.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -55.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       185009.0,       320207.0,       138022.0),
           (       249630.0,       332973.0,       114385.0),
           (        12248.0,         2553.0,          455.0)),
          ((      -251214.0,      -468126.0,      -202048.0),
           (         8237.0,        -3373.0,          188.0),
           (           42.0,          -45.0,          -20.0)),
          ((       256203.0,       241636.0,       103861.0),
           (       -11721.0,        24651.0,         8499.0),
           (            0.0,            0.0,            0.0)),
          ((        27437.0,        66503.0,        28830.0),
           (       -16091.0,       -15058.0,        -6236.0),
           (            0.0,            0.0,            0.0)),
          ((       -35381.0,        42977.0,        18817.0),
           (       -15546.0,        -6427.0,        -3457.0),
           (            0.0,            0.0,            0.0)),
          ((        -5752.0,        -2385.0,        -1009.0),
           (       -42777.0,       -76869.0,       -34744.0),
           (            0.0,            0.0,            0.0)),
          ((        25985.0,      -220601.0,       -97239.0),
           (        43525.0,       106915.0,      -161698.0),
           (            0.0,            0.0,            0.0)),
          ((        26568.0,        38786.0,        16776.0),
           (       -84638.0,      -129951.0,       -54327.0),
           (            0.0,            0.0,            0.0)),
          ((        -3512.0,        -3813.0,        -1679.0),
           (         6248.0,        -3842.0,        -1550.0),
           (            0.0,            0.0,            0.0)),
          ((        -3828.0,        -4302.0,        -1843.0),
           (        15584.0,         3652.0,         1945.0),
           (            0.0,            0.0,            0.0)),
          ((      -110273.0,       -74599.0,       -35341.0),
           (       -12248.0,         4810.0,         1893.0),
           (            0.0,            0.0,            0.0)),
          ((        -3776.0,        -8449.0,        -3664.0),
           (        33860.0,         6261.0,         4358.0),
           (            0.0,            0.0,            0.0)),
          ((         4148.0,         2353.0,         1137.0),
           (        -4067.0,          410.0,          227.0),
           (            0.0,            0.0,            0.0)),
          ((        -1555.0,         -457.0,         -209.0),
           (         -213.0,         -387.0,         -155.0),
           (            0.0,            0.0,            0.0)),
          ((         4457.0,         6337.0,         3064.0),
           (        -6174.0,        -4505.0,        -2119.0),
           (            0.0,            0.0,            0.0)),
          ((        16716.0,        14166.0,         7073.0),
           (          648.0,        -8755.0,        -3467.0),
           (            0.0,            0.0,            0.0)),
          ((       -12543.0,       -12263.0,        -6500.0),
           (          628.0,          168.0,          173.0),
           (            0.0,            0.0,            0.0)),
          ((         4708.0,         6013.0,         3361.0),
           (         -441.0,        -3736.0,        -1216.0),
           (            0.0,            0.0,            0.0)),
          ((         4536.0,       -12018.0,        -5573.0),
           (           83.0,           81.0,           17.0),
           (            0.0,            0.0,            0.0)),
          ((        -7422.0,        -2684.0,         -994.0),
           (           26.0,          -25.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         1985.0,           73.0,         -429.0),
           (           27.0,          -43.0,          -31.0),
           (            0.0,            0.0,            0.0)),
          ((        -5979.0,        -1362.0,          727.0),
           (          -89.0,         -132.0,          -49.0),
           (            0.0,            0.0,            0.0)),
          ((         3251.0,         3372.0,          277.0),
           (          -16.0,          -31.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((       -82958.0,        31111.0,        12685.0),
           (           -4.0,           -9.0,           10.0),
           (            0.0,            0.0,            0.0)),
          ((         2260.0,          443.0,          -74.0),
           (           -5.0,           -1.0,           -5.0),
           (            0.0,            0.0,            0.0)),
          ((        -4887.0,         1727.0,         1032.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -29274.0,        99145.0,        41302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6506.0,         3745.0,         4686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11073.0,         6305.0,        10239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23074.0,        49789.0,        22803.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28377.0,        25348.0,        10302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8464.0,        -8684.0,        -3315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -8679497842.0,   4554065998.0,   1974427519.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1953.0,         5541.0,         1804.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        41440.0,        41805.0,        17425.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43181.0,        68727.0,        30503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        72477.0,       -77903.0,       -34196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8560.0,        17815.0,         8024.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3317.0,        -5327.0,        -2398.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,         1290.0,          590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35765.0,       -30342.0,       -12686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4143.0,        -4015.0,        -1916.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,         1057.0,          477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          878.0,        -5098.0,        -2287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2007.0,         1914.0,          894.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2130.0,        -1483.0,         -793.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8165.0,         1042.0,          510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,         -631.0,         -285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -797.0,         2232.0,         1007.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          306.0,         3384.0,         1542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3740.0,         9787.0,         4735.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          628.0,         -912.0,         -492.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          578.0,          290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          729.0,         -128.0,         -118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16775.0,       -83613.0,       -35809.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7163.0,         1913.0,          498.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -262.0,          -82.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1355.0,         1064.0,          456.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7164.0,        -3719.0,        -1905.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2360.0,        -1465.0,         -636.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9230.0,         3327.0,         1431.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15265.0,        -7077.0,        -3038.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5934.0,         8232.0,         3598.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8224.0,         7112.0,         3161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5826.0,        79087.0,        34166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13827.0,       -11611.0,        -4980.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          514.0,         2117.0,          851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -79647020.0,    -23096453.0,    -10013839.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6387.0,         6323.0,         3093.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -704.0,         -505.0,         -280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          659.0,         1177.0,          337.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,         -266.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3132.0,        -1797.0,         -461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -219.0,          235.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          218.0,          122.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,          150.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -52637.0,       -14062.0,        -5028.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          675.0,         -110.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,         -647.0,         -276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -312.0,         -149.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          243.0,          546.0,          194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1613.0,         -452.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          197.0,          172.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,          -36.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          911.0,         -390.0,         -218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          228.0,          508.0,          220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,         -205.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,         -258.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3005.0,         7094.0,         2734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4036.0,        -8995.0,        -3491.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,         -145.0,          -84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          202.0,          126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2626.0,        -1119.0,         -619.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -271.0,         -386.0,         -239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          258.0,          338.0,          183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25541.0,         5316.0,         2089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6457.0,         -839.0,         -114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2901.0,        -2545.0,        -1547.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1103.0,          966.0,          410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          304.0,         -506.0,         -235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2093.0,         4590.0,         2135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -442959.0,      -873262.0,      -378720.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,         -256.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,          127.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -36.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          203.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           45.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -19.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -638.0,        -1198.0,         -521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           12.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,          874.0,          370.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          -26.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,          638.0,          197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           93.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,          -89.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,          180.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           33.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           34.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -358.0,          541.0,          245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -13.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,          502.0,          216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          140.0,            8.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           12.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -234.0,          201.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6420.0,       -13550.0,        -5830.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          121.0,         -126.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -6.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           -1.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -214.0,          -54.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          570.0,          248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,           -7.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           12.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -5.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -6.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           29.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -2.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -400.0,         -225.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,           80.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -40.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            2.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           14.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           11.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            9.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          232.0,          -66.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -2.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,          163.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           10.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -19.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,          -18.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -12.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -51.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           32.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           48.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -15.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      -115566.0,        65517.0,        22233.0),
           (      -137619.0,      -715951.0,      -271214.0),
           (        -5642.0,        26882.0,        11727.0)),
          ((       135943.0,       -22018.0,        -1497.0),
           (        30600.0,        35453.0,        13377.0),
           (           18.0,           24.0,            6.0)),
          ((      -285666.0,        56104.0,        22979.0),
           (       -17673.0,       -70880.0,       -28667.0),
           (            0.0,            0.0,            0.0)),
          ((        31439.0,        28109.0,        11790.0),
           (       -31068.0,         5808.0,         3807.0),
           (            0.0,            0.0,            0.0)),
          ((        42537.0,        33733.0,        14294.0),
           (       -20311.0,         -364.0,         -928.0),
           (            0.0,            0.0,            0.0)),
          ((         6624.0,        -3596.0,        -1403.0),
           (         2521.0,       -23522.0,        -5229.0),
           (            0.0,            0.0,            0.0)),
          ((          370.0,      -210649.0,       -90972.0),
           (       -20911.0,         6910.0,       -38849.0),
           (            0.0,            0.0,            0.0)),
          ((       -30105.0,        15297.0,         5675.0),
           (       176777.0,       146304.0,        67353.0),
           (            0.0,            0.0,            0.0)),
          ((         2435.0,         -659.0,         -204.0),
           (        12242.0,         1852.0,         1935.0),
           (            0.0,            0.0,            0.0)),
          ((         8940.0,         -842.0,         -187.0),
           (        14757.0,        -7382.0,        -1415.0),
           (            0.0,            0.0,            0.0)),
          ((      -121542.0,       -77364.0,       -36664.0),
           (       -10549.0,          159.0,         -671.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,        -5855.0,        -2566.0),
           (        29819.0,        32669.0,        16643.0),
           (            0.0,            0.0,            0.0)),
          ((        -2864.0,         1139.0,          500.0),
           (         1265.0,          812.0,           65.0),
           (            0.0,            0.0,            0.0)),
          ((         1576.0,         -509.0,         -269.0),
           (          241.0,           72.0,           10.0),
           (            0.0,            0.0,            0.0)),
          ((         2791.0,         3746.0,         1655.0),
           (          809.0,        -1330.0,         -292.0),
           (            0.0,            0.0,            0.0)),
          ((       -12588.0,        10849.0,         5329.0),
           (       -12741.0,       -10705.0,        -4491.0),
           (            0.0,            0.0,            0.0)),
          ((        12285.0,        -4183.0,        -2372.0),
           (         -773.0,        -2574.0,        -1159.0),
           (            0.0,            0.0,            0.0)),
          ((        -4063.0,         4269.0,         2363.0),
           (         2108.0,          560.0,          257.0),
           (            0.0,            0.0,            0.0)),
          ((        -3403.0,       -11976.0,        -5356.0),
           (           -2.0,          147.0,           62.0),
           (            0.0,            0.0,            0.0)),
          ((         9084.0,         -437.0,           48.0),
           (           41.0,          -29.0,          -11.0),
           (            0.0,            0.0,            0.0)),
          ((        -2308.0,        -1928.0,        -1289.0),
           (           14.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((         6834.0,         4473.0,         3234.0),
           (          100.0,           26.0,          -67.0),
           (            0.0,            0.0,            0.0)),
          ((        -7952.0,        -1421.0,        -1812.0),
           (            2.0,           -1.0,          -19.0),
           (            0.0,            0.0,            0.0)),
          ((       -84502.0,        25163.0,         9799.0),
           (           11.0,           -5.0,          -18.0),
           (            0.0,            0.0,            0.0)),
          ((        -2510.0,        -1893.0,        -1082.0),
           (            7.0,            3.0,            8.0),
           (            0.0,            0.0,            0.0)),
          ((         4774.0,            8.0,          549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       108513.0,        -8028.0,         4563.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2275.0,        10923.0,         6979.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3591.0,        25685.0,        17110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        56098.0,        12942.0,         6427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26352.0,       -32377.0,       -12910.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -778.0,        12828.0,         5131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -9837153068.0,   1640716845.0,    711250457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18681.0,       -12952.0,        -6804.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -30706.0,       -14182.0,        -4813.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34442.0,       -57556.0,       -25193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -70734.0,       106970.0,        47577.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9254.0,       -12230.0,        -5239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2921.0,         3832.0,         1623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -314.0,        -2222.0,        -1023.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51274.0,        -5774.0,        -1958.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2314.0,         4608.0,         2029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          154.0,        -1319.0,         -615.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2875.0,         5451.0,         2570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1110.0,        -4236.0,        -1959.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1402.0,         1802.0,          639.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          473.0,        -8074.0,        -3583.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          516.0,          288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1345.0,         -432.0,         -349.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3695.0,         3264.0,         1617.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6418.0,        -1735.0,          785.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1150.0,         2193.0,          818.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -149.0,        -1437.0,         -558.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          362.0,         1513.0,          610.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51169.0,       -79572.0,       -33465.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5171.0,         9004.0,         4065.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -278.0,        -1022.0,         -430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          467.0,         3674.0,         1556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7594.0,        -4307.0,        -2246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          164.0,        -3924.0,        -1686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4160.0,         7942.0,         3441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7694.0,        11407.0,         5013.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2195.0,         5639.0,         2363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10092.0,         7724.0,         3263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        94231.0,       -20052.0,        -8877.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10049.0,        24171.0,        10461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3751.0,        -5120.0,        -2193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -46256608.0,    -63721421.0,    -27619133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1248.0,         6152.0,         2521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          621.0,         -194.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          661.0,         -195.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1865.0,         -198.0,         -176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3561.0,        -1307.0,         -764.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -298.0,          -69.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          -47.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,         -394.0,         -167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14464.0,       -48242.0,       -20865.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          299.0,          641.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,         -102.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -753.0,           44.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -148.0,            9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          698.0,          328.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,        -1204.0,         -517.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          273.0,           25.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -120.0,           45.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -231.0,          914.0,          408.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          436.0,          253.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -177.0,           35.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,           89.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5011.0,         1483.0,          627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6476.0,        -1496.0,         -616.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          158.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -286.0,         -366.0,         -145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3781.0,         1267.0,          499.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1022.0,          789.0,          314.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1106.0,         -480.0,         -194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31731.0,         6128.0,         2363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13472.0,         -278.0,          172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3147.0,         1843.0,          367.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          521.0,         2449.0,         1051.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,          533.0,          210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1079.0,        -2379.0,         -797.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       569336.0,      -802070.0,      -347862.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          135.0,          119.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,          -49.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           -3.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -150.0,         -128.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,            5.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1138.0,         -781.0,         -359.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           -7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          779.0,         -554.0,         -162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -760.0,          145.0,          355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           12.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           47.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -102.0,           99.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          242.0,          -88.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           16.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,           13.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -551.0,          355.0,          131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -5.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -452.0,         -408.0,         -176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -16.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           95.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           49.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,           19.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            7.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14835.0,         2276.0,          941.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           17.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -18.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,         -117.0,         -168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -567.0,         -273.0,          -59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,           10.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           21.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -13.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           10.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,          289.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -71.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,           30.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           14.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -12.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -5.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -16.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          208.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,         -161.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -15.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           21.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,           47.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           16.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -10.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,           21.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -35.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           22.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,          -15.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       160096.0,      -231859.0,       -98737.0),
           (       193250.0,       -89548.0,       -73262.0),
           (       -29059.0,       -15264.0,        -6354.0)),
          ((      -187947.0,       228914.0,        96856.0),
           (       -36463.0,       -14428.0,        -4526.0),
           (           59.0,           13.0,            9.0)),
          ((       210322.0,      -263902.0,      -114218.0),
           (        34776.0,        -7920.0,        -4570.0),
           (            0.0,            0.0,            0.0)),
          ((        59645.0,       -19421.0,        -8381.0),
           (       -39451.0,         8393.0,         1094.0),
           (            0.0,            0.0,            0.0)),
          ((        37167.0,       -39604.0,       -17309.0),
           (       -21172.0,        -1920.0,        -2957.0),
           (            0.0,            0.0,            0.0)),
          ((        -3545.0,         5130.0,         2172.0),
           (        20972.0,       -43538.0,        -3151.0),
           (            0.0,            0.0,            0.0)),
          ((      -110948.0,      -155535.0,       -67069.0),
           (        25313.0,       -87558.0,        86738.0),
           (            0.0,            0.0,            0.0)),
          ((        21601.0,       -32868.0,       -13803.0),
           (      -189195.0,       -58066.0,       -28959.0),
           (            0.0,            0.0,            0.0)),
          ((        -1328.0,         4517.0,         1915.0),
           (       -15504.0,        13277.0,         4487.0),
           (            0.0,            0.0,            0.0)),
          ((        -1066.0,         7160.0,         3038.0),
           (       -15825.0,        25049.0,         8891.0),
           (            0.0,            0.0,            0.0)),
          ((      -112828.0,       -84263.0,       -39428.0),
           (        14306.0,       -12272.0,        -4571.0),
           (            0.0,            0.0,            0.0)),
          ((        -7232.0,         4280.0,         1585.0),
           (       -26911.0,        31991.0,        19656.0),
           (            0.0,            0.0,            0.0)),
          ((         1386.0,         -878.0,         -328.0),
           (        -3230.0,        -6175.0,        -2053.0),
           (            0.0,            0.0,            0.0)),
          ((         -612.0,         -305.0,         -205.0),
           (          787.0,         -573.0,         -236.0),
           (            0.0,            0.0,            0.0)),
          ((         6052.0,        -1904.0,         -873.0),
           (         7538.0,        -5112.0,        -1996.0),
           (            0.0,            0.0,            0.0)),
          ((        -3794.0,       -10714.0,        -3966.0),
           (         1300.0,          -34.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((         2896.0,         7948.0,         2420.0),
           (         1547.0,         1141.0,          385.0),
           (            0.0,            0.0,            0.0)),
          ((        -1752.0,         -888.0,          133.0),
           (         -134.0,         2013.0,          525.0),
           (            0.0,            0.0,            0.0)),
          ((       -12209.0,          277.0,           27.0),
           (          -31.0,          -40.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        -7614.0,         7680.0,         3481.0),
           (          -44.0,           55.0,           17.0),
           (            0.0,            0.0,            0.0)),
          ((         3219.0,        -3687.0,        -1875.0),
           (          -21.0,          -23.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((        -8825.0,        11060.0,         5591.0),
           (           -8.0,           64.0,           99.0),
           (            0.0,            0.0,            0.0)),
          ((         8136.0,        -7934.0,        -4192.0),
           (           12.0,           -2.0,           14.0),
           (            0.0,            0.0,            0.0)),
          ((       -90810.0,        18443.0,         6609.0),
           (            3.0,            7.0,           11.0),
           (            0.0,            0.0,            0.0)),
          ((         3513.0,        -1367.0,         -656.0),
           (           -7.0,           -1.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((        -5250.0,         4240.0,         1782.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -33511.0,      -101879.0,       -48807.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3310.0,        22921.0,        10956.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2887.0,        36658.0,        18074.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12970.0,       -55290.0,       -24078.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        24791.0,        35048.0,        14249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7989.0,       -12984.0,        -5830.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -9870778044.0,  -1460431122.0,   -632674365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31405.0,         9232.0,         4063.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14269.0,       -12271.0,        -6396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18244.0,        32876.0,        13466.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27223.0,       -35948.0,       -14799.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5414.0,         6240.0,         2515.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1702.0,        -1681.0,         -631.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          331.0,         1871.0,          814.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        44833.0,        27198.0,        12162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4884.0,          230.0,          308.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1541.0,          877.0,          321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4010.0,         -178.0,          132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6113.0,         3240.0,         1170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -955.0,         -500.0,         -338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8949.0,         2247.0,          888.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1317.0,         -488.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3970.0,          221.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          485.0,        -3718.0,        -1472.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4790.0,        -8240.0,        -2477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2363.0,         -720.0,         -424.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2080.0,          260.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2469.0,        -1048.0,         -430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        93072.0,       -20925.0,        -7564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6271.0,        -7270.0,        -3131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1616.0,          455.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5757.0,        -2161.0,         -882.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12392.0,          661.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5123.0,         3226.0,         1381.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6951.0,        -7129.0,        -3111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1802.0,       -12938.0,        -5674.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9838.0,        -4413.0,        -2012.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9026.0,         -605.0,         -468.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28188.0,       -80152.0,       -34509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1133.0,       -29400.0,       -12880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -370.0,         2255.0,         1105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      7445217.0,    -76165526.0,    -33004903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,       -18816.0,        -8763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -224.0,         1476.0,          728.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          294.0,         2962.0,         1488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1013.0,        -3652.0,        -1852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5301.0,         1584.0,          295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,         -382.0,         -169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -227.0,           46.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,         -353.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        37669.0,       -35875.0,       -16566.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -589.0,          291.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,          -74.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -270.0,          686.0,          291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,          116.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,         -542.0,         -208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          592.0,         1382.0,          585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,         -180.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          121.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1377.0,          559.0,          277.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          421.0,         -206.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          162.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,          272.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1414.0,        -7381.0,        -3190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1232.0,         9450.0,         4091.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,          113.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -725.0,         -115.0,          -79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5540.0,          544.0,          366.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2015.0,          -12.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1716.0,          106.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8350.0,       -11329.0,        -5654.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22248.0,        -3139.0,         -560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1243.0,         3282.0,         1680.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2849.0,         -762.0,         -325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          870.0,          -85.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4833.0,         -794.0,         -700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1046468.0,        18982.0,         8517.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          282.0,           60.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,          -32.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,           39.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          117.0,         -182.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -10.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           11.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1104.0,          845.0,          360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           -9.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -27.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -888.0,         -345.0,         -214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           33.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -204.0,         -671.0,         -296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,         -105.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,           19.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -102.0,         -142.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -29.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          -14.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -722.0,           83.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           18.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          532.0,          299.0,          130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,          -19.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           57.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           22.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -76.0,         -248.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1119.0,        14626.0,         6356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,          160.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          143.0,          180.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          439.0,         -395.0,         -222.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -7.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -27.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           -1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           14.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -14.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,         -313.0,         -171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           37.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,           41.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,          -58.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           12.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           15.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           12.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -247.0,           44.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,          161.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           17.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -24.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           77.0,          -71.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -17.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           16.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            1.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,           18.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           10.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -15.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,          -25.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // mar
    marct   : array [1..5,1..183,0..2,1..3] of double = (
         (((       239420.0,       100731.0,        39494.0),
           (     -4133539.0,      -331329.0,     -4560125.0),
           (      -382802.0,     -2919082.0,     -1143555.0)),
          ((      -699022.0,      1041959.0,       490722.0),
           (       716780.0,      -119673.0,      -108741.0),
           (            0.0,            0.0,            0.0)),
          ((      -262888.0,       109966.0,        54070.0),
           (      -229632.0,       -48016.0,       -37638.0),
           (            0.0,            0.0,            0.0)),
          ((       446037.0,      -426933.0,      -204515.0),
           (       547442.0,      3356053.0,      1666796.0),
           (            0.0,            0.0,            0.0)),
          ((      -268375.0,       243904.0,       114082.0),
           (       745145.0,      1462896.0,       816978.0),
           (            0.0,            0.0,            0.0)),
          ((      -185050.0,       887092.0,       408031.0),
           (         2459.0,           92.0,         1046.0),
           (            0.0,            0.0,            0.0)),
          ((      -132864.0,       -35024.0,       -13475.0),
           (       -56983.0,        -6175.0,       -12938.0),
           (            0.0,            0.0,            0.0)),
          ((        68237.0,         8641.0,         4684.0),
           (     -1092220.0,       479746.0,       145878.0),
           (            0.0,            0.0,            0.0)),
          ((      -555385.0,       -29719.0,        -8494.0),
           (       273383.0,      -169336.0,       -74357.0),
           (            0.0,            0.0,            0.0)),
          ((      3955743.0,       204108.0,        83127.0),
           (        -6651.0,         6871.0,         3415.0),
           (            0.0,            0.0,            0.0)),
          ((     -6033005.0,      -389028.0,      -183161.0),
           (         2298.0,          -59.0,         -443.0),
           (            0.0,            0.0,            0.0)),
          ((      3164476.0,       246172.0,       128020.0),
           (       -16705.0,        33199.0,        17391.0),
           (            0.0,            0.0,            0.0)),
          ((     -1021588.0,      -149284.0,       -85879.0),
           (         3501.0,         7570.0,         3289.0),
           (            0.0,            0.0,            0.0)),
          ((      1340078.0,      -878431.0,      -411440.0),
           (          -97.0,         -334.0,         -152.0),
           (            0.0,            0.0,            0.0)),
          ((      1774010.0,       -86691.0,        27942.0),
           (         4278.0,        13224.0,         5610.0),
           (            0.0,            0.0,            0.0)),
          ((     -2867152.0,       336669.0,        59566.0),
           (           21.0,          118.0,           53.0),
           (            0.0,            0.0,            0.0)),
          ((       488598.0,        45550.0,        39517.0),
           (           21.0,           51.0,           24.0),
           (            0.0,            0.0,            0.0)),
          ((      -489410.0,      -411577.0,      -216422.0),
           (          -16.0,         -112.0,          -60.0),
           (            0.0,            0.0,            0.0)),
          ((       883194.0,       -49198.0,        40128.0),
           (            4.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1345239.0,       344000.0,       -19882.0),
           (           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          (( -14883852610.0,   2559146900.0,   1579047226.0),
           (           44.0,            3.0,          -28.0),
           (            0.0,            0.0,            0.0)),
          ((       -97894.0,       105045.0,        69780.0),
           (           12.0,           36.0,           16.0),
           (            0.0,            0.0,            0.0)),
          ((      -227685.0,      -917269.0,      -444181.0),
           (           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((       281925.0,       476074.0,       233122.0),
           (            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((      -116323.0,       168678.0,        81905.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -38401.0,       -21850.0,        -9762.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        72079.0,       -56365.0,       -27266.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16597.0,         4920.0,         4112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,         4303.0,         2071.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25777.0,       -74979.0,       -32226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24146.0,       -38430.0,       -15177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       244298.0,       149321.0,        50702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       193417.0,       252414.0,       119119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -200541.0,      -146581.0,       -57336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        50578.0,        21999.0,         4989.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       460781.0,       246496.0,       104640.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -114359.0,       284138.0,       122854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63321.0,      -852786.0,      -343027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    706256567.0,     26811086.0,     -6985218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -378748.0,     -1188941.0,      -460331.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       297603.0,       958567.0,       373757.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19235.0,        84666.0,        33012.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10283.0,       -22721.0,        -9732.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1731.0,         7409.0,         3067.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18973.0,        -1069.0,          360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          833.0,         2239.0,          486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2763.0,        -3915.0,        -1532.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34036.0,         9585.0,         5068.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        38587.0,        58722.0,        24496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -106242.0,      -234231.0,      -103926.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13610.0,       -26880.0,       -10685.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        24657.0,        39875.0,        16328.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -81667.0,       -84408.0,       -35055.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        42401.0,        42820.0,        15922.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -112668.0,      -185069.0,       -74991.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       923414.0,      1691295.0,       694949.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -48524540.0,    -13315985.0,     -4737883.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -81608.0,      -138974.0,       -57139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        49927.0,       107576.0,        45193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -41869.0,      -110129.0,       -47010.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24139.0,      -113990.0,       -50231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        41764.0,       181739.0,        79526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          186.0,        -9582.0,        -4390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          362.0,          265.0,          179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -742.0,         4635.0,         2257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10428.0,       -11659.0,        -5675.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          164.0,         -604.0,         -383.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5373.0,        31858.0,        14438.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3281.0,           32.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          979.0,          -93.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5229.0,         8034.0,         3567.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1539.0,          272.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -333.0,         1721.0,          881.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3611951.0,      1763166.0,       710403.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5636.0,        -3698.0,        -2066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1968.0,          916.0,          515.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          410.0,          378.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          551.0,         -347.0,         -169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -400.0,          131.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,           28.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          853.0,          -27.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           70.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -29.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1911.0,         1363.0,          589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,           72.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          439.0,          -70.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          277.0,        -4148.0,        -1928.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          219.0,          -67.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -247.0,          244.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,         -482.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -643.0,         -738.0,         -388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          759.0,         -263.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1643.0,         1435.0,          204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -282189.0,      -221497.0,       -94309.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2823.0,        -3883.0,         -674.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1004.0,         1748.0,          409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          332.0,         -599.0,         -200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -395.0,          277.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           96.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,          125.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -174.0,           13.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           39.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          309.0,         -169.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,           -9.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,          -11.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -177.0,          465.0,          214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,            9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,           61.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          256.0,          179.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -234.0,          -37.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           91.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        23764.0,        28424.0,        12850.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2735.0,        -2125.0,        -1402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,           99.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -80.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,           18.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -67.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -29.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            7.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -15.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -28.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,           28.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,           13.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -16.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -47.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -21.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -24.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1419.0,        -3014.0,        -1343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          -47.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           75.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           28.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,            0.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -28.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -326.0,         -470.0,         -221.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -27.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           13.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,          331.0,          149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -29.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           -4.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,          -42.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,            5.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            0.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           25.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -8.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           35.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -84475.0,      -134075.0,       -58358.0),
           (     20308804.0,      4428644.0,      1303521.0),
           (     -2009011.0,      2647923.0,      1027186.0)),
          ((      -197129.0,     -1175000.0,      -530510.0),
           (      1187239.0,      -274497.0,      -146223.0),
           (            0.0,            0.0,            0.0)),
          ((        56623.0,      -337669.0,      -148067.0),
           (       317440.0,       374649.0,       154921.0),
           (            0.0,            0.0,            0.0)),
          ((       303069.0,        87183.0,        18372.0),
           (      -324202.0,     -2556954.0,     -1176492.0),
           (            0.0,            0.0,            0.0)),
          ((        -4170.0,      -202070.0,       -77944.0),
           (     -1949960.0,     -1601200.0,      -654488.0),
           (            0.0,            0.0,            0.0)),
          ((      1111744.0,       430224.0,       155660.0),
           (       -14318.0,        -8120.0,        -3220.0),
           (            0.0,            0.0,            0.0)),
          ((        -6542.0,        26434.0,        19079.0),
           (       146206.0,        92041.0,        35370.0),
           (            0.0,            0.0,            0.0)),
          ((       -10748.0,        37193.0,         8606.0),
           (      1661934.0,      1585854.0,       660381.0),
           (            0.0,            0.0,            0.0)),
          ((      -258891.0,      -458823.0,      -166458.0),
           (      -258714.0,      -324243.0,      -139340.0),
           (            0.0,            0.0,            0.0)),
          ((      1499140.0,      2531487.0,       767242.0),
           (         1346.0,         3555.0,         1727.0),
           (            0.0,            0.0,            0.0)),
          ((     -2198067.0,     -4208047.0,     -1275953.0),
           (         1140.0,        -1409.0,         -327.0),
           (            0.0,            0.0,            0.0)),
          ((      1072808.0,      2309222.0,       685896.0),
           (       -16937.0,        11661.0,         5768.0),
           (            0.0,            0.0,            0.0)),
          ((      -246773.0,      -753785.0,      -203773.0),
           (       -11861.0,        -9522.0,        -5632.0),
           (            0.0,            0.0,            0.0)),
          ((       819127.0,      2270407.0,       829516.0),
           (          139.0,          -28.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       436149.0,      2103572.0,       584087.0),
           (        -5336.0,         3783.0,         1720.0),
           (            0.0,            0.0,            0.0)),
          ((     -1027914.0,     -3629208.0,     -1040924.0),
           (           30.0,           99.0,           60.0),
           (            0.0,            0.0,            0.0)),
          ((       168307.0,       795381.0,       225296.0),
           (          -10.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((      -637121.0,      -449249.0,       -67947.0),
           (           40.0,           26.0,           19.0),
           (            0.0,            0.0,            0.0)),
          ((       243803.0,       981974.0,       196054.0),
           (           -6.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((       -32777.0,     -1662298.0,      -153190.0),
           (            3.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((   1412756867.0, -13705867614.0,  -6324913629.0),
           (          124.0,            1.0,          -29.0),
           (            0.0,            0.0,            0.0)),
          ((        75065.0,       -91139.0,      -104656.0),
           (          -69.0,           39.0,           27.0),
           (            0.0,            0.0,            0.0)),
          ((      -701879.0,       -12161.0,       103497.0),
           (            4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       260933.0,      -159147.0,      -146419.0),
           (           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14963.0,      -188738.0,       -95245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15374.0,       -28650.0,       -10891.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        74650.0,       -59471.0,       -30078.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4528.0,          972.0,         2650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13774.0,         3195.0,         1966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28582.0,       105021.0,        45669.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8792.0,        39729.0,        16364.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       101141.0,      -251284.0,      -107721.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       441299.0,        96291.0,        27712.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        84049.0,       135619.0,        54902.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        73341.0,        42517.0,        21659.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -241053.0,       570178.0,       259329.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51342.0,       -97919.0,       -32302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -483998.0,       744993.0,       289281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -580173066.0,   -372162954.0,   -154881761.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -894369.0,      1022661.0,       396251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       652127.0,      -760605.0,      -291852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        61517.0,       -78502.0,       -30411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23423.0,         1817.0,          449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          122.0,       -11951.0,        -5114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1250.0,         3934.0,         1120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3499.0,        -5603.0,        -2325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3514.0,         2711.0,         1095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3641.0,        38834.0,        17962.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        40847.0,       -35023.0,       -14170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        76335.0,      -152459.0,       -74524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17894.0,        17375.0,         6306.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32752.0,       -24488.0,       -10474.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17198.0,        73423.0,        31207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        73193.0,       -34398.0,       -14371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -191039.0,       144562.0,        61763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1688957.0,     -1346136.0,      -581305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -45911528.0,     20288107.0,     10479866.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -148016.0,       103994.0,        46387.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       112606.0,       -88054.0,       -39219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -108357.0,        97799.0,        44018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -110220.0,       107612.0,        49406.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       174948.0,      -164752.0,       -75316.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9007.0,         9456.0,         4301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1458.0,           71.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4096.0,        -6580.0,        -3230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8440.0,         1318.0,          526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,         1317.0,          691.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17738.0,       -26173.0,       -11437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -230.0,        -2195.0,        -1126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          679.0,         -102.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9502.0,        -2982.0,        -1638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -900.0,         1000.0,          509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4891.0,        -4253.0,        -2292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -253854.0,      3703593.0,      1705287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7433.0,         6724.0,         3832.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2024.0,        -1536.0,         -917.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          131.0,          731.0,          385.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,          549.0,          269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          349.0,          550.0,          230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -177.0,          144.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          921.0,         -186.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -273.0,          234.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,        -2247.0,        -1021.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -23.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,         -134.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4853.0,          104.0,          164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,         -115.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -266.0,          179.0,           69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          246.0,         -374.0,         -151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,         -689.0,         -330.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          513.0,         -237.0,         -125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1113.0,         1417.0,          710.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       311258.0,       188607.0,        78207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2317.0,        -3761.0,        -2083.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -702.0,         1339.0,          772.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,         -454.0,         -298.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,          240.0,          147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,           67.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          111.0,          -97.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,         -167.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -36.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           46.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -348.0,         -147.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -16.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -300.0,          410.0,          194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -28.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -149.0,           61.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,         -190.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,          129.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          111.0,           22.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35095.0,       -18431.0,        -8268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3517.0,         3933.0,          661.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,         -256.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            2.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,            0.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           55.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            5.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -31.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           41.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -55.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           72.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           41.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -51.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,           45.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           19.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,          -54.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           14.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,            8.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,        -3278.0,        -1504.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -32.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           17.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           51.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           16.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           13.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           29.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,            2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          613.0,         -103.0,         -120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           14.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -17.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -322.0,         -174.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           16.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           29.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -17.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           63.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -52.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -32.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           48.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -57.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        33499.0,       162122.0,        74553.0),
           (      1708618.0,     -9883333.0,     -5387629.0),
           (      1118156.0,      -412385.0,      -111662.0)),
          ((      1010980.0,       849789.0,       360691.0),
           (      -289987.0,      -527492.0,      -448298.0),
           (            0.0,            0.0,            0.0)),
          ((       304833.0,       136942.0,        51263.0),
           (       354460.0,      -146953.0,      -112576.0),
           (            0.0,            0.0,            0.0)),
          ((       154322.0,      -130942.0,       -60647.0),
           (     -2624316.0,      1583527.0,       956775.0),
           (            0.0,            0.0,            0.0)),
          ((       133154.0,        68483.0,        23462.0),
           (     -1241810.0,      1163709.0,       752928.0),
           (            0.0,            0.0,            0.0)),
          ((       200698.0,     -1089604.0,      -495138.0),
           (        -4767.0,         9938.0,         5585.0),
           (            0.0,            0.0,            0.0)),
          ((        50319.0,       -22952.0,       -14990.0),
           (        67030.0,      -127447.0,       -67745.0),
           (            0.0,            0.0,            0.0)),
          ((       -66944.0,       -77279.0,       -30079.0),
           (      1215805.0,     -2339718.0,     -1159038.0),
           (            0.0,            0.0,            0.0)),
          ((       237920.0,      -159174.0,       -90717.0),
           (      -241720.0,       526197.0,       248970.0),
           (            0.0,            0.0,            0.0)),
          ((     -2466813.0,      -463988.0,       -41813.0),
           (         2795.0,       -11096.0,        -5118.0),
           (            0.0,            0.0,            0.0)),
          ((      3960981.0,       830226.0,        93267.0),
           (        -2706.0,         6544.0,         2581.0),
           (            0.0,            0.0,            0.0)),
          ((     -2182250.0,      -495324.0,       -61902.0),
           (         9583.0,       -17357.0,        -8965.0),
           (            0.0,            0.0,            0.0)),
          ((       718649.0,       140085.0,         2554.0),
           (         9238.0,        -8309.0,        -4543.0),
           (            0.0,            0.0,            0.0)),
          ((     -1818090.0,     -1223657.0,      -478443.0),
           (         -103.0,          145.0,           30.0),
           (            0.0,            0.0,            0.0)),
          ((     -1756602.0,      -640439.0,      -151652.0),
           (         2985.0,        -5084.0,         -352.0),
           (            0.0,            0.0,            0.0)),
          ((      2897450.0,      1394083.0,       403456.0),
           (          -24.0,           95.0,           56.0),
           (            0.0,            0.0,            0.0)),
          ((      -442313.0,      -142647.0,       -25900.0),
           (            4.0,          -28.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((       461285.0,       682487.0,       258264.0),
           (          -70.0,           -8.0,          -11.0),
           (            0.0,            0.0,            0.0)),
          ((      -628615.0,      -218618.0,       -21199.0),
           (            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1522245.0,       718474.0,       143912.0),
           (            1.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((  14076660795.0,   5271715189.0,   2037500808.0),
           (           91.0,           34.0,          -10.0),
           (            0.0,            0.0,            0.0)),
          ((      -300746.0,        84309.0,        55784.0),
           (          -90.0,          -65.0,          -26.0),
           (            0.0,            0.0,            0.0)),
          ((      -653878.0,       380176.0,       169753.0),
           (            0.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((       183332.0,       -67175.0,       -24259.0),
           (            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       150183.0,       118334.0,        51130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19779.0,        26433.0,        11926.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        78429.0,       -58625.0,       -28651.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4031.0,        -9557.0,        -2843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2027.0,         5403.0,         2305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        40722.0,      -134487.0,       -59143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1899.0,       -32030.0,       -13156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        53353.0,        68752.0,        21201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       232130.0,      -443300.0,      -204581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -63189.0,      -123540.0,       -51969.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -114103.0,       -42665.0,       -18894.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -559421.0,          467.0,        17728.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -146759.0,       106292.0,        45885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       271462.0,      -142711.0,       -42702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    267115041.0,    597184364.0,    266655530.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       496405.0,       -61810.0,        -1010.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -436716.0,        46909.0,         1548.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -50785.0,       -11133.0,        -6600.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5255.0,        14569.0,         6982.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2290.0,         6700.0,         2820.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14783.0,        -8069.0,        -3566.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4966.0,        -1777.0,         -444.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3811.0,          479.0,          217.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34894.0,        -5056.0,        -3151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -45785.0,        -7050.0,        -4285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       251939.0,       -37242.0,       -22189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25203.0,        -3625.0,        -1895.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25820.0,        -8643.0,        -4192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14972.0,       -40650.0,       -17295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -81645.0,       -30561.0,       -13092.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       194405.0,        63284.0,        27500.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1748826.0,      -541144.0,      -231163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -18288634.0,     41417495.0,     19517513.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       141227.0,        27682.0,        11164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -119357.0,       -36772.0,       -15159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       114559.0,        30424.0,        12141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       130137.0,        37630.0,        14732.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -201247.0,       -59410.0,       -23444.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12217.0,         5086.0,         2014.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1810.0,         1589.0,          764.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5246.0,         -150.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17567.0,         -357.0,         -546.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          564.0,         -364.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29659.0,        17685.0,         7363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1399.0,         -295.0,         -174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          936.0,        -1048.0,         -561.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          320.0,        -8772.0,        -4001.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          247.0,         -649.0,         -338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2566.0,         -602.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -3834922.0,      1268602.0,       685605.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3406.0,         -698.0,         -575.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -720.0,            0.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          220.0,         -958.0,         -456.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,         -331.0,         -148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          229.0,           -8.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          -65.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          831.0,          -78.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          -46.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           74.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1998.0,         1205.0,          585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          105.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          180.0,         -437.0,         -177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          645.0,         4490.0,         2029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,         -444.0,         -189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -202.0,          605.0,          241.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          565.0,         -888.0,         -379.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          596.0,           30.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          477.0,         -962.0,         -384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1214.0,         4339.0,         1661.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -337658.0,      -145184.0,       -57752.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2525.0,        -9094.0,        -3384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -949.0,         3343.0,         1271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,         -886.0,         -358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          220.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,         -210.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,          121.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          151.0,           66.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,           63.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           10.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,          330.0,          154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,           51.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,          -29.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -395.0,          409.0,          203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -38.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,          -21.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,         -138.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,          138.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,         -223.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12746.0,       -38687.0,       -17016.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1201.0,         8283.0,         3342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,         -529.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          148.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,          -59.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -20.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            8.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           15.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           33.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,          -81.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           99.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           59.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -76.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -130.0,           67.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           20.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -61.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -14.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -13.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1661.0,        -2918.0,        -1381.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -15.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            6.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           11.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -88.0,           56.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,           -8.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           -4.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -15.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           10.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           39.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -13.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,          544.0,          305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           26.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           -7.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          369.0,          -97.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -18.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -12.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           69.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -57.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -53.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           43.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           47.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        28258.0,      -289091.0,      -136776.0),
           (    -20886110.0,    -12156718.0,     -5344203.0),
           (       125080.0,     -2566096.0,     -1136991.0)),
          ((     -1287543.0,      -149686.0,       -33006.0),
           (     -1758669.0,      -281357.0,        -1591.0),
           (            0.0,            0.0,            0.0)),
          ((      -296823.0,        83207.0,        40623.0),
           (       -89875.0,      -308239.0,      -135366.0),
           (            0.0,            0.0,            0.0)),
          ((       689279.0,       134962.0,        52972.0),
           (      4668436.0,      1384722.0,       496442.0),
           (            0.0,            0.0,            0.0)),
          ((      -460867.0,       -86827.0,       -39731.0),
           (      3276734.0,      1423688.0,       551833.0),
           (            0.0,            0.0,            0.0)),
          ((      -902334.0,      -219230.0,       -65724.0),
           (        18020.0,         4436.0,         1574.0),
           (            0.0,            0.0,            0.0)),
          ((      -139759.0,        35534.0,        14239.0),
           (      -189965.0,       -25104.0,        -5990.0),
           (            0.0,            0.0,            0.0)),
          ((       147363.0,        -6108.0,          984.0),
           (     -2459727.0,       134794.0,       103035.0),
           (            0.0,            0.0,            0.0)),
          ((      -140065.0,      -291436.0,      -159959.0),
           (       498790.0,      -123500.0,       -64730.0),
           (            0.0,            0.0,            0.0)),
          ((      1629786.0,      1835760.0,      1089689.0),
           (        -8431.0,         8442.0,         4203.0),
           (            0.0,            0.0,            0.0)),
          ((     -2996843.0,     -2870594.0,     -1728621.0),
           (         6203.0,        -7981.0,        -3575.0),
           (            0.0,            0.0,            0.0)),
          ((      1800102.0,      1521218.0,       933981.0),
           (         6897.0,         4499.0,         1943.0),
           (            0.0,            0.0,            0.0)),
          ((      -663674.0,      -388119.0,      -268598.0),
           (        -6861.0,          340.0,         -705.0),
           (            0.0,            0.0,            0.0)),
          ((      1926238.0,      1374502.0,       798048.0),
           (         -120.0,         -228.0,          -84.0),
           (            0.0,            0.0,            0.0)),
          ((      1666129.0,       737179.0,       612110.0),
           (         3921.0,         7403.0,         3163.0),
           (            0.0,            0.0,            0.0)),
          ((     -3073442.0,     -1722680.0,     -1239880.0),
           (          -66.0,          -56.0,          -11.0),
           (            0.0,            0.0,            0.0)),
          ((       663613.0,       254869.0,       213375.0),
           (           36.0,            7.0,           -9.0),
           (            0.0,            0.0,            0.0)),
          ((       158272.0,       -70831.0,      -134751.0),
           (          -25.0,          -39.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((       639023.0,        64628.0,       225439.0),
           (           -1.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((     -1144559.0,       840987.0,      -111461.0),
           (           -3.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -9456125779.0,  10693781146.0,   5160134372.0),
           (          -12.0,          -36.0,          -14.0),
           (            0.0,            0.0,            0.0)),
          ((       278247.0,      -175104.0,       -28906.0),
           (           26.0,          -28.0,          -17.0),
           (            0.0,            0.0,            0.0)),
          ((       398782.0,      1295936.0,       490746.0),
           (           -2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -71836.0,      -696623.0,      -249344.0),
           (            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((      -233727.0,      -113006.0,       -36652.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4051.0,        37015.0,        13225.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        81701.0,       -60899.0,       -28233.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3109.0,        -3115.0,        -1597.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4714.0,         3627.0,         1936.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43391.0,       147231.0,        64751.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7211.0,        42725.0,        18923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        54153.0,      -263759.0,      -121188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -263450.0,      -258366.0,      -115072.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        71598.0,       164758.0,        73601.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        58971.0,       -16991.0,        -9286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -219640.0,      -508430.0,      -224509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -188669.0,       228371.0,       107050.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       115877.0,      -811133.0,      -363376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    132485369.0,   -629112114.0,   -292145307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31405.0,     -1008127.0,      -444364.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9033.0,       742576.0,       327436.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16197.0,        77155.0,        34484.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15371.0,       -21083.0,        -9739.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1544.0,         5869.0,         2756.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14500.0,         7419.0,         3500.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          388.0,        11716.0,         5244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -619.0,        -4599.0,        -1975.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2815.0,       -37651.0,       -17557.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19988.0,        62426.0,        27796.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       167198.0,        76101.0,        30994.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15121.0,       -38470.0,       -16595.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          670.0,        35674.0,        15860.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        44246.0,         3881.0,         1121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        43511.0,        73138.0,        31111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -57600.0,      -248985.0,      -109299.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       478403.0,      2271318.0,      1000174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     13993214.0,     41156535.0,     18525763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24553.0,      -203354.0,       -90653.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26724.0,       158861.0,        70554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18867.0,      -156368.0,       -69705.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17856.0,      -170664.0,       -76626.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25765.0,       266905.0,       119737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          134.0,       -13485.0,        -6129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -635.0,        -3191.0,        -1525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1221.0,         7732.0,         3552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11255.0,        -3669.0,        -1902.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -122.0,         -758.0,         -392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -35502.0,        -5745.0,        -1712.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1529.0,         2159.0,          908.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,        -1169.0,         -583.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9775.0,        -1883.0,         -564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          200.0,         2393.0,         1070.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4311.0,        -3094.0,        -1131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2884005.0,     -2678254.0,     -1150564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7201.0,         4747.0,         1542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2182.0,        -1173.0,         -343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -163.0,          695.0,          276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,           29.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -356.0,         -824.0,         -356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          530.0,          227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          848.0,         -802.0,         -384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          434.0,          -59.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -73.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2202.0,          826.0,          349.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,         -162.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          219.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4343.0,        -1870.0,         -948.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          219.0,          659.0,          262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -346.0,         -936.0,         -360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          372.0,         1252.0,          507.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          863.0,        -1465.0,         -643.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         1205.0,          490.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          378.0,        -6403.0,        -2662.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       356140.0,       100620.0,        36731.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1021.0,        15531.0,         6564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          646.0,        -5870.0,        -2516.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -202.0,         1675.0,          711.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,         -789.0,         -327.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,           62.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,         -126.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -76.0,          113.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          124.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -17.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          211.0,         -242.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,         -102.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           80.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -465.0,          431.0,          218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          -97.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          185.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,         -155.0,         -118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,          245.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          183.0,         -363.0,         -237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -30580.0,        -1683.0,        -1890.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5359.0,         7731.0,         5805.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          244.0,         -517.0,         -378.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,          141.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -29.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          -21.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -25.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           14.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -62.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           27.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,         -101.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          113.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           60.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -77.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,          -34.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           28.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -67.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -14.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           36.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2879.0,        -2006.0,         -995.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,           10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -8.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -28.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -88.0,           49.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -29.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           11.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -23.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -538.0,         -284.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,            3.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,          311.0,          149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           10.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           23.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -18.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -49.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           13.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,          -15.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      -141754.0,       400391.0,       188365.0),
           (     13600618.0,     10158308.0,      4361814.0),
           (      3896715.0,       370465.0,      -100362.0)),
          ((       782469.0,      -590502.0,      -284659.0),
           (       144721.0,       768360.0,       483517.0),
           (            0.0,            0.0,            0.0)),
          ((       226265.0,      -283565.0,      -133637.0),
           (       539597.0,        66082.0,        30666.0),
           (            0.0,            0.0,            0.0)),
          ((       200927.0,       498266.0,       220359.0),
           (     -3628662.0,      -158189.0,       -59528.0),
           (            0.0,            0.0,            0.0)),
          ((      -140052.0,      -180744.0,       -76293.0),
           (     -2871139.0,       -58792.0,       -36602.0),
           (            0.0,            0.0,            0.0)),
          ((      -189856.0,      1072052.0,       487347.0),
           (       -15979.0,        -2219.0,        -1183.0),
           (            0.0,            0.0,            0.0)),
          ((        71898.0,       -86601.0,       -39681.0),
           (       170237.0,        45570.0,        20985.0),
           (            0.0,            0.0,            0.0)),
          ((        43257.0,        74701.0,        29920.0),
           (      2110511.0,       993081.0,       448676.0),
           (            0.0,            0.0,            0.0)),
          ((       185569.0,      -120873.0,       -47568.0),
           (      -354995.0,      -267119.0,      -118108.0),
           (            0.0,            0.0,            0.0)),
          ((       130112.0,      1599893.0,       612575.0),
           (         2752.0,         7774.0,         3396.0),
           (            0.0,            0.0,            0.0)),
          ((      -730023.0,     -2424233.0,      -883121.0),
           (        -3441.0,        -6755.0,        -3356.0),
           (            0.0,            0.0,            0.0)),
          ((       705432.0,      1251333.0,       428082.0),
           (       -23696.0,       -25191.0,       -10679.0),
           (            0.0,            0.0,            0.0)),
          ((      -427842.0,      -474587.0,      -152670.0),
           (        -7570.0,        -5401.0,         1162.0),
           (            0.0,            0.0,            0.0)),
          ((      -204277.0,       174474.0,       -34462.0),
           (           82.0,          -84.0,          -62.0),
           (            0.0,            0.0,            0.0)),
          ((      1245189.0,       651374.0,        68130.0),
           (         -441.0,         5291.0,         3260.0),
           (            0.0,            0.0,            0.0)),
          ((     -1502219.0,     -1193306.0,      -179503.0),
           (           15.0,         -115.0,          -48.0),
           (            0.0,            0.0,            0.0)),
          ((       425045.0,       218763.0,        18548.0),
           (          -29.0,           49.0,           24.0),
           (            0.0,            0.0,            0.0)),
          ((      -237857.0,      -754957.0,      -259582.0),
           (           80.0,          -51.0,          -28.0),
           (            0.0,            0.0,            0.0)),
          ((       365605.0,       696723.0,       142892.0),
           (           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -937224.0,     -1084785.0,       -44463.0),
           (            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -8673475957.0, -11381330252.0,  -4988053354.0),
           (          -11.0,          -30.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((      -209707.0,       114388.0,         6944.0),
           (          -30.0,           36.0,           11.0),
           (            0.0,            0.0,            0.0)),
          ((       985593.0,       149435.0,       131528.0),
           (            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((      -530848.0,      -141097.0,      -112298.0),
           (           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((        27640.0,      -115518.0,       -62044.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        94285.0,       -13524.0,        -5549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        33264.0,          826.0,        -1988.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14240.0,       -23391.0,       -10509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -985.0,         3885.0,         1143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34425.0,      -159551.0,       -70056.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15583.0,       -34300.0,       -15978.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20723.0,        64173.0,        31356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -440425.0,       -81609.0,       -28637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -84384.0,      -123281.0,       -56946.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22796.0,       -73078.0,       -31441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       465422.0,      -169296.0,       -92291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -543620.0,        -9829.0,        18086.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1428180.0,        58919.0,       -53419.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -491319626.0,    459129321.0,    223838945.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1071595.0,       239531.0,         8560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -823205.0,      -232407.0,       -28230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -69779.0,       -22953.0,        -3139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6353.0,        -5533.0,        -3148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6539.0,        -8132.0,        -3323.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10434.0,        -4552.0,        -2591.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3602.0,        -1467.0,         -578.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,         2773.0,          999.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -35524.0,         1558.0,         1671.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8159.0,       -29496.0,        -9115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27781.0,       222277.0,        98570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3333.0,        25392.0,        10251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5241.0,       -19585.0,        -6668.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -38695.0,       -24253.0,       -12204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4750.0,       -16320.0,        -3819.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23123.0,       125746.0,        47913.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       281211.0,     -1300410.0,      -518372.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     41498379.0,     25691475.0,     10596468.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21011.0,       116325.0,        47419.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        31266.0,      -101147.0,       -42779.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43669.0,       110356.0,        47816.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -68153.0,       133293.0,        60010.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        99234.0,      -198402.0,       -88667.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6091.0,        13103.0,         5963.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -745.0,         3097.0,         1520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7194.0,        -7989.0,        -3947.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4339.0,        13397.0,         6234.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1535.0,         1171.0,          680.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        31444.0,        -7095.0,        -4022.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4842.0,        -3890.0,        -1913.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1506.0,         1942.0,          941.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3931.0,         4549.0,         2153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3868.0,         -642.0,         -381.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2664.0,         -958.0,         -537.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1487006.0,     -3465243.0,     -1629610.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2058.0,         5995.0,         3194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1046.0,         -966.0,         -587.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,         1020.0,          488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          386.0,          185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,           14.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -421.0,          329.0,          174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1090.0,         -758.0,         -395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          182.0,         -147.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -33.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          592.0,        -2092.0,         -979.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,          -80.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -539.0,          124.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1798.0,        -3956.0,        -1777.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -330.0,          135.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          396.0,         -221.0,         -125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -154.0,          407.0,          192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1113.0,          150.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1105.0,         -414.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3823.0,         1691.0,          734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -366492.0,       -61471.0,       -18311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6716.0,        -2360.0,        -1031.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1982.0,          678.0,          323.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -352.0,          -35.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,         -374.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,         -300.0,         -134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,           99.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -130.0,          -89.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -125.0,          -46.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,           15.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -369.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           36.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           -4.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -471.0,          405.0,          203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            7.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,           28.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -80.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,            9.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -201.0,          -68.0,          -93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1441.0,        31840.0,        12000.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2542.0,         1087.0,         3308.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -224.0,         -150.0,         -272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -12.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,           43.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          112.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,            5.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           26.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           30.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           44.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -59.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           91.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           51.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -80.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           70.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           33.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -77.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -13.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3571.0,         -703.0,         -415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           26.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            1.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            3.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,           30.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -22.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           -7.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -43.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          494.0,         -334.0,         -233.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -13.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,         -315.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           -7.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -5.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           28.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -13.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            9.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           10.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,          -25.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // jup
    jupct   : array [1..1,1..134,0..2,1..3] of double = (
         (((     -1453032.0,       475744.0,       269545.0),
           (     81840040.0,    -76703160.0,    -34768142.0),
           (    275665824.0,    197679332.0,     78226373.0)),
          ((     -4361370.0,    -32394181.0,    -13990520.0),
           (     15264475.0,    -17225403.0,     -7748111.0),
           (        25843.0,        -3195.0,        -2131.0)),
          ((      4274244.0,     -2340440.0,     -1055522.0),
           (   -360038401.0,    431971458.0,    193912085.0),
           (          -29.0,          -42.0,          -14.0)),
          ((      -439394.0,      1264788.0,       485486.0),
           (    -48151558.0,     54300228.0,     24429284.0),
           (           99.0,          130.0,           48.0)),
          ((     -1143255.0,     -1071145.0,      -350957.0),
           (     84921189.0,   -111164917.0,    -49755361.0),
           (            0.0,            0.0,            0.0)),
          ((      6283559.0,      9968650.0,      3856532.0),
           (    -15288185.0,     20988477.0,      9380354.0),
           (            0.0,            0.0,            0.0)),
          ((      -268810.0,      -477678.0,      -120602.0),
           (     -2274727.0,      3584523.0,      1595047.0),
           (            0.0,            0.0,            0.0)),
          ((       360136.0,       392925.0,        90418.0),
           (     -1793144.0,      3103450.0,      1376808.0),
           (            0.0,            0.0,            0.0)),
          ((     -4221246.0,       -27059.0,       521768.0),
           (     -6424289.0,     12281838.0,      5439685.0),
           (            0.0,            0.0,            0.0)),
          ((      6926908.0,     -1896174.0,     -1723842.0),
           (       405366.0,      -866906.0,      -382860.0),
           (            0.0,            0.0,            0.0)),
          ((     -4418171.0,      1591242.0,      1298767.0),
           (        69128.0,      -228388.0,      -101167.0),
           (            0.0,            0.0,            0.0)),
          ((      2129283.0,     -1632802.0,     -1022779.0),
           (        -5313.0,        36802.0,        16198.0),
           (            0.0,            0.0,            0.0)),
          ((      5517256.0,    -18492713.0,     -8036594.0),
           (        -1083.0,         -581.0,         -221.0),
           (            0.0,            0.0,            0.0)),
          ((       564439.0,      1515845.0,       335718.0),
           (           45.0,          109.0,           46.0),
           (            0.0,            0.0,            0.0)),
          ((      2379185.0,      1932458.0,      6254854.0),
           (          -13.0,           33.0,           17.0),
           (            0.0,            0.0,            0.0)),
          ((     -6181574.0,     -2393520.0,    -10282950.0),
           (           -3.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0)),
          ((  42219547864.0,  28233408468.0,  11077606257.0),
           (           40.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0)),
          ((      -734912.0,       508381.0,       473595.0),
           (           22.0,          -11.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((      1153118.0,     -2570155.0,     -1411459.0),
           (          354.0,           18.0,          -10.0),
           (            0.0,            0.0,            0.0)),
          ((     11715030.0,     -5991289.0,     -2498584.0),
           (            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((     -2709559.0,         7824.0,      -108865.0),
           (          -10.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((       605781.0,       -60786.0,        -9755.0),
           (            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((       613847.0,      -541488.0,      -229828.0),
           (          127.0,           33.0,           13.0),
           (            0.0,            0.0,            0.0)),
          ((       941227.0,      3468918.0,      1533913.0),
           (            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((     -2989015.0,     -7698969.0,     -3389893.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -11328560.0,     -8985668.0,     -3631980.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      7241413.0,     11548968.0,      5013819.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -911498.0,     -1026210.0,      -432715.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       303476.0,     -4592377.0,     -1962941.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -740399.0,      -920542.0,      -389627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3676593.0,      3255827.0,      1328783.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    693978080.0,    974101361.0,    400437290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2585903.0,     -4448289.0,     -1552402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -3048326.0,     -2960225.0,     -1323241.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1834492.0,      1910595.0,       810688.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      8503080.0,     -1507170.0,      -867948.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2684498.0,      1938215.0,       789212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1667805.0,     -1313286.0,      -535190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       441205.0,       690462.0,       292118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1732146.0,      1352193.0,       545925.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -9509924.0,     -7317136.0,     -2939908.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     12153241.0,      9292429.0,      3732365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -9060171.0,     -6846637.0,     -2740391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     44260055.0,     33016065.0,     13172632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -698020.0,     -1848894.0,      -794417.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -63628650.0,    -46647106.0,    -18540018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     59719202.0,     43354381.0,     17197511.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -51390929.0,     -2901941.0,       -39170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     42397454.0,     30000118.0,     11846316.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -31845167.0,    -22323867.0,     -8803318.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     12107588.0,      8300901.0,      3266723.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      6361282.0,      4369683.0,      1719574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      7933660.0,      5244919.0,      2062950.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -8589592.0,     -5746786.0,     -2260893.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        59711.0,        37459.0,        15075.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -55288.0,       -46189.0,       -17687.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       268987.0,       180014.0,        69198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       394869.0,       193726.0,        72332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -823699.0,      -535512.0,      -206066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       454251.0,       289286.0,       111191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -75580.0,      1891754.0,       812402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -895048.0,      -557921.0,      -214096.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3190795.0,      1952647.0,       748066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1646422.0,     -1004638.0,      -384098.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1193312.0,      -713680.0,      -272812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       176941.0,       104762.0,        39987.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        40860.0,        18752.0,         7013.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3594.0,        -2224.0,         -846.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9740.0,        -3883.0,        -1855.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6858.0,        -2310.0,         -813.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9421.0,         2949.0,         1016.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -193051.0,       -62883.0,       -21011.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       190299.0,        60313.0,        20187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11740.0,          803.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -50607.0,        74085.0,        33054.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          909.0,        -1042.0,         -475.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2399.0,         1616.0,          619.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          241.0,          598.0,          248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -365.0,         -762.0,         -306.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1508.0,           11.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1811.0,            9.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -94.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          278.0,          476.0,          187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          589.0,         -927.0,         -429.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,          -94.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          358.0,         -100.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3246.0,         1767.0,          862.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          761.0,          315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,         -292.0,         -122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1400.0,         1203.0,          486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -905.0,        -1107.0,         -453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          367.0,          421.0,          175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4882.0,          141.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          206.0,         -135.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           69.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           87.0,         -178.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          161.0,         -127.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           95.0,           20.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          281.0,           68.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,          125.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,         -198.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          120.0,           63.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -217.0,            8.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           13.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          143.0,         -110.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,           31.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,          -24.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,          -49.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           23.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -20.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -32.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           57.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -31.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13268.0,        25079.0,        10877.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,           95.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -20.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          204.0,          115.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           13.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15774.0,        -7647.0,        -2442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -159.0,          183.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -59.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          327.0,         -513.0,         -297.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,          131.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          -19.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // sat
    satct   : array [1..1,1..117,0..2,1..3] of double = (
         (((     11911616.0,    -14362564.0,     -5250529.0),
           (  -9646696640.0,   7343129305.0,   3450251321.0),
           (      -207865.0,      -391916.0,      -126958.0)),
          ((   1309791891.0,   1616226274.0,    608531216.0),
           (  -1475289129.0,   1316383220.0,    607363122.0),
           (        -1977.0,          559.0,          334.0)),
          ((  -3141191721.0,  -3686077866.0,  -1384381695.0),
           (   -442692291.0,    740764985.0,    324291801.0),
           (            0.0,            0.0,            0.0)),
          ((   2022029716.0,   2483669790.0,    940147188.0),
           (    240544053.0,   -142655332.0,    -66702006.0),
           (            0.0,            0.0,            0.0)),
          ((   -514537558.0,   -586186351.0,   -221667784.0),
           (      9519069.0,     -2888981.0,     -1654463.0),
           (            0.0,            0.0,            0.0)),
          ((   3239319069.0,   3578550118.0,   1350807359.0),
           (      4233477.0,     -2078724.0,     -1067911.0),
           (            0.0,            0.0,            0.0)),
          ((  58643570058.0,  66848246686.0,  25078019831.0),
           (      -425592.0,       226091.0,       110289.0),
           (            0.0,            0.0,            0.0)),
          ((  -1798127387.0,  -2004952210.0,   -754874233.0),
           (         -284.0,         2407.0,         1265.0),
           (            0.0,            0.0,            0.0)),
          ((   1740502931.0,   1976793589.0,    742403067.0),
           (      -144129.0,        59568.0,        30909.0),
           (            0.0,            0.0,            0.0)),
          ((  -1904907736.0,  -2267906871.0,   -850634308.0),
           (       128188.0,       -59701.0,       -15736.0),
           (            0.0,            0.0,            0.0)),
          ((    894888374.0,   1177294792.0,    444951655.0),
           (          -74.0,          175.0,         -145.0),
           (            0.0,            0.0,            0.0)),
          ((   -243632711.0,   -205351638.0,    -75787779.0),
           (         1224.0,        -2891.0,         -815.0),
           (            0.0,            0.0,            0.0)),
          ((    494173928.0,    417507046.0,    152512685.0),
           (        -1185.0,         -179.0,          361.0),
           (            0.0,            0.0,            0.0)),
          ((   1572841559.0,   -731853004.0,   -370964196.0),
           (         -905.0,          235.0,          141.0),
           (            0.0,            0.0,            0.0)),
          ((   2173376891.0,   2504426826.0,    939344214.0),
           (         1341.0,        -1967.0,         -957.0),
           (            0.0,            0.0,            0.0)),
          ((   -748027689.0,   -874454798.0,   -329937853.0),
           (          172.0,         -144.0,          -93.0),
           (            0.0,            0.0,            0.0)),
          ((   -851987266.0,   -829650517.0,   -302913976.0),
           (           37.0,           90.0,           18.0),
           (            0.0,            0.0,            0.0)),
          ((    135573983.0,    119353005.0,     43714237.0),
           (           67.0,          -78.0,          -48.0),
           (            0.0,            0.0,            0.0)),
          ((    -70305600.0,    -58132867.0,    -20560775.0),
           (           25.0,          -24.0,          -17.0),
           (            0.0,            0.0,            0.0)),
          ((     80822370.0,     59493046.0,     20451163.0),
           (         -292.0,          247.0,          107.0),
           (            0.0,            0.0,            0.0)),
          ((    535985134.0,    163170205.0,     39948973.0),
           (            1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -510402194.0,   -253093295.0,    -77650625.0),
           (            5.0,           -2.0,           -5.0),
           (            0.0,            0.0,            0.0)),
          ((     17056423.0,      3198279.0,       429379.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2304555.0,      3077936.0,      1364898.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        54525.0,       781462.0,       308094.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3653182.0,      2918204.0,       842066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -3817371.0,    -11178658.0,     -4306469.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2275681.0,      6155879.0,      2326631.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -978848.0,     -2749459.0,     -1035540.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       422781.0,       538148.0,       177553.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -247675.0,      -419388.0,      -144700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       885481.0,      1581936.0,       541628.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -542429.0,       269309.0,       214928.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -98109.0,      -175956.0,       -58441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       251832.0,       257900.0,        52721.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3036.0,       -16308.0,        -5205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       122653.0,        52053.0,         7925.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27461.0,        -8068.0,        -1208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5964.0,         2075.0,          252.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -174920.0,       239506.0,       105866.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2226.0,         3499.0,          502.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7192.0,        -3743.0,          968.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        75413.0,        89363.0,        21858.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7291.0,        -8015.0,        -5965.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -610.0,         6090.0,         2989.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        62645.0,        73933.0,        24504.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1645.0,        -2605.0,         -958.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -79581.0,        40111.0,        23740.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1982.0,        37593.0,        10989.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -645.0,        -2737.0,         -556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15875.0,        38452.0,        12208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8733.0,        -4610.0,        -3214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5541.0,         5115.0,         2332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11214.0,       -10761.0,        -4327.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12764.0,        33128.0,        10496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4379.0,        -3444.0,        -1372.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12768.0,         9652.0,         3668.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -78790.0,        45369.0,        19676.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        53374.0,       -32244.0,       -12792.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1989.0,          744.0,          363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3232.0,        15193.0,         5613.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1515.0,          868.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          249.0,         -140.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8230.0,        13782.0,         4350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,          232.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16168.0,         5199.0,         2578.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -251.0,           75.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1204.0,         6512.0,         2665.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1013.0,         -494.0,         -262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -459.0,          268.0,          117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1093.0,         -464.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1351.0,         7719.0,         2595.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,           20.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -692.0,          309.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6375.0,          445.0,          435.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1861.0,         2787.0,         1250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          714.0,         -232.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,          122.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1128.0,         -130.0,          106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1351.0,         3887.0,         1190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          862.0,         -123.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -288.0,           63.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3105.0,         -326.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          425.0,          -49.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,           20.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1052.0,         1084.0,          589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,           88.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -469.0,         1677.0,          641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -15.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1146.0,         -413.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,           32.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4650.0,          378.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -512.0,          366.0,          226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -11.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           75.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -432.0,          698.0,          301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           26.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -431.0,         -270.0,         -113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -408.0,          160.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           52.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -346.0,          259.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,         -156.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,           55.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -224.0,          108.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -77.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -113.0,           10.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,           28.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -37.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,            1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13172.0,        25126.0,        10895.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          195.0,          111.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15778.0,        -7649.0,        -2443.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -159.0,          182.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -53.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          453.0,         -364.0,         -241.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,          -14.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // ura
    uract   : array [1..1,1..87,0..2,1..3] of double = (
         ((( 131840869162.0,-126849988516.0, -57421030082.0),
           (  -2879880321.0,  -2353279416.0,   -989940473.0),
           (          138.0,          -92.0,          -48.0)),
          ((    -33215732.0,    -67712086.0,    -33442537.0),
           (   1871713824.0,   1498241997.0,    629835819.0),
           (            0.0,            0.0,            0.0)),
          ((   -361331752.0,    572314832.0,    260579399.0),
           (    -12793668.0,    -10417253.0,     -4381549.0),
           (            0.0,            0.0,            0.0)),
          ((    662100031.0,   2361617853.0,   1020319175.0),
           (      3424112.0,      2684458.0,      1129007.0),
           (            0.0,            0.0,            0.0)),
          ((  -1510822580.0,   2340934014.0,   1049191237.0),
           (        35745.0,        27516.0,        11604.0),
           (            0.0,            0.0,            0.0)),
          ((   1118955412.0,  -1647782696.0,   -737021973.0),
           (        21000.0,        13152.0,         5352.0),
           (            0.0,            0.0,            0.0)),
          ((   -921268557.0,   1296732972.0,    578020192.0),
           (         2233.0,         2812.0,         1187.0),
           (            0.0,            0.0,            0.0)),
          ((   1119542351.0,  -1895239410.0,   -841402384.0),
           (         -621.0,         -653.0,         -275.0),
           (            0.0,            0.0,            0.0)),
          ((   -912906734.0,   1451354990.0,    644254501.0),
           (        -2000.0,        -3652.0,        -1558.0),
           (            0.0,            0.0,            0.0)),
          ((    238656911.0,   -365737728.0,   -162121937.0),
           (         -213.0,         -128.0,          -54.0),
           (            0.0,            0.0,            0.0)),
          ((    -89829442.0,    162993338.0,     71690305.0),
           (          212.0,          169.0,           70.0),
           (            0.0,            0.0,            0.0)),
          ((    122564697.0,   -216847409.0,    -95198544.0),
           (         1241.0,          970.0,          409.0),
           (            0.0,            0.0,            0.0)),
          ((      7881865.0,    -22589473.0,     -9530556.0),
           (          224.0,          183.0,           71.0),
           (            0.0,            0.0,            0.0)),
          ((    -64499873.0,    124410817.0,     54104777.0),
           (         2799.0,         2251.0,          954.0),
           (            0.0,            0.0,            0.0)),
          ((     16777729.0,    -30273452.0,    -13098880.0),
           (         -191.0,         -152.0,          -64.0),
           (            0.0,            0.0,            0.0)),
          ((    -19971059.0,     36997235.0,     15942628.0),
           (         -346.0,         -236.0,         -102.0),
           (            0.0,            0.0,            0.0)),
          ((     62627452.0,     -2719131.0,     -3453500.0),
           (           47.0,           35.0,           15.0),
           (            0.0,            0.0,            0.0)),
          ((    -85654208.0,    156645033.0,     68729343.0),
           (         -208.0,         -185.0,          -78.0),
           (            0.0,            0.0,            0.0)),
          ((     80844225.0,   -163927534.0,    -70455721.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -16637125.0,     36861679.0,     15632513.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      5313475.0,    -12557741.0,     -5282254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1245160.0,      2996448.0,      1251042.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       180566.0,      -456938.0,      -187416.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22336.0,        69941.0,        28422.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17302.0,        20445.0,         7744.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        37598.0,       -44004.0,       -17585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -114161.0,        72939.0,        32831.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -100702.0,        41595.0,        20465.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26699.0,        -9700.0,        -5149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25942.0,         6663.0,         3662.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34155.0,       -17270.0,        -8595.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -44755.0,        42263.0,        20121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        31663.0,       -20638.0,       -10027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       844217.0,       798503.0,       320612.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -206747.0,       138242.0,        65247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7038.0,        -5752.0,        -2700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7149.0,         5421.0,         2158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2839.0,        -3547.0,        -1490.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12585.0,        -2760.0,        -1043.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2489.0,        -1617.0,         -716.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9853.0,        22227.0,         9616.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,         1209.0,          511.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2592.0,         4561.0,         2155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3308.0,         -796.0,         -533.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          595.0,         -545.0,         -302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          167.0,           13.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          461.0,          184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9024.0,        40349.0,        16865.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          336.0,        -2142.0,         -700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -268.0,          324.0,          121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -703.0,         1857.0,          824.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1057.0,            2.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3379.0,        -1006.0,         -463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          920.0,          929.0,          391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,         -491.0,         -236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -140.0,          462.0,          227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          194.0,         -336.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,         -151.0,          -79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1359.0,         3198.0,         1455.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1592.0,        -1373.0,         -676.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -134.0,          254.0,          117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,         -708.0,         -311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          244.0,         -221.0,         -105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          -47.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,           74.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -83.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,           38.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,          -48.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,           33.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4921.0,          307.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13233.0,        25145.0,        10903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           54.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          -46.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          116.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15797.0,        -7621.0,        -2429.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -32.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -150.0,          178.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -63.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          426.0,         -367.0,         -242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,           -9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // nep
    nepct   : array [1..1,1..80,0..2,1..3] of double = (
         ((( -98527270512.0,-259450683536.0,-103740035118.0),
           (   8185539297.0,  -2064389350.0,  -1048074430.0),
           (      8378731.0,      9215140.0,      3277744.0)),
          ((     29762712.0,   1010626822.0,    411276417.0),
           (   -979028035.0,    486760041.0,    223767626.0),
           (           15.0,         -128.0,          -38.0)),
          ((  -2708362026.0,  -6406940067.0,  -2554004284.0),
           (    945867650.0,    -88495583.0,    -59415173.0),
           (           33.0,          -11.0,           -4.0)),
          ((   4318845529.0,  11248944083.0,   4494954907.0),
           (     -6925901.0,      1277515.0,       715578.0),
           (          -30.0,          -14.0,           -2.0)),
          ((  -1370559913.0,  -3444413872.0,  -1374407310.0),
           (       -20154.0,       -11198.0,        -4307.0),
           (            0.0,            0.0,            0.0)),
          ((   -122136377.0,   -275808200.0,   -109322979.0),
           (         -336.0,         -363.0,         -134.0),
           (            0.0,            0.0,            0.0)),
          ((   1068524420.0,   2314228555.0,    912250547.0),
           (        -1053.0,         -401.0,         -135.0),
           (            0.0,            0.0,            0.0)),
          ((   -916339971.0,  -1965682768.0,   -774665063.0),
           (         4338.0,            3.0,           10.0),
           (            0.0,            0.0,            0.0)),
          ((   -182580266.0,   -358370212.0,   -140158241.0),
           (          168.0,          104.0,           38.0),
           (            0.0,            0.0,            0.0)),
          ((     70317289.0,    133214343.0,     51914439.0),
           (          262.0,          135.0,           48.0),
           (            0.0,            0.0,            0.0)),
          ((    -16767225.0,    -29679890.0,    -11489104.0),
           (         -129.0,          -11.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((     29293357.0,     49138281.0,     18865385.0),
           (           26.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((    -29856945.0,    -49191352.0,    -18848993.0),
           (           12.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0)),
          ((      5953769.0,      9361184.0,      3569088.0),
           (           14.0,          -11.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((      -988344.0,     -1464827.0,      -549606.0),
           (          -12.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((      1332122.0,      3058199.0,      1183669.0),
           (           36.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((     -1394659.0,     -2040622.0,      -764016.0),
           (          -10.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((       554038.0,       606007.0,       218195.0),
           (           -8.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((     47239800.0,     -2501738.0,     -2092558.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2234487.0,      2341672.0,       821339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2260866.0,     -2132134.0,      -726914.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3461962.0,      3184525.0,      1076614.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -11120712.0,     -9689809.0,     -3211414.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     15461236.0,     13360521.0,      4409078.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -6192544.0,     -5255516.0,     -1722447.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       309562.0,       257337.0,        83060.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22567.0,       -19114.0,        -6058.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7386.0,         5379.0,         1770.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2255.0,         -808.0,         -111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          -59.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          307.0,          275.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          122.0,         5132.0,         2156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1222.0,           37.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26707.0,        12154.0,         4292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3398.0,          527.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5397.0,         -109.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13096.0,         8083.0,         2452.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31630.0,       -11862.0,        -3471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1175083.0,      -283253.0,      -150571.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1019.0,          280.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9339.0,         5891.0,         2779.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,         -207.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1313.0,           83.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1579.0,          462.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           16.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          426.0,          285.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          717.0,        -1960.0,         -829.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -138.0,         -261.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1320.0,          715.0,          250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -332.0,         -296.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        40469.0,       -19950.0,        -9483.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4311.0,         -684.0,         -256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1258.0,          264.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          108.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -654.0,          596.0,          272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          275.0,         -222.0,         -101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,          144.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -49.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -219.0,           19.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -89.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           -3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,          -22.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1247.0,        -1209.0,         -553.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -45.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          101.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           17.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,          -29.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,          -54.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          323.0,        -4432.0,        -2041.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24803.0,        16111.0,         6985.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,            8.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,          -11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -185.0,         -109.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16117.0,         6999.0,         2128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -248.0,          -26.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -62.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -516.0,          282.0,          204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,          -31.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // plu
    pluct   : array [1..1,1..79,0..2,1..3] of double = (
         ((( -16182909423.0, 298776956905.0,  98093717079.0),
           ( 102659273615.0,  26552608666.0, -22519893658.0),
           (   -289665538.0, -32558215719.0, -10058579663.0)),
          ((  -6100246460.0,  76591022463.0,  25730019069.0),
           (  -9394687472.0, -12695473663.0,  -1152386735.0),
           (     47585289.0, -12787759816.0,  -4018947101.0)),
          ((  23563025280.0, -35664693697.0, -18219331401.0),
           (   1411902358.0,  -1105358623.0,   -807816466.0),
           (      2974833.0,    -31284958.0,    -10129785.0)),
          ((  10751048838.0,  15791151685.0,   1668170789.0),
           (    471963107.0,      5229156.0,   -135265706.0),
           (         2336.0,        -1625.0,        -2911.0)),
          ((  -4552113061.0, -17461207781.0,  -4058669010.0),
           (   -611915884.0,    -66727676.0,    143117182.0),
           (         -934.0,          829.0,          812.0)),
          ((  -3582687208.0,   3122583126.0,   2039451793.0),
           (   1565347971.0,    383056304.0,   -169681509.0),
           (         -424.0,         1380.0,          495.0)),
          ((   1112219973.0,    230777793.0,   -259738603.0),
           (  -1611540868.0,   -316541749.0,    166214347.0),
           (            0.0,            0.0,            0.0)),
          ((   -706554345.0,   4931492067.0,   1746261575.0),
           (     -2798270.0,    -76941060.0,    -47393199.0),
           (            0.0,            0.0,            0.0)),
          ((    747589502.0,  -4923568168.0,  -1758263138.0),
           (   -733276480.0,   -378484435.0,   -113154209.0),
           (            0.0,            0.0,            0.0)),
          ((   -754465900.0,   5034654721.0,   1801774466.0),
           (    -16360240.0,     -5589953.0,     -1300299.0),
           (            0.0,            0.0,            0.0)),
          ((   1105801536.0,  -8156125400.0,  -2870224777.0),
           (      -685327.0,      -211410.0,      -105718.0),
           (            0.0,            0.0,            0.0)),
          ((   -195698460.0,   1456180908.0,    501746197.0),
           (    109739814.0,     35446430.0,     15245730.0),
           (            0.0,            0.0,            0.0)),
          ((   -391956340.0,   3114219655.0,   1097463752.0),
           (   -222499244.0,    -71882513.0,    -31591034.0),
           (            0.0,            0.0,            0.0)),
          ((    462217137.0,  -3467899164.0,  -1221726669.0),
           (       110233.0,        18489.0,        11075.0),
           (            0.0,            0.0,            0.0)),
          ((  -1176717340.0,   8780212371.0,   3073204957.0),
           (        -5517.0,        -4555.0,         -326.0),
           (            0.0,            0.0,            0.0)),
          ((    707781483.0,  -5319633941.0,  -1862848942.0),
           (       -26407.0,         9590.0,         7246.0),
           (            0.0,            0.0,            0.0)),
          ((     74175141.0,   -584492952.0,   -199772986.0),
           (         1421.0,          378.0,          -32.0),
           (            0.0,            0.0,            0.0)),
          ((   -110356092.0,    826373009.0,    290676938.0),
           (          130.0,            2.0,           12.0),
           (            0.0,            0.0,            0.0)),
          ((   -908004923.0,   6919736762.0,   2418505650.0),
           (         -373.0,          -77.0,          488.0),
           (            0.0,            0.0,            0.0)),
          ((   1021013597.0,  -7754803071.0,  -2714004881.0),
           (          121.0,          105.0,           10.0),
           (            0.0,            0.0,            0.0)),
          ((    -18400032.0,    144867321.0,     50090663.0),
           (         -112.0,           43.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((      5025980.0,    -39415728.0,    -13618712.0),
           (           60.0,          -36.0,           -5.0),
           (            0.0,            0.0,            0.0)),
          ((      1144429.0,     -9783336.0,     -3324298.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -343823.0,      3070133.0,      1041682.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       321804.0,     -3038032.0,     -1009420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -305880.0,      5184676.0,      1725125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       414864.0,     -4854183.0,     -1540301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -349754.0,      4352897.0,      1367829.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       536355.0,     -7273142.0,     -2254010.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     44920368.0,   -764334934.0,   -229973686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -163504424.0,   2882260450.0,    863840780.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     93451631.0,  -2141689994.0,   -645578961.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       589146.0,    -12641213.0,     -3701527.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -60044.0,      1584908.0,       453391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7060.0,      -256898.0,       -71992.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2411.0,        86079.0,        24554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9645.0,      -326915.0,       -90843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9421.0,       252373.0,        64711.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -841.0,         7810.0,         2336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -621.0,        -4122.0,         -756.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          720.0,         1961.0,          229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -516.0,         2834.0,          818.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -478.0,        -1024.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,         -480.0,         -137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7498.0,         4119.0,         1029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3088.0,         -947.0,          875.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3087.0,          532.0,         -815.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2830.0,         3050.0,         2032.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -425.0,         -714.0,         -294.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2649.0,        -9006.0,        -1321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          641.0,         3151.0,        -1975.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3268.0,          793.0,         -193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6062.0,         1036.0,          797.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1391.0,         -366.0,         1231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1374.0,         2218.0,         1213.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10321.0,         4868.0,       -13790.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -349775.0,      1046215.0,       462309.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14804.0,         5750.0,        15914.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6982.0,        -4132.0,        -7521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2424.0,         1914.0,         2539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11753.0,          122.0,         1049.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1596.0,        -1780.0,        -1826.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          319.0,          652.0,          476.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          266.0,        -1561.0,         -782.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -357.0,          763.0,          477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          628.0,         -491.0,         -414.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -253.0,          584.0,          306.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -231.0,         -825.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -421.0,         1276.0,          316.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          121.0,           -8.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        39184.0,       -17923.0,        -8735.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          694.0,          265.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1829.0,         -451.0,         -133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2698.0,        -3993.0,        -1950.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12159.0,        25748.0,        11203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,         -212.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17833.0,          744.0,         -628.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          264.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -399.0,         -445.0,         -205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
implementation

end.
