unit sincoeff;

    //
    // Sine.pas
    //
    // Ephemerides of planets between 1900 and 2100 Chapront J., Francou G.
    // Bureau des Longitudes, Group : Dynamics of Solar System (1996)
    //
    // Tables and coefficients from the series96.xxx files converted to
    // pascal structures. - J. Burton
    //

interface

const

    //
    // Sine Coefficients of Poisson Series
    //

    // mer
    merst   : array [1..20,1..224,0..2,1..3] of double = (
         (((        -8792.0,       -47610.0,       -23713.0),
           (        -1481.0,         5081.0,         2664.0),
           (       -16206.0,       -52242.0,       -25095.0)),
          ((         2076.0,         2228.0,          992.0),
           (        -3672.0,        12085.0,         4093.0),
           (        16466.0,        85891.0,        45851.0)),
          ((        -1192.0,        -1292.0,         -576.0),
           (         1290.0,         2594.0,         1366.0),
           (            0.0,            0.0,            0.0)),
          ((         -998.0,          -34.0,           68.0),
           (        -9337.0,       -13365.0,       -62137.0),
           (            0.0,            0.0,            0.0)),
          ((       -19923.0,        -4197.0,         -210.0),
           (        58937.0,       -11586.0,       -16022.0),
           (            0.0,            0.0,            0.0)),
          ((         2232.0,          339.0,          -57.0),
           (         1807.0,        -2596.0,        -1511.0),
           (            0.0,            0.0,            0.0)),
          ((         8310.0,        13227.0,         6454.0),
           (         8696.0,       -14754.0,       -15236.0),
           (            0.0,            0.0,            0.0)),
          ((        -1923.0,         -606.0,         -158.0),
           (        -2970.0,        10165.0,         4717.0),
           (            0.0,            0.0,            0.0)),
          ((         1577.0,          956.0,          355.0),
           (         -830.0,          -76.0,          152.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,           83.0,           54.0),
           (           44.0,         -131.0,            5.0),
           (            0.0,            0.0,            0.0)),
          ((          458.0,           52.0,          -18.0),
           (            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((          896.0,         1050.0,          446.0),
           (           49.0,           85.0,           11.0),
           (            0.0,            0.0,            0.0)),
          ((        -4427.0,         6063.0,         3314.0),
           (            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          375.0,          441.0,          191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1762.0,         1166.0,          977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,          954.0,          434.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3882.0,          953.0,          258.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1661.0,         3483.0,         1461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3106.0,        -3367.0,        -1181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -400.0,          751.0,          409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4771.0,        -3178.0,        -2021.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1075.0,         1248.0,          318.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          637.0,          318.0,         -138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -348.0,          334.0,          347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -148.0,         -762.0,         -477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,          417.0,          239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1107.0,        -2279.0,        -1104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -210.0,         2610.0,         1401.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14586.0,       -62991.0,       -31551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          752.0,        20809.0,        11068.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3671.0,        12260.0,         6560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2171.0,        -8364.0,        -4185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2936.0,         8117.0,         3821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3606.0,        -4898.0,        -1653.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14517.0,        77810.0,        43681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3754590252.0,   -165939164.0,    301805642.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18868.0,        67412.0,        33037.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1359.0,        -6468.0,        -3242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2369.0,        11184.0,         5631.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -45701.0,      -163949.0,       -81006.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        42281.0,       151198.0,        74669.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3881.0,         7926.0,         3767.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          472.0,        -2590.0,        -1376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8816.0,       -22402.0,       -12507.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1104.0,        -1304.0,         -616.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,        -1689.0,         -925.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          916.0,         1655.0,          734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,         -108.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          248.0,          396.0,          167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2394.0,         1015.0,          620.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          610.0,          681.0,          271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2641.0,        -2326.0,         -784.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,          150.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          588.0,          462.0,          186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2349.0,         1201.0,          897.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,           26.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          794.0,         -277.0,         -210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2150.0,         3679.0,         1588.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -268.0,          348.0,          218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,           66.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          -68.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -179.0,         -209.0,          -94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          999.0,         -216.0,         -189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -352.0,        -6794.0,        -3493.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -552.0,          399.0,          254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6397.0,        -1002.0,        -1218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,           89.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          197.0,          181.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,           88.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          354.0,         -593.0,         -328.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2655.0,        -1073.0,         -270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -531.0,         1088.0,          584.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -373219235.0,    -92009409.0,    -10324279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,        14815.0,         7791.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -171.0,         6413.0,         3561.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,        -2659.0,        -1529.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          660.0,          407.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,          -93.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,          -37.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          527.0,          490.0,          212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,          -15.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5977.0,        -8448.0,        -5139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -37.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          765.0,         -240.0,         -214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -264.0,         -180.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,           51.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           34.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -624.0,          250.0,          185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          155.0,           15.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -984.0,        -1000.0,         -417.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -38.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          166.0,          201.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,          -91.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1097.0,          372.0,          325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          216.0,          -11.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,           11.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          336.0,         1045.0,          507.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -264.0,           85.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           23.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -46.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          376.0,           -6.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          134.0,        -1438.0,         -769.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           35.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1993.0,          190.0,         -116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          116.0,          -38.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           70.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,         -142.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,        -1305.0,         -698.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          312.0,           85.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -52711578.0,    -24801868.0,     -7763056.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -154.0,          779.0,          432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1167.0,         4178.0,         2343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           -2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          133.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           10.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2243.0,        -1929.0,        -1263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          174.0,          -13.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -74.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           39.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           78.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -199.0,          -91.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,           68.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,         -317.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           30.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -359.0,           21.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           -3.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           13.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          266.0,          137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,           17.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -10.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,           27.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          125.0,         -331.0,         -188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           22.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          575.0,          142.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           43.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,         -123.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          305.0,          175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -40.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,         -584.0,         -337.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          162.0,           57.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -8285125.0,     -6207546.0,     -2453310.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,         -101.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -594.0,         1048.0,          627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           20.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,         -134.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,          119.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           35.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          708.0,         -375.0,         -274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,            2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          -81.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,          -13.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           66.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          -66.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          146.0,           79.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -20.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,         -101.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           35.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1315474.0,     -1514000.0,      -671689.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -163.0,          162.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,          108.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          -38.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          209.0,          -57.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -22.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           17.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           26.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,          -20.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -193830.0,      -362616.0,      -173480.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,            7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,           38.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,           -3.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22082.0,       -85256.0,       -43231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            8.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,       -19614.0,       -10463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1117.0,        -4390.0,        -2460.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          520.0,         -947.0,         -560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          180.0,         -194.0,         -122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          -37.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -10717.0,        21933.0,        11941.0),
           (         2351.0,         9800.0,         4871.0),
           (        59062.0,        18354.0,         3280.0)),
          ((         -761.0,          718.0,          438.0),
           (        43139.0,        91437.0,        44163.0),
           (       -33239.0,       -26315.0,        -8218.0)),
          ((          703.0,         -284.0,         -211.0),
           (          774.0,        -3326.0,        -1882.0),
           (            0.0,            0.0,            0.0)),
          ((          861.0,        -1086.0,         -629.0),
           (         2014.0,         9198.0,        60652.0),
           (            0.0,            0.0,            0.0)),
          ((         1938.0,       -17592.0,        -9221.0),
           (       -10398.0,       123891.0,        62973.0),
           (            0.0,            0.0,            0.0)),
          ((         1033.0,         1557.0,          716.0),
           (         1102.0,         7584.0,         3541.0),
           (            0.0,            0.0,            0.0)),
          ((         9020.0,        12132.0,         5784.0),
           (        -1544.0,       -10065.0,       -11217.0),
           (            0.0,            0.0,            0.0)),
          ((        -1823.0,          485.0,          421.0),
           (         5153.0,       -12270.0,        -6154.0),
           (            0.0,            0.0,            0.0)),
          ((         -538.0,         -690.0,         -309.0),
           (          787.0,         -532.0,         -276.0),
           (            0.0,            0.0,            0.0)),
          ((         -360.0,         -158.0,          -50.0),
           (          190.0,         -330.0,         -115.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,          -84.0,          -44.0),
           (            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        -1518.0,         -828.0,         -279.0),
           (          100.0,          -11.0,           21.0),
           (            0.0,            0.0,            0.0)),
          ((        -6615.0,        -3336.0,         -940.0),
           (            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -822.0,         -242.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -793.0,         2003.0,         1309.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1234.0,         -369.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7204.0,         3410.0,         1124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4191.0,        -1442.0,         -282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4770.0,         1648.0,          220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -337.0,         -155.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4045.0,         3163.0,         1991.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1521.0,          754.0,          452.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,         1044.0,          455.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1126.0,        -1097.0,         -411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          785.0,          592.0,          196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -523.0,         -349.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          187.0,         -675.0,         -392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1921.0,          764.0,          622.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        36854.0,        36798.0,        15056.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15061.0,        -1813.0,          749.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11348.0,        -8655.0,        -4216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7214.0,         1892.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9988.0,        -3141.0,         -611.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15717.0,         3486.0,          576.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        60651.0,         7995.0,         2197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3748860503.0,     -8013903.0,   -393996285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -60218.0,         5630.0,        10443.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          989.0,         2510.0,         1241.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4745.0,        -4805.0,        -1971.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        72414.0,        79802.0,        34063.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -67136.0,       -73759.0,       -31440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2011.0,        -5573.0,        -2951.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1807.0,         1109.0,          245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9288.0,        23469.0,        13160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2290.0,          801.0,          176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          966.0,         -107.0,         -239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1875.0,         -524.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          258.0,          143.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -265.0,         -118.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2799.0,         2002.0,          689.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -758.0,         -476.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2712.0,         2241.0,          735.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,          -86.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -551.0,         -480.0,         -207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2093.0,         1406.0,          981.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,          -51.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          866.0,         -310.0,         -231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,          -21.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3718.0,        -2042.0,        -1324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          328.0,         -327.0,         -214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -72.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           23.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          174.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          996.0,         -407.0,         -285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1383.0,        -6669.0,        -3322.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -391.0,          240.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -782.0,        -5396.0,        -2647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -422.0,         -144.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,         -230.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -239.0,          -47.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          508.0,          472.0,          231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1054.0,         2604.0,         1218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1242.0,         4219.0,         2094.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -379906846.0,    -57457151.0,      8810460.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11548.0,        -1983.0,        -2206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12882.0,         2715.0,         -168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8360.0,        -2174.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3298.0,          933.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -664.0,         -233.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,          109.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -563.0,          551.0,          382.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          203.0,          -35.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5140.0,        -8841.0,        -5254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,         -189.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -191.0,          -64.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,          489.0,          265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,         -197.0,         -101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           37.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,         -113.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -270.0,         -270.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,          -91.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -902.0,         -878.0,         -364.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -134.0,         -114.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          321.0,          346.0,          142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -303.0,         -270.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1288.0,         -246.0,         -281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,          -80.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          129.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          -57.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1058.0,          378.0,          301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -252.0,           83.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           33.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -26.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -367.0,           37.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,         1510.0,          776.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,         -105.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          239.0,         1991.0,         1031.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,          -72.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           97.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,         -139.0,          -79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -523.0,        -1503.0,         -702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          215.0,        -1362.0,         -714.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     55979541.0,     17401286.0,      3473192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,          374.0,          172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4645.0,         -654.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,         -101.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           52.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -32.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          131.0,         -137.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           27.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1937.0,         2155.0,         1351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           76.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -26.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,          -52.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           18.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,           -5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          193.0,          154.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,          -45.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          232.0,          294.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -26.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -348.0,           74.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          290.0,          -44.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          -20.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,            1.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,         -330.0,         -180.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -17.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,         -504.0,         -278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -51.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,          121.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -270.0,         -289.0,         -128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           44.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          286.0,          573.0,          269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,          323.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -9460521.0,     -4615618.0,     -1481344.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           19.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1106.0,          300.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,           63.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -372.0,         -242.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          338.0,          216.0,           69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,           23.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           -7.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          601.0,         -478.0,         -317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -20.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,            9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -32.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -77.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,           -4.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,           -3.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           81.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          137.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -23.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,         -145.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,          -65.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1691907.0,      1181830.0,       455234.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -176.0,          -55.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -177.0,         -112.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           80.0,           49.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -11.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -184.0,           98.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           19.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -22.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -38.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           46.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           11.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -306703.0,      -297809.0,      -127151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           27.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,           22.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           23.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          -17.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        54398.0,        74208.0,        33973.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9010.0,       -18303.0,        -8838.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1259.0,         4466.0,         2254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,        -1075.0,         -565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          256.0,          139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -61.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        36612.0,        21350.0,         7799.0),
           (         6197.0,       -12329.0,        -6761.0),
           (       -59564.0,       -33232.0,        -8868.0)),
          ((        -5469.0,        -4832.0,        -1981.0),
           (        52029.0,       -15759.0,       -12042.0),
           (        52202.0,        -7994.0,       -13747.0)),
          ((         2729.0,         2844.0,         1211.0),
           (        -4418.0,         1795.0,         1279.0),
           (            0.0,            0.0,            0.0)),
          ((         1548.0,          933.0,          331.0),
           (        -3242.0,       -11503.0,       -57548.0),
           (            0.0,            0.0,            0.0)),
          ((        19300.0,         1557.0,        -1078.0),
           (        94687.0,       -39877.0,       -27481.0),
           (            0.0,            0.0,            0.0)),
          ((         -797.0,          227.0,          202.0),
           (         2892.0,        -2682.0,        -1736.0),
           (            0.0,            0.0,            0.0)),
          ((        10393.0,        12766.0,         5960.0),
           (        -3011.0,       -20641.0,       -16033.0),
           (            0.0,            0.0,            0.0)),
          ((        -3034.0,         -151.0,          205.0),
           (        -3547.0,         5806.0,         2672.0),
           (            0.0,            0.0,            0.0)),
          ((         1268.0,         1026.0,          428.0),
           (          -48.0,          968.0,          615.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           55.0,           30.0),
           (          100.0,          293.0,          215.0),
           (            0.0,            0.0,            0.0)),
          ((          365.0,          138.0,           39.0),
           (           -2.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((         1246.0,          590.0,          180.0),
           (         -143.0,         -107.0,          -75.0),
           (            0.0,            0.0,            0.0)),
          ((         3328.0,        -7390.0,        -3861.0),
           (            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((          236.0,          737.0,          333.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1643.0,         1527.0,         1157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          476.0,          427.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3864.0,         1248.0,          376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2124.0,         1376.0,          421.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3298.0,        -1396.0,         -215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -262.0,          293.0,          216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4079.0,        -3575.0,        -2214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -463.0,         -856.0,         -181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1584.0,        -1063.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1376.0,          687.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1369.0,         -719.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          524.0,          244.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1635.0,        -1830.0,         -735.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4321.0,         1001.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -50984.0,       -45212.0,       -17029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        23796.0,         9127.0,         1408.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15504.0,         1855.0,         -689.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8750.0,        -4487.0,        -1144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10034.0,         5782.0,         1563.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9955.0,       -10553.0,        -3815.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4571.0,       -54766.0,       -28313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3733109421.0,    181939241.0,    485135853.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5233.0,       -48859.0,       -25836.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7247.0,        -6505.0,        -2390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9394.0,         6508.0,         2115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -125012.0,       -80548.0,       -24665.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       116175.0,        74173.0,        22560.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2215.0,         2493.0,          920.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3067.0,        -2279.0,         -778.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9210.0,       -23360.0,       -13098.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2919.0,         -775.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1100.0,        -1529.0,         -685.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1840.0,          863.0,          224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,          -61.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          453.0,          256.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          615.0,        -2623.0,        -1219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          803.0,          634.0,          219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2734.0,        -2309.0,         -764.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          228.0,          180.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          677.0,          477.0,          187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2270.0,         1270.0,          923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           92.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          694.0,         -613.0,         -372.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          112.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2673.0,        -3442.0,        -1427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -351.0,          304.0,          208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           75.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,         -238.0,         -118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1205.0,         -383.0,         -292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2497.0,        -6489.0,        -3122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          394.0,          197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5365.0,          763.0,          994.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -576.0,          216.0,          175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          224.0,          220.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -130.0,          117.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1149.0,         -533.0,         -396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7676.0,        -3582.0,         -974.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10362.0,        -1946.0,          268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -382506762.0,    -22286270.0,     27851289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3178.0,        -8830.0,        -5022.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3704.0,       -10756.0,        -5249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3646.0,         6501.0,         2996.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1786.0,        -2379.0,        -1041.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          341.0,          552.0,          248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,         -182.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -671.0,         -224.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,         -111.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4536.0,        -9166.0,        -5367.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -400.0,         -149.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,         -330.0,         -182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          330.0,         -223.0,         -150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,          101.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -25.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           76.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          475.0,         -629.0,         -360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,          261.0,          142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -969.0,        -1039.0,         -437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,           95.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          297.0,           13.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -248.0,          152.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -892.0,          358.0,          297.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,           68.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          301.0,         -157.0,         -111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -128.0,           57.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -564.0,         -977.0,         -449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -272.0,          123.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           12.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -70.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          417.0,          -80.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -531.0,        -1458.0,         -713.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          150.0,          109.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2177.0,           37.0,          259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          106.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           18.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           60.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,         -266.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2772.0,         -781.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3728.0,          -22.0,          415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -57891898.0,     -9579319.0,       900862.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1566.0,           23.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          278.0,        -3634.0,        -1951.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          243.0,          -92.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -204.0,           19.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,           -6.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -14.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,          -78.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1628.0,        -2386.0,        -1443.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,          -46.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,         -111.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,          -24.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           16.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,           54.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,         -194.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           76.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -260.0,         -282.0,         -120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           32.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -340.0,           99.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,          -19.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,         -262.0,         -125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,           25.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -28.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          117.0,            1.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -95.0,         -361.0,         -180.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,           27.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -654.0,          -41.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,           32.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -233.0,          -40.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          512.0,          100.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -60.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -917.0,         -239.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -701.0,         -121.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -10228912.0,     -2825113.0,      -445651.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -187.0,          196.0,          142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          378.0,         -948.0,         -550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -33.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          206.0,          -36.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -193.0,           36.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,          -30.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          506.0,         -584.0,         -365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -12.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -30.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -19.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          -72.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,           17.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -72.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -83.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -164.0,          -21.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -32.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -250.0,          -65.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -150.0,          -13.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1954837.0,      -770316.0,      -208214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,         -137.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,         -132.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           47.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          156.0,         -133.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -19.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,            2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -18.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -22.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -12.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -22.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          -18.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -390005.0,      -204262.0,       -68551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           25.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -51.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          -31.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -79597.0,       -53446.0,       -20269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -13.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -7.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16388.0,       -13874.0,        -5706.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3364.0,        -3580.0,        -1562.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -680.0,         -919.0,         -420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,         -235.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -59.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -16.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -13392.0,       -18863.0,        -8258.0),
           (       -16475.0,        -9132.0,        -3257.0),
           (        17912.0,       -26523.0,       -14645.0)),
          ((        -1789.0,         2944.0,         1639.0),
           (      -118690.0,        21167.0,        19705.0),
           (       -53578.0,        11433.0,        11752.0)),
          ((         1162.0,        -1977.0,        -1100.0),
           (        -1221.0,         4664.0,         2487.0),
           (            0.0,            0.0,            0.0)),
          ((          453.0,          170.0,           52.0),
           (        15396.0,         8430.0,        52493.0),
           (            0.0,            0.0,            0.0)),
          ((         -830.0,        18008.0,         9308.0),
           (      -107907.0,       122609.0,        67987.0),
           (            0.0,            0.0,            0.0)),
          ((          907.0,        -1776.0,         -992.0),
           (        -3813.0,         6201.0,         3364.0),
           (            0.0,            0.0,            0.0)),
          ((        10056.0,        13156.0,         6179.0),
           (        14111.0,        -8467.0,       -11523.0),
           (            0.0,            0.0,            0.0)),
          ((        -2577.0,         -472.0,          -23.0),
           (        -7290.0,        -9137.0,        -3227.0),
           (            0.0,            0.0,            0.0)),
          ((        -1247.0,         -943.0,         -377.0),
           (        -1682.0,         -214.0,          175.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,          -23.0,           -6.0),
           (         -337.0,          142.0,          174.0),
           (            0.0,            0.0,            0.0)),
          ((         -243.0,          -76.0,          -17.0),
           (           -4.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((        -1393.0,         -469.0,         -107.0),
           (         -185.0,           10.0,           60.0),
           (            0.0,            0.0,            0.0)),
          ((         8894.0,         3415.0,          780.0),
           (            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1251.0,         -397.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -698.0,         2101.0,         1345.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -873.0,         -248.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6934.0,         2141.0,          453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3048.0,         -802.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4056.0,          997.0,         -124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          246.0,         -246.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3623.0,         4042.0,         2374.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          479.0,         -580.0,         -268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1526.0,        -1218.0,         -680.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1656.0,          937.0,          583.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          977.0,         -896.0,         -508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -443.0,          364.0,          214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -635.0,        -2000.0,         -966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2263.0,          915.0,          686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35912.0,        15608.0,         4829.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10763.0,        11711.0,         6827.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1048.0,        11337.0,         6833.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3586.0,        -4271.0,        -2483.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2158.0,         4069.0,         2137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2306.0,        -4271.0,        -1569.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -72201.0,        23744.0,        16635.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3707374411.0,   -355366630.0,   -574976972.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        55432.0,        14127.0,          449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10261.0,        -4883.0,        -3631.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12536.0,         5930.0,         4479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       153538.0,       -77371.0,       -57991.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -141662.0,        70938.0,        53215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6014.0,         4504.0,         3309.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2035.0,          464.0,          121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8257.0,        25071.0,        13802.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2686.0,        -1034.0,         -734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1278.0,        -1287.0,         -860.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2304.0,          301.0,          398.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          320.0,           19.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -349.0,           -1.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1575.0,        -1116.0,         -413.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -796.0,         -201.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2768.0,         2152.0,          681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -313.0,          -49.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -735.0,         -483.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2035.0,         1557.0,         1047.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,         -143.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          705.0,         -382.0,         -255.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,         -148.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3421.0,         2560.0,         1553.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          310.0,         -355.0,         -228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,            7.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -30.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          212.0,          365.0,          167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1081.0,         -295.0,         -234.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3296.0,        -6229.0,        -2907.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -260.0,           84.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2088.0,         5068.0,         2371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,          273.0,          146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,         -203.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,         -452.0,         -219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -336.0,         2433.0,         1268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6935.0,        -8878.0,        -5285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3819.0,       -14272.0,        -7718.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -381000281.0,     13128414.0,     46596723.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15431.0,          296.0,         1780.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5967.0,        -7792.0,        -3360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          978.0,         7277.0,         3633.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          571.0,        -3149.0,        -1679.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -171.0,          762.0,          411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -89.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          129.0,         -535.0,         -327.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,         -132.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3610.0,        -9496.0,        -5445.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -237.0,            4.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -473.0,          871.0,          524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,         -389.0,         -215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,          207.0,          115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -71.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,          166.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          626.0,          -10.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -124.0,          293.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1013.0,        -1039.0,         -434.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,          150.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          408.0,         -159.0,         -121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -273.0,          329.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1163.0,         -769.0,         -539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -77.0,          145.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          -37.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           93.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          982.0,         -617.0,         -418.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -249.0,          139.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -18.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           19.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,          -87.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -399.0,           81.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          810.0,         1356.0,          632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,           58.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -299.0,        -1970.0,        -1009.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,           97.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -20.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          106.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,         -425.0,         -220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1977.0,         2957.0,         1759.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          612.0,         4741.0,         2412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     58404800.0,      1523525.0,     -5254549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -803.0,        -1591.0,         -743.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4883.0,         1667.0,          357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,         -345.0,         -172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          196.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -78.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -76.0,          128.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           26.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1244.0,         2572.0,         1501.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,           -6.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,         -184.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,           57.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -39.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,         -108.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,           88.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,          -83.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          296.0,          261.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -17.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -27.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -310.0,          147.0,          112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,          -18.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -256.0,          133.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,          -27.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           95.0,            1.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,         -358.0,         -174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           28.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,          496.0,          254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           64.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,         -308.0,         -152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          630.0,          313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,           21.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,         -964.0,         -535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          238.0,         -826.0,         -478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -10558096.0,      -912486.0,       609646.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,         -164.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1147.0,         -323.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,          -25.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          213.0,          178.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,         -158.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -40.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          373.0,         -665.0,         -395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           10.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           31.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           21.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          -64.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,          -34.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,          -34.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           82.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,         -150.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -40.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,          275.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          172.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2086756.0,       307020.0,       -52847.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          165.0,           55.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          199.0,          102.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,          -60.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           16.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,          164.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -10.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           18.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -16.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           48.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,          -64.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -56.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -435810.0,       -90967.0,        -3302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -30.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,          -12.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -23.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,          -40.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           17.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        94405.0,        25679.0,         3905.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20977.0,        -7081.0,        -1602.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4747.0,         1929.0,          537.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1087.0,         -522.0,         -166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          252.0,          143.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -40.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -34915.0,        -8570.0,        -1411.0),
           (        11112.0,        15141.0,         6733.0),
           (        98003.0,        28033.0,         5231.0)),
          ((         8359.0,          277.0,         -571.0),
           (        32903.0,        76429.0,        35463.0),
           (       -81403.0,        21918.0,        18023.0)),
          ((        -4633.0,          172.0,          480.0),
           (         3278.0,          815.0,          -16.0),
           (            0.0,            0.0,            0.0)),
          ((        -1388.0,          284.0,          264.0),
           (       -20847.0,       -33492.0,       -62615.0),
           (            0.0,            0.0,            0.0)),
          ((       -19825.0,          472.0,         2175.0),
           (        59310.0,        94758.0,        40837.0),
           (            0.0,            0.0,            0.0)),
          ((         1813.0,         -244.0,         -314.0),
           (         1581.0,         1356.0,          591.0),
           (            0.0,            0.0,            0.0)),
          ((        10299.0,        12737.0,         5934.0),
           (       -13305.0,         -401.0,        -3896.0),
           (            0.0,            0.0,            0.0)),
          ((        -2067.0,         -500.0,          -99.0),
           (        -2074.0,        20734.0,        10479.0),
           (            0.0,            0.0,            0.0)),
          ((         2069.0,          492.0,           57.0),
           (          971.0,        -1226.0,         -673.0),
           (            0.0,            0.0,            0.0)),
          ((         -307.0,           64.0,           61.0),
           (          777.0,        -1033.0,         -568.0),
           (            0.0,            0.0,            0.0)),
          ((          773.0,          -57.0,          -97.0),
           (           -8.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((          947.0,          410.0,          116.0),
           (           17.0,          137.0,           45.0),
           (            0.0,            0.0,            0.0)),
          ((        -2665.0,         6980.0,         3602.0),
           (           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         -387.0,          -89.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -696.0,         1886.0,         1262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -854.0,          155.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7095.0,         1002.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2028.0,          831.0,          395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -576.0,        -1149.0,         -207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1448.0,          195.0,          221.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3743.0,        -4343.0,        -2510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,          581.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2257.0,         1319.0,          236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2986.0,        -1277.0,         -230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2175.0,          813.0,          138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1074.0,         -358.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          225.0,         -316.0,         -223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4787.0,         -268.0,          353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        30701.0,       -20335.0,       -13701.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -41111.0,        -7216.0,          482.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22057.0,         -670.0,         1556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14214.0,         2947.0,          103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15922.0,        -5368.0,        -1304.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17612.0,         8349.0,         2889.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14180.0,        64158.0,        29269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3671698464.0,    527842421.0,    663285882.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15010.0,        65798.0,        35234.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10806.0,         2759.0,          543.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15042.0,        -2151.0,          179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       182642.0,        14756.0,        -7322.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -169232.0,       -14058.0,         6589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3992.0,          445.0,          412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4045.0,         -327.0,         -565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3749.0,       -24928.0,       -13307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          955.0,          613.0,          155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1015.0,         -777.0,         -571.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1288.0,           49.0,          105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,           -6.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          103.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1222.0,         2009.0,          999.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          322.0,          275.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2559.0,        -2170.0,         -713.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,          -32.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          920.0,          208.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2397.0,         1344.0,          973.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          209.0,           52.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          277.0,         -762.0,         -405.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          158.0,           74.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3022.0,         3040.0,         1199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -247.0,          297.0,          194.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,           41.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,          -67.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -442.0,          -77.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          955.0,         -276.0,         -210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4282.0,        -6000.0,        -2690.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          289.0,          894.0,          426.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4685.0,        -2659.0,        -1891.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1059.0,          745.0,          269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -342.0,          -45.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          899.0,          600.0,          207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3699.0,        -2474.0,         -854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12535.0,         9594.0,         3582.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15553.0,         6124.0,         1416.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -375400794.0,     48394398.0,     64839057.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7482.0,        11796.0,         7085.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9292.0,        12963.0,         5770.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8123.0,        -8011.0,        -3275.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3115.0,         2661.0,         1033.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -797.0,         -511.0,         -175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          161.0,           -9.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          671.0,           44.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,           30.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2697.0,        -9869.0,        -5546.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           82.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          732.0,         -291.0,         -229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -298.0,         -166.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          129.0,           49.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -10.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,           47.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -511.0,          442.0,          263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          286.0,           20.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1268.0,         -839.0,         -304.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,           -6.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,            5.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          205.0,           50.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1145.0,          549.0,          418.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          119.0,           30.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,         -200.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,           73.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          785.0,          800.0,          335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,          113.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,           17.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -10.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,          -24.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          329.0,          -98.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1083.0,        -1298.0,         -574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          253.0,          149.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1477.0,         -684.0,         -527.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          504.0,          136.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -211.0,           -8.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          167.0,           69.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -558.0,         -223.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3867.0,         2182.0,          729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5445.0,         1470.0,          166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -57504621.0,      6566729.0,      9480123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3224.0,         -773.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          364.0,         4978.0,         2588.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -297.0,         -162.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          139.0,           86.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,          -43.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           -4.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,           10.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,           12.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          814.0,        -2728.0,        -1538.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           22.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          147.0,          -58.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -63.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           31.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           56.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -191.0,           36.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,           32.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -316.0,         -223.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,            8.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -303.0,          172.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -34.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          200.0,          201.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,           31.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -14.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           74.0,          -30.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -211.0,         -290.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          586.0,         -122.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -25.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          161.0,          179.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -440.0,         -376.0,         -173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -150.0,          -30.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1181.0,          695.0,          260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          975.0,          188.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -10433495.0,      1038759.0,      1638484.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          150.0,          141.0,          103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -286.0,         1116.0,          609.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,          127.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1113.0,         -705.0,         -307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          997.0,          630.0,          274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           -5.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          212.0,         -725.0,         -409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -21.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           28.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,          -53.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,           45.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           53.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,          -75.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,          -37.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,          -17.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          371.0,          174.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          147.0,          -19.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2078783.0,       176755.0,       310324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          203.0,          117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          183.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -73.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,         -187.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -14.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           13.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -22.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -15.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,           43.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,            7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -439613.0,        31003.0,        62220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,            9.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,           61.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           25.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -48.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -96858.0,         5436.0,        12964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           19.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21991.0,          920.0,         2776.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5109.0,          141.0,          606.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1208.0,           16.0,          134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -289.0,            0.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,            0.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        31942.0,        18150.0,         6306.0),
           (         7083.0,        -8628.0,        -4977.0),
           (        20311.0,        43998.0,        22645.0)),
          ((        -8732.0,         -653.0,          461.0),
           (        63357.0,        -8710.0,        -9247.0),
           (        -1736.0,       -59248.0,       -32648.0)),
          ((         4806.0,            0.0,         -438.0),
           (         -383.0,        -4851.0,        -2354.0),
           (            0.0,            0.0,            0.0)),
          ((          940.0,        -1298.0,         -750.0),
           (        14970.0,        41959.0,        64783.0),
           (            0.0,            0.0,            0.0)),
          ((        -3638.0,       -17487.0,        -8622.0),
           (         1217.0,        -1187.0,          829.0),
           (            0.0,            0.0,            0.0)),
          ((         1763.0,         1132.0,          417.0),
           (         2118.0,         3854.0,         1587.0),
           (            0.0,            0.0,            0.0)),
          ((        10180.0,        11640.0,         5346.0),
           (       -29158.0,       -29503.0,       -16805.0),
           (            0.0,            0.0,            0.0)),
          ((        -1692.0,          454.0,          373.0),
           (         5543.0,        -4168.0,        -2239.0),
           (            0.0,            0.0,            0.0)),
          ((         -896.0,         -504.0,         -167.0),
           (         2613.0,         1306.0,          466.0),
           (            0.0,            0.0,            0.0)),
          ((         -304.0,          -83.0,          -18.0),
           (          592.0,           25.0,            5.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,           64.0,           48.0),
           (           -6.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((        -1622.0,         -228.0,           29.0),
           (          257.0,          126.0,           64.0),
           (            0.0,            0.0,            0.0)),
          ((        -7449.0,        -2460.0,         -439.0),
           (            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -460.0,         -136.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -592.0,         2271.0,         1446.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -745.0,         -419.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6970.0,         2045.0,          507.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2898.0,        -1870.0,         -679.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4014.0,         1871.0,          412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,         -389.0,         -267.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2420.0,         4705.0,         2638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1085.0,          691.0,          347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -532.0,         1288.0,          652.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,        -1418.0,         -692.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,         1041.0,          555.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -154.0,         -546.0,         -278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -955.0,          141.0,          166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          648.0,         -923.0,         -590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        23829.0,        46969.0,        22730.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2662.0,       -13661.0,        -7923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4296.0,       -13433.0,        -7736.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1478.0,         5835.0,         3070.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3601.0,        -7462.0,        -3854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9127.0,         4839.0,         2178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        61708.0,       -34353.0,       -23831.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3626239778.0,   -698905159.0,   -749830852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -67567.0,       -11653.0,         1920.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3591.0,         6320.0,         3881.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1444.0,       -10380.0,        -5814.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4702.0,       151277.0,        83610.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4074.0,      -139921.0,       -77284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,        -8370.0,        -4710.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          201.0,         2003.0,          966.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5609.0,        23674.0,        12819.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1486.0,          947.0,          425.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          687.0,          102.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1178.0,         -926.0,         -391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          218.0,          116.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,         -195.0,          -84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3118.0,         1291.0,          363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -668.0,         -482.0,         -175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2689.0,         2216.0,          729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,           43.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -773.0,          -69.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1939.0,         1413.0,          968.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,           44.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          537.0,         -803.0,         -452.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,           94.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3006.0,        -2943.0,        -1699.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,         -248.0,         -177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,          -76.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           47.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,           51.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          734.0,         -580.0,         -346.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4620.0,        -5372.0,        -2330.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -659.0,          586.0,          363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -941.0,        -5883.0,        -2893.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1110.0,          449.0,          343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          333.0,         -438.0,         -250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -808.0,          535.0,          370.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3356.0,        -2645.0,        -1756.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10637.0,        12477.0,         7664.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9272.0,        12138.0,         7371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -365755602.0,     83144161.0,     82386863.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17500.0,        -5534.0,        -4850.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6143.0,         8926.0,         4084.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1682.0,        -8112.0,        -4133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          360.0,         3529.0,         1838.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,         -833.0,         -448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          262.0,          137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,          608.0,          341.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           45.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1888.0,       -10074.0,        -5568.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,          -29.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,         -204.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -332.0,          714.0,          416.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          133.0,         -312.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           80.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,         -180.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -693.0,          163.0,          157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          166.0,         -284.0,         -168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1206.0,         -583.0,         -171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,         -212.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          187.0,          367.0,          174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,         -435.0,         -225.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          937.0,         -378.0,         -310.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,         -146.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          317.0,          169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,         -127.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -756.0,          817.0,          499.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -228.0,           87.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,           53.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -33.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           20.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -265.0,          179.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1159.0,         1126.0,          476.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,         -141.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          676.0,         2004.0,          991.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          202.0,         -218.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,          168.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,          -84.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -410.0,          355.0,          233.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2343.0,        -4077.0,        -2400.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2658.0,        -3604.0,        -2195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     55210807.0,    -14499137.0,    -13476979.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1932.0,         1993.0,         1272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4511.0,         -237.0,          351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -289.0,          162.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          140.0,         -113.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,           24.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -28.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,         -158.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -443.0,         2801.0,         1539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           36.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          101.0,         -112.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           53.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,           62.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          331.0,          -47.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,           29.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          343.0,          181.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -262.0,          208.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -9.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -26.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          183.0,         -204.0,         -125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,          -35.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,          -62.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -292.0,         -213.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -48.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -279.0,         -435.0,         -199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,         -105.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -173.0,          373.0,          236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          417.0,         -860.0,         -539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          -22.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -753.0,         1485.0,          895.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -255.0,          542.0,          282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -9859447.0,      2945578.0,      2596993.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -309.0,          315.0,          255.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1376.0,         -250.0,         -298.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           96.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,         -469.0,         -312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          421.0,          279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           15.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           41.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,         -734.0,         -400.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -16.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           12.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           -9.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,          -43.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          -58.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,           54.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,           60.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,          132.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           19.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,         -370.0,         -214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -67.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1931202.0,      -648738.0,      -547021.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,           53.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -180.0,          -11.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           64.0,           11.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          196.0,          106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           19.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -19.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -43.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           94.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           36.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -400924.0,       150009.0,       121748.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           43.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          -38.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           21.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -53.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -27.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        86618.0,       -35844.0,       -28138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,            8.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19261.0,         8763.0,         6680.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4377.0,        -2179.0,        -1618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1011.0,          550.0,          398.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          238.0,         -140.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           37.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        27271.0,       -16779.0,       -10974.0),
           (       -13831.0,        -1090.0,          665.0),
           (      -135664.0,      -102062.0,       -35216.0)),
          ((        -1544.0,         1733.0,         1038.0),
           (       -15202.0,        11062.0,         5974.0),
           (       106689.0,        37418.0,         8275.0)),
          ((         1148.0,         -500.0,         -366.0),
           (        -4591.0,         4678.0,         2864.0),
           (            0.0,            0.0,            0.0)),
          ((         1772.0,          386.0,           30.0),
           (       -13666.0,       -26823.0,       -55907.0),
           (            0.0,            0.0,            0.0)),
          ((        18856.0,        -2977.0,        -3360.0),
           (        57469.0,       -16679.0,       -16136.0),
           (            0.0,            0.0,            0.0)),
          ((         -502.0,          617.0,          379.0),
           (         1442.0,        -2745.0,        -1758.0),
           (            0.0,            0.0,            0.0)),
          ((        11829.0,        11594.0,         5139.0),
           (         2835.0,       -31063.0,       -20930.0),
           (            0.0,            0.0,            0.0)),
          ((        -2796.0,          751.0,          641.0),
           (        12190.0,         1832.0,         -827.0),
           (            0.0,            0.0,            0.0)),
          ((         1627.0,          741.0,          237.0),
           (         -948.0,         2103.0,         1273.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           89.0,           37.0),
           (         -658.0,          803.0,          532.0),
           (            0.0,            0.0,            0.0)),
          ((          344.0,         -106.0,          -77.0),
           (           11.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((         1449.0,           64.0,         -107.0),
           (         -156.0,          -25.0,          -11.0),
           (            0.0,            0.0,            0.0)),
          ((         1627.0,        -7989.0,        -3981.0),
           (            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((          759.0,          965.0,          389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1554.0,         1526.0,         1163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1142.0,          976.0,          358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2883.0,        -1604.0,         -930.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4443.0,         3401.0,         1160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5088.0,        -2828.0,         -685.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          461.0,          954.0,          479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1680.0,        -5222.0,        -2820.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,         -336.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2374.0,        -1862.0,         -467.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2752.0,         2249.0,          714.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2565.0,        -1973.0,         -614.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1165.0,          943.0,          312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3480.0,        -2607.0,         -951.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6810.0,         3244.0,          815.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -93390.0,       -71928.0,       -25998.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        47729.0,        33596.0,        11185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26773.0,        13310.0,         3935.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18885.0,       -14555.0,        -5120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22765.0,        16655.0,         5634.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26309.0,       -22000.0,        -7746.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16116.0,       -33001.0,       -12953.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3571089693.0,    868103920.0,    834382878.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17790.0,       -48068.0,       -26768.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12793.0,       -15203.0,        -6302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18653.0,        18674.0,         7439.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -254486.0,      -247773.0,       -97832.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       235651.0,       228681.0,        90165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7067.0,         9618.0,         4223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5263.0,        -4359.0,        -1564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7876.0,       -22579.0,       -12573.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4160.0,        -1835.0,         -514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1661.0,        -2183.0,         -971.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3216.0,         2204.0,          763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -243.0,         -226.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          646.0,          319.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -653.0,        -3137.0,        -1363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1119.0,          731.0,          243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2850.0,        -2316.0,         -759.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          328.0,           65.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          733.0,           22.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2123.0,         1515.0,         1031.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -53.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          300.0,         -762.0,         -412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,            3.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3457.0,        -2718.0,        -1006.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -415.0,          189.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          111.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -205.0,          -75.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          980.0,         -686.0,         -423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5552.0,        -4437.0,        -1755.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -157.0,         -130.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4641.0,         2451.0,         1781.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -954.0,         -487.0,         -147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          369.0,          454.0,          185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -624.0,         -562.0,         -226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2529.0,         2787.0,         1173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16556.0,       -11903.0,        -4342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11006.0,        -9131.0,        -3508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -352183622.0,    117008790.0,     99055897.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5545.0,        -4993.0,        -3321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10191.0,       -19199.0,        -8906.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9216.0,        13446.0,         5987.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4022.0,        -5102.0,        -2211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          860.0,         1097.0,          475.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -243.0,         -281.0,         -121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -651.0,          289.0,          236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,         -193.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1145.0,       -10070.0,        -5489.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -426.0,          -62.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,         -385.0,         -210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          259.0,         -194.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,           82.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,          -22.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           16.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          139.0,         -655.0,         -343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,          157.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1134.0,         -764.0,         -274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,           15.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          377.0,            9.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -309.0,            3.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -654.0,          714.0,          457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,          -11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          213.0,         -169.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -148.0,           16.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -957.0,         -652.0,         -242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -258.0,          123.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           17.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            0.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -26.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          314.0,         -236.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1449.0,         -786.0,         -269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,         -109.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1818.0,         1043.0,          750.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,         -204.0,          -96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,          148.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          -62.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,          318.0,          155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5274.0,        -2886.0,         -944.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3143.0,        -2530.0,         -995.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -51581679.0,     22082214.0,     17149016.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2584.0,         2238.0,          903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1234.0,        -4569.0,        -2286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          332.0,          118.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -251.0,         -108.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,           34.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,          -30.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -125.0,           76.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,          -28.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,        -2840.0,        -1519.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,           -7.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,         -123.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -35.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           19.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,            6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -144.0,         -168.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,           40.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -358.0,         -166.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,            2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -242.0,          234.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -39.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -267.0,         -151.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           39.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           12.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -17.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,          -48.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -406.0,         -195.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,           21.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -589.0,          248.0,          199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          116.0,           53.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -478.0,         -175.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1067.0,          435.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           49.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1951.0,         -821.0,         -193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -466.0,         -343.0,         -169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -8861880.0,      4726406.0,      3444334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -272.0,          112.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -77.0,         -966.0,         -529.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,         -130.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          720.0,          562.0,          144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -655.0,         -502.0,         -128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -13.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           27.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -22.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,         -740.0,         -389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           -3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -29.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -34.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,          -32.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           70.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,          -34.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -15.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,          -37.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -124.0,           89.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           29.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -472.0,         -190.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1654121.0,      1077316.0,       747116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,         -136.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -102.0,         -178.0,          -84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,           67.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -11.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,         -196.0,         -101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           20.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           31.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,          -36.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -323648.0,       254653.0,       169607.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -42.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -27.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -52.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           -6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -65052.0,        61577.0,        39641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -15.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13248.0,        15115.0,         9448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2702.0,         3749.0,         2283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -546.0,          936.0,          556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,          234.0,          136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           59.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           18.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -46242.0,        10027.0,         9358.0),
           (          813.0,         1629.0,          736.0),
           (        29068.0,       -33055.0,       -19189.0)),
          ((         -542.0,        -2497.0,        -1246.0),
           (       -12188.0,        93432.0,        47915.0),
           (       -51548.0,        65857.0,        38091.0)),
          ((         -129.0,         1055.0,          561.0),
           (         2488.0,         4340.0,         1786.0),
           (            0.0,            0.0,            0.0)),
          ((          213.0,          598.0,          292.0),
           (         8791.0,       -18098.0,        33635.0),
           (            0.0,            0.0,            0.0)),
          ((         4575.0,        17414.0,         8481.0),
           (        41438.0,       218058.0,       103063.0),
           (            0.0,            0.0,            0.0)),
          ((          207.0,        -1624.0,         -844.0),
           (         1842.0,         9087.0,         4422.0),
           (            0.0,            0.0,            0.0)),
          ((        11811.0,        12233.0,         5450.0),
           (        11795.0,         5973.0,        -1992.0),
           (            0.0,            0.0,            0.0)),
          ((        -2609.0,         -109.0,          167.0),
           (       -13485.0,       -12902.0,        -5002.0),
           (            0.0,            0.0,            0.0)),
          ((        -1444.0,         -593.0,         -157.0),
           (        -2358.0,        -1031.0,         -251.0),
           (            0.0,            0.0,            0.0)),
          ((         -117.0,          -28.0,           -3.0),
           (         -509.0,          189.0,          173.0),
           (            0.0,            0.0,            0.0)),
          ((         -173.0,           24.0,           27.0),
           (            2.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((        -1530.0,          241.0,          263.0),
           (         -200.0,         -138.0,          -45.0),
           (            0.0,            0.0,            0.0)),
          ((         9403.0,         1845.0,          -31.0),
           (           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        -1444.0,          105.0,          200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,         2046.0,         1260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1143.0,          190.0,          224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7721.0,         -404.0,         -917.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3875.0,          456.0,          762.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4665.0,         -107.0,         -798.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,          -97.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1332.0,         4975.0,         2632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          372.0,         -418.0,         -165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1586.0,        -1585.0,         -860.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1851.0,         1550.0,          925.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1195.0,        -1237.0,         -687.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -563.0,          514.0,          297.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1047.0,        -1486.0,         -675.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3092.0,         1979.0,         1282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51328.0,          989.0,        -4080.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15275.0,        17439.0,        10143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3313.0,        13588.0,         8181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5367.0,        -5912.0,        -3490.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4179.0,         4612.0,         2647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -177.0,        -2720.0,        -1039.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -50148.0,        52834.0,        35088.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3506364843.0,  -1034981242.0,   -916710554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        59137.0,        18604.0,         2822.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12945.0,        -6327.0,        -4610.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15216.0,         8323.0,         5940.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       184609.0,      -115666.0,       -80201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -170897.0,       106497.0,        73893.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5948.0,         6043.0,         4018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3122.0,         -154.0,         -325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5990.0,        26174.0,        14135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3068.0,        -1758.0,        -1129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1601.0,        -1489.0,        -1010.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2658.0,          994.0,          774.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          350.0,         -106.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -391.0,           51.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1772.0,         -416.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -862.0,          -64.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2789.0,         2073.0,          638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -345.0,          103.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -786.0,         -117.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1910.0,         1619.0,         1067.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,          -91.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,         -680.0,         -353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -98.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2560.0,         3276.0,         1822.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          368.0,         -280.0,         -201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,            5.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -36.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          321.0,          147.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          917.0,         -625.0,         -385.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6219.0,        -3930.0,        -1423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,          -95.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2383.0,         5065.0,         2347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          360.0,           64.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,         -122.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          379.0,         -494.0,         -304.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1467.0,         2376.0,         1426.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13188.0,       -13662.0,        -8729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13370.0,        -9704.0,        -6744.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -334836988.0,    149608274.0,    114659623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18106.0,         3885.0,         4043.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2684.0,        -9378.0,        -4789.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2667.0,         8893.0,         5091.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2241.0,        -3786.0,        -2286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -619.0,          938.0,          574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,         -153.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -454.0,         -493.0,         -232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          161.0,         -165.0,         -105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,       -10179.0,        -5431.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -169.0,           92.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -610.0,          854.0,          536.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          339.0,         -339.0,         -225.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -171.0,          186.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          -54.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,          144.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          918.0,         -284.0,         -245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -313.0,          289.0,          192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1050.0,         -811.0,         -312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,          127.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          626.0,         -266.0,         -207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -531.0,          311.0,          228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1117.0,         -970.0,         -647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -169.0,          117.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          180.0,           72.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -125.0,           44.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          578.0,         -940.0,         -545.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -253.0,          150.0,          109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,          -11.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -300.0,          256.0,          162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1667.0,          566.0,          130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -272.0,          161.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -756.0,        -1834.0,         -890.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -379.0,          167.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,          -32.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           88.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,         -204.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3757.0,         4599.0,         2855.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2497.0,         2330.0,         1523.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     46706303.0,    -29130612.0,    -20406617.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          244.0,        -1248.0,         -693.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4841.0,          104.0,         -453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,         -248.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,          121.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -48.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           10.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          126.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           16.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          388.0,         2851.0,         1478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,          -47.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,         -184.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           21.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          -17.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          -37.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           75.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,          -35.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          370.0,          133.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,            4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,          259.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -38.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           15.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,          245.0,          138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          -40.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -12.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -53.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -360.0,         -133.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           27.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          428.0,          374.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           74.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,         -372.0,         -191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -294.0,          798.0,          415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           18.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          834.0,        -1622.0,         -918.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          577.0,         -509.0,         -375.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -7484108.0,      6304013.0,      4143796.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          253.0,         -322.0,         -122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1271.0,          338.0,          286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          -55.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          161.0,          463.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,         -417.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -15.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -26.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -229.0,         -734.0,         -369.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           18.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           41.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           -8.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,          -19.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -77.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -68.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           17.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           93.0,           17.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -128.0,         -101.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           12.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,          415.0,          238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,           35.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1266170.0,     -1433635.0,      -897140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          184.0,          -71.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          -26.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,           -9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,          186.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           20.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,            5.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           31.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,         -113.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -26.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -215250.0,       334754.0,       201135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -63.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,           53.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -45.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           31.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34990.0,       -79258.0,       -45964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -13.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4979.0,        18895.0,        10609.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          448.0,        -4514.0,        -2457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,         1075.0,          568.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,         -254.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           58.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((         5341.0,        16648.0,         8045.0),
           (        15707.0,        -1683.0,        -2274.0),
           (        76378.0,        39500.0,        15248.0)),
          ((         4777.0,        -1577.0,        -1199.0),
           (        39870.0,       -14165.0,       -10695.0),
           (       -36007.0,        11850.0,         5070.0)),
          ((        -2208.0,         1071.0,          723.0),
           (         3068.0,        -3065.0,        -2104.0),
           (            0.0,            0.0,            0.0)),
          ((         -767.0,          508.0,          320.0),
           (       -23099.0,        -2016.0,       -38755.0),
           (            0.0,            0.0,            0.0)),
          ((       -19107.0,         5029.0,         4441.0),
           (        29166.0,       -59729.0,       -32668.0),
           (            0.0,            0.0,            0.0)),
          ((         1890.0,         -705.0,         -556.0),
           (          703.0,        -3349.0,        -1699.0),
           (            0.0,            0.0,            0.0)),
          ((        11932.0,        11938.0,         5287.0),
           (       -22573.0,         3623.0,         1450.0),
           (            0.0,            0.0,            0.0)),
          ((        -2487.0,         -194.0,           86.0),
           (         2393.0,        16077.0,         8091.0),
           (            0.0,            0.0,            0.0)),
          ((         1937.0,          -35.0,         -215.0),
           (         1103.0,        -1562.0,         -943.0),
           (            0.0,            0.0,            0.0)),
          ((         -194.0,          131.0,           81.0),
           (          456.0,        -1287.0,         -724.0),
           (            0.0,            0.0,            0.0)),
          ((          559.0,         -266.0,         -178.0),
           (           -7.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0)),
          ((         1085.0,         -201.0,         -210.0),
           (           57.0,           65.0,           32.0),
           (            0.0,            0.0,            0.0)),
          ((        -1870.0,         7579.0,         3830.0),
           (           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,         -335.0,         -196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -563.0,         2239.0,         1443.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -262.0,          -70.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6019.0,         -351.0,         -510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,          102.0,         -245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2032.0,         -321.0,          454.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -915.0,          191.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1246.0,        -5141.0,        -2672.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          405.0,          423.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1959.0,         1447.0,          355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2366.0,        -1428.0,         -391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1675.0,         1116.0,          389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -823.0,         -499.0,         -177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -418.0,          718.0,          392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2596.0,         -666.0,         -162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5258.0,        -6653.0,        -3154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28454.0,       -11610.0,        -3777.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17478.0,        -3105.0,         -529.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10407.0,         4769.0,         1718.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12868.0,        -7255.0,        -2873.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16804.0,         8778.0,         3417.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        50376.0,        39547.0,        12998.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3432258793.0,   1199053666.0,    996578164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7645.0,        64989.0,        34520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4859.0,         5740.0,         2825.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7038.0,        -6572.0,        -3167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        81875.0,        72549.0,        36782.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -76341.0,       -67141.0,       -33959.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -543.0,        -2686.0,        -1825.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2388.0,           50.0,         -184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2011.0,       -25965.0,       -13682.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,         1446.0,          681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          590.0,         -378.0,         -317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -347.0,         -439.0,         -279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           -7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,          -55.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,         2504.0,         1196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          498.0,           39.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2630.0,        -2067.0,         -651.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          210.0,         -191.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          889.0,         -149.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2250.0,         1536.0,         1055.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,           31.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -179.0,         -918.0,         -445.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,           82.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3794.0,         2235.0,          737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -313.0,          232.0,          172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -125.0,           26.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,          -58.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -357.0,          104.0,           82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1024.0,         -475.0,         -319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6955.0,        -3481.0,        -1116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          640.0,          879.0,          387.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3926.0,        -3446.0,        -2220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1275.0,          738.0,          250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -372.0,          -37.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          817.0,          627.0,          238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4055.0,        -2786.0,        -1011.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14058.0,        14351.0,         5994.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10765.0,        13114.0,         5682.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -313886071.0,    180590842.0,    129027760.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3572.0,         9788.0,         5681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8573.0,        16680.0,         7790.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7019.0,       -11091.0,        -5004.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2594.0,         3828.0,         1699.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -636.0,         -793.0,         -340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           97.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          553.0,         -439.0,         -313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,           83.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -805.0,       -10363.0,        -5437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          227.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          668.0,         -387.0,         -270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -289.0,          -76.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,            2.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           96.0,            6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -124.0,          590.0,          299.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          211.0,          -69.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1367.0,         -576.0,         -155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,          -43.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          143.0,          -98.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           82.0,           34.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -832.0,          794.0,          511.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,         -224.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,           69.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1111.0,          397.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,          133.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           15.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -7.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,           15.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          303.0,         -234.0,         -151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1813.0,         -401.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          334.0,           51.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1065.0,        -1152.0,         -735.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          506.0,           12.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -181.0,           41.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          135.0,           39.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -593.0,         -115.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4540.0,         3723.0,         1483.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3536.0,         3652.0,         1529.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -40699283.0,     35471913.0,     23169620.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2178.0,        -1567.0,         -564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1959.0,         5133.0,         2508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -211.0,         -221.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,          112.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,          -50.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          110.0,         -133.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -815.0,        -2817.0,        -1415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           55.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          124.0,         -101.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          -37.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           14.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          104.0,           -4.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,          152.0,           88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,          -27.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -381.0,          -86.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -23.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,          285.0,          171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -36.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          296.0,           48.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,           44.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           18.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,          -80.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -394.0,           -9.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -22.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          399.0,         -364.0,         -234.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          -49.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          251.0,          244.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -630.0,         -485.0,         -231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -163.0,           24.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1590.0,          930.0,          358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          740.0,          613.0,          202.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -5784226.0,      7609820.0,      4664794.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          333.0,           14.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          383.0,         1090.0,          514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          214.0,           87.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1412.0,         -381.0,         -135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1262.0,          339.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -39.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -388.0,         -703.0,         -336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -34.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           31.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,           -5.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           83.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,            2.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           15.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,            5.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,         -129.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           16.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          482.0,          174.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           78.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -793251.0,      1693712.0,       986977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,          166.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          150.0,          160.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,          -64.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -10.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,         -167.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,            3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           25.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,            5.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -37.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          124.0,           23.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,           29.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -86072.0,       382501.0,       213236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          -26.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           58.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,            8.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          -37.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,            0.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -342.0,        86555.0,        46267.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4142.0,        19455.0,         9962.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1905.0,         4309.0,         2104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          657.0,          932.0,          430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          202.0,          196.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,           38.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        29094.0,       -32256.0,       -19227.0),
           (        -8563.0,       -11826.0,        -5237.0),
           (        26983.0,        37198.0,        19049.0)),
          ((        -8902.0,         4619.0,         3190.0),
           (        -8428.0,       -31847.0,       -14601.0),
           (       -34728.0,       -93935.0,       -47530.0)),
          ((         4683.0,        -2819.0,        -1878.0),
           (        -3160.0,        -4059.0,        -1900.0),
           (            0.0,            0.0,            0.0)),
          ((          417.0,        -1678.0,         -900.0),
           (        34464.0,        35361.0,        51152.0),
           (            0.0,            0.0,            0.0)),
          ((        -8584.0,       -16068.0,        -7412.0),
           (       -50931.0,        -9417.0,         1211.0),
           (            0.0,            0.0,            0.0)),
          ((         2084.0,          601.0,          106.0),
           (         1757.0,         3315.0,         1331.0),
           (            0.0,            0.0,            0.0)),
          ((        11450.0,        11046.0,         4840.0),
           (       -27139.0,       -28294.0,       -14417.0),
           (            0.0,            0.0,            0.0)),
          ((        -1308.0,          632.0,          413.0),
           (         -553.0,          363.0,          318.0),
           (            0.0,            0.0,            0.0)),
          ((         -898.0,         -159.0,           24.0),
           (         2237.0,         1080.0,          312.0),
           (            0.0,            0.0,            0.0)),
          ((         -362.0,          -45.0,            8.0),
           (          377.0,         -110.0,         -100.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          241.0,          125.0),
           (           -7.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((        -1579.0,          396.0,          338.0),
           (          282.0,          115.0,           19.0),
           (            0.0,            0.0,            0.0)),
          ((        -7741.0,        -1414.0,           88.0),
           (            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -558.0,          -51.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,         2395.0,         1454.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -970.0,         -348.0,         -114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7532.0,          166.0,         -488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3881.0,        -1602.0,         -463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4908.0,         1465.0,           98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -181.0,         -440.0,         -288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,         5338.0,         2740.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1087.0,          777.0,          373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -375.0,         1265.0,          629.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -469.0,        -1220.0,         -564.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          307.0,         1018.0,          542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -318.0,         -475.0,         -235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -905.0,          648.0,          432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          467.0,        -1677.0,        -1000.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        36350.0,        41049.0,        18947.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -714.0,       -15683.0,        -8977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5657.0,       -13464.0,        -7752.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2945.0,         5446.0,         2857.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4671.0,        -6234.0,        -3168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7399.0,         1328.0,          334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35807.0,       -59022.0,       -38956.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3348999562.0,  -1359921020.0,  -1073790396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -70155.0,        -8680.0,         3234.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1087.0,         4809.0,         3051.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1782.0,        -8752.0,        -4911.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        39399.0,       124371.0,        68435.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -36706.0,      -114879.0,       -63165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3188.0,        -6995.0,        -3761.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          308.0,         1539.0,          803.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2973.0,        24609.0,        12977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1440.0,          131.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          766.0,         -218.0,         -237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1372.0,         -319.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          207.0,          -10.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -191.0,         -144.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3288.0,          199.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -700.0,         -270.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2687.0,         2110.0,          677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -216.0,          235.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -715.0,          322.0,          219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1867.0,         1478.0,          992.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,           80.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,         -917.0,         -457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -76.0,          109.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2077.0,        -3589.0,        -1929.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,         -206.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,          -31.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,            9.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          162.0,           11.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          521.0,         -505.0,         -283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6702.0,        -2765.0,         -772.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -518.0,          843.0,          485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1872.0,        -5986.0,        -2855.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1082.0,          783.0,          523.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          361.0,         -566.0,         -326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -685.0,          632.0,          415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3399.0,        -2496.0,        -1722.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12376.0,         8645.0,         6087.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16926.0,         7166.0,         5843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -289550594.0,    209641276.0,    142014761.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17398.0,        -7033.0,        -5660.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5142.0,         5015.0,         2240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -678.0,        -4521.0,        -2452.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -125.0,         1933.0,         1095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          158.0,         -386.0,         -235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,           98.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          576.0,          518.0,          224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,          -51.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1605.0,       -10218.0,        -5277.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          167.0,          -55.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -409.0,          537.0,          333.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,         -209.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,           52.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          -67.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -793.0,          282.0,          223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          260.0,         -136.0,         -101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1383.0,         -461.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,         -103.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          -13.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,         -107.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          613.0,         -790.0,         -492.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,          -29.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,          190.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -29.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -282.0,         1042.0,          568.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -214.0,          124.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           31.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           21.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -138.0,          200.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1611.0,          235.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,         -158.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1275.0,         1708.0,          769.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          228.0,         -274.0,         -172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,          202.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,         -100.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -409.0,          380.0,          247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3349.0,        -2763.0,        -1851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4849.0,        -3394.0,        -2347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     33704813.0,    -40956542.0,    -25373143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1700.0,         2503.0,         1526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4048.0,         1261.0,         1089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -291.0,          193.0,          137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          135.0,         -148.0,          -94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,           45.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -184.0,          -82.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -3.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1181.0,         2699.0,         1315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,         -147.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,           74.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -15.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,           98.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          259.0,         -228.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,           88.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          383.0,           39.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           29.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           28.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -122.0,          295.0,          171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -25.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,         -272.0,         -142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -47.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -72.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,           60.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -25.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -529.0,         -268.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -74.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,          275.0,          186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,         -680.0,         -447.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,          -59.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -745.0,         1191.0,          756.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -772.0,          591.0,          355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -3834532.0,      8588754.0,      4985416.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -252.0,          242.0,          232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1034.0,         -827.0,         -576.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           50.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -88.0,         -231.0,         -195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,          205.0,          173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,           20.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -21.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -492.0,         -616.0,         -278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           22.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,            7.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -10.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -86.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           73.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           16.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,          -28.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,           55.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           45.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,         -351.0,         -203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,         -134.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       266951.0,     -1840212.0,     -1010643.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,          191.0,          115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -124.0,           90.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          -24.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,          135.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -8.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           25.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          -13.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           88.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           63.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51513.0,       393368.0,       204773.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           38.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -85.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          -27.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -24.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34372.0,       -82521.0,       -40513.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           22.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12555.0,        16699.0,         7618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3847.0,        -3180.0,        -1300.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1076.0,          543.0,          178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -282.0,          -74.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,            5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -13437.0,        -1632.0,          233.0),
           (       -11304.0,        15864.0,         9162.0),
           (       -79430.0,       -76678.0,       -30027.0)),
          ((         3851.0,         2040.0,          709.0),
           (         1611.0,        94006.0,        47118.0),
           (        41836.0,        56889.0,        22761.0)),
          ((        -1870.0,         -795.0,         -245.0),
           (        -1351.0,         5825.0,         3092.0),
           (            0.0,            0.0,            0.0)),
          ((         1405.0,         -119.0,         -190.0),
           (       -35315.0,       -52398.0,       -57213.0),
           (            0.0,            0.0,            0.0)),
          ((        17559.0,        -7213.0,        -5410.0),
           (        82247.0,       126688.0,        54568.0),
           (            0.0,            0.0,            0.0)),
          ((         -625.0,          693.0,          438.0),
           (          843.0,         2088.0,          826.0),
           (            0.0,            0.0,            0.0)),
          ((        13400.0,        10571.0,         4375.0),
           (        -4483.0,       -17549.0,       -10976.0),
           (            0.0,            0.0,            0.0)),
          ((        -2627.0,         1025.0,          761.0),
           (        19916.0,         1363.0,        -1425.0),
           (            0.0,            0.0,            0.0)),
          ((         1960.0,          192.0,          -91.0),
           (         -657.0,         1088.0,          631.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,          120.0,           60.0),
           (         -823.0,          477.0,          320.0),
           (            0.0,            0.0,            0.0)),
          ((          440.0,         -326.0,         -198.0),
           (           13.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         1034.0,         -504.0,         -353.0),
           (           -5.0,            7.0,           23.0),
           (            0.0,            0.0,            0.0)),
          ((          722.0,        -8014.0,        -3904.0),
           (            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((          292.0,          775.0,          342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -639.0,         1800.0,         1207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          407.0,          832.0,          365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4137.0,        -2982.0,        -1766.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2268.0,         2913.0,         1135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3604.0,        -2342.0,         -559.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,          853.0,          496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -309.0,        -5588.0,        -2832.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -708.0,         -401.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1484.0,        -1661.0,         -479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1347.0,         2064.0,          809.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1288.0,        -1650.0,         -594.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          546.0,          812.0,          321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2803.0,        -1598.0,         -529.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3288.0,         2179.0,          700.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -66749.0,       -53312.0,       -19908.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22366.0,        26432.0,        10625.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16807.0,        10374.0,         3742.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10067.0,       -11423.0,        -4643.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13522.0,        12134.0,         4638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19750.0,       -14900.0,        -5546.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -45382.0,       -13471.0,        -2903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3256769444.0,   1517176515.0,   1148154860.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5131.0,       -43506.0,       -23596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3337.0,       -13493.0,        -6453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6264.0,        16168.0,         7520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -96137.0,      -218411.0,      -100851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        88999.0,       201548.0,        92998.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1901.0,         9475.0,         4851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2752.0,        -3403.0,        -1317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3439.0,       -23563.0,       -12609.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2527.0,        -1120.0,         -338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -827.0,        -1873.0,         -920.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1640.0,         1520.0,          591.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -134.0,         -208.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          387.0,          154.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1212.0,        -2887.0,        -1210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          798.0,          465.0,          146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2717.0,        -2190.0,         -712.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          201.0,         -143.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          663.0,         -376.0,         -241.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2056.0,         1681.0,         1110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,         -100.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -352.0,         -727.0,         -334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          -25.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4066.0,        -1830.0,         -521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,          134.0,          131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           83.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -360.0,           68.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          647.0,         -975.0,         -539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7201.0,        -1293.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,         -378.0,         -208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4687.0,         3468.0,         2315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -492.0,         -639.0,         -284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,          505.0,          245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -337.0,         -670.0,         -321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1254.0,         2804.0,         1350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8112.0,        -8780.0,        -3813.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5876.0,       -12417.0,        -5902.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -262112219.0,    236443608.0,    153481633.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2000.0,        -5041.0,        -2513.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3927.0,       -16995.0,        -8565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3458.0,        11621.0,         5773.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1651.0,        -4380.0,        -2140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          335.0,          890.0,          435.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -183.0,         -173.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -408.0,          664.0,          422.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,         -190.0,          -96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2411.0,       -10050.0,        -5100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -332.0,           40.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -179.0,         -503.0,         -257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          336.0,          -92.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,           30.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,          -23.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -203.0,         -530.0,         -251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           66.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1261.0,         -483.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          -27.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          240.0,          -82.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -203.0,          -22.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -474.0,          928.0,          549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,          -24.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,         -137.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -88.0,           14.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1156.0,         -197.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,          137.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           20.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           14.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,         -374.0,         -203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1624.0,          327.0,          334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,         -292.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1257.0,         1839.0,         1111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,         -408.0,         -199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          239.0,          121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,         -125.0,          -59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          162.0,          427.0,          207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2612.0,        -2303.0,         -942.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2554.0,        -4367.0,        -2035.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -25896881.0,     45450897.0,     26963929.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1100.0,         2769.0,         1345.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2441.0,        -4091.0,        -1917.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          206.0,          150.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -180.0,         -134.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           46.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          -14.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          174.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,          -23.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1534.0,        -2553.0,        -1200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           29.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,         -136.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,          -29.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           12.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,          -33.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -192.0,          -63.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,           -9.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -384.0,          -21.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -24.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,          314.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -23.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -301.0,           27.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           48.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,            7.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,            1.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -87.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -382.0,          142.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          -11.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -287.0,          481.0,          290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,           25.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -257.0,          -75.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          585.0,          249.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           88.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1022.0,         -569.0,         -157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -642.0,         -595.0,         -287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1720644.0,      9199062.0,      5092248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           73.0,          113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -748.0,         -638.0,         -288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,         -147.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          294.0,          741.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -271.0,         -663.0,         -247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -12.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           57.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -33.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -599.0,         -527.0,         -219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -19.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,           20.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           90.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,           27.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -24.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,           54.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          148.0,           80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           39.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -327.0,         -127.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -110.0,          -72.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       277234.0,      1863412.0,       966626.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -122.0,          -33.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -199.0,         -126.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,           53.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -196.0,         -100.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           23.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           44.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,          -12.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,           -6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       184108.0,       366446.0,       176650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -21.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,            3.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -17.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -13.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        64590.0,        67711.0,        29471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18833.0,        11099.0,         3976.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4963.0,         1366.0,          215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1210.0,            9.0,         -121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          273.0,          -74.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,          -35.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -9.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        -7082.0,        38232.0,        20395.0),
           (        14491.0,        -8200.0,        -5427.0),
           (        30917.0,        -7675.0,        -4709.0)),
          ((        -4391.0,        -7734.0,        -3592.0),
           (        60954.0,        47610.0,        18477.0),
           (       -23030.0,        57766.0,        31456.0)),
          ((         2150.0,         3964.0,         1852.0),
           (         4526.0,          361.0,         -379.0),
           (            0.0,            0.0,            0.0)),
          ((          579.0,         1125.0,          528.0),
           (        16758.0,        17125.0,        37510.0),
           (            0.0,            0.0,            0.0)),
          ((         9286.0,        15570.0,         7080.0),
           (       111136.0,       116022.0,        47879.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,        -1225.0,         -602.0),
           (         4497.0,         5241.0,         2396.0),
           (            0.0,            0.0,            0.0)),
          ((        13365.0,        11038.0,         4599.0),
           (       -16907.0,         2012.0,         1750.0),
           (            0.0,            0.0,            0.0)),
          ((        -2734.0,          639.0,          560.0),
           (        -9563.0,        -8760.0,        -3843.0),
           (            0.0,            0.0,            0.0)),
          ((        -1614.0,            0.0,          183.0),
           (         -457.0,        -1217.0,         -637.0),
           (            0.0,            0.0,            0.0)),
          ((         -151.0,          -92.0,          -33.0),
           (          169.0,            1.0,          -59.0),
           (            0.0,            0.0,            0.0)),
          ((         -121.0,          220.0,          122.0),
           (            5.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((        -1179.0,          786.0,          503.0),
           (         -109.0,         -184.0,         -113.0),
           (            0.0,            0.0,            0.0)),
          ((         9422.0,          622.0,         -613.0),
           (           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1253.0,          220.0,          232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          368.0,         2163.0,         1281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -973.0,          149.0,          179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6986.0,        -1972.0,        -1632.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3446.0,          222.0,          590.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4354.0,          -81.0,         -779.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -152.0,         -255.0,         -147.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1040.0,         5269.0,         2567.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          484.0,         -572.0,         -251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1310.0,        -1406.0,         -713.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1489.0,         1290.0,          740.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          968.0,         -820.0,         -415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -485.0,          298.0,          163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1258.0,         -132.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2760.0,         1599.0,         1015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        53748.0,         2148.0,        -3110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12196.0,         9655.0,         5410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2810.0,        10079.0,         6094.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4979.0,        -2514.0,        -1487.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5062.0,          315.0,          251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3359.0,         3084.0,         1702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15778.0,        62324.0,        37535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   3155829651.0,  -1670378037.0,  -1219460067.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51957.0,        17547.0,         3171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9416.0,        -2548.0,        -2356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10463.0,         3553.0,         2983.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       122211.0,       -56038.0,       -42251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -113554.0,        51657.0,        39019.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2705.0,         3516.0,         2238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2736.0,          511.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2725.0,        26081.0,        13749.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2440.0,        -1622.0,         -986.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1374.0,        -1210.0,         -854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1902.0,          702.0,          528.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          239.0,         -130.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -325.0,          -17.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1884.0,          534.0,          342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -704.0,          -98.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2704.0,         2060.0,          641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -235.0,          216.0,          128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -624.0,          279.0,          197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1796.0,         1619.0,         1055.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,            4.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -367.0,         -722.0,         -326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -18.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1512.0,         3783.0,         1971.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          394.0,         -168.0,         -149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,          -28.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -23.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          332.0,         -135.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          808.0,         -823.0,         -477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7441.0,         -760.0,          346.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,           62.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3075.0,         4260.0,         1857.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          406.0,          448.0,          168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,         -263.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          318.0,           71.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1464.0,          -74.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6383.0,        -5132.0,        -3612.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8778.0,        -3042.0,        -2788.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -231858032.0,    260694360.0,    163296141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15051.0,          865.0,         2168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4624.0,         -592.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -215.0,          811.0,          637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          980.0,         -276.0,         -334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -278.0,          121.0,          115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,          -56.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -755.0,         -321.0,          -90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,          -26.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3372.0,       -10003.0,        -4972.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -149.0,          282.0,          153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -536.0,          626.0,          409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          280.0,            5.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,            6.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           12.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,            0.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          702.0,         -302.0,         -228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -197.0,           45.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1207.0,         -419.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -151.0,          -29.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          403.0,         -106.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -359.0,          -17.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          747.0,         -913.0,         -580.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,          -24.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          277.0,          173.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,          -64.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,        -1055.0,         -550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -229.0,          141.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           28.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -23.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,           79.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,          352.0,          203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1730.0,         -487.0,         -432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -277.0,          145.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1266.0,        -1267.0,         -536.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -415.0,           32.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          154.0,           47.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,            0.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          361.0,           61.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2030.0,         1852.0,         1235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2563.0,          395.0,          510.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     17462777.0,    -48844030.0,    -27902156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1000.0,          545.0,          174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3820.0,        -1601.0,        -1252.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          189.0,           36.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -140.0,          -67.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           27.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          197.0,           36.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1913.0,         2390.0,         1073.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -84.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,         -172.0,          -90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -15.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,           33.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           56.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,           13.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          377.0,          -15.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           25.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          313.0,          171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -18.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          101.0,          255.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -50.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -29.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -93.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -292.0,          213.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,          -18.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          612.0,          176.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,          -48.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -434.0,           91.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,           40.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          620.0,         -530.0,         -325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          261.0,         -464.0,         -314.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       466990.0,      9413451.0,      4980087.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          355.0,          -44.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -943.0,          663.0,          430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           50.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,         -124.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          105.0,          113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,           11.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -714.0,         -441.0,         -162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           23.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           39.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -30.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,           32.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          -85.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,          -54.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           15.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           23.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,          -68.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -163.0,            6.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,            7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,          201.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,           77.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -802760.0,     -1761508.0,      -857748.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          104.0,         -142.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,         -134.0,          -78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           45.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          218.0,           57.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           19.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           24.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           -9.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -58.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -26.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       298895.0,       304160.0,       131490.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -49.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           74.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,            4.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -86347.0,       -44038.0,       -14573.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -19.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21904.0,         3610.0,         -342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5014.0,          740.0,          915.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1027.0,         -527.0,         -388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,          196.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -59.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           16.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        16830.0,       -14219.0,        -8653.0),
           (        -6399.0,        -8726.0,        -4017.0),
           (       -36381.0,        21949.0,        18751.0)),
          ((         1383.0,         4743.0,         2326.0),
           (       -61121.0,       -55015.0,       -23001.0),
           (        74444.0,       -17030.0,       -18507.0)),
          ((         -302.0,        -2308.0,        -1170.0),
           (         -676.0,        -3812.0,        -2282.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,           -9.0,           -3.0),
           (       -23722.0,        -7654.0,       -28609.0),
           (            0.0,            0.0,            0.0)),
          ((       -17327.0,         9656.0,         6635.0),
           (       -95556.0,      -150554.0,       -68586.0),
           (            0.0,            0.0,            0.0)),
          ((         2171.0,        -1470.0,         -968.0),
           (        -2591.0,        -6765.0,        -3235.0),
           (            0.0,            0.0,            0.0)),
          ((        13122.0,        11122.0,         4665.0),
           (       -21072.0,        -9883.0,        -3235.0),
           (            0.0,            0.0,            0.0)),
          ((        -2209.0,          112.0,          217.0),
           (         8485.0,          842.0,          -29.0),
           (            0.0,            0.0,            0.0)),
          ((         1754.0,         -504.0,         -447.0),
           (         1586.0,          -96.0,         -300.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          176.0,           82.0),
           (          143.0,         -362.0,         -259.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,         -397.0,         -202.0),
           (            0.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((          994.0,         -761.0,         -489.0),
           (          -30.0,           20.0,           43.0),
           (            0.0,            0.0,            0.0)),
          ((        -1082.0,         8097.0,         4023.0),
           (           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          733.0,         -633.0,         -424.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -675.0,         2509.0,         1594.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          931.0,         -259.0,         -303.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2892.0,        -1769.0,         -904.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3911.0,         -402.0,         -960.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5024.0,          312.0,         1145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          337.0,          233.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1851.0,        -5136.0,        -2376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1227.0,          415.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          635.0,         1291.0,          420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,        -1099.0,         -466.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -348.0,          991.0,          556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          203.0,         -408.0,         -246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2007.0,         1313.0,          876.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2989.0,         -616.0,         -720.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -58515.0,        -1264.0,         6471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14947.0,        -8384.0,        -6818.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          850.0,        -1684.0,        -1859.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5946.0,         2907.0,         2572.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5356.0,        -4009.0,        -3246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1450.0,         3407.0,         2469.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        84912.0,         3563.0,        -4873.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3046465480.0,   1819077271.0,   1287498600.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3650.0,        53876.0,        28658.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8154.0,         4693.0,         3712.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11686.0,        -5045.0,        -4450.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -164169.0,        53386.0,        54265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       151212.0,       -48987.0,       -49882.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7798.0,        -2849.0,        -2916.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1795.0,         -698.0,         -138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1837.0,       -25953.0,       -13662.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2029.0,         1492.0,          930.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -532.0,         -350.0,         -191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2095.0,         -181.0,         -411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,          -61.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          413.0,          -75.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,         2452.0,         1152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          821.0,           -8.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2733.0,        -2033.0,         -622.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          229.0,         -285.0,         -167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          563.0,         -485.0,         -296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1982.0,         1737.0,         1131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -29.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -517.0,         -686.0,         -302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,           17.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4337.0,         1332.0,          250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -403.0,          149.0,          142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,           29.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -52.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -158.0,          293.0,          163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          773.0,         -662.0,         -388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7481.0,          -13.0,          737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          242.0,          518.0,          249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4306.0,        -3561.0,        -2322.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          481.0,          227.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,           90.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           74.0,          177.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -132.0,         -403.0,         -243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3393.0,         4575.0,         2176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3429.0,         1438.0,          553.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -199104418.0,    282144200.0,    171357622.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2531.0,        12754.0,         6462.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,         9049.0,         4923.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          690.0,        -4577.0,        -2596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -465.0,         1169.0,          708.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,         -211.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,           72.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          203.0,         -648.0,         -395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4213.0,        -9786.0,        -4772.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,          257.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          597.0,         -317.0,         -229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -261.0,         -203.0,          -90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,           64.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -24.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,           37.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          335.0,          402.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1346.0,         -367.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           -9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          238.0,         -260.0,         -164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -177.0,          129.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -360.0,          908.0,          524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           40.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,         -154.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           59.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1210.0,          -88.0,         -168.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -229.0,          155.0,          109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           13.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           53.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          125.0,         -286.0,         -160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1541.0,          672.0,          506.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          111.0,          -19.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          922.0,        -1518.0,         -913.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,          -77.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           62.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,           17.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          992.0,         1592.0,          749.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,          637.0,          327.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -8603594.0,     51054608.0,     28165162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          726.0,         -534.0,         -349.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2279.0,         4160.0,         1965.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,         -108.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,           36.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -16.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           25.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,         -179.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2231.0,        -2163.0,         -918.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           64.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,         -123.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          -25.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,            5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,          -42.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          149.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          -55.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -375.0,           60.0,           71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -22.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -38.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          332.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           -3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          264.0,         -125.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           51.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -18.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           32.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -95.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -255.0,          297.0,          183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -42.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,         -477.0,         -260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -80.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,          329.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -113.0,         -651.0,         -361.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           85.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          432.0,          727.0,          365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,          -59.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2635351.0,      9222821.0,      4653690.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,          112.0,          103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          950.0,          701.0,          254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           94.0,           53.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -482.0,         -120.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          424.0,          106.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -37.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -788.0,         -329.0,          -94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           11.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -35.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,            7.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,           45.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           83.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          -53.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           16.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -19.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           77.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,         -148.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -14.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          147.0,          110.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -33.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1274590.0,      1541390.0,       691312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          217.0,           45.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,           76.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,          -39.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -224.0,          -12.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           17.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           21.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -34.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           12.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       384944.0,       212473.0,        73607.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,          -10.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,           18.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        96813.0,        14582.0,        -2244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           -9.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21242.0,        -4498.0,        -4604.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3987.0,        -2689.0,        -1850.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          571.0,         -923.0,         -552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,         -255.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -60.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -21841.0,       -36546.0,       -16987.0),
           (         -156.0,         7456.0,         3814.0),
           (        44924.0,       -14037.0,       -10629.0)),
          ((        -1101.0,         2418.0,         1383.0),
           (        39759.0,        43570.0,        19666.0),
           (       -62463.0,       -17126.0,        -6925.0)),
          ((           39.0,        -1751.0,         -926.0),
           (        -4043.0,        -1969.0,         -793.0),
           (            0.0,            0.0,            0.0)),
          ((         -695.0,        -1292.0,         -604.0),
           (        32017.0,         7663.0,        27541.0),
           (            0.0,            0.0,            0.0)),
          ((       -12417.0,       -13975.0,        -5964.0),
           (        55784.0,       109360.0,        52191.0),
           (            0.0,            0.0,            0.0)),
          ((         1732.0,          366.0,           14.0),
           (         5802.0,         6113.0,         2432.0),
           (            0.0,            0.0,            0.0)),
          ((        13125.0,        10227.0,         4177.0),
           (       -13196.0,        -9132.0,        -3567.0),
           (            0.0,            0.0,            0.0)),
          ((        -1537.0,          573.0,          389.0),
           (       -12935.0,          684.0,         1291.0),
           (            0.0,            0.0,            0.0)),
          ((         -917.0,          -35.0,           97.0),
           (          -59.0,          720.0,          293.0),
           (            0.0,            0.0,            0.0)),
          ((         -380.0,          121.0,           98.0),
           (         -196.0,           56.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((          226.0,          171.0,           64.0),
           (           -6.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((        -1200.0,         1105.0,          666.0),
           (           60.0,          106.0,           14.0),
           (            0.0,            0.0,            0.0)),
          ((        -7710.0,         -749.0,          378.0),
           (            0.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -727.0,          559.0,          342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          747.0,         1989.0,         1161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1314.0,          483.0,          360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7637.0,        -3217.0,        -2272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5199.0,         1146.0,         1131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5975.0,         -824.0,        -1247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -563.0,          132.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2575.0,         4710.0,         2178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1174.0,         1287.0,          642.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          533.0,          195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -971.0,          177.0,          247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          823.0,          -65.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -578.0,          144.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -366.0,          276.0,          169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1049.0,         -298.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        54772.0,         8251.0,         -310.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11099.0,          948.0,         1041.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9425.0,        -5634.0,        -3137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6400.0,        -1724.0,        -1351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7441.0,         2386.0,         1804.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6769.0,        -6983.0,        -4519.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8201.0,       -64754.0,       -35043.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2928939623.0,  -1962907916.0,  -1352100600.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -65352.0,       -15818.0,         -566.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4011.0,        -1338.0,         -699.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7841.0,          388.0,          515.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       119997.0,        -2139.0,        -6869.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -111100.0,         2392.0,         6601.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6839.0,        -1613.0,         -421.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1038.0,         -188.0,         -137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -687.0,        26154.0,        13531.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1613.0,        -1397.0,         -729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          934.0,         -871.0,         -603.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1713.0,         1037.0,          648.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,         -169.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -285.0,           24.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3128.0,        -1051.0,         -736.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -734.0,           64.0,          113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2671.0,         1973.0,          606.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -140.0,          420.0,          215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -418.0,          590.0,          332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1834.0,         1595.0,         1046.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -82.0,           73.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -627.0,         -658.0,         -272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           61.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          980.0,        -3958.0,        -2008.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          432.0,         -161.0,         -148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,           12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          -21.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,          -91.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          453.0,         -672.0,         -363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7120.0,          746.0,         1095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          527.0,          259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3562.0,        -5002.0,        -2162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -332.0,          324.0,          203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,         -342.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           64.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,         -690.0,         -371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -760.0,         -317.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1962.0,         -120.0,         -134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -164212961.0,    300565516.0,    177582374.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14008.0,        -1410.0,        -2345.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9231.0,        -3605.0,        -2805.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5015.0,         3438.0,         2257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1772.0,        -1498.0,         -937.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -336.0,          468.0,          273.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,         -175.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          740.0,          241.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,         -181.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5027.0,        -9360.0,        -4458.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,          -15.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,          154.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -202.0,          279.0,          174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          -63.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           56.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -474.0,          329.0,          208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          122.0,           43.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1377.0,         -332.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           23.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,         -340.0,         -184.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          218.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          403.0,        -1140.0,         -655.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           80.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          201.0,          -39.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           77.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          270.0,         1041.0,          511.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -211.0,          156.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           16.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           45.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          305.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1292.0,         -914.0,         -609.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,           97.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1900.0,          765.0,          202.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           91.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           30.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           34.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          133.0,           43.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1122.0,          786.0,          258.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -275.0,          454.0,          240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -463767.0,    -52031149.0,    -27747941.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -451.0,          302.0,          223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3658.0,         2469.0,         1696.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,          -52.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           21.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -183.0,           27.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           25.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2498.0,         1871.0,          736.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -14.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,         -129.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           60.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           75.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,         -271.0,         -146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           90.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          353.0,         -100.0,          -90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           26.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           39.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          316.0,          164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,            4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -194.0,         -220.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -53.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -71.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,          300.0,          178.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -616.0,          -22.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            0.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -116.0,          -39.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          243.0,           47.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -12.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -589.0,          -70.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -70.0,         -177.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      4690655.0,      8636100.0,      4127525.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,          -13.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          580.0,         -959.0,         -589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -52.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,          307.0,          104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,         -278.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -18.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,          -18.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -14.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -808.0,         -186.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           33.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,           54.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          -75.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,           37.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           17.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           18.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -84.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          147.0,          -59.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -9.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,          -23.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           34.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1661002.0,     -1217933.0,      -478538.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          199.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          151.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -57.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          213.0,          -37.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           19.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           30.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           23.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       433925.0,       100347.0,         8647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,            9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -63.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           31.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -94620.0,        16776.0,        18766.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           13.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -12.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16962.0,       -11840.0,        -8083.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2104.0,         4060.0,         2387.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,        -1078.0,         -572.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          136.0,          234.0,          111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,          -39.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((         8255.0,        35686.0,        17379.0),
           (         5131.0,         6760.0,         3078.0),
           (        15000.0,        -1704.0,        -2082.0)),
          ((         1329.0,         -620.0,         -453.0),
           (        60792.0,        64522.0,        27312.0),
           (       -44750.0,         6626.0,         6348.0)),
          ((          -42.0,          674.0,          358.0),
           (         1954.0,         3441.0,         1622.0),
           (            0.0,            0.0,            0.0)),
          ((         1495.0,         -402.0,         -334.0),
           (       -43531.0,       -30972.0,       -35667.0),
           (            0.0,            0.0,            0.0)),
          ((        14653.0,       -11119.0,        -7140.0),
           (       121954.0,       102527.0,        40568.0),
           (            0.0,            0.0,            0.0)),
          ((         -356.0,          874.0,          515.0),
           (          504.0,         2231.0,         1092.0),
           (            0.0,            0.0,            0.0)),
          ((        14583.0,         9301.0,         3518.0),
           (       -20404.0,        -1262.0,         2022.0),
           (            0.0,            0.0,            0.0)),
          ((        -2224.0,         1640.0,         1033.0),
           (        13889.0,         1163.0,         -355.0),
           (            0.0,            0.0,            0.0)),
          ((         2086.0,         -211.0,         -326.0),
           (         -540.0,         -344.0,         -204.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,           36.0,           24.0),
           (         -565.0,           15.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((          410.0,         -366.0,         -217.0),
           (            6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((          366.0,        -1036.0,         -557.0),
           (          128.0,           43.0,           42.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,        -7600.0,        -3632.0),
           (           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -355.0,          235.0,          138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          405.0,         2280.0,         1333.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -496.0,          121.0,          103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5136.0,        -3015.0,        -1913.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -517.0,          478.0,          190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1665.0,         -350.0,          276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -823.0,          220.0,          264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2121.0,        -4745.0,        -2248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1504.0,         -839.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -331.0,         -810.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -551.0,          750.0,          373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          449.0,         -328.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -322.0,          173.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1184.0,          418.0,          321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1425.0,         -777.0,         -315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22100.0,        -8019.0,        -1668.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13665.0,          743.0,         1448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1762.0,         -834.0,         -281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3562.0,         -843.0,         -720.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2184.0,         -142.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3604.0,          773.0,          475.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -67819.0,         9693.0,         7868.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -2803567209.0,   2101490526.0,   1413100734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13829.0,       -43457.0,       -24053.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6021.0,        -5069.0,        -3325.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6658.0,         4512.0,         3131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        79406.0,       -65288.0,       -43638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -73473.0,        60097.0,        40190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3681.0,         4575.0,         3065.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          372.0,         -671.0,         -260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1383.0,       -25494.0,       -13075.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -412.0,          672.0,          343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          187.0,         -967.0,         -587.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -263.0,         -281.0,         -114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -66.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,         -109.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1552.0,        -2144.0,         -839.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          341.0,           57.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2519.0,        -2023.0,         -647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,         -348.0,         -161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          395.0,         -631.0,         -350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1961.0,         1797.0,         1160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,          -74.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -861.0,         -457.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,          -17.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4414.0,         -817.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -384.0,          102.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -110.0,           36.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           14.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -261.0,          249.0,          152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          497.0,         -911.0,         -495.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7016.0,         1900.0,         1693.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          357.0,           22.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4739.0,         3262.0,         2217.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          163.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,          207.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,          196.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -343.0,         -103.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3405.0,         4748.0,         1908.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4173.0,         5262.0,         2085.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -127560115.0,    315752194.0,    181899390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3050.0,       -10339.0,        -5055.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,        -7775.0,        -4237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           59.0,         3622.0,         2015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -228.0,        -1145.0,         -622.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,          139.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,           43.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,          740.0,          410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -80.0,         -106.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5684.0,        -9107.0,        -4251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -219.0,          135.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -207.0,         -632.0,         -322.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          295.0,           31.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,          -32.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -41.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -550.0,         -283.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          106.0,            4.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1378.0,         -216.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,          -25.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,         -174.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           41.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -320.0,         1010.0,          576.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           14.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -163.0,          -60.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           35.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1105.0,          299.0,          265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -186.0,          158.0,          106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           18.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           49.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,         -347.0,         -182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1137.0,         1194.0,          743.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          111.0,         -177.0,         -105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -846.0,         1936.0,         1122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,         -138.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,          116.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           -1.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,          160.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2404.0,          189.0,         -176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2090.0,          136.0,         -180.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      9518698.0,     51747530.0,     26659353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -469.0,          459.0,          307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2516.0,        -3103.0,        -1380.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -83.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,            7.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           20.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          121.0,          135.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,            7.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2716.0,        -1604.0,         -571.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           52.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -102.0,         -141.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          -12.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -56.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -145.0,           78.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          -49.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -341.0,          124.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -23.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,          320.0,          161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           11.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -199.0,          191.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           53.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           12.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,          -79.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,          351.0,          195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -15.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,          537.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           19.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          -42.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -81.0,          156.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           47.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          551.0,         -407.0,         -259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          517.0,         -153.0,         -156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      6543682.0,      7678128.0,      3424080.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          201.0,         -101.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1039.0,         -317.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,         -116.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,          607.0,          290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,         -541.0,         -259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,           32.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -18.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -830.0,          -55.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           10.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           12.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,           63.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,           69.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           67.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -16.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           83.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          101.0,          121.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,           -5.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,         -176.0,         -100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,          -62.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1935948.0,       812804.0,       233691.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,           28.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -172.0,           -9.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,            2.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -199.0,           77.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,           20.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -50.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -31.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       441054.0,       -21411.0,       -57125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          -24.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           41.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           41.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        80048.0,       -45932.0,       -32830.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9794.0,       -17165.0,       -10185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -231.0,        -4560.0,        -2412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -633.0,         -946.0,         -440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -253.0,         -139.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,           -3.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        31660.0,         4745.0,         -144.0),
           (         -565.0,       -18397.0,        -9281.0),
           (         9385.0,        11932.0,         7236.0)),
          ((        -8853.0,        -3406.0,         -978.0),
           (        14216.0,       -12103.0,        -7479.0),
           (        10977.0,        19502.0,         7662.0)),
          ((         4340.0,         1784.0,          541.0),
           (         3703.0,        -1857.0,        -1425.0),
           (            0.0,            0.0,            0.0)),
          ((         1127.0,          822.0,          323.0),
           (        24957.0,        17874.0,        26143.0),
           (            0.0,            0.0,            0.0)),
          ((        13163.0,        13249.0,         5512.0),
           (        39639.0,         1670.0,        -3021.0),
           (            0.0,            0.0,            0.0)),
          ((         -211.0,        -1304.0,         -632.0),
           (         3059.0,          505.0,          193.0),
           (            0.0,            0.0,            0.0)),
          ((        14590.0,        10073.0,         3895.0),
           (       -26384.0,        -9832.0,         -689.0),
           (            0.0,            0.0,            0.0)),
          ((        -2314.0,          917.0,          664.0),
           (        -3128.0,         1727.0,          510.0),
           (            0.0,            0.0,            0.0)),
          ((        -1429.0,          498.0,          438.0),
           (         1244.0,         -821.0,         -660.0),
           (            0.0,            0.0,            0.0)),
          ((         -244.0,          -89.0,          -24.0),
           (          622.0,          -89.0,         -193.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          362.0,          180.0),
           (            5.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         -565.0,         1114.0,          610.0),
           (          -70.0,         -122.0,          -92.0),
           (            0.0,            0.0,            0.0)),
          ((         9287.0,         -589.0,        -1174.0),
           (           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1033.0,          262.0,          226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          737.0,         2236.0,         1272.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -717.0,           27.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5425.0,        -3160.0,        -2093.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2680.0,         -224.0,          290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3791.0,          110.0,         -657.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,         -383.0,         -232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3169.0,         4678.0,         2075.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          704.0,         -769.0,         -365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          882.0,        -1177.0,         -539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -800.0,          978.0,          509.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          473.0,         -387.0,         -126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -261.0,          111.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1128.0,         1101.0,          694.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1133.0,         1101.0,          556.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        42419.0,         1650.0,        -2015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1609.0,         1964.0,          243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1007.0,         6872.0,         4000.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1483.0,          501.0,          441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1944.0,        -2627.0,        -1482.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2424.0,         6808.0,         3337.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        31043.0,        56097.0,        25027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2670725259.0,  -2234451587.0,  -1470333232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        49421.0,        34911.0,        11940.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5111.0,         -160.0,         -517.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3874.0,          156.0,          331.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        33714.0,       -10214.0,        -6585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31665.0,         9634.0,         6253.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1258.0,         1042.0,          330.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1672.0,          826.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          957.0,        25882.0,        13259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1295.0,        -1472.0,         -795.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          883.0,         -906.0,         -662.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -671.0,          512.0,          293.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,         -124.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -219.0,          -24.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1773.0,         1377.0,          726.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -449.0,         -104.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2570.0,         2046.0,          649.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,          298.0,          150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -273.0,          538.0,          300.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1730.0,         1643.0,         1059.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,           60.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -771.0,         -426.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -391.0,         4072.0,         2010.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          419.0,          -70.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,          -43.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -30.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           77.0,         -283.0,         -156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          346.0,         -909.0,         -473.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6396.0,         2568.0,         1967.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -279.0,          164.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4582.0,         3496.0,         1286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -244.0,          689.0,          376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          213.0,         -392.0,         -220.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -224.0,          203.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1374.0,        -1453.0,         -885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10836.0,         3033.0,         2704.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9645.0,         6968.0,         4604.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -89529506.0,    327542544.0,    184261649.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10241.0,        -3836.0,         -865.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13745.0,         4263.0,         3654.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8472.0,        -3571.0,        -2718.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2892.0,         1599.0,         1119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          665.0,         -292.0,         -215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,          -21.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -669.0,           71.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,           26.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6424.0,        -8815.0,        -4017.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -148.0,          349.0,          193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -359.0,          533.0,          335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,          148.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -63.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           30.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -68.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,         -407.0,         -227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,          -39.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1360.0,         -112.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,          -71.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          -52.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,         -113.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          292.0,         -955.0,         -550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -65.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          263.0,           67.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,          -65.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -521.0,         -972.0,         -451.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,          151.0,          105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           34.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -21.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           83.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,          318.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          810.0,        -1280.0,         -755.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           75.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1975.0,         -537.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,          -80.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,          108.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -16.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          201.0,          257.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1583.0,        -2759.0,        -1605.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,        -4104.0,        -2131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -18345064.0,    -50210879.0,    -24925499.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          725.0,         2273.0,         1126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3668.0,        -2504.0,        -1717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          106.0,          173.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,         -163.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           62.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          135.0,          -85.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -26.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2928.0,         1299.0,          386.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          -88.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,         -165.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -23.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           64.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           42.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           33.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          322.0,         -152.0,         -114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           18.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          126.0,          303.0,          148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,           23.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          257.0,          158.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -57.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,          -71.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,          365.0,          189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -29.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          608.0,         -101.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,          -55.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          140.0,          182.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -350.0,         -448.0,         -207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,          -26.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          366.0,          982.0,          485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          260.0,          657.0,          307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      8115798.0,      6389460.0,      2573125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          172.0,           45.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -603.0,          819.0,          494.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          128.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -193.0,         -596.0,         -315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          180.0,          529.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           15.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           35.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -834.0,           69.0,          124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           18.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,           32.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -25.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,           70.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,          -56.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -79.0,           -2.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           10.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,          -71.0,          -33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          107.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          -21.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -253.0,         -180.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -130.0,          -39.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2081034.0,      -353032.0,        26898.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,         -168.0,          -93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,         -156.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,           56.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,         -118.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -20.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,            4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -11.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -37.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,           11.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           56.0,           -3.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       405636.0,      -141170.0,      -117425.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -10.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           54.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,            8.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           49.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -54992.0,        69088.0,        42605.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          955.0,       -19566.0,       -10552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2516.0,         4078.0,         1918.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1062.0,         -565.0,         -192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          289.0,           -1.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,           34.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -15.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -32086.0,       -20261.0,        -7445.0),
           (       -15593.0,         3366.0,         3008.0),
           (       -92639.0,       -18605.0,         1698.0)),
          ((         5091.0,         4901.0,         2068.0),
           (       -99147.0,         1619.0,         8716.0),
           (       139197.0,        27637.0,        -2686.0)),
          ((        -2354.0,        -2879.0,        -1273.0),
           (        -4305.0,        -3212.0,        -1374.0),
           (            0.0,            0.0,            0.0)),
          ((         -292.0,         -151.0,          -52.0),
           (       -27784.0,       -16270.0,       -22106.0),
           (            0.0,            0.0,            0.0)),
          ((       -13695.0,        13337.0,         8167.0),
           (      -135059.0,       -88382.0,       -33283.0),
           (            0.0,            0.0,            0.0)),
          ((         1563.0,        -1850.0,        -1097.0),
           (        -3252.0,        -4868.0,        -2287.0),
           (            0.0,            0.0,            0.0)),
          ((        14683.0,         9962.0,         3827.0),
           (        -1382.0,       -10066.0,        -3116.0),
           (            0.0,            0.0,            0.0)),
          ((        -2521.0,          590.0,          492.0),
           (        12827.0,       -13057.0,        -7457.0),
           (            0.0,            0.0,            0.0)),
          ((         1303.0,         -929.0,         -632.0),
           (         1063.0,         1179.0,          402.0),
           (            0.0,            0.0,            0.0)),
          ((          182.0,          215.0,           82.0),
           (         -265.0,          541.0,          242.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,         -495.0,         -217.0),
           (            7.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((          559.0,        -1035.0,         -587.0),
           (         -190.0,          -43.0,           23.0),
           (            0.0,            0.0,            0.0)),
          ((          215.0,         8086.0,         3892.0),
           (           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          861.0,         -601.0,         -417.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -370.0,         2463.0,         1522.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1417.0,          -84.0,         -252.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          586.0,        -3623.0,        -1662.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5669.0,          188.0,         -808.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6242.0,          102.0,         1166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1008.0,          479.0,          128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4338.0,        -4416.0,        -1792.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1596.0,          652.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          750.0,          193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1114.0,         -156.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1269.0,          308.0,          281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          673.0,          -20.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2227.0,         1227.0,          842.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5784.0,          199.0,         -526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -88399.0,        -6220.0,         6265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35847.0,         3425.0,        -2360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9670.0,         3496.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13918.0,        -2278.0,          503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14535.0,         3149.0,         -135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11408.0,        -5237.0,        -1480.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        83768.0,       -32346.0,       -22814.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -2530757556.0,   2361421503.0,   1523640190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21466.0,        53721.0,        30278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13667.0,         1604.0,         2604.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19767.0,         -488.0,        -2829.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -267332.0,        -9775.0,        30735.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       246934.0,         9768.0,       -27960.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10547.0,        -1034.0,        -2214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3953.0,        -1966.0,         -602.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,       -25278.0,       -13125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2807.0,         1104.0,          801.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1008.0,         -485.0,         -232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3072.0,          456.0,         -160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -331.0,         -115.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          440.0,           -7.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          845.0,         1912.0,          844.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          819.0,           87.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2701.0,        -2059.0,         -637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          116.0,         -302.0,         -167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          124.0,         -669.0,         -356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1731.0,         1891.0,         1187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -128.0,          -78.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -724.0,         -195.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,          -57.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4540.0,          345.0,         -245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -464.0,           70.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           24.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -14.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,          280.0,          135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          528.0,        -1065.0,         -575.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6102.0,         3606.0,         2485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,         -237.0,         -120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4188.0,        -2962.0,        -1980.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,         -794.0,         -415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          434.0,          219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -429.0,         -504.0,         -211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1666.0,         2798.0,         1245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2917.0,       -12737.0,        -6238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8762.0,       -13901.0,        -6156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -50537925.0,    335819572.0,    184649821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1236.0,        19374.0,         9970.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -799.0,          135.0,          389.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1382.0,         3659.0,         1581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -685.0,        -2346.0,        -1081.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,          575.0,          273.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -45.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -314.0,         -546.0,         -281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,         -119.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7272.0,        -8261.0,        -3638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          222.0,          222.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          416.0,         -171.0,         -136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,         -412.0,         -211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          166.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -48.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           93.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          720.0,           -1.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -164.0,          148.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1255.0,         -187.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,           74.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          283.0,         -387.0,         -233.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -402.0,          290.0,          196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,          894.0,          468.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -164.0,          100.0,           71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -62.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,           72.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1043.0,         -562.0,         -396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,          187.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,            8.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,         -385.0,         -189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -472.0,         1524.0,          847.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -192.0,         -195.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          715.0,        -1531.0,         -894.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -359.0,         -281.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          171.0,          137.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -150.0,          -16.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          541.0,          114.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3517.0,        -2369.0,         -861.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5323.0,         -900.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     26728787.0,     47460211.0,     22589288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2298.0,          845.0,          187.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2630.0,         3721.0,         1693.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          322.0,          -27.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -233.0,          -22.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,            1.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           20.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,         -106.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3070.0,         -946.0,         -183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,           49.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,         -127.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          -18.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,            1.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          -62.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           68.0,           70.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -56.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -303.0,          190.0,          132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -14.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          -22.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          174.0,          298.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           31.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,         -246.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           55.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -74.0,          -85.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          109.0,          384.0,          191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -45.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -263.0,         -449.0,         -211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -65.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,          242.0,          136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          329.0,         -436.0,         -263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          165.0,           85.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1171.0,          170.0,          215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -995.0,          238.0,          231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      9339745.0,      4826369.0,      1611642.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,           20.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1410.0,          452.0,           90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,          -37.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          483.0,          292.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -446.0,         -262.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,            6.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -795.0,          211.0,          196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -22.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           78.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,           45.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,          -70.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,           -6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,           58.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,          -83.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -43.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -262.0,          228.0,          148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,          164.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2086684.0,      -130480.0,      -285743.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          236.0,          -77.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,           -3.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          -19.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,          154.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           20.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           -7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           78.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           53.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       331188.0,      -247389.0,      -166458.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           56.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          -80.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           52.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           16.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22736.0,       -83222.0,       -46817.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -22.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           13.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8050.0,       -18636.0,        -9123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4263.0,        -2722.0,        -1013.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1212.0,          -32.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -236.0,          142.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           59.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -18832.0,         2554.0,         2842.0),
           (        18269.0,         8937.0,         2985.0),
           (        72254.0,       -65293.0,       -41676.0)),
          ((         2228.0,        -2199.0,        -1320.0),
           (       122424.0,        51968.0,        17262.0),
           (      -110329.0,        29987.0,        28150.0)),
          ((        -1498.0,         1395.0,          845.0),
           (        -4342.0,        -1315.0,         -323.0),
           (            0.0,            0.0,            0.0)),
          ((        -1053.0,         -365.0,          -89.0),
           (        43444.0,        21197.0,        20059.0),
           (            0.0,            0.0,            0.0)),
          ((       -15897.0,       -11288.0,        -4259.0),
           (       182768.0,       130284.0,        53662.0),
           (            0.0,            0.0,            0.0)),
          ((         1774.0,          459.0,           67.0),
           (         9470.0,         6323.0,         2333.0),
           (            0.0,            0.0,            0.0)),
          ((        14456.0,         9015.0,         3336.0),
           (       -14175.0,        17985.0,        13298.0),
           (            0.0,            0.0,            0.0)),
          ((        -1346.0,          917.0,          543.0),
           (       -20346.0,          254.0,         1489.0),
           (            0.0,            0.0,            0.0)),
          ((         -742.0,          183.0,          199.0),
           (        -2799.0,          119.0,          249.0),
           (            0.0,            0.0,            0.0)),
          ((         -398.0,          248.0,          170.0),
           (         -848.0,            0.0,           29.0),
           (            0.0,            0.0,            0.0)),
          ((          474.0,           33.0,          -36.0),
           (            0.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((         -586.0,         1547.0,          841.0),
           (          -91.0,          108.0,           43.0),
           (            0.0,            0.0,            0.0)),
          ((        -7582.0,         -139.0,          619.0),
           (            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         -761.0,         1171.0,          682.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1344.0,         1484.0,          809.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1512.0,         1326.0,          843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6819.0,        -6365.0,        -3898.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5992.0,         4009.0,         2793.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6555.0,        -3229.0,        -2645.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -893.0,          746.0,          414.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4738.0,         3224.0,         1209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1225.0,         1763.0,          903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          272.0,         -183.0,         -255.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1378.0,         1535.0,         1063.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1274.0,        -1129.0,         -726.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -782.0,          750.0,          478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          469.0,         -155.0,         -169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3092.0,         1513.0,         1153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        72625.0,       -26651.0,       -21319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23857.0,        18518.0,        12320.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14142.0,         2343.0,         1841.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10834.0,        -8930.0,        -5808.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11700.0,        11104.0,         7048.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9201.0,       -14597.0,        -8794.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -58490.0,       -47751.0,       -16473.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2383988210.0,  -2482088960.0,  -1572887393.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -72151.0,       -29211.0,        -6117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7044.0,        -6493.0,        -4235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12188.0,         7975.0,         5589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       182343.0,      -107846.0,       -77330.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -168563.0,       100270.0,        71787.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9533.0,         3231.0,         2821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1804.0,        -1384.0,         -909.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1617.0,        27104.0,        13844.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1613.0,        -2523.0,        -1383.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1048.0,        -1288.0,         -875.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2001.0,         1925.0,         1218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          139.0,         -237.0,         -135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -372.0,          172.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2628.0,        -2056.0,        -1186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -735.0,          285.0,          242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2635.0,         1894.0,          563.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          509.0,          256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          680.0,          342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1806.0,         1727.0,         1110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           57.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -999.0,         -166.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,            6.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -212.0,        -4064.0,        -1949.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          466.0,         -109.0,         -129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           47.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -49.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -90.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          556.0,         -698.0,         -390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5776.0,         3880.0,         2597.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          843.0,          388.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5623.0,        -4247.0,        -1539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          784.0,          153.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -314.0,         -203.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1005.0,         -110.0,         -172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4358.0,          501.0,          754.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16064.0,        -2154.0,        -2884.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15352.0,        -5492.0,        -4554.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -11009301.0,    340479215.0,    183052464.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7701.0,          941.0,         -364.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        22872.0,        -6375.0,        -5822.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17014.0,         5673.0,         4821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6678.0,        -2353.0,        -1958.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1471.0,          681.0,          518.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,         -249.0,         -161.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          469.0,         -193.0,         -174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          193.0,         -184.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8043.0,        -7687.0,        -3247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          323.0,          -22.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,          209.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          159.0,          231.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,          -31.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,          -11.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -95.0,           71.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          101.0,          445.0,          208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,           52.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1251.0,          -90.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,           56.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,         -369.0,         -209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -230.0,          283.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,        -1259.0,         -698.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          102.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          252.0,         -221.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          124.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          770.0,          824.0,          349.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,          176.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           33.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          320.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          410.0,        -1565.0,         -864.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -238.0,          252.0,          157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2417.0,         -142.0,         -335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -252.0,          328.0,          197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          123.0,          -93.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,          140.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          338.0,         -440.0,         -269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2510.0,         3454.0,         2086.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1986.0,         4374.0,         2517.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -34463459.0,    -43560924.0,    -19706973.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1337.0,        -2443.0,        -1432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3807.0,         4000.0,         2520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          180.0,         -353.0,         -205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -156.0,          189.0,          117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,          -80.0,          -47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           49.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,           99.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           58.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3157.0,          546.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           36.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -36.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,          -82.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,           30.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           25.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,         -179.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,           50.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          266.0,         -213.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,            8.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           22.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          199.0,          269.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           32.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -293.0,          -70.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -55.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           15.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -22.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -42.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,          293.0,          138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           31.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,          225.0,          174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           83.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,         -412.0,         -216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,          906.0,          478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           42.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,        -1490.0,         -778.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -383.0,         -650.0,         -295.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     10161412.0,      3055929.0,       581033.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,         -220.0,         -143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,        -1033.0,         -549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,         -165.0,          -93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          899.0,          489.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,         -806.0,         -439.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -27.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -28.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -722.0,          341.0,          258.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -17.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           15.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           80.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,          -31.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           75.0,          -27.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -85.0,          -42.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,         -141.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,          -40.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          321.0,          279.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          110.0,            4.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1952350.0,       605127.0,       525379.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          153.0,           72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          176.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,          -78.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,         -179.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -18.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           15.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           -3.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           39.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -109.0,           -9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,            4.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       224835.0,      -329655.0,      -199398.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          -25.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,           -5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           51.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -13.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12481.0,        86450.0,        44897.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15683.0,       -14531.0,        -6141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5094.0,          780.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1045.0,          506.0,          379.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,         -237.0,         -138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           62.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        44199.0,        10727.0,         1509.0),
           (        -4758.0,        -9885.0,        -4613.0),
           (        83015.0,        61195.0,        22441.0)),
          ((        -6101.0,         3113.0,         2085.0),
           (         3794.0,         -250.0,          135.0),
           (      -112141.0,       -53727.0,       -16175.0)),
          ((         3586.0,        -1770.0,        -1195.0),
           (         4550.0,         2090.0,          405.0),
           (            0.0,            0.0,            0.0)),
          ((         1535.0,        -1370.0,         -824.0),
           (       -63200.0,       -37377.0,       -21972.0),
           (            0.0,            0.0,            0.0)),
          ((        10637.0,       -13710.0,        -8089.0),
           (        28912.0,        38397.0,        16917.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,          409.0,          246.0),
           (        -3654.0,         1103.0,         1038.0),
           (            0.0,            0.0,            0.0)),
          ((        15522.0,         8398.0,         2873.0),
           (       -46546.0,         4448.0,        10612.0),
           (            0.0,            0.0,            0.0)),
          ((        -1767.0,         1538.0,          930.0),
           (         1427.0,         -567.0,          441.0),
           (            0.0,            0.0,            0.0)),
          ((         1954.0,         -777.0,         -619.0),
           (         -350.0,        -2329.0,        -1281.0),
           (            0.0,            0.0,            0.0)),
          ((         -168.0,           -5.0,            9.0),
           (          -42.0,         -585.0,         -376.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,         -349.0,         -198.0),
           (            0.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((         -393.0,        -1262.0,         -598.0),
           (          247.0,           79.0,           30.0),
           (            0.0,            0.0,            0.0)),
          ((         -790.0,        -7283.0,        -3400.0),
           (            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         -826.0,          -92.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1305.0,         2547.0,         1357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1121.0,         -434.0,         -109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5053.0,        -3002.0,        -1942.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2479.0,        -1577.0,         -643.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -282.0,         1448.0,         1064.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1361.0,         -290.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3634.0,        -3166.0,        -1319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2093.0,        -1140.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          471.0,         -134.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1883.0,         -296.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1702.0,          739.0,          293.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -936.0,         -330.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          514.0,         2105.0,         1018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4854.0,        -2695.0,         -899.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13388.0,        29916.0,        13785.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -40006.0,       -18884.0,        -5630.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9227.0,        -9797.0,        -3588.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13519.0,         7871.0,         2641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13287.0,       -10334.0,        -3843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9072.0,        14671.0,         6211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -62971.0,        28411.0,        20656.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -2230811089.0,   2596109480.0,   1617932056.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18763.0,       -53883.0,       -29792.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13908.0,          596.0,        -1244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17367.0,        -3515.0,          117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       220350.0,        46546.0,        -1029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -203771.0,       -43237.0,          832.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7647.0,          653.0,         1459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3029.0,         1312.0,          469.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5837.0,       -26721.0,       -13230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1448.0,         1971.0,          822.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1037.0,         -221.0,         -311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1847.0,        -1697.0,         -674.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           83.0,           76.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -259.0,         -267.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1779.0,        -1302.0,         -433.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,         -203.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2323.0,        -1926.0,         -615.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -258.0,         -459.0,         -197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,         -700.0,         -354.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1818.0,         1893.0,         1195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          -17.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1051.0,          -14.0,           84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,           -2.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4480.0,          241.0,          520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -389.0,           65.0,          100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -139.0,           -8.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,           21.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,          313.0,          172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          168.0,         -811.0,         -411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4792.0,         4556.0,         2855.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          242.0,          466.0,          210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4014.0,         2892.0,         1934.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         1046.0,          528.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -110.0,         -201.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          905.0,          457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -295.0,        -3995.0,        -2040.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5108.0,        16732.0,         8131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3764.0,        13607.0,         6714.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     28649773.0,    341473394.0,    179485487.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2022.0,       -16644.0,        -8526.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3545.0,          -83.0,          288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2995.0,        -3741.0,        -2266.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1403.0,         2026.0,         1212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          362.0,         -610.0,         -361.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          200.0,          112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          647.0,          490.0,          205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -207.0,           17.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8339.0,        -7312.0,        -3015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -164.0,          180.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -134.0,         -774.0,         -410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          104.0,          183.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,         -111.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           31.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          -60.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -860.0,           85.0,          133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          318.0,          -88.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1502.0,           75.0,          216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           74.0,          -31.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -487.0,         -114.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          406.0,           26.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -244.0,         1063.0,          598.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,           31.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -272.0,           62.0,           61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,           33.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -833.0,          734.0,          463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -144.0,          172.0,          109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,           21.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           49.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -130.0,         -239.0,         -112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,         1477.0,          782.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          130.0,            9.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -252.0,         1731.0,          951.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          286.0,          188.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -125.0,          -38.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          102.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -323.0,         -421.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4952.0,         2676.0,          870.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3151.0,         2330.0,          889.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     41365079.0,     38604904.0,     16346567.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1234.0,        -1722.0,         -769.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3359.0,        -2162.0,         -792.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,         -338.0,         -162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,          171.0,           88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -72.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           36.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,           -7.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           35.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3179.0,         -236.0,          206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           46.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,         -125.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           -7.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -7.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,          -34.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,          155.0,           87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -53.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -245.0,          239.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,            4.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          244.0,          251.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           31.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          269.0,          141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           54.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           10.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,          -26.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          290.0,          265.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,            7.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          456.0,          406.0,          167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,           27.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          426.0,          -49.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -963.0,          114.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -87.0,          -38.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1879.0,         -254.0,         -351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          586.0,         -106.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     10545899.0,      1153371.0,      -474939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          636.0,          -64.0,         -131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1398.0,           25.0,          171.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,          -11.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -619.0,            2.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          555.0,            1.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -1.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,          -26.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           15.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -647.0,          437.0,          301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           -2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           32.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           81.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,           17.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           57.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,           14.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          158.0,           25.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           -7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          307.0,         -308.0,         -197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -56.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1686904.0,     -1039277.0,      -729835.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -140.0,          120.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -119.0,           97.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,          -35.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,          191.0,          106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -12.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -13.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,         -102.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -39.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        96698.0,      -380140.0,      -213113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -45.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           81.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           47.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -19.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -46077.0,       -78355.0,       -37098.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20642.0,        -7949.0,        -2111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4831.0,         1328.0,         1209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -597.0,          909.0,          548.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          256.0,          131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           57.0,           42.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((         1240.0,       -21497.0,       -11013.0),
           (        -9897.0,         2175.0,         1999.0),
           (        -9825.0,        17529.0,        10369.0)),
          ((        -1234.0,           25.0,          110.0),
           (       -13400.0,        51144.0,        26839.0),
           (        79862.0,        30891.0,         4806.0)),
          ((          461.0,         -134.0,         -100.0),
           (         1821.0,        -2482.0,        -1611.0),
           (            0.0,            0.0,            0.0)),
          ((          905.0,          529.0,          192.0),
           (        33656.0,        11816.0,         9700.0),
           (            0.0,            0.0,            0.0)),
          ((        16740.0,         9912.0,         3456.0),
           (       -11495.0,        37174.0,        19297.0),
           (            0.0,            0.0,            0.0)),
          ((         -772.0,        -1150.0,         -494.0),
           (         2017.0,          -20.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        16067.0,         8791.0,         3006.0),
           (       -22254.0,       -17863.0,        -3138.0),
           (            0.0,            0.0,            0.0)),
          ((        -2233.0,         1506.0,          956.0),
           (         2169.0,        13292.0,         5842.0),
           (            0.0,            0.0,            0.0)),
          ((        -1102.0,         1036.0,          693.0),
           (         2834.0,         -536.0,         -666.0),
           (            0.0,            0.0,            0.0)),
          ((         -321.0,          -60.0,           -1.0),
           (          848.0,         -133.0,         -230.0),
           (            0.0,            0.0,            0.0)),
          ((          295.0,          429.0,          189.0),
           (            3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((           95.0,         1127.0,          558.0),
           (           10.0,          -12.0,          -16.0),
           (            0.0,            0.0,            0.0)),
          ((         8938.0,        -1828.0,        -1740.0),
           (           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -862.0,          290.0,          230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1158.0,         2210.0,         1190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -602.0,          -58.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3669.0,        -4040.0,        -2421.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2357.0,         -463.0,          186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3562.0,          117.0,         -691.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,         -414.0,         -247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4802.0,         3272.0,         1208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          844.0,         -938.0,         -448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          573.0,        -1045.0,         -447.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -272.0,          829.0,          406.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,         -134.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,           45.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -495.0,         1790.0,          987.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          261.0,          677.0,          261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        30032.0,        -3525.0,        -4060.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6822.0,        -2635.0,        -2503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4114.0,         5341.0,         3086.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1478.0,         1877.0,         1257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1154.0,        -3064.0,        -1729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2129.0,         6965.0,         2994.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63312.0,        33403.0,         7543.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   2071710787.0,  -2703173589.0,  -1658659023.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        59044.0,        39663.0,        13880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1102.0,         2383.0,         1170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2205.0,        -2310.0,        -1535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -48703.0,        23639.0,        18936.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        44890.0,       -21361.0,       -17198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4556.0,        -1139.0,        -1236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          406.0,          821.0,          332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4685.0,        25607.0,        12757.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,        -1401.0,         -673.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          341.0,         -631.0,         -496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          632.0,          383.0,          121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -110.0,          -82.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,           11.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1406.0,         2025.0,          999.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,         -103.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2417.0,         2046.0,          664.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          199.0,          319.0,          139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          152.0,          583.0,          287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1684.0,         1723.0,         1093.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,           75.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -859.0,           66.0,          109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            0.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          742.0,         4093.0,         1918.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          420.0,            6.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,          -35.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,          -40.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -140.0,         -270.0,         -132.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,        -1027.0,         -508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3714.0,         5275.0,         3109.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -602.0,           19.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5754.0,         3149.0,          977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -702.0,          475.0,          321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          403.0,         -291.0,         -195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -887.0,           79.0,          141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3574.0,         -171.0,         -500.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20889.0,         3308.0,         4116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14826.0,          985.0,         2290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     68004752.0,    338806815.0,    173996041.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3537.0,        -1477.0,         -463.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26575.0,         3590.0,         4824.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19926.0,        -2573.0,        -3580.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7812.0,         1086.0,         1452.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1819.0,         -163.0,         -291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -445.0,          -23.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -215.0,          499.0,          312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -328.0,          -15.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8824.0,        -6732.0,        -2657.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -164.0,          299.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          610.0,          339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -218.0,           66.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,          -14.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,            7.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          152.0,          -50.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -433.0,         -515.0,         -209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          341.0,            9.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1512.0,          118.0,          239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,          -25.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -403.0,          -25.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          320.0,          -68.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -267.0,        -1024.0,         -518.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,          -49.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          -92.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -22.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -962.0,         -688.0,         -260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,          163.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           38.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -21.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,           51.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          277.0,          295.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -516.0,        -1518.0,         -745.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          288.0,           87.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2484.0,          108.0,          331.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          305.0,          -49.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -126.0,           99.0,           68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          105.0,          -29.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -268.0,           22.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3560.0,        -4499.0,        -2773.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2312.0,        -2760.0,        -1729.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -47268547.0,    -32716124.0,    -12591216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1216.0,         2882.0,         1655.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3366.0,        -3791.0,        -2363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -93.0,          216.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,         -186.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           64.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -29.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,         -128.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -39.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3169.0,         -127.0,         -400.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          -65.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,         -159.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           51.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,           13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           30.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,         -257.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          -10.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          260.0,          232.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           45.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -18.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          303.0,           -7.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -58.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           18.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,          -38.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          350.0,          248.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,          -52.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          422.0,         -327.0,         -222.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,         -112.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          114.0,          441.0,          210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -278.0,        -1006.0,         -477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,            4.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          475.0,         1926.0,          950.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,          545.0,          323.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     10478188.0,      -798890.0,     -1510739.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           66.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          946.0,          519.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,          179.0,           79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -310.0,         -862.0,         -377.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          282.0,          769.0,          336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           24.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           17.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           13.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -539.0,          539.0,          345.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           20.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,            2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           82.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,           -1.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,           52.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -10.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,            9.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          148.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            6.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -413.0,         -297.0,         -114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -11.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1308366.0,      1403817.0,       885427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -77.0,         -108.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,         -140.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          106.0,           59.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,         -197.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -14.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -15.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -18.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -17.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,          -29.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,           -8.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -40759.0,      -394107.0,      -206360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           19.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,            1.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           37.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           24.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        73662.0,        60029.0,        24455.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22089.0,          -11.0,         2279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3527.0,        -3152.0,        -2049.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,         1076.0,          574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -204.0,         -194.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,            6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // ven
    venst   : array [1..8,1..178,0..2,1..3] of double = (
         (((        87078.0,        -2428.0,        -5098.0),
           (        29072.0,        -7794.0,        -4578.0),
           (         1254.0,          777.0,          854.0)),
          ((        15140.0,         8180.0,         1211.0),
           (       -10690.0,        20241.0,         8952.0),
           (            0.0,            0.0,            0.0)),
          ((         7132.0,       -10552.0,        -3334.0),
           (         -327.0,        -6662.0,        -3264.0),
           (            0.0,            0.0,            0.0)),
          ((        -8026.0,        -3288.0,        -2809.0),
           (         4144.0,        12966.0,         4729.0),
           (            0.0,            0.0,            0.0)),
          ((       191383.0,        58856.0,        13799.0),
           (          377.0,       104113.0,       -31274.0),
           (            0.0,            0.0,            0.0)),
          ((        -2870.0,        -3002.0,        -2832.0),
           (         1336.0,         3735.0,          690.0),
           (            0.0,            0.0,            0.0)),
          ((        38488.0,        32034.0,        25496.0),
           (         5178.0,        24756.0,        11586.0),
           (            0.0,            0.0,            0.0)),
          ((      -305809.0,       -80766.0,       -31191.0),
           (           54.0,         -923.0,         -362.0),
           (            0.0,            0.0,            0.0)),
          ((        -8886.0,        -6217.0,        -4882.0),
           (          186.0,          277.0,          145.0),
           (            0.0,            0.0,            0.0)),
          ((       -63389.0,       -13313.0,       -17555.0),
           (        -2797.0,         2195.0,          956.0),
           (            0.0,            0.0,            0.0)),
          ((        64309.0,        13067.0,        18618.0),
           (         2207.0,        -2696.0,        -1238.0),
           (            0.0,            0.0,            0.0)),
          ((         1249.0,        -1275.0,         -788.0),
           (        -7361.0,         -292.0,          370.0),
           (            0.0,            0.0,            0.0)),
          ((         2357.0,          676.0,          501.0),
           (        17253.0,         3729.0,          667.0),
           (            0.0,            0.0,            0.0)),
          ((        -7471.0,        -7710.0,        -4827.0),
           (          -76.0,          -38.0,          -12.0),
           (            0.0,            0.0,            0.0)),
          ((         6486.0,        10927.0,         6715.0),
           (          208.0,          370.0,          151.0),
           (            0.0,            0.0,            0.0)),
          ((       -19922.0,        -7698.0,        -3120.0),
           (        -1302.0,         2492.0,         1201.0),
           (            0.0,            0.0,            0.0)),
          ((          643.0,          867.0,          669.0),
           (          225.0,         -569.0,         -270.0),
           (            0.0,            0.0,            0.0)),
          ((       -11598.0,        -9794.0,        -5560.0),
           (            9.0,          147.0,           63.0),
           (            0.0,            0.0,            0.0)),
          ((       -14716.0,         2315.0,         8216.0),
           (            7.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((        -3161.0,        -6765.0,        -5650.0),
           (            4.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((          995.0,         2335.0,         1823.0),
           (            5.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((       102735.0,         9343.0,         1742.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2712.0,         2136.0,         1410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5409.0,        -3812.0,        -2812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2128.0,          838.0,          882.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1181.0,         1027.0,          711.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3577.0,        -3565.0,        -2223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6490.0,         3839.0,         2551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -32621.0,       -20231.0,       -12151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63945.0,        34369.0,        18440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29007.0,         1455.0,        -4904.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31963.0,         5964.0,        10208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6030.0,        33066.0,        10642.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2193.0,       -31730.0,       -17447.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        20646.0,         2397.0,         7067.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -7220620572.0,   -246727311.0,    346555260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20479.0,       -32104.0,       -19497.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1065.0,       -11051.0,        -1980.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13177.0,         2824.0,        -2841.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5770.0,         2807.0,         1343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15195.0,        15417.0,        10962.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -36351.0,           13.0,         -149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -998.0,         -792.0,         -491.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          551.0,         -357.0,          149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -261.0,         -144.0,         -135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2218.0,        -3458.0,        -2740.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          284.0,          598.0,          392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -66877.0,         4244.0,         3159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          193.0,          -58.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          431.0,          409.0,          197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,          135.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          710.0,          -15.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9614.0,        -1541.0,        -1155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          485.0,          141.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -723.0,         -573.0,         -287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          293.0,          190.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -339.0,         -935.0,         -451.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1266.0,         2124.0,         1254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -224.0,          -99.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       112874.0,       -16983.0,       -13880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2255.0,         1546.0,          694.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1505.0,         -779.0,         -336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2474.0,         -200.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1192.0,        -1482.0,         -640.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2505.0,         -711.0,         -395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25292.0,       -17320.0,        -6341.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        33758.0,        41840.0,        16764.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -48140.0,       -30889.0,       -10876.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -520.0,         8335.0,         3617.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -18783093.0,     14070409.0,      7515868.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5586.0,       -11516.0,        -4412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          127.0,         3741.0,         1486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1397.0,          553.0,          186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          854.0,        -2058.0,         -826.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        57991.0,       -27508.0,       -15571.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4368.0,        17425.0,         7080.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          385.0,        -1075.0,         -454.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,          234.0,          112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,         -135.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           70.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1284.0,          584.0,          378.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          336.0,          154.0,           41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          192.0,           49.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -447.0,          338.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           34.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          473.0,          362.0,          188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -24.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5762.0,         1685.0,         1352.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -547.0,         -234.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -24.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -33.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           45.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          202.0,         -139.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -70.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          756.0,         -775.0,         -378.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,          235.0,          100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           17.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          466.0,         -732.0,         -387.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,          -59.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3854.0,        -1752.0,         -533.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1160.0,         -502.0,         -445.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,         -104.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,         -160.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19326.0,       113133.0,        52104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           74.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,          -28.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          314.0,         -449.0,         -226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          117.0,          157.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -465.0,          226.0,          205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -18.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -30.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,           75.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -16.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -17.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          -90.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -23.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          167.0,          -90.0,          -93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           17.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           93.0,           80.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,           30.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -27.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          424.0,          649.0,          267.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           24.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -65.0,           39.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -16.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -7.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -44.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -18.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            8.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -56.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -26.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -7.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        71219.0,        28711.0,         8490.0),
           (        27047.0,        35564.0,        16111.0),
           (        -1828.0,        -7634.0,        -3323.0)),
          ((         9548.0,        41228.0,        18007.0),
           (         9631.0,        -6520.0,        -5319.0),
           (            0.0,            0.0,            0.0)),
          ((         9518.0,       -28060.0,       -13194.0),
           (        -1278.0,        -1392.0,        -1233.0),
           (            0.0,            0.0,            0.0)),
          ((        -2644.0,        24416.0,        11216.0),
           (       -18409.0,       -17556.0,       -12999.0),
           (            0.0,            0.0,            0.0)),
          ((        35592.0,      -194766.0,       -90952.0),
           (        44621.0,       -95518.0,       175308.0),
           (            0.0,            0.0,            0.0)),
          ((         -595.0,        15565.0,         7074.0),
           (         1060.0,         8315.0,          104.0),
           (            0.0,            0.0,            0.0)),
          ((         7848.0,      -141850.0,       -64596.0),
           (        28856.0,         4506.0,         2923.0),
           (            0.0,            0.0,            0.0)),
          ((       199811.0,       -52833.0,       -33179.0),
           (        -1267.0,          -62.0,         -196.0),
           (            0.0,            0.0,            0.0)),
          ((        -2806.0,        29272.0,        13378.0),
           (         -103.0,          -52.0,           24.0),
           (            0.0,            0.0,            0.0)),
          ((       -57467.0,       196928.0,        92044.0),
           (        -1687.0,         4893.0,         2279.0),
           (            0.0,            0.0,            0.0)),
          ((        63459.0,      -208497.0,       -97861.0),
           (          256.0,         2154.0,          801.0),
           (            0.0,            0.0,            0.0)),
          ((          381.0,          783.0,          285.0),
           (        -8938.0,        -2898.0,         -945.0),
           (            0.0,            0.0,            0.0)),
          ((        -2147.0,        -2829.0,        -1269.0),
           (        24805.0,         1804.0,         -373.0),
           (            0.0,            0.0,            0.0)),
          ((         8697.0,        19976.0,         8746.0),
           (          -88.0,           52.0,           27.0),
           (            0.0,            0.0,            0.0)),
          ((       -13747.0,       -26787.0,       -11525.0),
           (          290.0,           81.0,            7.0),
           (            0.0,            0.0,            0.0)),
          ((        -1091.0,        21872.0,         9928.0),
           (         1767.0,        -1733.0,         -823.0),
           (            0.0,            0.0,            0.0)),
          ((         -716.0,        -2210.0,         -999.0),
           (           29.0,           58.0,          -15.0),
           (            0.0,            0.0,            0.0)),
          ((         2970.0,        14634.0,         6911.0),
           (          -14.0,           94.0,           56.0),
           (            0.0,            0.0,            0.0)),
          ((       -21860.0,       -56196.0,       -23816.0),
           (            1.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         8493.0,        24284.0,        10743.0),
           (          -17.0,           -2.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((        -2759.0,        -7555.0,        -3348.0),
           (           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       104500.0,        -4931.0,        -4183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          282.0,        -5286.0,        -2388.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1012.0,        12045.0,         5395.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -941.0,       -10937.0,        -5242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -454.0,        -2795.0,        -1248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2373.0,         2884.0,         1066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3205.0,       -11975.0,        -5269.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        15962.0,        42403.0,        18631.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -33615.0,       -46737.0,       -20060.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21028.0,        68740.0,        31672.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19803.0,         9730.0,         -918.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13759.0,       -64517.0,       -21382.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5741.0,        66260.0,        30246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18683.0,       -66396.0,       -29230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   6118004081.0,   3649203763.0,   1253458449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        20847.0,        68517.0,        30123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24053.0,       -13774.0,        -5824.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6566.0,          610.0,          703.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5789.0,        -1674.0,        -1250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14887.0,       -12174.0,        -5090.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8866.0,        47661.0,        22196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          281.0,         2571.0,         1209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -287.0,        -1939.0,         -586.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          124.0,          865.0,          373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1833.0,         1945.0,         -842.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -167.0,        -1835.0,         -818.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23203.0,       -56468.0,       -24307.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -228.0,         -688.0,         -296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,         -374.0,         -182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -120.0,         -474.0,         -204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          345.0,         1091.0,          457.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5730.0,         4591.0,         2291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,          331.0,          127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          415.0,          677.0,          338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,         -172.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,         -216.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -729.0,         2099.0,          535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          301.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34380.0,       -99278.0,       -46305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -838.0,        -2831.0,        -1222.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          476.0,         1165.0,          512.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          671.0,        -1261.0,         -594.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1174.0,          811.0,          326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1914.0,        -3814.0,        -1636.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5902.0,        -8880.0,        -3287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6648.0,        18435.0,         7960.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -18100.0,       -27582.0,       -10822.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11211.0,          939.0,          896.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -22667002.0,     -9167044.0,     -2686589.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2381.0,        -6592.0,        -2843.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1970.0,         3115.0,         1427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -348.0,          941.0,          373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          393.0,        -3879.0,        -1719.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43720.0,         -437.0,         3528.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2024.0,        31727.0,        13815.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,        -2136.0,         -915.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          609.0,          248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,         -145.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,          113.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1004.0,          916.0,          332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,          260.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          112.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          320.0,          397.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           43.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          399.0,          412.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -15.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6291.0,         -518.0,          396.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          563.0,          292.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,           40.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           20.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          280.0,          128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -11.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           27.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,         1059.0,          472.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,         -157.0,          -59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           38.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -195.0,          731.0,          390.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          214.0,           44.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3110.0,         2741.0,         1034.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1046.0,         -716.0,         -177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          189.0,          -57.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          -39.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       122008.0,        29051.0,         5334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           72.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           22.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -21.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          397.0,         -370.0,         -196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          217.0,          -70.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,         -474.0,         -203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           22.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           14.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           67.0,           -9.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,          -93.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,         -165.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           22.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           73.0,           73.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,          -30.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,            9.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -754.0,         -117.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -24.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           77.0,          -25.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -33.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           10.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            6.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           11.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,           22.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((        31770.0,        69208.0,        24147.0),
           (       -41441.0,       -11188.0,        -4356.0),
           (        -2685.0,        -5502.0,        -5117.0)),
          ((           96.0,        35241.0,        21943.0),
           (        33546.0,        28797.0,        11841.0),
           (            0.0,            0.0,            0.0)),
          ((         7150.0,          272.0,        -7561.0),
           (        -3010.0,        -3411.0,         -291.0),
           (            0.0,            0.0,            0.0)),
          ((         2864.0,         7748.0,        10680.0),
           (        -8608.0,         1056.0,         9441.0),
           (            0.0,            0.0,            0.0)),
          ((      -193752.0,       -21966.0,        -2390.0),
           (       -14772.0,       119492.0,      -235297.0),
           (            0.0,            0.0,            0.0)),
          ((        -1747.0,         6079.0,         9426.0),
           (        -9094.0,        -9694.0,         -103.0),
           (            0.0,            0.0,            0.0)),
          ((        -1433.0,       -52174.0,       -78123.0),
           (        30609.0,        -9536.0,        -8558.0),
           (            0.0,            0.0,            0.0)),
          ((        32982.0,       296120.0,       175660.0),
           (        -1545.0,          366.0,          402.0),
           (            0.0,            0.0,            0.0)),
          ((         1130.0,        11364.0,        16045.0),
           (           47.0,         -237.0,         -182.0),
           (            0.0,            0.0,            0.0)),
          ((         7843.0,        77719.0,       103064.0),
           (        -7640.0,        -1333.0,         -179.0),
           (            0.0,            0.0,            0.0)),
          ((       -11215.0,       -82549.0,      -109200.0),
           (          193.0,         1030.0,          533.0),
           (            0.0,            0.0,            0.0)),
          ((          846.0,          448.0,          836.0),
           (         5755.0,        -4661.0,        -1965.0),
           (            0.0,            0.0,            0.0)),
          ((         -419.0,         1250.0,         -155.0),
           (       -14757.0,         8422.0,         3803.0),
           (            0.0,            0.0,            0.0)),
          ((         8153.0,         1847.0,         6235.0),
           (           20.0,           58.0,           13.0),
           (            0.0,            0.0,            0.0)),
          ((       -14162.0,        -5071.0,        -8664.0),
           (         -262.0,           10.0,           53.0),
           (            0.0,            0.0,            0.0)),
          ((        16538.0,        -4159.0,         -250.0),
           (        -2024.0,         1243.0,          608.0),
           (            0.0,            0.0,            0.0)),
          ((          360.0,         -502.0,        -1459.0),
           (           35.0,           92.0,           90.0),
           (            0.0,            0.0,            0.0)),
          ((         2431.0,        -1245.0,         8299.0),
           (          124.0,            8.0,          -37.0),
           (            0.0,            0.0,            0.0)),
          ((         3428.0,       -30799.0,       -30931.0),
           (           -4.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((         1194.0,         7426.0,        13044.0),
           (           23.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        -1178.0,        -2001.0,        -4026.0),
           (            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((       100218.0,       -25554.0,       -11861.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          851.0,        -1533.0,        -2786.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1317.0,         4406.0,         7131.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          396.0,        -7575.0,        -6717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,         -984.0,        -1617.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1968.0,         3470.0,         3585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2158.0,        -4997.0,        -6962.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9986.0,        26946.0,        31313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22846.0,       -53528.0,       -49688.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17338.0,       -26884.0,         5326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5582.0,       -24747.0,        -8759.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20048.0,        14274.0,       -17407.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10494.0,       -13837.0,        10027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15490.0,       -13238.0,       -29505.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -3220082646.0,  -5980711832.0,  -2485993581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9495.0,        21168.0,        34974.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        20148.0,         1536.0,       -11196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23107.0,        17303.0,        20459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7832.0,        -2206.0,        -3596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9408.0,         4958.0,       -11977.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17167.0,        35324.0,        24017.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          361.0,         1341.0,         1609.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -667.0,        -1247.0,         -702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          197.0,          490.0,          499.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -427.0,         5892.0,         1502.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -802.0,        -1015.0,        -1009.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        54342.0,       -35175.0,       -16122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -418.0,         -616.0,         -479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,         1026.0,          549.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -117.0,         -211.0,         -226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          475.0,          955.0,          662.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6561.0,        -2620.0,        -1668.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          207.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,         -689.0,         -302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,          121.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -433.0,         -397.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -931.0,        -2080.0,        -1263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,         -147.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -83806.0,       -75439.0,       -28748.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -182.0,          548.0,         -373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          450.0,          -92.0,          253.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -338.0,          210.0,         -175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          774.0,          255.0,          361.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2483.0,        -3076.0,        -1633.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3624.0,       -13136.0,        -3902.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9294.0,        14431.0,         4029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6529.0,       -30546.0,       -11678.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8070.0,        -8011.0,        -3425.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1928246.0,    -22370974.0,     -9939120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2551.0,         7257.0,         2686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2189.0,        -2198.0,         -551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1641.0,         -445.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1376.0,         2261.0,          591.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        38773.0,       -27651.0,       -18149.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8299.0,       -15229.0,        -3114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          433.0,         1076.0,          248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,         -375.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -62.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          827.0,        -1114.0,         -582.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -355.0,           51.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -252.0,          -84.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          640.0,         -120.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          313.0,          434.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           18.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5830.0,        -2443.0,         -506.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          655.0,         -247.0,         -219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           14.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           46.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           18.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,            4.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -234.0,         -142.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -10.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          -63.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -434.0,         -977.0,         -430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,            8.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,         -106.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -151.0,         -599.0,         -299.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,         -116.0,          -83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1954.0,        -3604.0,        -1516.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -540.0,         1267.0,          718.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          155.0,         -343.0,         -190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -94.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        37437.0,      -107818.0,       -50910.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           88.0,           39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           32.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -14.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          475.0,         -290.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,         -196.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          546.0,           62.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -28.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            5.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,          -46.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,            0.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,         -183.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            9.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           -2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           25.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,           74.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -60.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           19.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          459.0,         -456.0,         -240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,           10.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           11.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,          -16.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           30.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -22.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           -3.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,           30.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           21.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -21344.0,        76047.0,        35270.0),
           (       -36805.0,       -18174.0,        -7061.0),
           (         -501.0,        -1836.0,        -1063.0)),
          ((        -8396.0,        25240.0,        11840.0),
           (        55973.0,        20312.0,         7293.0),
           (            0.0,            0.0,            0.0)),
          ((       -10954.0,         1379.0,         1111.0),
           (        -2060.0,        -4688.0,        -1626.0),
           (            0.0,            0.0,            0.0)),
          ((         3559.0,         -608.0,         -376.0),
           (        10312.0,          459.0,        -2119.0),
           (            0.0,            0.0,            0.0)),
          ((         6232.0,       175099.0,        78703.0),
           (        48581.0,       -48834.0,       237724.0),
           (            0.0,            0.0,            0.0)),
          ((         3473.0,         4290.0,         1836.0),
           (        -4646.0,       -16455.0,        -5318.0),
           (            0.0,            0.0,            0.0)),
          ((       -30493.0,       -23631.0,        -9818.0),
           (         5518.0,       -11740.0,        -6862.0),
           (            0.0,            0.0,            0.0)),
          ((      -218504.0,      -110322.0,       -37404.0),
           (         -495.0,         -792.0,         -352.0),
           (            0.0,            0.0,            0.0)),
          ((         6157.0,         4593.0,         1929.0),
           (          486.0,         -258.0,         -165.0),
           (            0.0,            0.0,            0.0)),
          ((        43024.0,        44133.0,        19474.0),
           (        -1449.0,          170.0,          364.0),
           (            0.0,            0.0,            0.0)),
          ((       -43121.0,       -46671.0,       -21021.0),
           (        -1822.0,        -1754.0,         -782.0),
           (            0.0,            0.0,            0.0)),
          ((         1088.0,        -1603.0,         -761.0),
           (         8597.0,         -432.0,         -536.0),
           (            0.0,            0.0,            0.0)),
          ((          724.0,        -1334.0,         -589.0),
           (       -18241.0,         5843.0,         3304.0),
           (            0.0,            0.0,            0.0)),
          ((         -143.0,         2498.0,         1047.0),
           (           45.0,          -43.0,          -25.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,        -4968.0,        -2111.0),
           (         -443.0,          347.0,          196.0),
           (            0.0,            0.0,            0.0)),
          ((        -8775.0,       -14702.0,        -6134.0),
           (         2947.0,         -214.0,         -313.0),
           (            0.0,            0.0,            0.0)),
          ((         -977.0,         -844.0,         -325.0),
           (           53.0,          215.0,          103.0),
           (            0.0,            0.0,            0.0)),
          ((        11259.0,         1054.0,         -725.0),
           (          135.0,           13.0,          -20.0),
           (            0.0,            0.0,            0.0)),
          ((        10447.0,       -19483.0,       -12095.0),
           (            4.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((         4694.0,         5376.0,         2642.0),
           (          -20.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((        -2029.0,        -1224.0,         -548.0),
           (            0.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((        92797.0,       -43142.0,       -20738.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          710.0,        -1748.0,         -812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          824.0,         2367.0,         1043.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -656.0,        -6146.0,        -2521.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -204.0,         -525.0,         -231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1360.0,         2522.0,         1417.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           72.0,        -1508.0,         -682.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8173.0,         7579.0,         3978.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29809.0,        -8553.0,        -5471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        46767.0,        12020.0,         3607.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2110.0,        21953.0,        14226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8127.0,       -31039.0,        -4887.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2851.0,        30591.0,        12907.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10282.0,         4173.0,         1779.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -622811165.0,   6557133348.0,   2989101357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5902.0,        -1387.0,         -146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1642.0,       -26627.0,       -11967.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21273.0,        22343.0,         8442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2200.0,        -1710.0,        -1018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19271.0,        -1966.0,        -2420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34600.0,         9352.0,         2670.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          502.0,          402.0,          230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -844.0,         -680.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          157.0,          194.0,           78.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1140.0,         6559.0,         2834.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          166.0,           11.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        53082.0,        38302.0,        15756.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          106.0,         -145.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          485.0,          223.0,          229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,         -170.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -285.0,          675.0,          263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6163.0,         6982.0,         3438.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -253.0,          555.0,          266.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,          650.0,          293.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,         -143.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -556.0,          361.0,          150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          515.0,        -2170.0,         -529.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           44.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -109327.0,        33625.0,        21068.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -446.0,         -253.0,         -145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,          346.0,          159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1220.0,          122.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -450.0,          829.0,          343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1343.0,        -3486.0,        -1401.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9146.0,       -11237.0,        -4890.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9246.0,         5403.0,         2555.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6626.0,       -27773.0,       -12522.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3186.0,        -9355.0,        -4139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     20831753.0,    -11236673.0,     -6373282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5013.0,        12232.0,         5476.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          235.0,        -3093.0,        -1283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2156.0,         -277.0,         -172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          489.0,         1601.0,          585.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9506.0,        63962.0,        28987.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3844.0,       -16187.0,        -6383.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          351.0,          959.0,          374.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,         -134.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,          265.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -71.0,          -82.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1342.0,         -387.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,         -301.0,         -138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,         -173.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,         -491.0,         -208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -49.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          223.0,          457.0,          204.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           29.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4135.0,        -4106.0,        -1459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -592.0,         -505.0,         -199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           20.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -60.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           14.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          260.0,         -131.0,          -93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           93.0,          150.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          876.0,          458.0,          148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -94.0,          189.0,           91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           30.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          323.0,          572.0,          283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -58.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          635.0,         3809.0,         1673.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1459.0,          304.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -160.0,         -171.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          216.0,           88.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -115676.0,       -44964.0,       -12915.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -16.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           77.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           37.0,           74.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -25.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          539.0,         -191.0,         -124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,          -13.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -192.0,          439.0,          254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           31.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -22.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           72.0,           34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -14.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,           99.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -17.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,         -147.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -14.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           24.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,           97.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          -39.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          225.0,          607.0,          267.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,            7.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -51.0,            4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           24.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           23.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -55.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -21.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -72606.0,        47713.0,        27435.0),
           (        10642.0,        35879.0,        18031.0),
           (        -1888.0,        -1853.0,          764.0)),
          ((       -12045.0,        12956.0,         4346.0),
           (         -159.0,       -20871.0,       -12513.0),
           (            0.0,            0.0,            0.0)),
          ((       -17059.0,        -7718.0,           64.0),
           (        -1577.0,         1629.0,          191.0),
           (            0.0,            0.0,            0.0)),
          ((         7788.0,         4628.0,         -920.0),
           (       -29125.0,       -18576.0,        -9676.0),
           (            0.0,            0.0,            0.0)),
          ((       180999.0,       -29678.0,       -24991.0),
           (        28432.0,        47069.0,      -143798.0),
           (            0.0,            0.0,            0.0)),
          ((        11804.0,         2866.0,        -1749.0),
           (       -14282.0,        -3718.0,        -4251.0),
           (            0.0,            0.0,            0.0)),
          ((       -80949.0,       -30377.0,        10534.0),
           (        -6791.0,       -15852.0,        -4639.0),
           (            0.0,            0.0,            0.0)),
          ((       321713.0,       -76432.0,       -67037.0),
           (        -1108.0,          538.0,          -56.0),
           (            0.0,            0.0,            0.0)),
          ((        14991.0,         7864.0,        -1285.0),
           (          483.0,          322.0,          203.0),
           (            0.0,            0.0,            0.0)),
          ((        80083.0,        90364.0,        10536.0),
           (        -6176.0,        -5363.0,        -2140.0),
           (            0.0,            0.0,            0.0)),
          ((       -88733.0,       -95107.0,       -10261.0),
           (          448.0,        -1048.0,         -401.0),
           (            0.0,            0.0,            0.0)),
          ((          533.0,         -781.0,         -668.0),
           (         -834.0,         6150.0,         2316.0),
           (            0.0,            0.0,            0.0)),
          ((        -3050.0,          618.0,          608.0),
           (           88.0,       -12506.0,        -4678.0),
           (            0.0,            0.0,            0.0)),
          ((         9866.0,         3497.0,         -924.0),
           (          -14.0,          -42.0,          -14.0),
           (            0.0,            0.0,            0.0)),
          ((        -9238.0,        -8960.0,        -1059.0),
           (         -149.0,          194.0,           79.0),
           (            0.0,            0.0,            0.0)),
          ((        -5457.0,        10244.0,         3929.0),
           (        -2641.0,         -663.0,          -26.0),
           (            0.0,            0.0,            0.0)),
          ((        -3044.0,          874.0,          973.0),
           (          245.0,          -14.0,          -87.0),
           (            0.0,            0.0,            0.0)),
          ((        26799.0,        -2992.0,        -6645.0),
           (           35.0,         -122.0,          -37.0),
           (            0.0,            0.0,            0.0)),
          ((        -8255.0,         9959.0,         7496.0),
           (           -9.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((        17369.0,        -4323.0,        -5889.0),
           (           16.0,          -11.0,           -7.0),
           (            0.0,            0.0,            0.0)),
          ((        -6424.0,         1374.0,         1971.0),
           (           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        83465.0,       -59618.0,       -28327.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3116.0,         -509.0,          703.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7102.0,          627.0,        -1953.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4135.0,        -2978.0,          438.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1889.0,           37.0,          553.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          873.0,        -1604.0,        -1727.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7389.0,         1324.0,         2764.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34576.0,        -9774.0,       -13467.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -54025.0,        29006.0,        25586.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8341.0,        37645.0,         9759.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27884.0,        -3787.0,       -10202.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -32680.0,        19723.0,         6557.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19619.0,       -19971.0,       -15596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21777.0,        -5752.0,         7107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   4282946367.0,  -5209406476.0,  -2614861305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        47906.0,        21813.0,        -2047.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -40181.0,        -2566.0,         4507.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        20340.0,       -17149.0,       -13869.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1043.0,         2687.0,         1802.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1088.0,       -18761.0,        -3242.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        34451.0,       -16123.0,       -12591.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1283.0,          226.0,         -334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1328.0,         -269.0,          334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          417.0,          110.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2326.0,         3756.0,         2718.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -571.0,         -690.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24647.0,        57093.0,        25319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,         -306.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -343.0,         -360.0,         -346.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,         -172.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,          494.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2618.0,        -4882.0,        -2283.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -381.0,          108.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          503.0,         -635.0,         -289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,          158.0,           99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -345.0,          -57.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          993.0,         2525.0,         1353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          171.0,          -22.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13585.0,       105074.0,        47656.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1841.0,           17.0,          291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          728.0,         -101.0,         -181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2258.0,          373.0,          296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -304.0,          112.0,          -99.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2157.0,        -1876.0,         -485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3327.0,       -16918.0,        -8382.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9257.0,        16791.0,         8645.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5145.0,       -33452.0,       -16289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7292.0,        -4137.0,        -1981.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     20894026.0,     12059033.0,      4103481.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11119.0,         6808.0,         3659.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3631.0,         -858.0,         -629.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1201.0,           21.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3164.0,          499.0,          508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22048.0,       -50659.0,       -20638.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16706.0,         -173.0,        -2022.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -928.0,            9.0,          123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          130.0,          -75.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,         -171.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           60.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -192.0,         1377.0,          644.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          302.0,         -212.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          224.0,          -42.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -617.0,         -157.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          143.0,          498.0,          212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           -3.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2186.0,        -5419.0,        -2279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -238.0,          533.0,          310.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,            6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,          -85.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            8.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,          -25.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          319.0,          141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           27.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,          -10.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1188.0,          -53.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          167.0,          -44.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -78.0,          -36.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -352.0,         -448.0,         -203.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -67.0,           47.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          740.0,        -3797.0,        -1750.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -683.0,        -1459.0,         -578.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          240.0,          176.0,           74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          247.0,         -260.0,         -152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -54639.0,       100546.0,        48713.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           15.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          107.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -17.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           16.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          578.0,         -110.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,          193.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -428.0,         -325.0,          -72.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -31.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           36.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,          -41.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           20.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,           92.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -202.0,          -95.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -22.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          123.0,           51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           60.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -13.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -734.0,         -216.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,           10.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,          -22.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           24.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           -4.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            2.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,           34.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,            0.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -4.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -98969.0,          271.0,         3551.0),
           (        -5347.0,       -28549.0,       -12628.0),
           (        -7996.0,          195.0,        -1200.0)),
          ((       -11509.0,         1631.0,         4594.0),
           (        15831.0,        18021.0,         6405.0),
           (            0.0,            0.0,            0.0)),
          ((        -9268.0,       -16688.0,       -10603.0),
           (        -5588.0,          936.0,          798.0),
           (            0.0,            0.0,            0.0)),
          ((        17646.0,         6805.0,         5772.0),
           (       -18033.0,         8243.0,         8659.0),
           (            0.0,            0.0,            0.0)),
          ((       -71715.0,      -174184.0,       -77537.0),
           (        33501.0,        -6540.0,        19704.0),
           (            0.0,            0.0,            0.0)),
          ((        15291.0,         1118.0,         2932.0),
           (       -21814.0,        -6370.0,         -362.0),
           (            0.0,            0.0,            0.0)),
          ((      -134069.0,       -22294.0,       -29977.0),
           (       -14924.0,         3432.0,         1139.0),
           (            0.0,            0.0,            0.0)),
          ((        53300.0,       266239.0,       138312.0),
           (         -158.0,          499.0,          322.0),
           (            0.0,            0.0,            0.0)),
          ((        27626.0,         6206.0,         6788.0),
           (          347.0,          354.0,          144.0),
           (            0.0,            0.0,            0.0)),
          ((       169112.0,        58009.0,        51641.0),
           (         1928.0,        -2721.0,        -1228.0),
           (            0.0,            0.0,            0.0)),
          ((      -176290.0,       -61958.0,       -55528.0),
           (         -269.0,         1415.0,          686.0),
           (            0.0,            0.0,            0.0)),
          ((         2343.0,         -259.0,          138.0),
           (        -6017.0,         5568.0,         2762.0),
           (            0.0,            0.0,            0.0)),
          ((         -311.0,          688.0,           50.0),
           (        21000.0,       -15276.0,        -7829.0),
           (            0.0,            0.0,            0.0)),
          ((        12586.0,         2209.0,         3063.0),
           (          -35.0,           41.0,           19.0),
           (            0.0,            0.0,            0.0)),
          ((       -14305.0,        -5711.0,        -5205.0),
           (         -230.0,         -172.0,          -69.0),
           (            0.0,            0.0,            0.0)),
          ((        22736.0,         5749.0,         2524.0),
           (         2644.0,         1509.0,          505.0),
           (            0.0,            0.0,            0.0)),
          ((        -2204.0,          377.0,         -301.0),
           (         -652.0,          -11.0,           30.0),
           (            0.0,            0.0,            0.0)),
          ((        24284.0,         3032.0,         4231.0),
           (          179.0,          -20.0,          -23.0),
           (            0.0,            0.0,            0.0)),
          ((       -15352.0,        20616.0,          882.0),
           (           -2.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((        19763.0,        -3561.0,         2337.0),
           (          -16.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0)),
          ((        -6988.0,         1009.0,         -771.0),
           (            1.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((        69502.0,       -72683.0,       -32684.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3637.0,         -948.0,        -1237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10913.0,         1476.0,         2596.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4692.0,        -1605.0,        -1771.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2653.0,         -280.0,         -574.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5432.0,        -1348.0,         -237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11250.0,        -2297.0,        -2997.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        52899.0,        14709.0,        13946.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -84708.0,       -35175.0,       -25299.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8135.0,       -31102.0,        -6243.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21557.0,        -1821.0,         5949.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24166.0,       -25448.0,        -7840.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        39973.0,        26384.0,        16737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -49585.0,          223.0,       -10490.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -6686237269.0,   2333167165.0,   1472899050.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        52300.0,          676.0,        10428.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12082.0,        19922.0,         3826.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1170.0,         2917.0,         7348.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3054.0,         1980.0,         -516.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -32129.0,       -22983.0,       -15871.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17455.0,       -28125.0,        -8802.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2119.0,         -249.0,          315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1849.0,          178.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          794.0,          -59.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2404.0,        -1252.0,         1211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1763.0,          195.0,         -165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -65270.0,        -6915.0,        -1458.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -907.0,           17.0,         -105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          365.0,          473.0,          316.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -100.0,          134.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          -87.0,          113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5602.0,         6830.0,         3298.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -843.0,          227.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -504.0,          458.0,          195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -25.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,          475.0,          264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -968.0,         1599.0,          248.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,           95.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        97625.0,        61493.0,        22371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1804.0,         -271.0,         -284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1175.0,         -158.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2238.0,          115.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          765.0,         -464.0,         -119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4202.0,         -374.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10657.0,       -18620.0,        -7964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8461.0,        26364.0,        10570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        26432.0,       -32917.0,       -15355.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6707.0,         1445.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1729320.0,     22180288.0,     10090898.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7370.0,        -2093.0,         -518.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3470.0,         2361.0,          931.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1756.0,          480.0,          287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1833.0,        -2108.0,         -957.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2817.0,        29885.0,        11964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25435.0,        17611.0,         7624.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1742.0,        -1169.0,         -504.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          503.0,          366.0,          160.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,           -9.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,           33.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1354.0,         -244.0,         -229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,          287.0,          126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,           45.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -363.0,          434.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           40.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          519.0,          210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -19.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,        -5700.0,        -2599.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          595.0,          -33.0,         -107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -18.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           27.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -11.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,          -13.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -273.0,         -128.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          158.0,         -124.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          858.0,         -356.0,         -224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,         -154.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           24.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          486.0,          326.0,          111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -151.0,            8.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2114.0,         3377.0,         1642.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -783.0,         1233.0,          733.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,         -274.0,         -152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          256.0,          106.0,           71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       106457.0,        59424.0,        19987.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            1.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           44.0,           72.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,            0.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           -1.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          602.0,           -6.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          196.0,           84.0,           50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          453.0,         -242.0,         -210.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           28.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -43.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,          -29.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,            6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -116.0,            2.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,          -11.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -24.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          123.0,           58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,           44.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          676.0,         -380.0,         -206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -16.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           78.0,           36.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,           38.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -27.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -59.0,           17.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           20.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -8.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -3.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -66680.0,       -51702.0,       -19356.0),
           (       -12606.0,       -24319.0,       -10743.0),
           (         7173.0,        -2472.0,        -1868.0)),
          ((       -36830.0,        -1070.0,         2056.0),
           (        65809.0,        -3710.0,        -4518.0),
           (            0.0,            0.0,            0.0)),
          ((        24257.0,        -7540.0,        -5081.0),
           (        -4032.0,        -1890.0,         -841.0),
           (            0.0,            0.0,            0.0)),
          ((       -12460.0,         8801.0,         4968.0),
           (        12610.0,         2089.0,        -3725.0),
           (            0.0,            0.0,            0.0)),
          ((      -166678.0,        55026.0,        37743.0),
           (        24565.0,       -94988.0,        79341.0),
           (            0.0,            0.0,            0.0)),
          ((       -13921.0,         9671.0,         5493.0),
           (        -4858.0,         6481.0,         3755.0),
           (            0.0,            0.0,            0.0)),
          ((        98164.0,       -75234.0,       -42030.0),
           (       -14636.0,        19859.0,         9299.0),
           (            0.0,            0.0,            0.0)),
          ((      -269188.0,      -129835.0,       -41435.0),
           (          893.0,         -412.0,         -203.0),
           (            0.0,            0.0,            0.0)),
          ((       -18767.0,        14563.0,         8109.0),
           (         -402.0,           59.0,           23.0),
           (            0.0,            0.0,            0.0)),
          ((      -100310.0,        86758.0,        47247.0),
           (         -575.0,        -5110.0,        -2180.0),
           (            0.0,            0.0,            0.0)),
          ((       103553.0,       -90427.0,       -49041.0),
           (         2027.0,         2795.0,         1094.0),
           (            0.0,            0.0,            0.0)),
          ((        -1925.0,         -655.0,         -157.0),
           (        -6783.0,        -3505.0,        -1216.0),
           (            0.0,            0.0,            0.0)),
          ((          999.0,        -2185.0,        -1139.0),
           (         9710.0,         8840.0,         3579.0),
           (            0.0,            0.0,            0.0)),
          ((       -12606.0,         6306.0,         4035.0),
           (          -39.0,           15.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((        19324.0,        -6213.0,        -4499.0),
           (         -309.0,          -44.0,           20.0),
           (            0.0,            0.0,            0.0)),
          ((        -1353.0,       -12149.0,        -5103.0),
           (        -1709.0,        -1871.0,         -738.0),
           (            0.0,            0.0,            0.0)),
          ((         1856.0,        -2501.0,        -1303.0),
           (          -76.0,          271.0,          134.0),
           (            0.0,            0.0,            0.0)),
          ((       -13862.0,        21130.0,        10984.0),
           (           65.0,         -108.0,          -65.0),
           (            0.0,            0.0,            0.0)),
          ((        25998.0,        -1120.0,        -2519.0),
           (           -5.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15494.0,        11697.0,         6654.0),
           (            5.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         4706.0,        -3864.0,        -2207.0),
           (           -3.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((        47666.0,       -81241.0,       -36392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3888.0,        -4070.0,        -2139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10144.0,         8550.0,         4655.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10464.0,        -5003.0,        -3065.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2350.0,        -1853.0,        -1031.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1167.0,         4010.0,         2231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10166.0,        -8659.0,        -4784.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -47354.0,        34172.0,        19029.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        81328.0,       -50463.0,       -28284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10888.0,        16141.0,         8209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -289.0,        22339.0,         9895.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16511.0,           89.0,       -11928.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -33230.0,         -710.0,         2789.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        44476.0,       -50853.0,       -27208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   7127497541.0,   1227475661.0,    101757141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43255.0,        23138.0,        14644.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        35499.0,       -33320.0,       -17095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2050.0,        29908.0,        15186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8804.0,        -1942.0,        -1665.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2666.0,       -31250.0,       -15285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -41732.0,        -4313.0,         1018.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2090.0,         1192.0,          721.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          397.0,         -168.0,         -148.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -611.0,          307.0,          202.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          277.0,        -5193.0,         -969.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1733.0,         -619.0,         -440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12763.0,       -60142.0,       -26027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          872.0,            0.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,         -107.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          332.0,         -228.0,         -107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1112.0,           79.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          349.0,        -7965.0,        -3559.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,         -409.0,         -157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          758.0,         -226.0,          -77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,          -77.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,          623.0,          302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1292.0,        -2443.0,        -1249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -48.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       101780.0,       -50055.0,       -27914.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -286.0,          487.0,          133.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          476.0,         -199.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -458.0,         -127.0,          -75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1106.0,         -726.0,         -324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5057.0,          374.0,          359.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18489.0,       -13104.0,        -6359.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21959.0,        25776.0,        11781.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        42216.0,       -22968.0,       -12163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2887.0,         6299.0,         2343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -22398240.0,      8186128.0,      5100855.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6618.0,        -4984.0,        -2342.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2317.0,         3633.0,         1670.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2811.0,          484.0,          176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1021.0,        -2565.0,        -1193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27307.0,       -49077.0,       -19819.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15800.0,        15398.0,         7623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1055.0,         -810.0,         -407.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -319.0,           93.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -97.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -52.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -509.0,        -1359.0,         -576.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -145.0,          381.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -215.0,          -22.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          451.0,          412.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,          503.0,          196.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2440.0,        -4993.0,        -2539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -373.0,         -524.0,         -229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           21.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           70.0,           19.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           38.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -49.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          228.0,         -192.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,          -41.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -102.0,          133.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -454.0,          733.0,          343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,          210.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -27.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -546.0,         -247.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          237.0,          -70.0,          -42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3253.0,        -2595.0,        -1381.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1423.0,          237.0,         -107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          136.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          150.0,           76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        69987.0,       -90966.0,       -45370.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -32.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           90.0,           14.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           55.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -38.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          578.0,          109.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,         -153.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          158.0,          472.0,          199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -25.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           42.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           83.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -33.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -19.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,          -84.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -16.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -167.0,           94.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -20.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,           -3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,           99.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           58.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,          683.0,          302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          -51.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           45.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -28.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            1.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -54.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -19.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           10.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       -19228.0,       -75126.0,       -32165.0),
           (         7453.0,        33372.0,        14295.0),
           (         6342.0,         -655.0,         -378.0)),
          ((       -36677.0,       -12660.0,        -3845.0),
           (        -6695.0,       -11199.0,        -5173.0),
           (            0.0,            0.0,            0.0)),
          ((        27271.0,        11462.0,         3968.0),
           (        -1406.0,         2850.0,         1821.0),
           (            0.0,            0.0,            0.0)),
          ((       -20843.0,         -489.0,          526.0),
           (       -30594.0,         6089.0,         9405.0),
           (            0.0,            0.0,            0.0)),
          ((       114133.0,       151731.0,        60928.0),
           (       -70001.0,       115358.0,      -191227.0),
           (            0.0,            0.0,            0.0)),
          ((       -14738.0,         4811.0,         2566.0),
           (         9709.0,        19066.0,        10531.0),
           (            0.0,            0.0,            0.0)),
          ((       127522.0,       -33978.0,       -18913.0),
           (         5505.0,        11931.0,         3245.0),
           (            0.0,            0.0,            0.0)),
          ((       173534.0,         4832.0,        -9051.0),
           (          490.0,          839.0,          586.0),
           (            0.0,            0.0,            0.0)),
          ((       -24978.0,         8206.0,         4365.0),
           (         -549.0,         -593.0,         -283.0),
           (            0.0,            0.0,            0.0)),
          ((      -138942.0,        95370.0,        45918.0),
           (         6619.0,         -942.0,         -698.0),
           (            0.0,            0.0,            0.0)),
          ((       151194.0,      -103485.0,       -50128.0),
           (          329.0,         1527.0,          703.0),
           (            0.0,            0.0,            0.0)),
          ((        -2949.0,         -411.0,          -56.0),
           (        -3182.0,        -9715.0,        -3961.0),
           (            0.0,            0.0,            0.0)),
          ((         1498.0,         1927.0,          785.0),
           (         6529.0,        22291.0,         8888.0),
           (            0.0,            0.0,            0.0)),
          ((       -20370.0,        -4692.0,        -1212.0),
           (          -30.0,          -27.0,          -15.0),
           (            0.0,            0.0,            0.0)),
          ((        30155.0,         6592.0,         1563.0),
           (         -102.0,          -84.0,          -10.0),
           (            0.0,            0.0,            0.0)),
          ((       -21617.0,         3071.0,         2337.0),
           (         1198.0,         2376.0,          951.0),
           (            0.0,            0.0,            0.0)),
          ((         1873.0,          403.0,          128.0),
           (          -71.0,           86.0,           75.0),
           (            0.0,            0.0,            0.0)),
          ((       -21359.0,         1551.0,         1983.0),
           (           39.0,          -47.0,          -42.0),
           (            0.0,            0.0,            0.0)),
          ((        14096.0,        22884.0,        12434.0),
           (            2.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((       -16256.0,        -5265.0,        -2133.0),
           (          -11.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((         5744.0,         1683.0,          626.0),
           (           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28357.0,       -90844.0,       -40391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5103.0,         -593.0,         -391.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10615.0,         1344.0,          839.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11954.0,         -281.0,         -902.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2633.0,           -2.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4524.0,         2015.0,         1310.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9645.0,          335.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -39805.0,       -10366.0,        -3822.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        51812.0,        32805.0,        13771.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -40595.0,        43664.0,        20643.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          892.0,       -27965.0,       -16823.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        42670.0,       -27049.0,        -3644.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23251.0,        31239.0,        14062.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        40342.0,       -11291.0,        -5828.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -5477280738.0,  -4427722982.0,  -1646997093.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -41810.0,        19984.0,         9754.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2835.0,       -10082.0,        -4223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13304.0,       -15998.0,        -7249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11270.0,         1794.0,          215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5762.0,         5343.0,         3034.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -49195.0,         8938.0,         6054.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2444.0,          335.0,          193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          687.0,          481.0,          102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -667.0,            7.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1410.0,        -6261.0,        -2621.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          941.0,         -375.0,         -179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        59223.0,       -26225.0,       -12737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          642.0,         -287.0,         -118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -593.0,           18.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          152.0,          215.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -556.0,         -470.0,         -141.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1857.0,         5601.0,         2646.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -350.0,         -347.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -788.0,           57.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,          -28.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          298.0,          492.0,          154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1201.0,        -1541.0,         -235.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,          -34.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4934.0,      -106201.0,       -47296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,           19.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,          210.0,           80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          960.0,         -525.0,         -236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1869.0,           -6.0,         -104.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6398.0,         -422.0,          174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25324.0,         4541.0,          677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -39211.0,          961.0,         2555.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        54540.0,         4928.0,         -903.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5074.0,         2592.0,         1353.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -18676776.0,    -14645218.0,     -5412324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16638.0,         1846.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5658.0,          569.0,          537.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          979.0,         -203.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4793.0,           77.0,         -200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        84160.0,        16374.0,         1898.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -35327.0,         3855.0,         3714.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2220.0,         -299.0,         -268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -567.0,          137.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          199.0,            5.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,           23.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1061.0,          808.0,          461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -226.0,         -192.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -290.0,          -56.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          564.0,         -245.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -16.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,          504.0,          190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4368.0,        -3895.0,        -2249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -124.0,          485.0,          281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,           12.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,          -20.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           12.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -33.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           81.0,          293.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           42.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,          -85.0,          -38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           86.0,         -990.0,         -437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,            8.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           87.0,          -38.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          507.0,          217.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -77.0,           21.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3835.0,         1451.0,          904.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -865.0,        -1201.0,         -471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          463.0,          205.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,          173.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -95219.0,       -72705.0,       -26704.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           -6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          127.0,           11.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -72.0,           -7.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,           -2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          531.0,          186.0,           49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -130.0,         -152.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -548.0,          -38.0,           92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           21.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -35.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          -45.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           -5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -14.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,          -96.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,            8.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,          154.0,          105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -7.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -14.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           70.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,           32.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -18.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -515.0,         -418.0,         -146.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,           60.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           43.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           -4.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           16.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -11.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           44.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           -3.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            1.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // emb
    embst   : array [1..5,1..208,0..2,1..3] of double = (
         (((        83586.0,      2060481.0,       828412.0),
           (      -119669.0,       171568.0,        66864.0),
           (        51367.0,        17854.0,         5711.0)),
          ((       -28432.0,     -2364571.0,      -952574.0),
           (        12117.0,        -8569.0,        -2830.0),
           (          -19.0,           14.0,            7.0)),
          ((      -181921.0,       391737.0,       153807.0),
           (       -18030.0,        -2380.0,        -2748.0),
           (            0.0,            0.0,            0.0)),
          ((       -74923.0,        47673.0,        19017.0),
           (       -15315.0,       -12246.0,        -5834.0),
           (            0.0,            0.0,            0.0)),
          ((        41122.0,        37520.0,        15970.0),
           (       -12682.0,        -7891.0,        -5297.0),
           (            0.0,            0.0,            0.0)),
          ((        -1971.0,       -14497.0,        -5823.0),
           (       -39026.0,       -57232.0,       -24446.0),
           (            0.0,            0.0,            0.0)),
          ((      -128328.0,      -224868.0,       -94858.0),
           (         6097.0,       -83997.0,       182937.0),
           (            0.0,            0.0,            0.0)),
          ((          166.0,        77445.0,        31989.0),
           (        66559.0,        60847.0,        22996.0),
           (            0.0,            0.0,            0.0)),
          ((         -905.0,        -7327.0,        -3071.0),
           (         7356.0,        -2072.0,        -1187.0),
           (            0.0,            0.0,            0.0)),
          ((         -321.0,        -6187.0,        -2685.0),
           (         6941.0,        -7019.0,        -3885.0),
           (            0.0,            0.0,            0.0)),
          ((        87990.0,       -98927.0,       -39208.0),
           (        -4738.0,         8539.0,         4054.0),
           (            0.0,            0.0,            0.0)),
          ((         6555.0,        -6237.0,        -2581.0),
           (        35764.0,        11792.0,         9142.0),
           (            0.0,            0.0,            0.0)),
          ((         -539.0,         4704.0,         2064.0),
           (          691.0,         5368.0,         2917.0),
           (            0.0,            0.0,            0.0)),
          ((         1758.0,        -1463.0,         -699.0),
           (         -150.0,         -732.0,         -287.0),
           (            0.0,            0.0,            0.0)),
          ((        -3582.0,         6580.0,         2919.0),
           (        -8458.0,        -2983.0,        -1789.0),
           (            0.0,            0.0,            0.0)),
          ((        -5041.0,         3379.0,         1754.0),
           (        14038.0,        -2686.0,        -2278.0),
           (            0.0,            0.0,            0.0)),
          ((         2077.0,       -10014.0,        -4720.0),
           (          187.0,         -571.0,         -257.0),
           (            0.0,            0.0,            0.0)),
          ((        -3575.0,         3859.0,         1994.0),
           (         2998.0,         -582.0,         -582.0),
           (            0.0,            0.0,            0.0)),
          ((       -10370.0,         3005.0,         1317.0),
           (          -17.0,          -19.0,           20.0),
           (            0.0,            0.0,            0.0)),
          ((         2230.0,         7379.0,         3295.0),
           (           -6.0,          -65.0,          -42.0),
           (            0.0,            0.0,            0.0)),
          ((          268.0,          -52.0,         -236.0),
           (          -51.0,           -6.0,            5.0),
           (            0.0,            0.0,            0.0)),
          ((        -2930.0,         -331.0,          338.0),
           (           36.0,          139.0,           94.0),
           (            0.0,            0.0,            0.0)),
          ((         5272.0,        -1272.0,         -968.0),
           (           11.0,           22.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((         5185.0,       -64289.0,       -26834.0),
           (          -11.0,           -7.0,          -17.0),
           (            0.0,            0.0,            0.0)),
          ((        -2272.0,         -874.0,         -566.0),
           (           -1.0,            9.0,            9.0),
           (            0.0,            0.0,            0.0)),
          ((         2523.0,        -1758.0,         -357.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -64031.0,       -81679.0,       -42410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2822.0,         3015.0,         4192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4275.0,         2312.0,         8552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3998.0,        56024.0,        25528.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15976.0,        34737.0,        13719.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5633.0,        -1513.0,         -113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -9314255269.0,  -3334314513.0,  -1446479129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5034.0,        22602.0,         9634.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -52028.0,        40931.0,        18679.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13361.0,       -12227.0,        -3897.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4349.0,        11947.0,         4274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1397.0,         -983.0,           52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          278.0,        -1892.0,         -963.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -299.0,        -1476.0,         -576.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        41482.0,       -25743.0,       -10545.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4113.0,          688.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1419.0,        -1452.0,         -520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1583.0,         4348.0,         1627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5959.0,        -3653.0,        -1226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2980.0,         1516.0,          311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1526.0,         5461.0,         2485.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1786.0,         2153.0,          823.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1443.0,        -4386.0,        -1754.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5974.0,         1884.0,          533.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2786.0,         9801.0,         4106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2916.0,         -238.0,         -229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3151.0,        -1039.0,         -326.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3048.0,         1580.0,          561.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10991.0,       -88116.0,       -37042.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18678.0,         5339.0,         1581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -830.0,         -898.0,         -369.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4748.0,         2593.0,         1003.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4662.0,        -7490.0,        -2908.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3475.0,        -1750.0,         -681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2464.0,         5238.0,         2188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27327.0,        -5711.0,        -2477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19029.0,         8986.0,         3934.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9139.0,         8450.0,         3858.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        88054.0,         5165.0,         1862.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3721.0,       -20132.0,        -8640.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1528.0,         -796.0,         -497.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -72518362.0,    -38331228.0,    -16628789.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5222.0,        -1983.0,          -73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -162.0,          515.0,          121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          462.0,         1728.0,          492.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2238.0,        -1106.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5207.0,          178.0,          431.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -105.0,          231.0,          106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          218.0,          -99.0,          -43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -95.0,           66.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -53504.0,         9949.0,         5530.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,         -473.0,         -208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -537.0,          471.0,          200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -221.0,           79.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          134.0,         -497.0,         -222.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          866.0,         1009.0,          442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           66.0,         -129.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,           30.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          334.0,         -631.0,         -313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          148.0,         -436.0,         -177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          105.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          159.0,          -88.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          330.0,         9878.0,         4215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,       -12242.0,        -5212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          225.0,         -121.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -559.0,          -30.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3323.0,         1679.0,          825.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1067.0,         -115.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -857.0,          -38.0,          -65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31321.0,        -2858.0,        -1587.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12425.0,         6614.0,         3218.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1620.0,        -3163.0,        -1215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1903.0,          997.0,          435.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,         -112.0,          -39.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          486.0,         -136.0,         -162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -821982.0,      -601120.0,      -260684.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           37.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -19.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           29.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          175.0,           25.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -39.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            7.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1429.0,           58.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -25.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -648.0,         -647.0,         -320.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           25.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          571.0,         -388.0,         -415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,         -116.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           59.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          156.0,           40.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           43.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -7.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -575.0,          468.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -644.0,           40.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           27.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,         -112.0,          -46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           44.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          222.0,           69.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11162.0,       -10378.0,        -4548.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -166.0,           -8.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           15.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,          128.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          487.0,         -318.0,         -170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           28.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            9.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -33.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          403.0,          152.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,         -150.0,          -44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           54.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           26.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,          -20.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -144.0,         -207.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,           21.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           11.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -17.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -20.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -5.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,          -91.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -49.0,          -40.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            9.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,           20.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           23.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           44.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -9.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      -903158.0,     -1063215.0,      -461413.0),
           (       -34951.0,      -117273.0,       -48435.0),
           (       -16469.0,        -2316.0,          261.0)),
          ((       979486.0,      1232567.0,       535293.0),
           (       -15640.0,        20741.0,         8866.0),
           (           28.0,          -20.0,           -2.0)),
          ((        10188.0,      -304232.0,      -132894.0),
           (          459.0,       -56165.0,       -23540.0),
           (            0.0,            0.0,            0.0)),
          ((       -86430.0,       -25772.0,       -11394.0),
           (       -12744.0,        18701.0,         6892.0),
           (            0.0,            0.0,            0.0)),
          ((        37786.0,       -41956.0,       -18404.0),
           (        -3499.0,        -7941.0,        -5584.0),
           (            0.0,            0.0,            0.0)),
          ((         7620.0,         5888.0,         2599.0),
           (       -41740.0,       -62275.0,       -20245.0),
           (            0.0,            0.0,            0.0)),
          ((      -155011.0,       -89437.0,       -35915.0),
           (        -2591.0,      -100095.0,       312742.0),
           (            0.0,            0.0,            0.0)),
          ((       -30110.0,       -30593.0,       -13869.0),
           (      -108235.0,        46253.0,        25029.0),
           (            0.0,            0.0,            0.0)),
          ((         1874.0,         4028.0,         1800.0),
           (        -6572.0,         8225.0,         4029.0),
           (            0.0,            0.0,            0.0)),
          ((         3687.0,         5046.0,         2231.0),
           (          -18.0,         6107.0,         3678.0),
           (            0.0,            0.0,            0.0)),
          ((        84696.0,      -109267.0,       -43761.0),
           (        -2551.0,        -4196.0,        -2261.0),
           (            0.0,            0.0,            0.0)),
          ((         5753.0,        -3551.0,        -1285.0),
           (        23934.0,        21255.0,        12860.0),
           (            0.0,            0.0,            0.0)),
          ((        -1874.0,        -1027.0,         -496.0),
           (         4269.0,        -6241.0,        -2704.0),
           (            0.0,            0.0,            0.0)),
          ((         1011.0,        -1046.0,         -462.0),
           (         -461.0,        -1027.0,         -470.0),
           (            0.0,            0.0,            0.0)),
          ((        -2688.0,         3388.0,         1301.0),
           (        -5207.0,        -2410.0,         -926.0),
           (            0.0,            0.0,            0.0)),
          ((        -5688.0,        -5558.0,        -2831.0),
           (         9712.0,       -11873.0,        -4355.0),
           (            0.0,            0.0,            0.0)),
          ((        17828.0,        12468.0,         5742.0),
           (          502.0,        -1474.0,         -521.0),
           (            0.0,            0.0,            0.0)),
          ((        -7654.0,        -1639.0,         -819.0),
           (         3823.0,         -369.0,          379.0),
           (            0.0,            0.0,            0.0)),
          ((         1779.0,        12649.0,         5642.0),
           (          -29.0,           50.0,           -5.0),
           (            0.0,            0.0,            0.0)),
          ((        -7755.0,        -2743.0,        -1120.0),
           (           51.0,            0.0,            8.0),
           (            0.0,            0.0,            0.0)),
          ((         4756.0,        -1860.0,         -906.0),
           (            2.0,           14.0,           -5.0),
           (            0.0,            0.0,            0.0)),
          ((       -12787.0,         3898.0,         2018.0),
           (         -126.0,          -72.0,           19.0),
           (            0.0,            0.0,            0.0)),
          ((        12725.0,        -5274.0,        -2573.0),
           (          -15.0,          -20.0,            8.0),
           (            0.0,            0.0,            0.0)),
          ((         1137.0,       -53324.0,       -21953.0),
           (           -6.0,           -6.0,            9.0),
           (            0.0,            0.0,            0.0)),
          ((         2025.0,        -2621.0,        -1334.0),
           (           -5.0,            0.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((        -2964.0,         5106.0,         2315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -63710.0,        83724.0,        32119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3212.0,        11138.0,         7091.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -544.0,        22862.0,        15407.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        57070.0,       -39826.0,       -17879.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13255.0,       -25802.0,       -11873.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8009.0,        -4552.0,        -2214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -7571202902.0,  -5991192713.0,  -2598208613.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12987.0,       -14326.0,        -6402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        41748.0,       -41128.0,       -17989.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4888.0,        31083.0,        12634.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10528.0,       -62929.0,       -25534.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1674.0,         7507.0,         3224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3113.0,         -822.0,         -296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -662.0,          949.0,          338.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        50836.0,         6829.0,         3461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2872.0,        -3552.0,        -1461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          342.0,          912.0,          302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           89.0,         -233.0,          206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          411.0,         3624.0,         1268.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1196.0,           13.0,         -158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7453.0,         1929.0,          806.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -634.0,          -17.0,          111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           80.0,        -1674.0,         -968.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -932.0,        -2945.0,        -1031.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7015.0,         6675.0,         3986.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -813.0,        -1675.0,         -786.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1300.0,          755.0,          274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -874.0,         -328.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        68989.0,       -60560.0,       -25421.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8043.0,         1550.0,         1175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -472.0,          485.0,          198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          591.0,         -260.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -693.0,        -5956.0,        -2539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -860.0,         -758.0,         -387.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4417.0,        -2037.0,         -829.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11854.0,       -16923.0,        -7415.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1134.0,        13164.0,         5748.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2060.0,        15794.0,         6794.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2216.0,       -89189.0,       -38582.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8427.0,        20951.0,         9057.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1683.0,        -7781.0,        -3304.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -30311627.0,    -71499162.0,    -31006360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2363.0,        11005.0,         4474.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          116.0,        -1183.0,         -488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          299.0,        -2031.0,         -885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1346.0,         1913.0,          821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2342.0,        -2510.0,         -816.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -313.0,          247.0,           96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,         -120.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -261.0,         -321.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -37326.0,       -36783.0,       -15332.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          588.0,           -5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,         -115.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          431.0,          656.0,          284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -139.0,          118.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,          252.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1106.0,          289.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          165.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,         -122.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          764.0,          177.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,          105.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,         -221.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          221.0,         -199.0,          -84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5019.0,        -1122.0,         -771.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6413.0,         1172.0,          838.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -182.0,          211.0,           60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          273.0,         -336.0,          -96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          316.0,         2872.0,         1108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -497.0,          319.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          289.0,         -375.0,         -169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18517.0,       -17007.0,        -8498.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4205.0,         4685.0,         3228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3186.0,        -1870.0,        -1344.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1286.0,        -1930.0,         -852.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -131.0,         -158.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,          264.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       122546.0,      -955727.0,      -414321.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,          -36.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           30.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -28.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,          223.0,          113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           19.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -348.0,        -1293.0,         -565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           21.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -30.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -338.0,          840.0,          317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -32.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          479.0,          613.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           44.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,          -14.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          109.0,          -38.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,         -118.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,          -66.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           10.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,           -9.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -765.0,           62.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -20.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          619.0,         -183.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,            7.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          120.0,          -28.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           18.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,          -24.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -13.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -215.0,           87.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9419.0,       -11702.0,        -5027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          178.0,          -19.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,            2.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,            6.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -141.0,           37.0,          202.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          161.0,          546.0,          224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -69.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           10.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -11.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -24.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           17.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -19.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          109.0,         -292.0,         -124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           23.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           36.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,          -37.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           10.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          216.0,         -103.0,          -53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            5.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            3.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,          -28.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           18.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           13.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           98.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           65.0,            4.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -9.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -38.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,            1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            3.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,           -9.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      -423244.0,      -727115.0,      -253038.0),
           (       171977.0,       289165.0,       124759.0),
           (       -37614.0,        -3425.0,        -2243.0)),
          ((       541763.0,       804409.0,       279009.0),
           (       -30164.0,        -4559.0,        -1869.0),
           (          -25.0,          -30.0,          -21.0)),
          ((      -257097.0,        -8693.0,        12588.0),
           (        37497.0,        48804.0,        21123.0),
           (            0.0,            0.0,            0.0)),
          ((       -53464.0,        -6029.0,        -1244.0),
           (       -26906.0,       -16798.0,        -8487.0),
           (            0.0,            0.0,            0.0)),
          ((       -44675.0,       -35817.0,       -15207.0),
           (        -5322.0,       -16179.0,        -9053.0),
           (            0.0,            0.0,            0.0)),
          ((        -1499.0,         3953.0,         1290.0),
           (        14329.0,        -1191.0,        10527.0),
           (            0.0,            0.0,            0.0)),
          ((      -213945.0,       -41462.0,       -16233.0),
           (       -32853.0,      -132785.0,       394799.0),
           (            0.0,            0.0,            0.0)),
          ((         6780.0,       -18290.0,        -6332.0),
           (       144399.0,      -144329.0,       -65687.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         2732.0,         1049.0),
           (         5104.0,        -6859.0,        -3232.0),
           (            0.0,            0.0,            0.0)),
          ((        -5828.0,         4300.0,         1681.0),
           (        10535.0,         3370.0,          994.0),
           (            0.0,            0.0,            0.0)),
          ((        92769.0,      -108794.0,       -43356.0),
           (        -2619.0,        -2452.0,        -1065.0),
           (            0.0,            0.0,            0.0)),
          ((        -4349.0,        -2784.0,        -1185.0),
           (        -8657.0,        46252.0,        24386.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,         -532.0,         -292.0),
           (           22.0,          789.0,          639.0),
           (            0.0,            0.0,            0.0)),
          ((        -1373.0,         -479.0,         -133.0),
           (          138.0,          648.0,          280.0),
           (            0.0,            0.0,            0.0)),
          ((          801.0,         3848.0,         1493.0),
           (         4670.0,        -7655.0,        -2922.0),
           (            0.0,            0.0,            0.0)),
          ((        -5924.0,          781.0,          174.0),
           (        -3708.0,        -1137.0,         -423.0),
           (            0.0,            0.0,            0.0)),
          ((         6701.0,        10474.0,         4771.0),
           (         -772.0,          442.0,          120.0),
           (            0.0,            0.0,            0.0)),
          ((        -3857.0,        -1825.0,        -1043.0),
           (         1933.0,        -1572.0,         -808.0),
           (            0.0,            0.0,            0.0)),
          ((        11043.0,         7657.0,         3376.0),
           (          -28.0,          173.0,           79.0),
           (            0.0,            0.0,            0.0)),
          ((         3515.0,        -1723.0,         -827.0),
           (           40.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3567.0,         2753.0,         1320.0),
           (          -20.0,           31.0,           11.0),
           (            0.0,            0.0,            0.0)),
          ((       -12297.0,        -8183.0,        -3892.0),
           (           87.0,          -54.0,         -112.0),
           (            0.0,            0.0,            0.0)),
          ((        11047.0,         6238.0,         3034.0),
           (            8.0,          -22.0,          -26.0),
           (            0.0,            0.0,            0.0)),
          ((        -4963.0,       -48531.0,       -20117.0),
           (           13.0,          -12.0,          -12.0),
           (            0.0,            0.0,            0.0)),
          ((         -269.0,         -142.0,          -97.0),
           (            5.0,            2.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((         -810.0,        -1729.0,         -548.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       108959.0,        25880.0,        19942.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13456.0,         8288.0,         3614.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5347.0,        26356.0,        12202.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -40847.0,       -10795.0,        -3053.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31232.0,        26803.0,        12365.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14381.0,        -7408.0,        -3495.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -4963081724.0,  -7963647541.0,  -3452657893.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10152.0,        -1405.0,        -2172.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -36917.0,        38271.0,        16737.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -33205.0,       -37393.0,       -15430.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        30589.0,        61294.0,        25705.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8370.0,        -7598.0,        -3069.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2285.0,         2975.0,         1275.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          448.0,          223.0,          175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        37322.0,        32357.0,        14451.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1676.0,         2938.0,         1228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1375.0,         -235.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1519.0,          -61.0,          -94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5826.0,        -1324.0,         -612.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1069.0,         1958.0,          616.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          580.0,        -7244.0,        -3183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1307.0,         -480.0,         -188.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          347.0,          311.0,           82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5633.0,        -1289.0,         -347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9036.0,        -5547.0,         -851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4424.0,         1230.0,          359.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2881.0,            3.0,          100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2670.0,          -90.0,         -124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       100274.0,        -8580.0,        -1926.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9035.0,        -3310.0,        -1677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1874.0,         -811.0,         -340.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5795.0,         1949.0,          788.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5006.0,        -6616.0,        -2831.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3705.0,        -2314.0,         -969.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6898.0,         8629.0,         3718.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10378.0,        14353.0,         6356.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10769.0,         5368.0,         2199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6136.0,         7389.0,         3083.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -87864.0,         5093.0,         2478.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14588.0,       -12544.0,        -5472.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4712.0,          321.0,          154.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     25164448.0,    -73078867.0,    -31683440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5296.0,         6600.0,         2840.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -854.0,         -690.0,         -297.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2936.0,         1056.0,          505.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2466.0,         -519.0,         -274.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4486.0,        -1237.0,         -811.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,         -171.0,          -80.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -142.0,          176.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,         -100.0,          -51.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14848.0,       -48035.0,       -21412.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          258.0,          714.0,          312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,         -154.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          781.0,          -86.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          188.0,         -148.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          195.0,          758.0,          347.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,        -1834.0,         -798.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          108.0,          302.0,          135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -92.0,         -180.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -303.0,          712.0,          331.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          191.0,          441.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -153.0,           -5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          150.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3695.0,        -3582.0,        -1705.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4642.0,         4903.0,         2319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -220.0,          269.0,          117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          151.0,         -742.0,         -312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,         4906.0,         2068.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,         1520.0,          607.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -211.0,        -1073.0,         -423.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3489.0,         9500.0,         4420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1564.0,         8943.0,         3558.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3029.0,         2435.0,          717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1040.0,         -991.0,         -435.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          475.0,          440.0,          162.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3865.0,        -3771.0,        -1284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       950888.0,      -405022.0,      -175901.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          190.0,          182.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,          -91.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           44.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -159.0,          -97.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,            3.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -16.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1281.0,         -556.0,         -263.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           24.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          969.0,         -158.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,          -22.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -671.0,          279.0,          404.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           68.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,           16.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -97.0,           28.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -118.0,          -31.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -89.0,           30.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -46.0,           26.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -611.0,         -360.0,         -176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           13.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -15.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -549.0,          315.0,          139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -38.0,          -20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          139.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -26.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -84.0,          -16.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14213.0,         5064.0,         2176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -88.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -4.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,         -205.0,         -107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -618.0,          -41.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           51.0,           59.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -15.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           18.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           27.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,          -23.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -20.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           21.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -230.0,          220.0,          112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           10.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,          -42.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,            6.0,          -76.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,            1.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           40.0,           -3.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -18.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,          212.0,          107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            2.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -5.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          176.0,           35.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,          -16.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,          -97.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            6.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           13.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,           33.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            7.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            0.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,           21.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -24.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            0.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,          -31.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((       325607.0,      1522676.0,       578318.0),
           (      -108569.0,        81659.0,        30653.0),
           (       -32013.0,        10565.0,          803.0)),
          ((      -436095.0,     -1718767.0,      -653480.0),
           (        30005.0,        11409.0,         5433.0),
           (            4.0,           48.0,           20.0)),
          ((        47501.0,       163634.0,        50277.0),
           (       -38980.0,        19688.0,         7315.0),
           (            0.0,            0.0,            0.0)),
          ((       -10751.0,        65811.0,        26352.0),
           (        27805.0,        -6045.0,        -3336.0),
           (            0.0,            0.0,            0.0)),
          ((       -35609.0,        40401.0,        17652.0),
           (         3553.0,       -13794.0,        -7501.0),
           (            0.0,            0.0,            0.0)),
          ((        -2363.0,        -9848.0,        -3805.0),
           (       -27744.0,        48383.0,        31397.0),
           (            0.0,            0.0,            0.0)),
          ((      -222062.0,       -34194.0,       -10258.0),
           (        15248.0,      -201634.0,       408518.0),
           (            0.0,            0.0,            0.0)),
          ((        13827.0,        52629.0,        21476.0),
           (      -129659.0,       144401.0,        63322.0),
           (            0.0,            0.0,            0.0)),
          ((         -172.0,        -4983.0,        -2055.0),
           (         2241.0,         8386.0,         3778.0),
           (            0.0,            0.0,            0.0)),
          ((        -3525.0,        -2813.0,        -1264.0),
           (        -6737.0,        14132.0,         6333.0),
           (            0.0,            0.0,            0.0)),
          ((        90238.0,      -103276.0,       -41024.0),
           (          398.0,        -8262.0,        -3473.0),
           (            0.0,            0.0,            0.0)),
          ((        -2105.0,        -1725.0,         -809.0),
           (       -38279.0,        28134.0,        13941.0),
           (            0.0,            0.0,            0.0)),
          ((        -1139.0,         3103.0,         1273.0),
           (        -2089.0,          340.0,           16.0),
           (            0.0,            0.0,            0.0)),
          ((         -246.0,            1.0,           62.0),
           (          642.0,          328.0,          134.0),
           (            0.0,            0.0,            0.0)),
          ((         1820.0,         4604.0,         1862.0),
           (         7706.0,         -993.0,         -309.0),
           (            0.0,            0.0,            0.0)),
          ((       -14160.0,          731.0,          491.0),
           (        10610.0,          731.0,          448.0),
           (            0.0,            0.0,            0.0)),
          ((        13771.0,        -6247.0,        -3120.0),
           (         2758.0,         -165.0,         -152.0),
           (            0.0,            0.0,            0.0)),
          ((        -6337.0,          633.0,          138.0),
           (        -1072.0,         3960.0,         1453.0),
           (            0.0,            0.0,            0.0)),
          ((        13756.0,        -5612.0,        -2587.0),
           (         -118.0,          -75.0,          -21.0),
           (            0.0,            0.0,            0.0)),
          ((        -3482.0,         2802.0,          979.0),
           (           -1.0,           35.0,           12.0),
           (            0.0,            0.0,            0.0)),
          ((         2559.0,          253.0,          540.0),
           (          -58.0,            8.0,           17.0),
           (            0.0,            0.0,            0.0)),
          ((        -7755.0,        -3268.0,        -2792.0),
           (           59.0,           80.0,           74.0),
           (            0.0,            0.0,            0.0)),
          ((         4837.0,         3122.0,         2668.0),
           (            8.0,            6.0,            6.0),
           (            0.0,            0.0,            0.0)),
          ((       -13683.0,       -41495.0,       -17740.0),
           (            4.0,           17.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((         1651.0,        -2220.0,         -650.0),
           (           -3.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((         -654.0,         2560.0,          600.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1831.0,      -105238.0,       -46745.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2882.0,        14070.0,         3772.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -627.0,        30008.0,         8707.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4824.0,        51866.0,        22812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        32133.0,       -33436.0,       -14690.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11865.0,         2341.0,         1810.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -1787834477.0,  -9026290709.0,  -3912379079.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13442.0,        15191.0,         6821.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17039.0,       -25097.0,        -9892.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        74023.0,        46804.0,        23245.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -133735.0,       -85948.0,       -41351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16023.0,        11816.0,         5812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4903.0,        -3834.0,        -1926.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2991.0,           94.0,          177.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7372.0,        46677.0,        20534.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6869.0,          586.0,         -163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1879.0,          854.0,          610.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7012.0,        -4320.0,        -2586.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6139.0,         1276.0,         1286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2194.0,          730.0,          -94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9136.0,         1102.0,          635.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1002.0,         -793.0,         -581.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1185.0,         2914.0,         1674.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4255.0,          878.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1953.0,       -11431.0,        -4239.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2889.0,          929.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1936.0,         -937.0,         -113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2364.0,          538.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        86318.0,        38866.0,        19125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9051.0,         5508.0,         1260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1474.0,          392.0,          280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4856.0,         -394.0,         -531.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6197.0,        -4449.0,        -1630.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4734.0,          428.0,          384.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8954.0,        -4137.0,        -2117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11888.0,        -7399.0,        -3042.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6447.0,        -1850.0,        -1040.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8191.0,        -9239.0,        -3999.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21787.0,        86231.0,        37189.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -26133.0,         9405.0,         4208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5312.0,         3271.0,         1261.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     69446821.0,    -42444844.0,    -18398699.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5721.0,         -406.0,          612.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,          466.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1040.0,         -187.0,         -176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -784.0,         -713.0,         -191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4127.0,         1542.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          111.0,         -342.0,         -139.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,          125.0,           54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          446.0,           81.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        52626.0,       -12853.0,        -6797.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -589.0,          235.0,          103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,          -11.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,         -662.0,         -280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,         -288.0,         -115.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,           85.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1041.0,          374.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          111.0,           67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          -48.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -976.0,          165.0,          105.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -180.0,          -86.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          121.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          425.0,          186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          100.0,         3191.0,         1640.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -124.0,        -3138.0,        -1654.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          198.0,         -106.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -379.0,         -253.0,         -153.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1585.0,         3503.0,         1663.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1121.0,         1069.0,          535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -921.0,        -1113.0,         -511.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22165.0,       -30272.0,       -12799.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16847.0,        13596.0,         5547.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2214.0,         2683.0,         1539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2649.0,          474.0,          216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -392.0,          -53.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          528.0,        -1377.0,         -621.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       875690.0,       522747.0,       226579.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          152.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -95.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           63.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,         -192.0,         -106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -36.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -17.0,          -38.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          897.0,         1043.0,          450.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           35.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -585.0,         -689.0,         -330.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           31.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -281.0,         -624.0,         -233.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,         -116.0,          -50.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           20.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           43.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          130.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           37.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -30.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -415.0,         -486.0,         -215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           11.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          452.0,         -416.0,         -180.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           15.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -107.0,          -24.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,          -63.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,         -226.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2670.0,        14529.0,         6276.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          206.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            7.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,           -6.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          189.0,          159.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          258.0,         -535.0,         -257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -8.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -33.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -9.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -18.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            8.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,          -23.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           15.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          338.0,         -217.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,           81.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,          -46.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -45.0,           69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            7.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -2.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -45.0,           28.0,           23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -1.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -245.0,           52.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            1.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            8.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           -5.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -173.0,          -42.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           13.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            7.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           46.0,           88.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -6.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -10.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,          -58.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -4.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           15.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            6.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           55.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      -494594.0,        78038.0,       108012.0),
           (       177047.0,      -232307.0,       -99507.0),
           (        39115.0,       -47340.0,       -16331.0)),
          ((       630659.0,       -74902.0,      -115196.0),
           (        17124.0,          408.0,         -133.0),
           (           24.0,          -54.0,          -20.0)),
          ((        72961.0,       170019.0,        91799.0),
           (        28399.0,       -38820.0,       -16285.0),
           (            0.0,            0.0,            0.0)),
          ((        -8250.0,        39626.0,        18702.0),
           (        14303.0,        22958.0,         8348.0),
           (            0.0,            0.0,            0.0)),
          ((        38955.0,        33308.0,        14153.0),
           (         1155.0,        -2403.0,        -2251.0),
           (            0.0,            0.0,            0.0)),
          ((         6338.0,        -4230.0,        -2209.0),
           (        24860.0,        48831.0,        23345.0),
           (            0.0,            0.0,            0.0)),
          ((      -166545.0,        74057.0,        34892.0),
           (        50254.0,      -145607.0,       416984.0),
           (            0.0,            0.0,            0.0)),
          ((       -24973.0,         5662.0,         3159.0),
           (        90205.0,      -125893.0,       -53522.0),
           (            0.0,            0.0,            0.0)),
          ((         2979.0,        -1294.0,         -558.0),
           (        -8882.0,        -4625.0,        -1657.0),
           (            0.0,            0.0,            0.0)),
          ((         5547.0,          741.0,          445.0),
           (       -12144.0,       -11055.0,        -4718.0),
           (            0.0,            0.0,            0.0)),
          ((        81400.0,      -106377.0,       -42589.0),
           (        13178.0,         4964.0,         2044.0),
           (            0.0,            0.0,            0.0)),
          ((         8067.0,          422.0,          466.0),
           (       -44478.0,       -39636.0,       -18830.0),
           (            0.0,            0.0,            0.0)),
          ((        -3050.0,         2481.0,          875.0),
           (         2061.0,        -7354.0,        -3593.0),
           (            0.0,            0.0,            0.0)),
          ((         2003.0,         -182.0,           -7.0),
           (          703.0,         -223.0,          -79.0),
           (            0.0,            0.0,            0.0)),
          ((          924.0,         1720.0,          313.0),
           (          388.0,         8169.0,         3521.0),
           (            0.0,            0.0,            0.0)),
          ((       -13219.0,         -668.0,        -2023.0),
           (        18248.0,        -2106.0,         -214.0),
           (            0.0,            0.0,            0.0)),
          ((        27831.0,        -5516.0,         -533.0),
           (         -251.0,         1280.0,          651.0),
           (            0.0,            0.0,            0.0)),
          ((       -10040.0,        -1083.0,        -1440.0),
           (         1971.0,         -263.0,          196.0),
           (            0.0,            0.0,            0.0)),
          ((         5686.0,       -11940.0,        -4867.0),
           (         -145.0,          -67.0,          -47.0),
           (            0.0,            0.0,            0.0)),
          ((        -3348.0,        -4205.0,        -1891.0),
           (          -14.0,          -11.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((         3364.0,         5406.0,         2620.0),
           (          -41.0,          -33.0,          -14.0),
           (            0.0,            0.0,            0.0)),
          ((        -6101.0,       -17398.0,        -8127.0),
           (         -126.0,          -37.0,           37.0),
           (            0.0,            0.0,            0.0)),
          ((         1843.0,        17798.0,         8222.0),
           (          -12.0,            5.0,           16.0),
           (            0.0,            0.0,            0.0)),
          ((       -24164.0,       -43817.0,       -18994.0),
           (          -12.0,           -9.0,            7.0),
           (            0.0,            0.0,            0.0)),
          ((         -105.0,         2344.0,         1135.0),
           (           -3.0,            2.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((        -2113.0,        -3261.0,        -1618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -102452.0,        41790.0,        10861.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1354.0,        -3756.0,        -3851.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1812.0,         2689.0,        -5717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        57074.0,        -2796.0,        -5257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -40029.0,        21315.0,         9024.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11654.0,         9392.0,         3254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   1591789382.0,  -9057576223.0,  -3925008972.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7158.0,       -30740.0,       -11825.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8040.0,        16228.0,         5563.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -64922.0,        -2862.0,        -3839.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        74924.0,         8194.0,         5909.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14760.0,        -1443.0,        -1483.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4532.0,          229.0,          335.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3499.0,         1346.0,          377.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -32641.0,        41154.0,        17199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4077.0,        -6184.0,        -2089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2953.0,         2547.0,          874.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5601.0,        -6844.0,        -2201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9773.0,         9269.0,         3315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2854.0,        -2253.0,         -814.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3710.0,         9153.0,         3849.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2021.0,        -2403.0,         -847.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2182.0,         5819.0,         2238.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5157.0,        -2142.0,         -696.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10503.0,        -2660.0,        -2618.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5520.0,        -2608.0,         -684.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3173.0,         2741.0,          857.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3142.0,        -3170.0,        -1038.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17964.0,        93917.0,        39477.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8984.0,       -15588.0,        -5459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2299.0,         1485.0,          545.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6738.0,        -5856.0,        -2190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,        -1108.0,         -623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5500.0,         5088.0,         2032.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10168.0,        -6996.0,        -2763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11624.0,        -1039.0,         -629.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6209.0,        -9150.0,        -3833.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -683.0,        -7847.0,        -3535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        88977.0,       -26563.0,       -11559.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        30234.0,         -298.0,         -329.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -218.0,        -1259.0,         -324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     83014720.0,      6826455.0,      2957608.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9332.0,         5231.0,         1089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -306.0,         -919.0,         -254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1527.0,        -2006.0,         -595.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1704.0,         3905.0,         1344.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2582.0,          893.0,          544.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          291.0,          131.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,         -224.0,          -89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          233.0,           22.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        39624.0,        34722.0,        14383.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -447.0,         -622.0,         -270.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          144.0,           97.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -798.0,         -196.0,          -90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -206.0,          -49.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -309.0,         -141.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -824.0,          690.0,          301.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,           -2.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           27.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          159.0,        -1466.0,         -626.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -567.0,          394.0,          166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          177.0,          -88.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -113.0,          -15.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5080.0,        -4974.0,        -1997.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6449.0,         6136.0,         2443.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,            6.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -91.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -682.0,         2361.0,         1043.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -301.0,         -169.0,          -48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          212.0,           74.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16424.0,        11858.0,         5279.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -923.0,         -350.0,         -287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3637.0,         -796.0,          144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          815.0,         2600.0,         1137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           58.0,          495.0,          246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1005.0,         -714.0,         -659.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20657.0,       957089.0,       415060.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,           10.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,           14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -48.0,          -25.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          149.0,          149.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -7.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           15.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -947.0,          983.0,          444.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           32.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -410.0,          818.0,          302.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -21.0,           -9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          748.0,          -15.0,         -300.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,           46.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -47.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,         -115.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          172.0,          -11.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          -51.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           19.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -23.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,         -679.0,         -273.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -333.0,          493.0,          214.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -76.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,          -54.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -68.0,           29.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           85.0,           56.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15134.0,          455.0,          244.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,          -96.0,          -35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -5.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           15.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -174.0,           37.0,          201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          452.0,          447.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,           43.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -2.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            6.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -3.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -5.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -439.0,           97.0,           40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          163.0,          -52.0,          -45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -61.0,           33.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           47.0,           -8.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           -7.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          -20.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -6.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,         -227.0,          -88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           -2.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            0.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          171.0,           49.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -6.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -6.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -73.0,          -70.0,          -34.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            2.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            5.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           58.0,           26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,          -37.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -4.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            0.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            7.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           19.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            4.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -51.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // mar
    marst   : array [1..5,1..183,0..2,1..3] of double = (
         (((        45338.0,       499193.0,       391677.0),
           (     -6536664.0,       925257.0,       267033.0),
           (      1365318.0,      3754681.0,      2030824.0)),
          ((     -1279006.0,      -918089.0,      -523065.0),
           (        37606.0,       830527.0,       418453.0),
           (            0.0,            0.0,            0.0)),
          ((        18815.0,        80721.0,       197879.0),
           (      -538525.0,      -522711.0,      -203663.0),
           (            0.0,            0.0,            0.0)),
          ((       287659.0,       -45169.0,      -261281.0),
           (     -2186652.0,      1076308.0,       496230.0),
           (            0.0,            0.0,            0.0)),
          ((       -91265.0,       243842.0,       363810.0),
           (       464388.0,      1369019.0,       529471.0),
           (            0.0,            0.0,            0.0)),
          ((     -1083628.0,      -466147.0,      -335482.0),
           (         5998.0,         7488.0,         2778.0),
           (            0.0,            0.0,            0.0)),
          ((       120444.0,         5916.0,        84694.0),
           (       -67751.0,      -103775.0,       -41795.0),
           (            0.0,            0.0,            0.0)),
          ((       -88529.0,      -110364.0,      -137452.0),
           (      -883350.0,     -1656365.0,      -683819.0),
           (            0.0,            0.0,            0.0)),
          ((       231830.0,       -36665.0,       273144.0),
           (       103232.0,       356916.0,       154852.0),
           (            0.0,            0.0,            0.0)),
          ((     -1892738.0,     -1285519.0,     -2965352.0),
           (         1932.0,        -6394.0,        -3176.0),
           (            0.0,            0.0,            0.0)),
          ((      2951676.0,      1982626.0,      4747133.0),
           (        -7265.0,         1680.0,          718.0),
           (            0.0,            0.0,            0.0)),
          ((     -1510712.0,     -1011594.0,     -2546880.0),
           (         1056.0,        -9421.0,        -4812.0),
           (            0.0,            0.0,            0.0)),
          ((       355183.0,       313736.0,       815938.0),
           (         5866.0,        -7292.0,        -1311.0),
           (            0.0,            0.0,            0.0)),
          ((     -1710831.0,      -228111.0,     -1058512.0),
           (          100.0,           49.0,            9.0),
           (            0.0,            0.0,            0.0)),
          ((      -784578.0,       232873.0,      -989442.0),
           (        -1874.0,        -1224.0,         -838.0),
           (            0.0,            0.0,            0.0)),
          ((      1596012.0,        24884.0,      1998162.0),
           (           82.0,           13.0,           -8.0),
           (            0.0,            0.0,            0.0)),
          ((      -326922.0,        56224.0,      -305276.0),
           (            4.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((       619816.0,      -278708.0,       109919.0),
           (          -14.0,           45.0,           21.0),
           (            0.0,            0.0,            0.0)),
          ((      -168726.0,       638705.0,       -81174.0),
           (           -2.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((       -40229.0,     -1146123.0,       207365.0),
           (            4.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((  -2963518471.0, -13539530851.0,  -6129456305.0),
           (           81.0,           -9.0,          -27.0),
           (            0.0,            0.0,            0.0)),
          ((       -88749.0,       -35511.0,       -40327.0),
           (           -2.0,          -27.0,          -17.0),
           (            0.0,            0.0,            0.0)),
          ((       893545.0,      -282687.0,      -155990.0),
           (           -3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((      -437194.0,       320578.0,       160084.0),
           (           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((      -165099.0,      -112779.0,       -40505.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        18357.0,       -29539.0,       -17514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        62598.0,        53109.0,        24373.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28668.0,       -17710.0,        -8035.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7406.0,        -2917.0,        -1055.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -38477.0,       121202.0,        53751.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1659.0,        26362.0,        12711.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3504.0,       -50478.0,       -31471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -394434.0,       328425.0,       164524.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        98626.0,       -79933.0,       -34017.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19088.0,       -40392.0,       -25084.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -322910.0,       523813.0,       254461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -197523.0,      -344698.0,      -170899.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       419401.0,      1174098.0,       617418.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -21721801.0,    640129416.0,    294076237.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       626383.0,      1289227.0,       717954.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -528525.0,      -994048.0,      -553012.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -50088.0,      -102982.0,       -56346.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14936.0,        28473.0,        14117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5335.0,        -5314.0,        -2827.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2414.0,         7171.0,         3678.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5136.0,         -198.0,         -608.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1426.0,         -797.0,           93.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4621.0,       -39414.0,       -18271.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -36862.0,        -2445.0,        -6337.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       237100.0,       -66744.0,       -32512.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13443.0,         4019.0,         4813.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27177.0,         6109.0,          838.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        73908.0,       -57722.0,       -25381.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15880.0,        15084.0,         2420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       113085.0,       -45865.0,       -10558.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1089308.0,       454669.0,       128552.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     13484821.0,    -43634597.0,    -20284518.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        94684.0,       -61691.0,       -23579.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -72486.0,        47321.0,        18672.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        80030.0,       -46698.0,       -19291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        92474.0,       -55885.0,       -25573.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -141525.0,        85979.0,        38465.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7420.0,        -5872.0,        -2759.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2072.0,         -301.0,         -290.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4627.0,         4925.0,         2441.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12705.0,         4153.0,         1605.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          688.0,        -1123.0,         -499.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34889.0,         3164.0,         2427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -360.0,         -409.0,         -127.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          378.0,           22.0,          -90.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8838.0,         5770.0,         2939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -154.0,        -2803.0,        -1376.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3187.0,         7018.0,         3651.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1903183.0,      3270908.0,      1552251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7455.0,       -11422.0,        -6128.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1648.0,         2070.0,         1156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          152.0,        -1055.0,         -550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          101.0,         -176.0,          -98.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          265.0,          780.0,          360.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,         -472.0,         -231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,         1188.0,          565.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          362.0,         -246.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,           21.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1466.0,        -1809.0,         -831.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -15.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          210.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4715.0,          627.0,          155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -40.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,          204.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,         -346.0,         -145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1154.0,         -107.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,         -274.0,         -117.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          781.0,         1202.0,          488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       243273.0,      -251386.0,      -121885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2295.0,        -4718.0,        -2129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1110.0,         2176.0,          993.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -313.0,         -746.0,         -352.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          267.0,          553.0,          284.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           90.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           99.0,           69.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,         -140.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -79.0,          -85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           46.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          171.0,          268.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,          147.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,         -141.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -618.0,         -282.0,         -110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,          126.0,           56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,         -296.0,         -142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          501.0,          225.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -153.0,         -450.0,         -197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          205.0,          724.0,          320.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23346.0,        36993.0,        17101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6059.0,       -19576.0,        -8264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          313.0,         1303.0,          555.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,         -281.0,         -116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           69.0,          144.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,          -76.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,           14.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           55.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -57.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          114.0,           57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,         -125.0,          -60.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,          -64.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           76.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          112.0,           32.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -41.0,           51.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,          -11.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -26.0,          -22.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3328.0,        -1267.0,         -670.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           16.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -10.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           -4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           64.0,           28.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           -1.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           19.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -25.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           15.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           24.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,            8.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -28.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          529.0,         -285.0,         -209.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -18.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -45.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           56.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -372.0,           40.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,           14.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -8.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,          -16.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           13.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,          -12.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           11.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,          -17.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            1.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -15.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,           -5.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,            6.0,           43.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -3.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            9.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -24.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -6.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -38.0,           44.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      -992062.0,      -253442.0,       -87668.0),
           (     -2340904.0,     -8035392.0,     -2440397.0),
           (     -4175717.0,     -3165300.0,     -1283037.0)),
          ((      2260634.0,         7662.0,       -60934.0),
           (       307551.0,      -381235.0,      -306815.0),
           (            0.0,            0.0,            0.0)),
          ((      -717936.0,      -138712.0,       -38698.0),
           (      -178906.0,       445443.0,       183709.0),
           (            0.0,            0.0,            0.0)),
          ((      1471407.0,       532171.0,       197599.0),
           (      2339780.0,        22734.0,        42006.0),
           (            0.0,            0.0,            0.0)),
          ((     -1462196.0,      -262254.0,       -74253.0),
           (      1067090.0,     -1182881.0,      -458806.0),
           (            0.0,            0.0,            0.0)),
          ((       563035.0,      1177886.0,       513935.0),
           (         9172.0,        -5931.0,        -2360.0),
           (            0.0,            0.0,            0.0)),
          ((      -562080.0,       -69572.0,       -13210.0),
           (       -89443.0,        76279.0,        31011.0),
           (            0.0,            0.0,            0.0)),
          ((       574918.0,        82446.0,        19584.0),
           (     -1648968.0,      1310569.0,       567179.0),
           (            0.0,            0.0,            0.0)),
          ((     -1530306.0,      -541579.0,      -197187.0),
           (       347614.0,      -322234.0,      -146626.0),
           (            0.0,            0.0,            0.0)),
          ((     12399023.0,      3310703.0,      1104102.0),
           (        -4732.0,         9356.0,         4628.0),
           (            0.0,            0.0,            0.0)),
          ((    -20080528.0,     -5236085.0,     -1732711.0),
           (         2141.0,        -4994.0,        -2443.0),
           (            0.0,            0.0,            0.0)),
          ((     10812055.0,      2752776.0,       902981.0),
           (       -20275.0,         9806.0,         5939.0),
           (            0.0,            0.0,            0.0)),
          ((     -3310593.0,      -853630.0,      -283069.0),
           (         4842.0,        -6004.0,        -3226.0),
           (            0.0,            0.0,            0.0)),
          ((      5628136.0,       856933.0,       190019.0),
           (          -26.0,          -25.0,            5.0),
           (            0.0,            0.0,            0.0)),
          ((      4128913.0,      1056205.0,       328200.0),
           (        -2139.0,         2519.0,          386.0),
           (            0.0,            0.0,            0.0)),
          ((     -8616249.0,     -1901865.0,      -562260.0),
           (           10.0,          110.0,           38.0),
           (            0.0,            0.0,            0.0)),
          ((      1033503.0,       300860.0,        91772.0),
           (           -8.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((      -676999.0,      -655010.0,      -263817.0),
           (          -27.0,          -30.0,          -13.0),
           (            0.0,            0.0,            0.0)),
          ((       292825.0,       327493.0,       122490.0),
           (           -9.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((       186061.0,      -185370.0,       -45992.0),
           (            1.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((  15110844637.0,   1415606076.0,    239293253.0),
           (            7.0,          -36.0,          -18.0),
           (            0.0,            0.0,            0.0)),
          ((      -184022.0,        66197.0,        28806.0),
           (           50.0,           61.0,           27.0),
           (            0.0,            0.0,            0.0)),
          ((       980889.0,      -630727.0,      -303797.0),
           (           -2.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((      -552542.0,       230893.0,       114768.0),
           (           -1.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        45230.0,        16886.0,         5939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        96481.0,        14657.0,         5015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16245.0,        67313.0,        29706.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1261.0,        -1124.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16839.0,        -4760.0,        -1753.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        31116.0,       -98073.0,       -44142.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7544.0,       -33188.0,       -14208.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       158633.0,       177131.0,        72492.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24650.0,       329074.0,       155547.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -136330.0,        36409.0,        22469.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -41187.0,        97309.0,        44659.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -642418.0,      -259774.0,      -100111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       157731.0,       126455.0,        50681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1119627.0,      -806629.0,      -324749.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    404128414.0,   -522859311.0,   -250789824.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1688587.0,     -1352785.0,      -551444.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1273497.0,      1006203.0,       408988.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       123510.0,        97295.0,        39539.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11575.0,       -27749.0,       -12095.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        12562.0,         2263.0,          611.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6800.0,       -13514.0,        -6004.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10589.0,         3720.0,         1692.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7219.0,        -2323.0,         -813.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -33241.0,         8097.0,         4428.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        69953.0,        44465.0,        16411.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       145218.0,        65259.0,        27795.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -34129.0,       -18673.0,        -6882.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        41756.0,        32182.0,        13165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -94482.0,        12525.0,         8749.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        58018.0,        70627.0,        30081.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -204896.0,      -185278.0,       -76927.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1760169.0,      1628536.0,       678054.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -23038331.0,    -41999943.0,    -18620020.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -118156.0,      -139003.0,       -59339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        90538.0,       103065.0,        43775.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -89688.0,      -101867.0,       -43421.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -72999.0,      -106393.0,       -46185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       123594.0,       163672.0,        70677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5899.0,       -10741.0,        -4789.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1664.0,        -1631.0,         -611.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1379.0,         3176.0,         1490.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3856.0,         7603.0,         3420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -239.0,          198.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        29988.0,       -15628.0,        -8040.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -35.0,         -384.0,         -264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1149.0,          662.0,          252.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2439.0,         8521.0,         3842.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,         -515.0,         -223.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1337.0,         2755.0,         1280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -4083854.0,      -271308.0,       -13649.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5137.0,        -4032.0,        -1938.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1507.0,          861.0,          448.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,          596.0,          277.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,         -111.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,         -139.0,         -123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          231.0,         -126.0,          -68.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,          862.0,          408.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,          103.0,           53.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,          -42.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2454.0,          -58.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -161.0,         -110.0,          -52.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          432.0,          365.0,          167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -492.0,        -4491.0,        -2063.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          367.0,          334.0,          163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -568.0,         -499.0,         -262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1045.0,          651.0,          319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          390.0,         -303.0,         -197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          759.0,          402.0,          232.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2876.0,        -2535.0,        -1461.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -204778.0,       280443.0,       133899.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         6239.0,         4162.0,         2593.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2311.0,        -1417.0,         -885.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          560.0,          286.0,          179.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           43.0,           63.0,           30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          160.0,          100.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -181.0,          -27.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          139.0,           51.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,            7.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,            7.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          183.0,         -322.0,         -144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,          -40.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           79.0,           -2.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -570.0,         -272.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -90.0,            8.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          173.0,           26.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -198.0,           59.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          284.0,         -156.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -103.0,          151.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9436.0,        34540.0,        15431.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7796.0,        -5881.0,        -2746.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -539.0,          302.0,          157.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          133.0,          -16.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           38.0,           -4.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          123.0,          -59.0,          -26.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           12.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -16.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -3.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -6.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           75.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          -89.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,          -53.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           61.0,           36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,          -39.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -23.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,           51.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          -13.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           12.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           20.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3631.0,          140.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -37.0,           30.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,          -14.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            0.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,           28.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,           -4.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,          -12.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,           16.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            8.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,            5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -19.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          114.0,          543.0,          281.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,          -14.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -25.0,          -18.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          202.0,         -282.0,         -134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -9.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -22.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           18.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           15.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            0.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          131.0,          -44.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,           12.0,           42.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           -2.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,            7.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           10.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,           36.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          -21.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -2.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,           10.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            5.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,           -9.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      -276257.0,       358426.0,       258724.0),
           (      6728244.0,      3150619.0,       958028.0),
           (     -2540996.0,      2501837.0,      1632575.0)),
          ((      -816552.0,       612956.0,       222725.0),
           (       590561.0,        44970.0,        25128.0),
           (            0.0,            0.0,            0.0)),
          ((      -254539.0,       584602.0,       355252.0),
           (       -73197.0,      -163881.0,       -56680.0),
           (            0.0,            0.0,            0.0)),
          ((       289880.0,      -284585.0,      -252753.0),
           (     -1208048.0,     -1414084.0,      -683072.0),
           (            0.0,            0.0,            0.0)),
          ((      -225951.0,       578193.0,       393012.0),
           (      -564567.0,        32529.0,       -15935.0),
           (            0.0,            0.0,            0.0)),
          ((      1288311.0,       -61710.0,      -137947.0),
           (        -5273.0,        -1418.0,         -695.0),
           (            0.0,            0.0,            0.0)),
          ((       -50560.0,       221516.0,       142464.0),
           (        80068.0,        13804.0,         7477.0),
           (            0.0,            0.0,            0.0)),
          ((       112974.0,      -205219.0,      -137269.0),
           (      1654847.0,       540593.0,       228957.0),
           (            0.0,            0.0,            0.0)),
          ((         5737.0,       717808.0,       450784.0),
           (      -386713.0,      -114422.0,       -45577.0),
           (            0.0,            0.0,            0.0)),
          ((       847332.0,     -5460303.0,     -3450052.0),
           (         8227.0,          278.0,          -87.0),
           (            0.0,            0.0,            0.0)),
          ((     -1737437.0,      8950494.0,      5607337.0),
           (        -5124.0,        -1011.0,         -415.0),
           (            0.0,            0.0,            0.0)),
          ((      1096951.0,     -4918948.0,     -3044840.0),
           (        14699.0,        -4017.0,        -2808.0),
           (            0.0,            0.0,            0.0)),
          ((      -408237.0,      1604918.0,       966194.0),
           (         4908.0,       -18301.0,        -6936.0),
           (            0.0,            0.0,            0.0)),
          ((      -243779.0,     -1804778.0,     -1048131.0),
           (          -75.0,          -21.0,          -20.0),
           (            0.0,            0.0,            0.0)),
          ((       574574.0,     -2581086.0,     -1300015.0),
           (          772.0,         5277.0,         2180.0),
           (            0.0,            0.0,            0.0)),
          ((      -800037.0,      4390886.0,      2310844.0),
           (          -16.0,          143.0,           60.0),
           (            0.0,            0.0,            0.0)),
          ((        48311.0,      -648888.0,      -308653.0),
           (            5.0,          -19.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((      -712781.0,       609251.0,       250145.0),
           (          -36.0,          -34.0,          -25.0),
           (            0.0,            0.0,            0.0)),
          ((       143994.0,      -782658.0,      -204170.0),
           (            8.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((      -675724.0,      1716648.0,       300654.0),
           (            2.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((  -5670769260.0,  12730086788.0,   5992501553.0),
           (          -26.0,          -39.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       -45133.0,      -276359.0,       -40981.0),
           (          -49.0,           68.0,           33.0),
           (            0.0,            0.0,            0.0)),
          ((      -611107.0,      -602218.0,      -411970.0),
           (            4.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       215380.0,       171039.0,       188086.0),
           (            1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((       -91585.0,       132343.0,        79001.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43724.0,        23614.0,         6216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        75566.0,        69611.0,        31888.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27062.0,        16940.0,         7114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5118.0,         2030.0,         1650.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8421.0,        73057.0,        32107.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -23197.0,        20181.0,        11854.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       188256.0,       -73087.0,       -54144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       329376.0,       271532.0,       127015.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        39703.0,       -16211.0,         -744.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       116383.0,      -136186.0,       -73138.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -93589.0,      -468277.0,      -199093.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        69381.0,      -221953.0,      -128035.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -682684.0,       654961.0,       432550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -653837269.0,    236131642.0,    125859804.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1061651.0,      1024123.0,       661331.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       818333.0,      -841866.0,      -533907.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        86845.0,       -86353.0,       -53226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -31069.0,        10775.0,         7246.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3573.0,        -1298.0,        -1530.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16606.0,        16431.0,         8608.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1826.0,        -2251.0,        -1698.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1197.0,         6069.0,         3240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,        38238.0,        17603.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        36538.0,       -68153.0,       -35887.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21886.0,       248074.0,       116456.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9702.0,        38531.0,        19657.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        23848.0,       -39999.0,       -21089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28678.0,         5681.0,         3952.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        59715.0,      -106915.0,       -53823.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -146415.0,       276101.0,       138450.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1251783.0,     -2461108.0,     -1221190.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -46361741.0,    -16291481.0,     -6168380.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -81795.0,       198728.0,        95936.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        76417.0,      -163494.0,       -78761.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -72001.0,       158303.0,        75674.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -81624.0,       172293.0,        80943.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       122928.0,      -270139.0,      -127300.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7446.0,        13388.0,         6215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          404.0,         2818.0,         1174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1935.0,        -7400.0,        -3393.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1044.0,        17684.0,         8170.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,          898.0,          496.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19570.0,        26900.0,        12939.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          836.0,        -1879.0,         -763.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1036.0,          917.0,          385.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9914.0,          102.0,         -224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          500.0,          211.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3781.0,        -3172.0,        -1631.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1433620.0,     -3499435.0,     -1566532.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6764.0,         5237.0,         2964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1455.0,        -1024.0,         -613.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          392.0,          219.0,          134.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -108.0,          132.0,           83.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -310.0,         -661.0,         -311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -204.0,          -71.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          371.0,          837.0,          350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -355.0,          327.0,          114.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -33.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1356.0,         1853.0,          838.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           22.0,           16.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          179.0,           17.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4650.0,          678.0,          409.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -99.0,           82.0,           81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          102.0,         -122.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -101.0,          264.0,          182.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          864.0,          563.0,          186.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -264.0,          211.0,          151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2335.0,        -1821.0,        -1086.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       161068.0,      -306486.0,      -144871.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5184.0,         5215.0,         2599.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2137.0,        -1968.0,         -920.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -605.0,          665.0,          258.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          262.0,         -250.0,          -81.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -39.0,          154.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          150.0,          -61.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          116.0,           65.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -91.0,           92.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,          -53.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -370.0,          109.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -27.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           59.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -517.0,         -254.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -70.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -175.0,           97.0,           18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          103.0,          -86.0,           31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -127.0,          195.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          138.0,         -356.0,          -84.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        38477.0,       -17058.0,        -6042.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4785.0,         7437.0,          471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          437.0,         -488.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -87.0,          140.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -31.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,          125.0,           44.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           -5.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           -6.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -36.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -42.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           31.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,          -47.0,          -23.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,          -24.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           40.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,          -32.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -12.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,           44.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -20.0,           12.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -29.0,          -19.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3237.0,         1519.0,          612.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,           42.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            0.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           33.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           49.0,           55.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -17.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           -1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -35.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,           18.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -20.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           42.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           16.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -613.0,         -102.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,          -10.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,            4.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,          321.0,          143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           10.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,          -12.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -9.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           14.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -27.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           18.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,            1.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,          -16.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,            0.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,           -8.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,           97.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -114.0,          -71.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           11.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -12.0,          -15.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -17.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,          -31.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           52.0,           49.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            8.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            7.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            2.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          -31.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((      1453270.0,       804446.0,       318086.0),
           (     -6641423.0,     -3941762.0,     -2703618.0),
           (      7344284.0,      3174942.0,      1221015.0)),
          ((      -843830.0,     -1808970.0,      -791729.0),
           (      -631021.0,     -1305782.0,      -457648.0),
           (            0.0,            0.0,            0.0)),
          ((       966949.0,       462593.0,       179941.0),
           (        72803.0,       542853.0,       244899.0),
           (            0.0,            0.0,            0.0)),
          ((     -1638971.0,      -431177.0,      -143132.0),
           (      -419113.0,      2774535.0,      1304022.0),
           (            0.0,            0.0,            0.0)),
          ((      1677068.0,       719941.0,       275329.0),
           (        48115.0,      1197291.0,       528005.0),
           (            0.0,            0.0,            0.0)),
          ((      -717856.0,     -1499479.0,      -655817.0),
           (          988.0,        10963.0,         4754.0),
           (            0.0,            0.0,            0.0)),
          ((       444633.0,       236664.0,        93462.0),
           (       -41925.0,      -128345.0,       -56293.0),
           (            0.0,            0.0,            0.0)),
          ((      -485682.0,      -278477.0,      -112040.0),
           (      -456464.0,     -2163298.0,      -954804.0),
           (            0.0,            0.0,            0.0)),
          ((      1909214.0,      1223497.0,       503371.0),
           (        58283.0,       517234.0,       231527.0),
           (            0.0,            0.0,            0.0)),
          ((    -14407139.0,     -8210049.0,     -3343552.0),
           (        -1258.0,       -10805.0,        -4990.0),
           (            0.0,            0.0,            0.0)),
          ((     23012527.0,     13085341.0,      5345171.0),
           (         2664.0,         7201.0,         3115.0),
           (            0.0,            0.0,            0.0)),
          ((    -12302368.0,     -6887729.0,     -2819985.0),
           (        22991.0,        13748.0,         4893.0),
           (            0.0,            0.0,            0.0)),
          ((      3807927.0,      2019663.0,       829172.0),
           (        -1242.0,        29153.0,        12266.0),
           (            0.0,            0.0,            0.0)),
          ((     -4552802.0,     -3528381.0,     -1569026.0),
           (          176.0,         -180.0,         -115.0),
           (            0.0,            0.0,            0.0)),
          ((     -4624239.0,     -2425248.0,     -1101396.0),
           (        -3936.0,         5331.0,         3964.0),
           (            0.0,            0.0,            0.0)),
          ((      9120390.0,      4898357.0,      2187800.0),
           (          -24.0,          125.0,           59.0),
           (            0.0,            0.0,            0.0)),
          ((     -1295444.0,      -510225.0,      -253833.0),
           (            1.0,            2.0,            6.0),
           (            0.0,            0.0,            0.0)),
          ((       751458.0,       828896.0,       400581.0),
           (           70.0,            2.0,          -11.0),
           (            0.0,            0.0,            0.0)),
          ((      -780008.0,      -169373.0,      -164954.0),
           (           -9.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -61089.0,      -236914.0,       179399.0),
           (            3.0,            2.0,            1.0),
           (            0.0,            0.0,            0.0)),
          (( -11870505667.0,  -8689067627.0,  -3665932507.0),
           (          -28.0,           -1.0,           33.0),
           (            0.0,            0.0,            0.0)),
          ((       275072.0,       423036.0,       147122.0),
           (          -61.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((     -1649290.0,      -176082.0,        16277.0),
           (           -1.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((       944548.0,       339561.0,        84906.0),
           (            0.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       140715.0,      -133721.0,       -70788.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -42500.0,       -30186.0,       -10527.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63584.0,        93893.0,        39227.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11120.0,        13728.0,         6943.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15407.0,         8323.0,         4305.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3153.0,       -44775.0,       -20280.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        21191.0,         5681.0,         1875.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -37478.0,       -72103.0,       -31404.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       496563.0,      -175483.0,       -93219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -51278.0,       119929.0,        55908.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -82148.0,         1804.0,         4159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       686972.0,      -124123.0,       -75693.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -531236.0,      -339520.0,      -139074.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2153735.0,       917368.0,       352089.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    692618846.0,    125857421.0,     39081064.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2827734.0,      1235167.0,       478165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2145047.0,      -986247.0,      -384820.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -210338.0,      -109859.0,       -43916.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        36476.0,        29693.0,        12519.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -15181.0,        -6180.0,        -2264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11804.0,         3554.0,         1103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6515.0,         -718.0,         -438.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7847.0,         1369.0,          410.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        33230.0,        -3453.0,        -2367.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -93901.0,       -17386.0,        -4740.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -63369.0,       177005.0,        82698.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        53787.0,         1754.0,         -865.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -49810.0,       -13780.0,        -5468.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5711.0,        54071.0,        25036.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -92092.0,        18606.0,        10163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       298189.0,        -2813.0,        -6498.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2616765.0,       101178.0,        90343.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -45048689.0,     12554529.0,      6995136.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       217182.0,       -13479.0,       -10289.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -162247.0,        27679.0,        15318.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       156858.0,       -25619.0,       -14256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       160672.0,       -45805.0,       -23808.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -254768.0,        67399.0,        35175.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14057.0,        -5014.0,        -2647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          414.0,        -1972.0,         -817.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6071.0,         5823.0,         2871.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1876.0,         5501.0,         2514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          441.0,        -1100.0,         -622.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5882.0,       -33164.0,       -15512.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1772.0,         3490.0,         1665.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1183.0,         -920.0,         -408.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1975.0,        -8155.0,        -3801.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2735.0,        -1376.0,         -533.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1898.0,         1755.0,          778.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2914585.0,     -2587875.0,     -1265442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -508.0,        -4430.0,        -2199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,         1240.0,          606.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -128.0,        -1173.0,         -542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          323.0,         -471.0,         -228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -293.0,          302.0,          205.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,         -334.0,         -151.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          501.0,         1066.0,          483.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -111.0,          117.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           25.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -817.0,        -2044.0,         -967.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,          -47.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          145.0,          -86.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1460.0,         3999.0,         1797.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           36.0,           24.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -135.0,         -174.0,          -91.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,          406.0,          206.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          113.0,          885.0,          380.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          626.0,          -36.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1576.0,          199.0,          -96.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -114304.0,       322291.0,       150686.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3964.0,         -538.0,          254.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1716.0,          -14.0,         -185.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          554.0,           58.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -349.0,          -22.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          155.0,          -10.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,           87.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -165.0,          -55.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          -84.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           18.0,           -5.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          278.0,          181.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           28.0,           57.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -54.0,          -48.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -419.0,         -335.0,         -143.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           92.0,           35.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -47.0,         -243.0,         -108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          133.0,          186.0,           66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -238.0,         -257.0,          -97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,          298.0,          137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -954.0,       -22607.0,       -10514.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5840.0,       -10570.0,        -4506.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          449.0,          787.0,          334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -120.0,         -151.0,          -55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -58.0,           66.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -117.0,          -52.0,          -36.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -6.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,          -18.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,            4.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,           10.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           35.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -69.0,          -13.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           11.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -24.0,          -23.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           71.0,          -26.0,          -18.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,           11.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -64.0,           26.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           17.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,          -33.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           52.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2226.0,         2609.0,         1137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           58.0,           24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            2.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -4.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           45.0,           78.0,           25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -20.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           18.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -14.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,            1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -6.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -31.0,          -20.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           36.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,           -3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          326.0,         -468.0,         -288.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           -9.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,            8.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,           21.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -349.0,         -151.0,          -58.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           16.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            2.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           24.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -8.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -19.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           14.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,            3.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,          -45.0,          -16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           13.0,           49.0,           17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,            5.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -8.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,          -30.0,          -17.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           14.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,            3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           -2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            4.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            4.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            1.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            3.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           56.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))),

         (((     -1403833.0,      -578615.0,      -255799.0),
           (     -3038198.0,     -2264462.0,      -331103.0),
           (     -6297242.0,      1354160.0,       597244.0)),
          ((      1540247.0,      1045635.0,       461562.0),
           (      -753958.0,      -153861.0,      -168839.0),
           (            0.0,            0.0,            0.0)),
          ((      -617433.0,      -150377.0,       -83134.0),
           (       167723.0,      -242820.0,      -123041.0),
           (            0.0,            0.0,            0.0)),
          ((       757293.0,       663441.0,       327306.0),
           (      -601371.0,     -1420207.0,      -613740.0),
           (            0.0,            0.0,            0.0)),
          ((     -1167996.0,      -610434.0,      -292933.0),
           (     -1128819.0,      -280401.0,       -13279.0),
           (            0.0,            0.0,            0.0)),
          ((      -350272.0,        96500.0,        81804.0),
           (        -2265.0,        -3575.0,        -1005.0),
           (            0.0,            0.0,            0.0)),
          ((      -357908.0,      -104370.0,       -52239.0),
           (        11030.0,        53536.0,        16958.0),
           (            0.0,            0.0,            0.0)),
          ((       376104.0,       217786.0,       102838.0),
           (      -295733.0,      1323731.0,       534151.0),
           (            0.0,            0.0,            0.0)),
          ((     -1252140.0,      -304702.0,      -146052.0),
           (       180533.0,      -380606.0,      -168717.0),
           (            0.0,            0.0,            0.0)),
          ((     10332733.0,      4084908.0,      1880484.0),
           (       -10222.0,        12036.0,         5988.0),
           (            0.0,            0.0,            0.0)),
          ((    -16360901.0,     -6773542.0,     -3101550.0),
           (        10519.0,       -10637.0,        -5012.0),
           (            0.0,            0.0,            0.0)),
          ((      8704544.0,      3759031.0,      1705487.0),
           (          194.0,        16700.0,         8669.0),
           (            0.0,            0.0,            0.0)),
          ((     -2688594.0,     -1162971.0,      -511390.0),
           (       -14581.0,        -5881.0,        -2563.0),
           (            0.0,            0.0,            0.0)),
          ((      3071528.0,      2828691.0,      1298671.0),
           (          -15.0,          101.0,           11.0),
           (            0.0,            0.0,            0.0)),
          ((      2917817.0,      2422772.0,       980211.0),
           (         1732.0,          961.0,         2294.0),
           (            0.0,            0.0,            0.0)),
          ((     -5646737.0,     -4437419.0,     -1853450.0),
           (          -21.0,          174.0,           86.0),
           (            0.0,            0.0,            0.0)),
          ((       762130.0,       824273.0,       319811.0),
           (            3.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((       198943.0,      -446680.0,      -156089.0),
           (          -43.0,           18.0,           12.0),
           (            0.0,            0.0,            0.0)),
          ((      -121960.0,       618076.0,       155379.0),
           (           -3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       464628.0,     -1255973.0,      -212721.0),
           (            5.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0)),
          ((  12453801299.0,  -7765534446.0,  -3896593434.0),
           (          -42.0,           28.0,           44.0),
           (            0.0,            0.0,            0.0)),
          ((      -217102.0,      -206135.0,      -140020.0),
           (           34.0,           -2.0,           -9.0),
           (            0.0,            0.0,            0.0)),
          ((        71041.0,       959429.0,       529105.0),
           (            0.0,           -1.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((       -28369.0,      -530074.0,      -311004.0),
           (            1.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       117811.0,        22780.0,        -3717.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8877.0,        85449.0,        43100.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9023.0,        39573.0,        15164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11261.0,        34548.0,        16051.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          432.0,         -908.0,         -557.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        25179.0,         -198.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -36487.0,        -5091.0,        -2102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       314483.0,        58865.0,        23797.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -143008.0,      -448603.0,      -206354.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -19989.0,      -119584.0,       -58337.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       206268.0,        61224.0,        25486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27699.0,       371564.0,       162961.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       362226.0,      -372497.0,      -169499.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1686654.0,       718826.0,       322724.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -508702827.0,   -450323556.0,   -192851053.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2521296.0,        51443.0,        15040.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1997482.0,       -36245.0,       -11062.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       176891.0,         9373.0,         5000.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -20199.0,       -16557.0,        -7732.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        17500.0,        -1780.0,         -964.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7450.0,       -10945.0,        -4774.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9888.0,        -5180.0,        -2324.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8299.0,        -1598.0,         -804.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5928.0,       -38126.0,       -17749.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        88511.0,        20188.0,         8808.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -287674.0,         3496.0,         7775.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -56601.0,       -19035.0,        -8575.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        49142.0,        11930.0,         5501.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1578.0,       -53230.0,       -25694.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        59497.0,        35654.0,        17200.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -243606.0,      -112163.0,       -51350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2182031.0,      1025620.0,       463513.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -28495388.0,     36784010.0,     17593063.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -160538.0,       -75605.0,       -34027.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       124726.0,        67242.0,        29912.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -119463.0,       -78387.0,       -34779.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -104564.0,       -92632.0,       -40948.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       172767.0,       139362.0,        61635.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8294.0,        -8960.0,        -4069.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          236.0,          -26.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1940.0,         5678.0,         2499.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8962.0,         4030.0,         2174.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          182.0,         -979.0,         -385.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9603.0,        29239.0,        13193.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1145.0,         3216.0,         1583.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1146.0,        -1029.0,         -401.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6415.0,        -4313.0,        -1804.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3290.0,         4345.0,         1924.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7821.0,        -7684.0,        -3315.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3820304.0,      1382360.0,       531641.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14319.0,        10443.0,         4231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3252.0,        -2008.0,         -789.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          465.0,         1072.0,          442.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           84.0,          671.0,          287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          156.0,          489.0,          230.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           98.0,          154.0,           45.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          466.0,          594.0,          287.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          419.0,          -13.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,          -43.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2304.0,          567.0,          296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -133.0,         -164.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          219.0,           69.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3991.0,        -2301.0,        -1123.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          247.0,          393.0,          118.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -129.0,         -553.0,         -166.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          159.0,          798.0,          240.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -514.0,          127.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          834.0,          330.0,          -41.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1941.0,        -1401.0,          191.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        63360.0,      -334801.0,      -154629.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3550.0,         2936.0,         -626.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1143.0,         -691.0,          420.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          374.0,          178.0,         -156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -374.0,          143.0,          145.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -27.0,          -50.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          154.0,          -96.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          118.0,         -133.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           24.0,         -124.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            5.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,         -345.0,         -165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,          -21.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           -5.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -327.0,         -437.0,         -198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           38.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,           -8.0,          -25.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          107.0,          219.0,          129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,         -205.0,         -121.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -56.0,          352.0,          207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -37008.0,         7563.0,         5578.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          949.0,        -7749.0,        -4795.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -137.0,          545.0,          354.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,         -166.0,          -95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,           50.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           62.0,          -54.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           26.0,            3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -43.0,          -15.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           15.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           63.0,          -29.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,           33.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,           38.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,          -48.0,          -21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,          130.0,           63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           16.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,           13.0,           14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -23.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,            6.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          799.0,         3221.0,         1456.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           46.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,          -11.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           17.0,          -13.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           95.0,           47.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -13.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,           28.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -18.0,           16.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,            2.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,            9.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -23.0,           11.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -40.0,           19.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           14.0,          -31.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          386.0,          439.0,          195.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           15.0,           -5.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           20.0,           -1.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -44.0,           39.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          362.0,         -150.0,          -79.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           -3.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           10.0,           16.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -14.0,          -27.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -15.0,            1.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -21.0,           -3.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,          -16.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           -5.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,            5.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,            0.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -34.0,          -72.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,           48.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -6.0,            9.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -10.0,            5.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           17.0,            6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           27.0,          -35.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -7.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,           -4.0,           -4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            0.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -5.0,            0.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            2.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,           -4.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -4.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -1.0,            1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           25.0,          -50.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,            1.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // jup
    jupst   : array [1..1,1..134,0..2,1..3] of double = (
         (((     19820380.0,    -22094577.0,     -9923606.0),
           (     13570154.0,     24069721.0,     10505137.0),
           (   -561871964.0,    702108697.0,    314731889.0)),
          ((    -53574306.0,     45595116.0,     20705246.0),
           (      5381293.0,      4285640.0,      1740676.0),
           (        -5773.0,        47504.0,        21146.0)),
          ((     22145035.0,    -18046786.0,     -8268838.0),
           (   -158258414.0,   -117239454.0,    -46698075.0),
           (           81.0,           68.0,           28.0)),
          ((    -20228022.0,     19408074.0,      8793351.0),
           (    -18049511.0,    -14285024.0,     -5741672.0),
           (         -136.0,          -34.0,          -14.0)),
          ((     20811922.0,    -20136534.0,     -9112887.0),
           (     46369699.0,     32095049.0,     12638590.0),
           (            0.0,            0.0,            0.0)),
          ((    -28138972.0,     27265472.0,     12401460.0),
           (     -9070925.0,     -6072760.0,     -2388539.0),
           (            0.0,            0.0,            0.0)),
          ((     11194807.0,    -11804185.0,     -5321236.0),
           (     -1720707.0,     -1136130.0,      -437381.0),
           (            0.0,            0.0,            0.0)),
          ((     -8336913.0,      8809546.0,      3971881.0),
           (     -1508872.0,      -962245.0,      -369754.0),
           (            0.0,            0.0,            0.0)),
          ((     32095978.0,    -33624796.0,    -15059013.0),
           (     -6123860.0,     -3723083.0,     -1425591.0),
           (            0.0,            0.0,            0.0)),
          ((    -51670936.0,     56983194.0,     25476767.0),
           (       430566.0,       254580.0,        97315.0),
           (            0.0,            0.0,            0.0)),
          ((     31517274.0,    -35097889.0,    -15673831.0),
           (       126876.0,        37656.0,        12443.0),
           (            0.0,            0.0,            0.0)),
          ((    -11176310.0,     12941736.0,      5721103.0),
           (       -16048.0,         -636.0,          206.0),
           (            0.0,            0.0,            0.0)),
          ((     22249989.0,     -1464640.0,     -1433615.0),
           (         -555.0,        -1071.0,         -427.0),
           (            0.0,            0.0,            0.0)),
          ((     -8658971.0,      8776014.0,      4016842.0),
           (          -56.0,          -50.0,          -23.0),
           (            0.0,            0.0,            0.0)),
          ((     43558627.0,    -49230344.0,    -22314635.0),
           (          -32.0,          -17.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((    -51526015.0,     56255633.0,     25559315.0),
           (          -14.0,           -4.0,            0.0),
           (            0.0,            0.0,            0.0)),
          (( -30294062001.0,  38502992927.0,  17241854140.0),
           (           28.0,           20.0,            7.0),
           (            0.0,            0.0,            0.0)),
          ((     -4117661.0,      2841467.0,      1327414.0),
           (           31.0,           30.0,           12.0),
           (            0.0,            0.0,            0.0)),
          ((     11431889.0,     -7105972.0,     -3365250.0),
           (          160.0,          138.0,           50.0),
           (            0.0,            0.0,            0.0)),
          ((     -5819964.0,     23067737.0,     10005962.0),
           (          -11.0,           -9.0,           -4.0),
           (            0.0,            0.0,            0.0)),
          ((      6840903.0,     -9016876.0,     -4042346.0),
           (          -11.0,           -5.0,           -6.0),
           (            0.0,            0.0,            0.0)),
          ((     -1477874.0,      1885644.0,       850525.0),
           (           15.0,           10.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((      -195450.0,      -489488.0,      -120623.0),
           (          119.0,          146.0,           55.0),
           (            0.0,            0.0,            0.0)),
          ((    -10356864.0,      8981720.0,      4080679.0),
           (           -6.0,           -6.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((     24457521.0,    -21975776.0,     -9973173.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     26102192.0,    -28863106.0,    -12965508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -39832328.0,     37819708.0,     17136453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3316074.0,     -3273724.0,     -1482486.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2619386.0,      3131998.0,      1262051.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3211968.0,     -3147235.0,     -1425329.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -9574673.0,     10041129.0,      4537588.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((  -1029860910.0,    601378217.0,    282933041.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -29981445.0,     37048598.0,     16526742.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     23255889.0,    -25391525.0,    -11418155.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -9181948.0,      9689539.0,      4368840.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      8715686.0,      -151895.0,      -262106.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -7760764.0,      8904168.0,      4001432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      5012144.0,     -5636196.0,     -2535550.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1181360.0,       914067.0,       419124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -4706253.0,      5375647.0,      2417286.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     24229601.0,    -28257739.0,    -12693603.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -31328283.0,     36367139.0,     16345427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     22110225.0,    -25957543.0,    -11661437.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -103243416.0,    122882952.0,     55178163.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      7368572.0,     -6137549.0,     -2798900.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    139936827.0,   -169863890.0,    -76220721.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -127134475.0,    156176311.0,     70049607.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     80734622.0,   -147734437.0,    -65311812.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -82837202.0,    105406497.0,     47221091.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     60341425.0,    -77880670.0,    -34873169.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -22029437.0,     28475671.0,     12750392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -11373661.0,     15098848.0,      6754906.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -13182445.0,     18225652.0,      8145111.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     14449504.0,    -19891854.0,     -8888722.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -99031.0,       141246.0,        63072.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        77194.0,      -103180.0,       -46150.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -366587.0,       560786.0,       249793.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -399136.0,       701748.0,       311704.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1062085.0,     -1701563.0,      -756998.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -564547.0,       930418.0,       413664.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2149641.0,       124818.0,       107398.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1010990.0,     -1814259.0,      -804722.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -3437428.0,      6427639.0,      2848313.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1812955.0,     -3359048.0,     -1488675.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1204064.0,     -2398260.0,     -1061144.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -174214.0,       354101.0,       156620.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28805.0,        56610.0,        25219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2676.0,        -7527.0,        -3296.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2674.0,        12377.0,         5890.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5760.0,       -11898.0,        -5339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6585.0,        17221.0,         7664.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       105972.0,      -351117.0,      -155311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -105037.0,       341868.0,       151333.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3957.0,        19653.0,         8740.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -78337.0,       -58896.0,       -23471.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          127.0,          898.0,          432.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3454.0,         1096.0,          734.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -271.0,          601.0,          253.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,         -686.0,         -277.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1819.0,         1528.0,          770.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           34.0,         -194.0,          -87.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          223.0,         -149.0,          -32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          114.0,          253.0,          108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -700.0,         -806.0,         -334.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          254.0,          758.0,          408.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          132.0,           58.0,           22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -551.0,           56.0,           59.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2979.0,        -3218.0,        -1253.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -234.0,           78.0,            9.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          119.0,         -186.0,          -74.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -190.0,          809.0,          344.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          193.0,        -1152.0,         -469.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -115.0,          591.0,          250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -311.0,         4201.0,         1949.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -130.0,         -109.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           31.0,          100.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -251.0,         -343.0,         -126.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          251.0,          119.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -75.0,          -25.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           88.0,          -47.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -86.0,           26.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          123.0,          -77.0,          -40.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -136.0,          -89.0,          -19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -50.0,         -195.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           48.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -104.0,          -72.0,          -14.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           50.0,          -32.0,           -8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -28.0,          -65.0,          -22.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -63.0,          -83.0,          -27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           41.0,           38.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           16.0,          -17.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           11.0,          -28.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -13.0,          -37.0,          -12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            3.0,           -6.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,          -14.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,          -18.0,           -6.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            6.0,          -18.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           30.0,           23.0,            8.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           -3.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          125.0,           10.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27352.0,       -12123.0,        -5256.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -174.0,           -1.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           23.0,           -2.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -119.0,          178.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -7.0,            7.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -3.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7971.0,       -14225.0,        -6904.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,           -1.0,            0.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -22.0,           14.0,            7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -202.0,         -145.0,          -63.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           48.0,           10.0,            1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          573.0,          454.0,          192.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -112.0,          -31.0,          -13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           21.0,          -53.0,          -30.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // sat
    satst   : array [1..1,1..117,0..2,1..3] of double = (
         (((   -983495385.0,    725219354.0,    341700109.0),
           (  -6937329785.0,  -7691462763.0,  -2879206244.0),
           (      -878322.0,       381260.0,       198807.0)),
          ((  -1475082148.0,   1087700929.0,    513258996.0),
           (    593821265.0,    938820424.0,    353475230.0),
           (        -2439.0,         1757.0,         1252.0)),
          ((   5277327001.0,  -3939025410.0,  -1853346606.0),
           (  -3920100203.0,  -3995558480.0,  -1473366623.0),
           (            0.0,            0.0,            0.0)),
          ((  -4184629996.0,   3006361186.0,   1421637551.0),
           (   -522553764.0,   -293788530.0,    -93902268.0),
           (            0.0,            0.0,            0.0)),
          ((   1085898500.0,   -804275203.0,   -378702366.0),
           (     -2094892.0,     -9697098.0,     -3979297.0),
           (            0.0,            0.0,            0.0)),
          ((  -5196179618.0,   3931257951.0,   1845607362.0),
           (     -1414369.0,     -2152691.0,      -732193.0),
           (            0.0,            0.0,            0.0)),
          (( -69987883366.0,  52271496493.0,  24600802929.0),
           (       150933.0,       246429.0,        91190.0),
           (            0.0,            0.0,            0.0)),
          ((   1701861797.0,  -1307435362.0,   -612307763.0),
           (        19361.0,       -28945.0,       -14119.0),
           (            0.0,            0.0,            0.0)),
          ((   -765704194.0,    588275630.0,    274990845.0),
           (        -9263.0,        41224.0,        13329.0),
           (            0.0,            0.0,            0.0)),
          ((   -232102981.0,    234961885.0,    108054404.0),
           (        72992.0,       -48781.0,       -18604.0),
           (            0.0,            0.0,            0.0)),
          ((    295472584.0,   -340725721.0,   -153589455.0),
           (        -1415.0,          127.0,          314.0),
           (            0.0,            0.0,            0.0)),
          ((    309708383.0,   -297467564.0,   -136046279.0),
           (          275.0,         -330.0,          120.0),
           (            0.0,            0.0,            0.0)),
          ((   -333987699.0,    383522234.0,    172565396.0),
           (          632.0,          -43.0,          191.0),
           (            0.0,            0.0,            0.0)),
          ((    132210786.0,   1974831956.0,    810226936.0),
           (         -265.0,           13.0,           81.0),
           (            0.0,            0.0,            0.0)),
          ((   -678419823.0,    513306943.0,    240268283.0),
           (         1790.0,          -13.0,          -90.0),
           (            0.0,            0.0,            0.0)),
          ((    372575309.0,   -281112275.0,   -131631451.0),
           (          -79.0,           68.0,           31.0),
           (            0.0,            0.0,            0.0)),
          ((    -30596841.0,    -88918930.0,    -35435468.0),
           (           -3.0,          -63.0,          -48.0),
           (            0.0,            0.0,            0.0)),
          ((     -2597453.0,     23723483.0,     10020357.0),
           (          -29.0,           60.0,           28.0),
           (            0.0,            0.0,            0.0)),
          ((    -22786241.0,      7136573.0,      3822006.0),
           (           -4.0,           22.0,            9.0),
           (            0.0,            0.0,            0.0)),
          ((     34264617.0,    -15723433.0,     -7752054.0),
           (           25.0,           -9.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((    262338258.0,    -48897345.0,    -28734098.0),
           (            8.0,            4.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((   -230060526.0,    145388471.0,     66946502.0),
           (            7.0,            5.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((      5279324.0,     -5369803.0,     -2266580.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2440049.0,       434366.0,         1265.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       746339.0,      -313380.0,      -174661.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     15202678.0,     -2807257.0,     -1921354.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -15319105.0,      5257724.0,      2948570.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      8985439.0,     -3276536.0,     -1815137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -4152916.0,      1469360.0,       824655.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       947981.0,      -615806.0,      -288039.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -810986.0,       376983.0,       193434.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      3278617.0,     -1474116.0,      -764780.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -2497851.0,      1834607.0,       851703.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -368453.0,       164179.0,        85183.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       321683.0,       -97851.0,       -77369.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -41597.0,        13943.0,         7614.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        57021.0,      -130889.0,       -51775.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -28928.0,         3351.0,         2677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        11840.0,        -4312.0,        -2164.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       244086.0,       160748.0,        52003.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4140.0,        -2309.0,        -2096.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8721.0,         -212.0,         2451.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       108343.0,        -2937.0,       -17312.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -13216.0,        16645.0,         4859.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        10076.0,        -9375.0,        -3627.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -14933.0,       -33728.0,       -12937.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -12255.0,         5519.0,         2647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -43477.0,       131681.0,        55917.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       128385.0,       -54275.0,       -28890.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -9746.0,         2785.0,         1946.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        56227.0,         1504.0,        -5917.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8460.0,        10624.0,         2251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13241.0,        -8568.0,        -2791.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27333.0,        13792.0,         4704.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        44578.0,       -27193.0,       -11311.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -10924.0,         5724.0,         1987.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        27167.0,       -13049.0,        -4088.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        80831.0,        10408.0,         8681.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -83193.0,        39018.0,         9079.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1327.0,        -1021.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19644.0,         5586.0,         -646.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          967.0,         1554.0,         -319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -266.0,         -552.0,         -137.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        13139.0,        -4257.0,        -2622.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -151.0,          367.0,          180.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5101.0,        17072.0,         6579.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -492.0,           93.0,          165.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9996.0,         3633.0,          156.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          658.0,          756.0,         -197.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,         -552.0,            5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -155.0,          442.0,         -130.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7650.0,         -774.0,         -637.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -66.0,          358.0,           73.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1168.0,        -2697.0,         -677.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -664.0,         9962.0,         3584.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4233.0,         2050.0,          392.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          592.0,          424.0,         -207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -297.0,         -392.0,           97.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          715.0,          595.0,         -264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2906.0,         -277.0,          285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          707.0,          746.0,         -181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -266.0,         -169.0,           88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -799.0,         3207.0,         1402.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          580.0,           29.0,         -207.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -274.0,           -2.0,           89.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2062.0,         1313.0,          219.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -57.0,         -105.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1996.0,          898.0,          140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -116.0,           79.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -647.0,         1438.0,          580.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          577.0,         -279.0,         -158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -214.0,         4450.0,         2053.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          426.0,          851.0,          251.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          127.0,          -95.0,          -56.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -301.0,          181.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1324.0,          336.0,          -15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -200.0,          180.0,           94.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -279.0,          560.0,          212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          305.0,          385.0,          116.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -60.0,           26.0,           28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          496.0,          340.0,           70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -173.0,          238.0,           88.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          128.0,          188.0,           62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          185.0,          218.0,           64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -96.0,          110.0,           38.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,           83.0,           29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           76.0,          120.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -62.0,           57.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,           49.0,           15.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -312.0,          267.0,          122.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27099.0,       -12357.0,        -5363.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -146.0,          200.0,           95.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7940.0,       -14204.0,        -6894.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -178.0,         -162.0,          -71.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           91.0,           -4.0,           -7.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          471.0,          397.0,          167.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            4.0,          -51.0,          -29.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // ura
    urast   : array [1..1,1..87,0..2,1..3] of double = (
         ((( 138123590390.0, 120548530335.0,  50840544173.0),
           (   -215744940.0,    397225297.0,    173438856.0),
           (         -495.0,         -405.0,         -172.0)),
          ((   2888255375.0,   2128720476.0,    895764460.0),
           (     30152589.0,    -74938646.0,    -31213540.0),
           (            0.0,            0.0,            0.0)),
          ((  -5016815236.0,  -3727672347.0,  -1564819370.0),
           (      -149804.0,       354034.0,       142067.0),
           (            0.0,            0.0,            0.0)),
          ((   2433356302.0,   4525209124.0,   1949482765.0),
           (      -226052.0,       142804.0,        67762.0),
           (            0.0,            0.0,            0.0)),
          ((  -5732879782.0,  -3942208745.0,  -1645927206.0),
           (        -2995.0,         5412.0,         2246.0),
           (            0.0,            0.0,            0.0)),
          ((   1721258203.0,   1047448956.0,    433869605.0),
           (          375.0,        -2592.0,         -873.0),
           (            0.0,            0.0,            0.0)),
          ((    642047753.0,    674899667.0,    287581991.0),
           (         -948.0,            2.0,           55.0),
           (            0.0,            0.0,            0.0)),
          ((  -2702633632.0,  -2568770606.0,  -1088057623.0),
           (          319.0,         -174.0,          -90.0),
           (            0.0,            0.0,            0.0)),
          ((   2986712612.0,   2624876276.0,   1108380121.0),
           (         1256.0,        -1049.0,         -530.0),
           (            0.0,            0.0,            0.0)),
          ((  -1094547407.0,   -918684338.0,   -387132931.0),
           (           83.0,          -36.0,          -20.0),
           (            0.0,            0.0,            0.0)),
          ((    816722442.0,    679690119.0,    286221072.0),
           (          -73.0,           76.0,           37.0),
           (            0.0,            0.0,            0.0)),
          ((  -1264421153.0,  -1042091314.0,   -438654606.0),
           (          -22.0,           21.0,           15.0),
           (            0.0,            0.0,            0.0)),
          ((   -333559480.0,   -269780636.0,   -113401294.0),
           (           14.0,          -12.0,           -2.0),
           (            0.0,            0.0,            0.0)),
          ((    993664949.0,    810997817.0,    341099437.0),
           (           97.0,          -59.0,          -30.0),
           (            0.0,            0.0,            0.0)),
          ((   -372800674.0,   -299534458.0,   -125958425.0),
           (           -9.0,            9.0,            4.0),
           (            0.0,            0.0,            0.0)),
          ((    484588974.0,    389501507.0,    163790357.0),
           (          -13.0,          -16.0,          -11.0),
           (            0.0,            0.0,            0.0)),
          ((    798538879.0,    685397285.0,    288895695.0),
           (           15.0,          -10.0,           -5.0),
           (            0.0,            0.0,            0.0)),
          ((    561027113.0,    479431273.0,    202074872.0),
           (           -5.0,          -21.0,          -10.0),
           (            0.0,            0.0,            0.0)),
          ((  -1957740396.0,  -1587092256.0,   -667533339.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    607943409.0,    490705078.0,    206354542.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -238752526.0,   -192715876.0,    -81044422.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     65522155.0,     53006228.0,     22297319.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -11732874.0,     -9480270.0,     -3985623.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1595210.0,      1277177.0,       537699.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2028256.0,      1682111.0,       707503.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -774888.0,      -628658.0,      -265371.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       554469.0,       494570.0,       209108.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -286225.0,      -198409.0,       -82743.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       216103.0,       165692.0,        69308.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -322083.0,      -245447.0,      -103113.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       458441.0,       346916.0,       145508.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -475484.0,      -362053.0,      -153945.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       471922.0,       370935.0,       156066.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1825246.0,      2814756.0,      1213520.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -3151574.0,     -2471768.0,     -1039796.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        78301.0,        60091.0,        25257.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -75520.0,       -53372.0,       -22474.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        53862.0,        47913.0,        20350.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -25356.0,        -6770.0,        -2620.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19721.0,        15646.0,         6648.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        16395.0,       -15035.0,        -6793.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4623.0,        -3610.0,        -1575.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         5070.0,         3452.0,         1407.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          319.0,        -3196.0,        -1488.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1414.0,         1213.0,          494.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -959.0,         -700.0,         -310.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1099.0,          354.0,          110.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -24161.0,        19916.0,         9336.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -21612.0,       -12849.0,        -5215.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2855.0,         1627.0,          665.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          284.0,          588.0,          260.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          990.0,         1565.0,          663.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1765.0,        -4252.0,        -1835.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          417.0,        -1292.0,         -591.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1490.0,         1577.0,          672.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1618.0,        -1607.0,         -684.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1110.0,          577.0,          278.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          348.0,          186.0,           75.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          172.0,          293.0,          125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3615.0,        -4006.0,        -1668.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2225.0,         3742.0,         1592.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           53.0,         -322.0,         -140.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -751.0,          255.0,          120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -83.0,         -192.0,          -82.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           92.0,           68.0,           32.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -151.0,         -134.0,          -57.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -98.0,           42.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -9.0,           83.0,           37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          239.0,          203.0,           86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -651.0,         -535.0,         -226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1336.0,         1068.0,          450.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1141.0,         3771.0,         1762.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27268.0,       -12020.0,        -5212.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -193.0,         -155.0,          -66.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1350.0,         1086.0,          460.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -664.0,         -536.0,         -228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -695.0,         -558.0,         -236.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          142.0,          114.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -208.0,          106.0,           55.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,          109.0,           46.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         7663.0,       -14443.0,        -6995.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          349.0,          248.0,          103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -170.0,         -123.0,          -54.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          106.0,           44.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -185.0,         -167.0,          -70.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          581.0,          540.0,          229.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            9.0,          -56.0,          -31.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // nep
    nepst   : array [1..1,1..80,0..2,1..3] of double = (
         ((( 274984744244.0, -87426663564.0, -42629038758.0),
           (   4355795971.0,  13039662330.0,   5232020163.0),
           (     16323244.0,     -2752369.0,     -1561912.0)),
          ((    -10373482.0,   -317285640.0,   -129066296.0),
           (  -2262594667.0,  -5512816241.0,  -2195912211.0),
           (           74.0,          188.0,           71.0)),
          ((   1983794100.0,   -912972345.0,   -423091871.0),
           (  -1278419385.0,  -2658170810.0,  -1045032024.0),
           (           81.0,           49.0,           18.0)),
          ((  -4374189008.0,   1467452168.0,    709523144.0),
           (      3250132.0,      3414106.0,      1185307.0),
           (          -21.0,           -4.0,           -1.0)),
          ((    979133641.0,   -382243331.0,   -180839464.0),
           (        34463.0,         4898.0,          565.0),
           (            0.0,            0.0,            0.0)),
          ((    -22592110.0,     -8655432.0,     -2979774.0),
           (         1411.0,          -38.0,         -112.0),
           (            0.0,            0.0,            0.0)),
          ((    683872279.0,    -66989947.0,    -43488782.0),
           (         1296.0,         -158.0,         -127.0),
           (            0.0,            0.0,            0.0)),
          ((   -583272125.0,     47760898.0,     33556496.0),
           (        -2454.0,         -400.0,         -178.0),
           (            0.0,            0.0,            0.0)),
          ((   -171646201.0,     20281971.0,     12572306.0),
           (         -182.0,           -5.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((     73870577.0,     -9573627.0,     -5766990.0),
           (         -156.0,           50.0,           21.0),
           (            0.0,            0.0,            0.0)),
          ((    -20096145.0,      2993947.0,      1729055.0),
           (          184.0,           39.0,            3.0),
           (            0.0,            0.0,            0.0)),
          ((     40823540.0,     -5653959.0,     -3346681.0),
           (          -69.0,          -17.0,            1.0),
           (            0.0,            0.0,            0.0)),
          ((    -42312776.0,      5994842.0,      3524557.0),
           (           30.0,            9.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((      8794252.0,     -1286043.0,      -749479.0),
           (         -121.0,          -18.0,           -3.0),
           (            0.0,            0.0,            0.0)),
          ((     -1598659.0,       266191.0,       151551.0),
           (           38.0,            5.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2198694.0,      -838451.0,      -387734.0),
           (          -21.0,           -4.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((     -2375636.0,       454352.0,       245378.0),
           (           36.0,           11.0,            2.0),
           (            0.0,            0.0,            0.0)),
          ((       945525.0,      -188076.0,      -102007.0),
           (           11.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -4109594.0,     46340133.0,     19960120.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      4506170.0,      -767761.0,      -435551.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -4848470.0,       826166.0,       471022.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      7565327.0,     -1295293.0,      -738572.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -25400117.0,      4396228.0,      2507354.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     35787636.0,     -6259609.0,     -3562754.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    -14535490.0,      2549510.0,      1451689.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       775939.0,      -136072.0,       -77756.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -66587.0,        11112.0,         6479.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19577.0,        -2773.0,        -1746.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -8419.0,         1387.0,          814.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2708.0,         -578.0,         -314.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -4329.0,         1342.0,          679.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5538.0,          151.0,          249.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          407.0,          376.0,          158.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        14955.0,       -15494.0,        -6598.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1117.0,          735.0,          291.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3694.0,        -1759.0,         -636.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3909.0,         1296.0,          655.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -16031.0,        -5159.0,        -1989.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       338279.0,      1067392.0,       449404.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          887.0,           81.0,           12.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -6241.0,        -8502.0,        -3489.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -2.0,          438.0,          181.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -995.0,         -331.0,         -103.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          292.0,          658.0,          264.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -33.0,           33.0,           16.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -346.0,          275.0,          124.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2212.0,         -352.0,          -92.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -139.0,         -270.0,         -101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          238.0,          611.0,          250.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           29.0,         -224.0,          -69.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        28354.0,        34534.0,        14228.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7988.0,          -25.0,           33.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         2296.0,           42.0,           -5.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           61.0,          -52.0,          -24.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -813.0,         -577.0,         -226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          395.0,          266.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -223.0,         -175.0,          -67.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -11.0,           17.0,           10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            8.0,           48.0,           19.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           55.0,          251.0,          101.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -106.0,          -33.0,          -11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -42.0,           70.0,           27.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           97.0,           58.0,           21.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1315.0,         1128.0,          454.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          137.0,          131.0,           48.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -214.0,         -167.0,          -61.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            7.0,           -7.0,           -3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -55.0,            6.0,            3.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          101.0,           30.0,           11.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4874.0,          329.0,           20.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           -8.0,           -5.0,           -2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -17558.0,       -22775.0,        -9876.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           12.0,            3.0,            2.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -16.0,           -1.0,           -1.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          115.0,         -181.0,          -86.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7252.0,        14554.0,         7005.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           19.0,         -237.0,         -102.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((            5.0,           15.0,            4.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -341.0,         -486.0,         -224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           33.0,          -48.0,          -28.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
    // plu
    plust   : array [1..1,1..79,0..2,1..3] of double = (
         (((-308141951618.0, -53474238507.0,  76194312726.0),
           ( -23619870084.0, 113651312909.0,  42550902070.0),
           (  39698021724.0,   2711526731.0, -11149057957.0)),
          (( -69004997669.0,  -8902380103.0,  17953428486.0),
           (  11901342393.0, -11519133987.0,  -7191098560.0),
           (   3923378445.0,   -573334908.0,  -1252249379.0)),
          ((  28560333346.0,  23633073446.0,  -1147392365.0),
           (   1846281006.0,    356486329.0,   -421431702.0),
           (      5042282.0,      1606275.0,       512676.0)),
          ((  -2070809805.0,  13648825300.0,   4750586133.0),
           (   -144014530.0,    271758149.0,    115200605.0),
           (        -2101.0,          146.0,         -123.0)),
          ((   8062798375.0,  -5966828316.0,  -4184651506.0),
           (    402093700.0,   -941559728.0,   -390326982.0),
           (         -122.0,         -370.0,          -11.0)),
          ((  -1173981973.0,  -2888890261.0,   -582182371.0),
           (  -1339223298.0,  10101126241.0,   3546859299.0),
           (         -480.0,           53.0,          132.0)),
          ((  -1151015709.0,    767399673.0,    604375592.0),
           (   1302836962.0,  -9798244929.0,  -3414672613.0),
           (            0.0,            0.0,            0.0)),
          ((   -930342842.0,   -141773773.0,    171875767.0),
           (    144384578.0,  -1000028233.0,   -363879232.0),
           (            0.0,            0.0,            0.0)),
          ((   1212312524.0,    285591293.0,   -215445577.0),
           (   1001872207.0,  -7548975979.0,  -2649575504.0),
           (            0.0,            0.0,            0.0)),
          ((   -817449709.0,   -324390277.0,     69830441.0),
           (     15485537.0,   -126495028.0,    -43331129.0),
           (            0.0,            0.0,            0.0)),
          ((   1404965895.0,    307852499.0,   -192914545.0),
           (       266187.0,     -3580011.0,     -1106585.0),
           (            0.0,            0.0,            0.0)),
          ((   -391832163.0,     11807430.0,     93543846.0),
           (    -33174914.0,    548136189.0,    165489877.0),
           (            0.0,            0.0,            0.0)),
          ((   -370043993.0,   -149936097.0,     14873887.0),
           (     58779770.0,  -1079543718.0,   -322286686.0),
           (            0.0,            0.0,            0.0)),
          ((    447176753.0,    154405518.0,    -20378540.0),
           (        -2180.0,       319918.0,        89480.0),
           (            0.0,            0.0,            0.0)),
          ((  -1358503056.0,   -311059889.0,    129006600.0),
           (        -4356.0,        -1394.0,         1378.0),
           (            0.0,            0.0,            0.0)),
          ((    848010526.0,    184811593.0,    -79417739.0),
           (       -33874.0,         4230.0,        42625.0),
           (            0.0,            0.0,            0.0)),
          ((    119824894.0,     10471932.0,    -17783261.0),
           (         3195.0,        -3590.0,        -3621.0),
           (            0.0,            0.0,            0.0)),
          ((    -81884916.0,    -39983731.0,    -10126334.0),
           (           70.0,            2.0,          -79.0),
           (            0.0,            0.0,            0.0)),
          ((   -729771959.0,   -332339467.0,    -87706529.0),
           (          226.0,          511.0,         -365.0),
           (            0.0,            0.0,            0.0)),
          ((    795238194.0,    378397761.0,    105082526.0),
           (         -594.0,          782.0,          558.0),
           (            0.0,            0.0,            0.0)),
          ((    -17522508.0,     -6526550.0,     -1476356.0),
           (           14.0,          -84.0,          -36.0),
           (            0.0,            0.0,            0.0)),
          ((      4827827.0,      1690687.0,       384216.0),
           (          -19.0,          145.0,           74.0),
           (            0.0,            0.0,            0.0)),
          ((      1387714.0,       456787.0,        96880.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -426528.0,      -146730.0,       -52135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       450273.0,       141913.0,        57427.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1458496.0,      -143922.0,       -12088.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       824845.0,       272045.0,        97440.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -786380.0,      -250056.0,       -91112.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      1370990.0,       428324.0,       173889.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    154393557.0,     49983575.0,     21637399.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((   -587671449.0,   -190406620.0,    -83076216.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((    473166111.0,    117444514.0,     50513453.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      2721328.0,       907311.0,       413247.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((      -359174.0,      -119206.0,       -53358.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        61210.0,        21474.0,         8351.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -22308.0,        -6444.0,        -3865.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        99746.0,        25273.0,        15702.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -47626.0,       -24514.0,       -16262.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -2275.0,         -139.0,         -459.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          367.0,         1131.0,          647.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -616.0,        -1382.0,         -129.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1558.0,        -1216.0,         -703.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1019.0,         -540.0,         -224.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           39.0,         -109.0,          -49.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -5839.0,         3746.0,         2176.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          491.0,         3379.0,         1054.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -377.0,        -3142.0,         -914.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         3566.0,         3061.0,         -317.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -1213.0,          777.0,          689.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -7093.0,          227.0,         2374.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         8832.0,        -7521.0,        -4318.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1410.0,        -3258.0,        -1135.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1258.0,         8787.0,          464.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          841.0,        -3840.0,         -159.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1075.0,          415.0,          628.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -11858.0,         5506.0,         3535.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((     -1144960.0,      -334404.0,      -115911.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         9338.0,        -2262.0,        -2449.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        -3751.0,          232.0,         1064.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1145.0,          151.0,         -285.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         1462.0,        -9903.0,        -4259.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -806.0,         -127.0,           13.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((           54.0,          756.0,          282.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          -53.0,         -211.0,           77.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -261.0,          366.0,           85.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -229.0,         -572.0,          -62.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          169.0,          200.0,          -37.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -558.0,         -344.0,          198.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -264.0,         -251.0,         -231.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -257.0,          -49.0,         -125.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((        19748.0,        36079.0,        14970.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -596.0,          439.0,          199.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          558.0,        -1593.0,         -708.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         4160.0,         2378.0,          976.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((       -27891.0,       -11080.0,        -4805.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          194.0,            6.0,          -10.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -845.0,        16023.0,         7201.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((         -259.0,         -111.0,          -64.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0)),
          ((          486.0,         -388.0,         -226.0),
           (            0.0,            0.0,            0.0),
           (            0.0,            0.0,            0.0))));
implementation

end.
